import {ChangeDetectionStrategy, Component, EventEmitter, OnInit, ViewEncapsulation} from '@angular/core';
import { ActivatedRoute } from '@angular/router';

import {DisplayGrid, GridsterConfig, GridsterItem, GridsterItemComponentInterface, GridType} from 'angular-gridster2';
import { ALARMiatorService } from '../services/alarmiator.service';

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.css'],
  encapsulation: ViewEncapsulation.None
})
export class DashboardComponent implements OnInit {

  static eventStop(item: GridsterItem, itemComponent: GridsterItemComponentInterface, event: MouseEvent): void {
    // tslint:disable-next-line:no-console
    console.info('eventStop', item, itemComponent, event);
  }

  static eventStart(item: GridsterItem, itemComponent: GridsterItemComponentInterface, event: MouseEvent): void {
    // tslint:disable-next-line:no-console
    console.info('eventStart', item, itemComponent, event);
  }
  
  constructor(
    private alarmiatorService: ALARMiatorService,
    private route: ActivatedRoute,
  ) { }

  private apiKey: string;

  ngOnInit(): void {
    this.route.queryParams.subscribe(params => {
      this.apiKey = params['apiKey'];
      console.log("API-Key:",this.apiKey);
      this.alarmiatorService.setAPIKey(this.apiKey);
    });

    this.options = {
      gridType: GridType.Fit,
      displayGrid: DisplayGrid.None,
      disableWindowResize: false,
      scrollToNewItems: false,
      disableWarnings: false,
      ignoreMarginInRow: false,
      itemResizeCallback: (item) => {
        // update DB with new size
        // send the update to widgets
        this.resizeEvent.emit(item);
        //console.log(item);
      }
    };

    this.dashboard = [
      {cols: 1, rows: 1, y: 0, x: 0, type: 'auth-needed'},
    ];

    this.alarmiatorService.getConfig().subscribe( (config) => { 
      this.dashboard = config;
    })

    this.alarmiatorService.getDisconnect().subscribe( (disconnect) => { 
      this.dashboard = [
        {"cols": 1, "rows": 1, "y": 0, "x": 0, "type": "error"}
      ];
    })
  }

  options: GridsterConfig;
  dashboard: GridsterItem[];
  resizeEvent: EventEmitter<GridsterItem> = new EventEmitter<GridsterItem>();
}
