import { AfterViewInit, ChangeDetectorRef, Component, Input } from '@angular/core';
import { icon, latLng, Layer, marker, tileLayer, geoJSON, Map} from 'leaflet';
import * as L from "leaflet";
import { ALARMiatorService } from '../services/alarmiator.service';
import * as moment from "moment";

@Component({
  selector: 'app-operation-map',
  templateUrl: './operation-map.component.html',
  styleUrls: ['./operation-map.component.css']
})
export class OperationMapComponent implements AfterViewInit {

  @Input()
  widget;
  
  private map;
  private directionMap;
  private lat;
  private lon;
  private operation;
  public center;
  public distance;
  public duration;
  public zoom = 17;
  public geoJSONdata;
  public geo;
  static baseLayer: Layer[] = [
    tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '' }),
    tileLayer('http://openfiremap.de/hytiles/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' })
  ];

  options = {
    layers: [
      tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '' }),
      tileLayer('http://openfiremap.de/hytiles/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' })
    ],
    zoom: 17
  };
  markers: Layer[] = [];
  layers: Layer[] = []
  fitBounds:any;

  constructor(private alarmiatorService: ALARMiatorService, private changeDetector: ChangeDetectorRef) {
    this.alarmiatorService.getOperations().subscribe( (operation) => { 
      this.operation = operation;
        this.lat = operation.lat;
        this.lon = operation.lon;
        this.center = latLng(operation.lat, operation.lon);
        const newMarker = marker(
          [ this.lat, this.lon]
        );
        this.markers = [];
    //    this.markers = this.markers.concat(OperationMapComponent.baseLayer);
        this.markers.push(newMarker);

        if (this.map){
          setTimeout(() => {
            console.log("invalidate size");
            this.map.invalidateSize();
            this.changeDetector.detectChanges();
            window.dispatchEvent(new Event('resize'));
          },1000)
        }
          this.changeDetector.detectChanges();
          window.dispatchEvent(new Event('resize'));
    })
    this.alarmiatorService.getDirections().subscribe( (geoJSONdata) => {
      this.geoJSONdata = geoJSONdata;
      console.log(geoJSONdata);

      if (geoJSONdata != null) {
        var geo = geoJSON(geoJSONdata,  {style: {
          "color": "blue", 
          "weight": 5,
        }});
        this.geo = geo;
        this.layers = [];
       // this.layers = this.layers.concat(OperationMapComponent.baseLayer);
  
        this.layers.push(geo);
        var destination = geoJSONdata['metadata']['query']['coordinates'][1];
        const newMarker = marker(
          [ destination[1], destination[0]]
        );
    
        this.layers.push(newMarker);
        this.fitBounds = geo.getBounds();
        var distance = geoJSONdata['features'][0]['properties']['summary']['distance'] 
        this.distance = Math.round((distance /1000 + Number.EPSILON) * 100) / 100
        var duration = geoJSONdata['features'][0]['properties']['summary']['duration'];
        this.duration = moment.utc(moment.duration(duration, "seconds").asMilliseconds()).format("HH:mm:ss")
        if (this.directionMap){
          setTimeout(() => {
            console.log("invalidate size");
            this.directionMap.invalidateSize();
            this.fitBounds = geo.getBounds();
            this.changeDetector.detectChanges();
            window.dispatchEvent(new Event('resize'));
          },1000)
        }
      }
      this.changeDetector.detectChanges();
      window.dispatchEvent(new Event('resize'));
    })

  }

  ngAfterViewInit(): void {
    this.changeDetector.detectChanges();
    console.log(this.widget.type);
  }

  addMarker() {
		const newMarker = marker(
			[ this.lat, this.lon]
		);

		this.markers.push(newMarker);
  }

  removeMarker() {
		this.markers.pop();
  }

  onMapReady(map: Map){
    this.map = map;
    console.log("map ready");
    setTimeout(() => {
      console.log("invalidate size");
      map.invalidateSize();
      this.changeDetector.detectChanges();
      window.dispatchEvent(new Event('resize'));
     }, 1000);
  }

  onDirectionsMapReady(map: Map){
    this.directionMap = map;
    console.log("map ready");
    setTimeout(() => {
      console.log("invalidate size");
      map.invalidateSize();
      this.fitBounds = this.geo.getBounds();
      this.changeDetector.detectChanges();
      window.dispatchEvent(new Event('resize'));
     }, 1000);
  }
}
