import { Injectable } from '@angular/core';
import { Socket } from 'ngx-socket-io';
import { Observable } from 'rxjs';
import { SocketIOService } from './socket-io.service';
import * as hash from 'object-hash';

@Injectable({
  providedIn: 'root'
})
export class ALARMiatorService {

  public messageslist = [];

  private apiKey:string;

  private config = null;
  private gearStates = null;
  private operation = null;
  private feedbackDetails = null;
  private feedbackOverview = null;
  private feedback = null;
  private configHash = null;
  private disconnect = null;
  private directions = null;
  private assets_maintenance = null;

  constructor(private socket: SocketIOService) { 
    socket.on('connect', () => {
      console.log('connected to server');
      if (this.apiKey){
        this.authenticate();
        this.configHash = null;
        this.disconnect = null;
      }
      else {
        this.authenticate();
        console.error("NO API KEY SET")
      }
    })
    
    this.socket.on('config', (config) => {
      this.config = config;
    });

    this.socket.on('gear-state', (gearStates) => {
      this.gearStates = gearStates;
    });

    this.socket.on('operation', (operation) => {
      this.operation = operation;
    });

    this.socket.on('directions', (directions) => {
      this.directions = directions;
    });

    this.socket.on('feedback', (feedback) => {
      this.feedback = feedback;

      var feedbackDetails = {};
      feedback['feedback'].forEach((entry) => { 
        var feedbackStateLabel = Object.keys(entry)[0];         
        feedbackDetails[feedbackStateLabel] = entry[feedbackStateLabel];
      })
      this.feedbackDetails = feedbackDetails;

      var feedbackOverview = {};
        feedback['feedback'].forEach((entry) => {
          
          var feedbackStateLabel = Object.keys(entry)[0];

          feedbackOverview[feedbackStateLabel] = {};
          
          feedbackOverview[feedbackStateLabel]['count']= entry[feedbackStateLabel].length;

          feedbackOverview[feedbackStateLabel]['qual'] = {}
          feedback['qualifications'].forEach((qual) => {
            feedbackOverview[feedbackStateLabel]['qual'][qual]=0;
          })

          entry[feedbackStateLabel].forEach(person => {

            if (person.qualificationsList != null){
              if (person.qualificationsList != ""){
                let qualifications = person.qualificationsList.split(',');
                qualifications.forEach((qual) => {
                  feedbackOverview[feedbackStateLabel]['qual'][qual]+=1;
                })
              }
            }
          });         
        })

        feedback['feedbackStates'].forEach(state => {
          feedbackOverview[state.stateLabel]['stateColor'] = state.stateColor;
          feedbackOverview[state.stateLabel]['stateTextColor'] = state.stateTextColor;
        });

        this.feedbackOverview = feedbackOverview;
    });

    this.socket.on('disconnect', (disconnect) => {
      this.disconnect = disconnect;
    });

    this.socket.on('maintenance-assets', (data) => {
      this.assets_maintenance = data;
    });
  }

  
  public sendMessage(msg: string){
    console.log('Sending to server: ' + msg);
    this.socket.emit("message", msg);
  }


  public getOperations() {
    return Observable.create((observer) => {
      if (this.operation != null) {
        //observer.next(this.operation);
      }
      this.socket.on('operation', (operation) => {
        console.log('operation recieved: ' + operation);
          observer.next(operation);
      });
    });
  }

  public getMessages() {
    return Observable.create((observer) => {
      this.socket.on('updateData', (message) => {
        console.log('message recieved: ' + message);
          observer.next(message);
      });
    });
  }

  public getFeedback() {
    return Observable.create((observer) => {
      if (this.feedback != null) {
        observer.next(this.feedback);
      }
      this.socket.on('feedback', (feedback) => {
        console.log('feedback recieved: ' + feedback);
          observer.next(feedback);
      });
    });
  }

  public getFeedbackOverview() {
    return Observable.create((observer) => {
      if (this.feedbackOverview != null) {
        observer.next(this.feedbackOverview);
      }

      this.socket.on('feedback', (feedback) => {

        var feedbackOverview = {};
        feedback['feedback'].forEach((entry) => {
          
          var feedbackStateLabel = Object.keys(entry)[0];

          feedbackOverview[feedbackStateLabel] = {};
          
          feedbackOverview[feedbackStateLabel]['count']= entry[feedbackStateLabel].length;

          feedbackOverview[feedbackStateLabel]['qual'] = {}
          feedback['qualifications'].forEach((qual) => {
            feedbackOverview[feedbackStateLabel]['qual'][qual]=0;
          })

          entry[feedbackStateLabel].forEach(person => {

            if (person.qualificationsList != null){
              if (person.qualificationsList != ""){
                let qualifications = person.qualificationsList.split(',');
                qualifications.forEach((qual) => {
                  feedbackOverview[feedbackStateLabel]['qual'][qual]+=1;
                })
              }
            }
          });         
        })

        feedback['feedbackStates'].forEach(state => {
          feedbackOverview[state.stateLabel]['stateColor'] = state.stateColor;
          feedbackOverview[state.stateLabel]['stateTextColor'] = state.stateTextColor;
        });
        console.log('feedback overview received: ' + JSON.stringify(feedbackOverview));
          observer.next(feedbackOverview);
      });
    });
  }

  public getFeedbackDetails() {
    return Observable.create((observer) => {
      if (this.feedbackDetails != null) {
        observer.next(this.feedbackDetails);
      }
      this.socket.on('feedback', (feedback) => {

        var feedbackDetails = {};
        feedback['feedback'].forEach((entry) => { 
          var feedbackStateLabel = Object.keys(entry)[0];         
          feedbackDetails[feedbackStateLabel] = entry[feedbackStateLabel];
        })

        console.log('feedback details received: ' + JSON.stringify(feedbackDetails));
        observer.next(feedbackDetails);
      });
    });
  }

  public getConfig() {
    return Observable.create((observer) => {
      if (this.config != null) {
        observer.next(this.config);
      }
      this.socket.on('config', (config) => {
        console.log('config recieved: ' + config);
          var configHash = hash(config);
          if (configHash != this.configHash){
            observer.next(config);
            this.configHash = configHash;
          }
          else {
            console.log("known hash");
          }

      });
    });
  }

  public setAPIKey(apikey:string){
    this.apiKey = apikey;
  }

  public authenticate(){
    console.log("sending authenticate for apikey",this.apiKey);
    if (this.apiKey != undefined){
      this.socket.emit("authenticate", this.apiKey);
    }
    else {
      this.socket.emit("authenticate", null);
    }
  }

  public getGearStates() {
    return Observable.create((observer) => {
      if (this.gearStates != null) {
        observer.next(this.gearStates);
      }

      this.socket.on('gear-state', (gearStates) => {
        console.log('gear-state: ' + gearStates);
        this.gearStates = gearStates;
          observer.next(gearStates);
      });
    });
  }

  public getDisconnect() {
    return Observable.create((observer) => {
      if (this.disconnect != null) {
        observer.next(this.disconnect);
      }
      this.socket.on('disconnect', (disconnect) => {
        console.log(disconnect);
          observer.next(disconnect);
      });
    });
  }  


  public getDirections() {

    return Observable.create((observer) => {
      if (this.directions != null) {
        observer.next(this.directions);
      }
      this.socket.on('directions', (directions) => {
          observer.next(directions);
      });
    });
  }

  public getAssetsMaintenance() {
    return Observable.create((observer) => {
      if (this.assets_maintenance != null) {
        observer.next(this.assets_maintenance);
      }
      this.socket.on('maintenance-assets', (assets_maintenance) => {
        observer.next(assets_maintenance);
      });
    })
  }
}
