/**
 * Virtual PEI implementation.
 *
 * Main entry point for application.
 *
 * @link   http://wwww.alarmiator.de
 * @file   This files defines the main entry point.
 * @author Stefan Bauer.
 * @since  2020.12.24
 */

var express = require('express');
var path = require('path');
var bodyParser = require('body-parser');

require('console-stamp')(console, 'HH:MM:ss.l');

// include routers
var routeindex = require('./routes/index');
var routeTetra = require('./routes/tetra');
var routeSettings = require('./routes/settings');

var app = express();

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');


app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(express.static(path.join(__dirname, 'public')));

// make npm packages available
app.use('/bootstrap', express.static(__dirname + '/node_modules/bootstrap/dist'));
app.use('/jquery', express.static(__dirname + '/node_modules/jquery/dist'));
app.use('/mdi', express.static(__dirname + '/node_modules/@mdi/font'));


app.use('/', routeindex);
app.use('/tetra', routeTetra);
app.use('/settings', routeSettings);

// catch 404 and forward to error handler
app.use(function(req, res, next) {
  var err = new Error('Not Found');
  err.status = 404;
  next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
  app.use(function(err, req, res, next) {
    res.status(err.status || 500);
    res.render('error', {
      message: err.message,
      error: err
    });
  });
}

// production error handler
// no stacktraces leaked to user
app.use(function(err, req, res, next) {
  res.status(err.status || 500);
  res.render('error', {
    message: err.message,
    error: {}
  });
});

const server = app.listen(3000, () => console.log(`Express server listening on port 3000`));


module.exports = app;

