var inquirer = require('inquirer'); // required for console interaction
var mysql = require('mysql'); // required for database access
var log4js = require('log4js'); // required for logging to logfile and console
var mysqlimport = require('mysql-import'); // required for importing mysql .sql file into new database
var bcrypt = require('bcrypt'); // required for generating new admin password
var fs = require('fs'); // required for writing config file
var os =  require('os'); // required for getting host related information
var oshostname = os.hostname();

// Configure logger
log4js.configure({
    appenders: { mylogger: { type: "dateFile", filename: "logs/installer/installer.log" }, console: { type:"console" } },
    categories: { default: { appenders:["mylogger", "console"], level:"ALL" } }
});
const logger = log4js.getLogger("default");


// installer variables
var databaseOptions = {
    dateStrings: [
      'DATE',
      'DATETIME'
    ],
    host: '',
    user: '',
    password: '',
    database: '',
    connectionLimit: 100 // important
};

var serverOptions = {
  uiport: 4000  
}

var questions = [
    {
      type: 'input',
      name: 'dbservername',
      message: "Servername des Datenbankservers",
      default: function() {
          return 'localhost';
      }
    },
    {
      type: 'input',
      name: 'dbdatabase',
      message: "Datenbankname für ALARMiator Server",
      default: function() {
          return 'alarmiator';
      }
    },
    {
      type: 'input',
      name: 'dbusername',
      message: "Benutzername für den Datenbankserver",
      default: function() {
        return 'root';
      }
    },
    {
        type: 'input',
        name: 'dbpassword',
        message: "Passwort für den Datenbankserver",
    },
    {
        type: 'input',
        name: 'dbport',
        message: "Port für den Datenbankserver",
        default: function() {
          return '3306';
        }
    },
    {
      type: 'input',
      name: 'adminpassword',
      message: "Passwort für den Admin-User in ALARMiator",
      default: function() {
        return 'admin';
      }
    },
    {
      type: 'input',
      name: 'uiport',
      message: "Port für Zugriff auf die Benutzeroberfläche im Browser (http://servername:port)",
      default: function() {
        return '5000';
      }
    },
    {
      type: 'input',
      name: 'servername',
      message: "interner Servername, mit dem der Zugriff auf die Weboberfläche erfolgen soll (z.b. http://" + oshostname + ".fritz.box)",
      default: function() {
        return 'http://'+oshostname;
      }
    }
  ];

  var questionsReadyToInstall = [
    {
      type: 'confirm',
      name: 'overwritedbconfirm',
      message: "Die angegebene Datenbank existiert bereits. Soll sie überschrieben werden",
      default: function() {
          return 'false';
      }
    }
  ];

/** 
 * Start installer
 */
logger.info('ALARMiator-INSTALLER | Start installer')
inquirer
.prompt(questions).then(answers => {
    logger.info('Cheching MySQL connectivity');
    //var answersArray = JSON.stringify(answers);

    databaseOptions.host = answers.dbservername;
    databaseOptions.database = answers.dbdatabase;
    databaseOptions.user = answers.dbusername;
    databaseOptions.password = answers.dbpassword;
    databaseOptions.port = answers.dbport;
    serverOptions.uiport = answers.uiport;
    serverOptions.servername = answers.servername;
    
    checkMySqlConnection(databaseOptions, (result) => {
        // Check if connection successful or not
        if (result === true) {
          logger.info('ALARMiator-INSTALLER | Database Connection successfully checked');
          checkMySqlIfDatabaseExists(databaseOptions, (result) => {
            if (result === true) {
              // Database exists
              logger.info('ALARMiator-INSTALLER | Database ' + databaseOptions.database + ' successfully checked');
              inquirer.prompt(questionsReadyToInstall).then(answersOverwrite => {
                  if (answersOverwrite.overwritedbconfirm === true) {
                    importStartupDatabase(databaseOptions, (result) => {
                      if (result === true) {
                        logger.info('ALARMiator-INSTALLER | Successfully setup startup database.');
                        setAdminPasswort(answers.adminpassword, databaseOptions, (result) => {
                          if (result === true) {
                            logger.info('ALARMiator-INSTALLER | Successfully set Password for Admin user.');
                            writeConfigFile(databaseOptions, serverOptions, (result) => {
                              if (result === true) {
                                logger.info('ALARMiator-INSTALLER | Successfully wrote config file to disk');
                                process.exit();
                              } else {
                                logger.error('ALARMiator-INSTALLER | could not write config file to disk.');
                                process.exit();
                              }
                            });
                          } else {
                            logger.error('ALARMiator-INSTALLER | Error setting admin password. Setup canceled.');
                            process.exit();
                          }
                        });
                      } else {
                        logger.error('ALARMiator-INSTALLER | Could not import startup database. See log for detailled error.');
                        process.exit();
                      }
                    });
                  } else {
                    logger.warn('ALARMiator-INSTALLER | User chose to cancel installation. Restart and set another database name to use for ALARMiator Server.');
                    process.exit();
                  }
                });
            } else {
              logger.info('ALARMiator-INSTALLER | Database ' + databaseOptions.database + ' does not exist');
              createDatabase(databaseOptions, (result) => {
                if (result === true) {
                  // Database has been created successfully
                  importStartupDatabase(databaseOptions, (result) => {
                    if (result === true) {
                      logger.info('ALARMiator-INSTALLER | Successfully setup startup database.');
                      setAdminPasswort(answers.adminpassword, databaseOptions, (result) => {
                        logger.info('Set Admin password');
                        if (result === true) {
                          logger.info('ALARMiator-INSTALLER | Successfully set Password for Admin user.');
                          writeConfigFile(databaseOptions, serverOptions, (result) => {
                            if (result === true) {
                              logger.info('ALARMiator-INSTALLER | Successfully wrote config file to disk');
                              process.exit();
                            } else {
                              logger.error('ALARMiator-INSTALLER | could not write config file to disk.');
                              process.exit();
                            }
                          });
                        } else {
                          logger.error('ALARMiator-INSTALLER | Error setting admin password. Setup canceled.');
                          process.exit();
                        }
                      });
                    } else {
                      logger.error('ALARMiator-INSTALLER | Could not import startup database. See log for detailled error.');
                      process.exit();
                    }
                  });
                } else {
                  // Database could not be created
                  logger.error('ALARMiator-INSTALLER | Database could not be created. Check access rights for given database user and try again.');
                  process.exit();
                }
              })
            }
          });
        } else {
          logger.error('ALARMiator-INSTALLER | Can not connect to database with given credentials');
          process.exit();
        }
    });
})

function checkMySqlConnection(databaseOptions, callback) {
    var connectionConfig = {
      dateStrings: [
        'DATE',
        'DATETIME'
      ],
      host: databaseOptions.host,
      user: databaseOptions.user,
      password: databaseOptions.password,
      port: databaseOptions.port,
      //database: databaseOptions.database,
      connectionLimit: 100 // important
    };
    const db = mysql.createConnection(connectionConfig);
    logger.info('ALARMiator-INSTALLER | Connecting to database ... ');
    // Connect to database
    db.connect((err) => {
        if (err) {
          logger.debug('ALARMiator-INSTALLER | Error connecting to database: ' + err);
          callback(false);
        } else {
          callback(true);
        }
    });
    global.db = db;
}

function checkMySqlIfDatabaseExists(databaseOptions, callback) {
  logger.info('ALARMiator-INSTALLER | Connecting to database for check if database exists ... ');
  // Connect to database
  let query = "SELECT SCHEMA_NAME FROM INFORMATION_SCHEMA.SCHEMATA WHERE SCHEMA_NAME = '" + databaseOptions.database + "'";
  db.query(query, (err, result) => {
      if (err) {
        logger.debug('ALARMiator-INSTALLER | Check for Database ' + databaseOptions.database + ' not possible: ' + err);
        callback(false);
      } else {
        if (result.length > 0) {
          // Database exists
          callback(true);
        } else {
          // Database does not exist
          callback(false);
        }
      }
  });
}

function createDatabase(databaseOptions, callback) {
  logger.info('ALARMiator-INSTALLER | try to create database schema for database ' + databaseOptions.database + ' ... ');
  // Connect to database
  let query = "CREATE DATABASE " + databaseOptions.database;
  db.query(query, (err, result) => {
      if (err) {
        logger.debug('ALARMiator-INSTALLER | Creation of database with name ' + databaseOptions.database + ' not possible: ' + err);
        callback(false);
      } else {
        logger.debug('ALARMiator-INSTALLER | created new database successfully');
        callback(true);
      } 
  });
}

function importStartupDatabase(databaseOptions, callback) {
  logger.info('ALARMiator-INSTALLER | Starting import of startup database into database ' + databaseOptions.database)
  

  const mydb_importer = mysqlimport.config({
    host: databaseOptions.host,
    port: databaseOptions.port,
    user: databaseOptions.user,
    password: databaseOptions.password,
    database: databaseOptions.database,
    onerror: (err) => {
      logger.error('ALARMiator-INSTALLER | Fehler beim Einlesen der Startup-Datenbank in die neue Datenbank: ' + err.message);
      callback(false);
    }
  });

  mydb_importer.import('./../sql/startup.sql').then(() => {
    logger.info('ALARMiator-INSTALLER | Einlesen der Startdatenbank erfolgreich abgeschlossen.');
    callback(true);
  });
}

function setAdminPasswort(password, databaseOptions, callback) {
  bcrypt.hash(password, 10, function(err, hash) {
    if (err) { 
        logger.error('ALARMiator-INSTALLER | error hasing (1) user password for Admin user: ' + err);
        callback(false); 
    }

    bcrypt.compare(password, hash, function(err, result) {
        if (err) {
            logger.error('ALARMiator-INSTALLER | error hasing (2) user password for Admin user: ' + err); 
            callback(false);
        }
        // Store hash in the database
        // send the user's details to the database
        // First select new database;
        let query = "USE " + databaseOptions.database;
        db.query(query, (error, result) => {
          if (error) {
            // Could not connect to new database
            logger.error('ALARMiator-INSTALLER | Could not connect to new database ' + databaseOptions.database + ' for setting admin user password');
            callback(false);
          } else {
            let query = "SELECT * FROM `users` WHERE username = 'admin'";
            db.query(query, (err, result) => {
              if (err) {
                logger.error('ALARMiator-INSTALLER | Error checking for Admin user in database: ' + err.message);
                callback(false);
              } else {
                if (result.length > 0) {
                  // Admin user exists, update user
                  let query = "UPDATE `users` SET password = '" + hash + "', superuser = 1 WHERE id = " + result[0].id;
                  db.query(query, (err2, result2) => {
                    if (err2) {
                      logger.error('ALARMiator-INSTALLER | Error updating password for Admin user: ' + err2.message);
                      callback(false);
                    } else {
                      // Successfully set new admin password
                      logger.info('ALARMiator-INSTALLER | Successfully set new Admin password.');
                      callback(true);
                    }
                  })
                } else {
                  // No Admin User found, create one
                  let query = "INSERT INTO `users` (username, password, superuser, basedataid) VALUES ('Admin', '" + hash + "', 1, 1)";
                  db.query(query, (err, result) => {
                      if (err) {
                          logger.error('ALARMiator-INSTALLER | error inserting new admin user to database');
                          callback(false);
                      } else {
                        logger.info('ALARMiator-INSTALLER | successfully inserted new admin user to database');
                        callback(true);
                      }
                  });
                }
              }
            });
          }
        })
    });
});
}


function writeConfigFile(databaseOptions, serverOptions, callback) {
  logger.info('ALARMiator-INSTALLER | Write config file');

  var configContent = " /* Config File created by ALARMiator-Installer */\r\n" +
                      "module.exports.databaseOptions = { \r\n" + 
                      " dateStrings: [ \r\n" +
                      "   'DATE', \r\n" +
                      "   'DATETIME' \r\n" +
                      " ], \r\n" +
                      " host: '" + databaseOptions.host + "', \r\n" +
                      " port: '" + databaseOptions.port + "', \r\n" +
                      " user: '" + databaseOptions.user + "', \r\n" +
                      " password: '" + databaseOptions.password + "', \r\n" +
                      " database: '" + databaseOptions.database + "', \r\n" +
                      " connectionLimit: 100 // important \r\n" +
                      "};\r\n" + 
                      "module.exports.serverOptions = {  \r\n" + 
                      " uiport: " + serverOptions.uiport + "  \r\n" +
                      " servername: " + serverOptions.servername + "  \r\n" +                       
                      "} \r\n";

  fs.writeFile("./../config/config_generated.js", configContent, function(err) {
      if(err) {
          logger.error('ALARMiator-INSTALLER | Error writing config file to filesystem (/config/config_generated.js with error:' + err);
          callback(false);
      } else {
        logger.info('ALARMiator-INSTALLER | config file has been written successfully to disk (/config/config_generated.js)');
        callback(true);
      }
  }); 
}