 
/**
 * Migration Coredatabase
 * 
 * @autor Jens Dinstühler
 * @description prefills table accessGroupsStore in coredatabase
 * 
 */


 'use strict';

 const { Sequelize } = require('sequelize');
 
 module.exports = {
   up: async (queryInterface) => {
     await queryInterface.bulkInsert('accessGroupsStore', 
       [
         // General
         {
           id: 1,
           alias: 'general', 
           title: 'Allgemein',
           description: 'Systemweite Berechtigungen',
           parentGroup: null
         },
         // Administration
         { 
           id: 2,
           alias: 'administration', 
           title: 'Administration',
           description: 'Berechtigungen im Bereich der Systemadministration.',
           parentGroup: null
         },
         { 
           id: 3,
           alias: 'serversettings', 
           title: 'Grundeinstellungen',
           description: 'Allgemeine Grundeinstellungen des ALAMRiator Servers.',
           parentGroup: 2
         },
         { 
           id: 4,
           alias: 'plugins', 
           title: 'Plugin-Einstellungen',
           description: 'Konfiguration der ALARMiator Server Plugins.',
           parentGroup: 2
         },
         { 
           id: 5,
           alias: 'accessmanagement', 
           title: 'Berechtigungsmanagement',
           description: 'Verwaltung der Benutzerberechtigungen in ALARMiator Server.',
           parentGroup: 2
         },
         { 
           id: 6,
           alias: 'logfiles', 
           title: 'Logfiles',
           description: 'Logfilemanagement',
           parentGroup: 2
         },
         { 
           id: 7,
           alias: 'backupmanagement', 
           title: 'Backup & Restore',
           description: 'Backupmanagement',
           parentGroup: 2
         },
         { 
           id: 8,
           alias: 'cronmanagement', 
           title: 'Aufgabenverwaltung',
           description: 'Verwaltung von automatisierten Aufgaben in ALARMiator Server.',
           parentGroup: 2
         },
         { 
           id: 9,
           alias: 'generalsystem', 
           title: 'System',
           description: 'Herunterfahren und Neustart des ALARMiator Servers.',
           parentGroup: 2
         },
         // Basedata
         { 
           id: 10,
           alias: 'basedata', 
           title: 'Stammdaten',
           description: 'Berechtigungen im Bereich der Stammdatenverwaltung',
           parentGroup: null
         },
         { 
           id: 11,
           alias: 'basedataexplorer', 
           title: 'Stammdatenexplorer',
           description: 'Nutzung des Stammdatenexplorers.',
           parentGroup: 10
         },
         { 
           id: 12,
           alias: 'organizations', 
           title: 'Organisationen',
           description: 'Verwaltung der Organisationseinheiten und deren Eigenschaften',
           parentGroup: 10
         },
         { 
           id: 13,
           alias: 'locations', 
           title: 'Standorte',
           description: 'Verwaltung der Standorte und deren Eigenschaften.',
           parentGroup: 10
         },
         { 
           id: 14,
           alias: 'members', 
           title: 'Mitglieder',
           description: 'Verwaltung der Mitglieder und deren Daten.',
           parentGroup: 10
         },
         { 
           id: 15,
           alias: 'groups', 
           title: 'Gruppenverwaltung',
           description: 'Verwaltung der Gruppen',
           parentGroup: 10
         },
         { 
           id: 16,
           alias: 'roles', 
           title: 'Rollenverwaltung',
           description: 'Verwaltung der Rollen',
           parentGroup: 10
         },
         { 
           id: 17,
           alias: 'qualifications', 
           title: 'Qualifikationen',
           description: 'Verwaltung der Qualifikationen',
           parentGroup: 10
         },
         { 
           id: 18,
           alias: 'users', 
           title: 'Benutzerverwaltung',
           description: 'Verwaltung der Benutzer des ALARMiator Servers und Apps',
           parentGroup: 10
         },
         // Inventory
         { 
           id: 19,
           alias: 'assets', 
           title: 'Invenatar',
           description: 'Berechtigungen im Bereich der Inventarverwaltung',
           parentGroup: null
         },
         { 
           id: 20,
           alias: 'assetmanagement', 
           title: 'Inventarverwaltung',
           description: 'Verwaltung von Inventargegenständen',
           parentGroup: 19
         },
         { 
           id: 21,
           alias: 'assetgroups', 
           title: 'Inventargruppen',
           description: 'Verwaltung der Inventargruppen',
           parentGroup: 19
         },
         { 
           id: 22,
           alias: 'warehouses', 
           title: 'Lagerorte',
           description: 'Verwaltung der Lagerorte',
           parentGroup: 19
         },
         { 
           id: 23,
           alias: 'orders', 
           title: 'Bestellungen',
           description: 'Verwaltung von Bestellungen',
           parentGroup: 19
         },
         { 
           id: 24,
           alias: 'goodsreceipts', 
           title: 'Wareneingang',
           description: 'Verwaltung des Wareneingangs',
           parentGroup: 19
         },
         { 
           id: 25,
           alias: 'suppliers', 
           title: 'Lieferanten',
           description: 'Vwerwaltung der Lieferantenstammdaten',
           parentGroup: 19
         },
         { 
           id: 26,
           alias: 'hydrants', 
           title: 'Hydranten',
           description: 'Verwaltung der Hydranten',
           parentGroup: 19
         },
         // Training
         { 
           id: 27,
           alias: 'trainings', 
           title: 'Ausbildung',
           description: 'Berechtigungen im Bereich der Ausbildungsverwaltung',
           parentGroup: null
         },
         { 
           id: 28,
           alias: 'exercises', 
           title: 'Übungen',
           description: 'Verwaltung von Übungen',
           parentGroup: 27
         },
         // Respiratory
         { 
           id: 29,
           alias: 'respiratory', 
           title: 'Atemschutz',
           description: 'Berechtigungen im Bereich Atemschutzverwaltung',
           parentGroup: null
         },
         { 
           id: 30,
           alias: 'suitability', 
           title: 'Tauglichkeit',
           description: 'Verwaltung der Tauglichkeitsbescheinigungen (G26.3)',
           parentGroup: 29
         },
         // alarm
         { 
           id: 31,
           alias: 'alarm', 
           title: 'Alarmierung',
           description: 'Berechtigungen im Bereich der Alarmierung',
           parentGroup: null
         },
         { 
           id: 32,
           alias: 'alarmmatrix', 
           title: 'Matrix',
           description: 'Verwaltung der aktiven Alarmierungswege',
           parentGroup: 31
         },
         { 
           id: 33,
           alias: 'devices', 
           title: 'Endgeräte',
           description: 'Verwaltung der Endgeräte',
           parentGroup: 31
         },
         { 
           id: 34,
           alias: 'fcmtokens', 
           title: 'Tokens',
           description: 'Verwaltung der Tokens',
           parentGroup: 31
         },
         { 
           id: 35,
           alias: 'wallboards', 
           title: 'Wallboardprofile',
           description: 'Verwaltung der Wallboardprofile',
           parentGroup: 31
         },
         { 
           id: 36,
           alias: 'feedback', 
           title: 'Rückmeldungen',
           description: 'Verwaltung des Einsatzrückmeldungen',
           parentGroup: 31
         },
         { 
           id: 37,
           alias: 'gear', 
           title: 'Einsatzmittel',
           description: 'Verwaltung der Einsatzmittel für Alarmierungen',
           parentGroup: 31
         },
         { 
           id: 38,
           alias: 'alarmprofiles', 
           title: 'Alarmierungsprofile',
           description: 'Verwaltung der Alarmierungsprofile',
           parentGroup: 31
         },
         { 
           id: 39,
           alias: 'testalarms', 
           title: 'Probealarmierungen',
           description: 'Verwaltung von Probealarmierungen',
           parentGroup: 31
         },
         // alarm
         { 
           id: 40,
           alias: 'operations', 
           title: 'Einsatzverwaltung',
           description: 'Berechtigungen im Bereich der Einsatzverwaltung',
           parentGroup: null
         },
         { 
           id: 41,
           alias: 'listactiveoperations', 
           title: 'Liste aktive Einsätze',
           description: 'Verwaltung der aktiven Einsätze',
           parentGroup: 40
         },
         { 
           id: 42,
           alias: 'listalloperations', 
           title: 'Liste alle Einsätze',
           description: 'Verwaltung aller Einsätze',
           parentGroup: 40
         },
         { 
           id: 43,
           alias: 'operationsmap', 
           title: 'Übersichtskarte',
           description: 'Nutzung der Übersichtskarte',
           parentGroup: 40
         },
         { 
           id: 44,
           alias: 'listgearstates', 
           title: 'Liste Statusmeldungen',
           description: 'Nutzung der Liste der Statusmeldungen',
           parentGroup: 40
         }
       ]
     );
   }
 };