/**
 * Migration Coredatabase
 * 
 * @autor Jens Dinstühler
 * @description Adds a new table assets_parameters_options to coredatabase
 * 
 */


 'use strict';

 const { Sequelize } = require('sequelize');
 const uuid = require('uuid').v1;
 
 module.exports = {
  up: async (queryInterface) => {

    queryInterface.describeTable("assets_parameters")
      .then(async function () {
        console.log('MIGRATION-TASK | --> migrating textual assets parameters select definitions to database');
        await queryInterface.sequelize.query('SELECT * FROM assets_parameters WHERE `type` = "select"')
          .then(function (rows) {
            if (rows[0] !== null) {
              console.log('MIGRATION-TASK | --> found ' + rows.length + ' entries to migrate' );
              if (rows[0].length > 0) {
                var i;
                for (i = 0; i < rows[0].length; i++) {
                  let parameterId = rows[0][i].id;
                  console.log('MIGRATION-TASK | Processing assets_parameter with id ' + parameterId );
                  let selectOptions = null;
                  try {
                    selectOptions = JSON.parse(rows[0][i].values);
                  } catch(err) {
                    console.log('MIGRATION-TASK | Error interpreting values of assets_parameter with id ' + parameterId )
                  }
                  
                  if (selectOptions.length > 0) {
                    // found select values
                    
                    selectOptions.forEach(value => {
                      for (const [key, insideValue] of Object.entries(value)) {
                        let valueStr = insideValue;
                        let valueAlias = uuid();
                        let legacyAlias = key;
                        
                        let query = `INSERT INTO assets_parameters_options(parameterId, label, alias, legacyAlias, sortorder, state) VALUES (${parameterId}, '${valueStr}', '${valueAlias}', ${legacyAlias}, ${legacyAlias}, 1)`;
                        queryInterface.sequelize.query(query);
                      }
                    });
                  } else {
                    // there are no values defined
                    console.log('MIGRATION-TASK | No values found for assets_parameter with id ' + parameterId )
                  }
                }
              }
            } else {
              console.log('MIGRATION-TASK | No asset_parameters with select defined in core database');
            }
          });
      });
  },
};
