const EventEmitter = require('events');
const Groups = require('../models/groups');
const Operations = require('../models/operationshelper');
const Geargroups = require('../models/gearGroups');
class Alertrules extends EventEmitter {

    /**
     * constructor
     */
    constructor() {
        super();
    }

    /**
     * Returns all alert rules from core database
     * @param {function} callback rows or null
     */
    getAllRules(callback) {
        let query = "SELECT * FROM alertrulesStore ORDER BY `order` ASC";
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all active alert rules from core database
     * @param {function} callback rows or null
     */
    getAllActiveRules(callback) {
        let query = "SELECT * FROM alertrulesStore WHERE state=1 ORDER BY `order` ASC";
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns a single alert rule identified by id
     * @param {integer} ruleId id of alert rule
     * @param {function} callback rows or null
     */
    getRuleWithId(ruleId, callback) {
        let query = "SELECT * FROM alertrulesStore WHERE id = ? ORDER BY `order` ASC";
        global.coreDb.all(query, [
            ruleId
        ], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Stores a new alertrule to core database
     * @param {object} reqBody req.body of request containing the form fields
     * @param {function} callback success (true/false), insertId or null
     */
    addrule(reqBody, callback) {
        var title = reqBody.title;
        var state = reqBody.state;
        var order = 0;

        this.doesRuleExist(reqBody, (exists, rows) => {
            if (exists != true) {
                let query = 'INSERT INTO alertrulesStore (`title`, `state`, `order`) VALUES (?, ?, ?)';
                global.coreDb.run(query, [
                    title,
                    state,
                    order
                ], function (err) {
                    if (err) {
                        console.log(`ERROR Adding new alertrule to core database: ${err.message}`);
                        callback(false, null);
                    } else {
                        var newAssetId = this.lastID;
                        callback(true, newAssetId);
                    }
                });
            } else {
                return callback(false, null);
            }
        })
    }

    /**
     * Activates an existing rule
     * @param {integer} ruleId id of rule to activate
     * @param {function} callback success (true/false)
     */
    activateRule(ruleId, callback) {
        let query = "UPDATE alertrulesStore SET state = 1 WHERE id = ?";
        global.coreDb.run(query, [
            ruleId
        ], function (err) {
            if (err) {
                console.log(`ERROR setting rule to active: ${err.message}`);
                callback(false);
            } else {
                callback(true);
            }
        });
    }

    /**
     * Deactivates an existing rule
     * @param {integer} ruleId id of rule to deactivate
     * @param {function} callback success (true/false)
     */
    deactivateRule(ruleId, callback) {
        let query = "UPDATE alertrulesStore SET state = 0 WHERE id = ?";
        global.coreDb.run(query, [
            ruleId
        ], function (err) {
            if (err) {
                console.log(`ERROR setting rule to active: ${err.message}`);
                callback(false);
            } else {
                callback(true);
            }
        });
    }


    /**
     * Updates an existingalertrule at core database
     * @param {object} reqBody req.body of request containing the form fields
     * @param {function} callback success (true/false), insertId or null
     */
    editrule(alertRuleId, reqBody, callback) {
        var title = reqBody.title;
        var state = reqBody.state;
        var order = 0;

        console.lo

        this.doesRuleExist(reqBody, (exists, rows) => {
            var proceed = false;
            if (exists === true) {
                if (rows[0].id === parseInt(alertRuleId)) {
                    console.log('editing existing rule --> same title --> ok');
                    proceed = true;
                }
            } else {
                proceed = true;
            }
            console.log('editRule -> proceed:' + proceed);
            if (proceed === true) {
                let query = 'UPDATE alertrulesStore SET title = ?, `state` = ?, `order` = ? WHERE id = ?';
                //let query = 'INSERT INTO alertrulesStore (`title`, `state`, `order`) VALUES (?, ?, ?)';
                global.coreDb.run(query, [
                    title,
                    state,
                    order,
                    alertRuleId
                ], function (err) {
                    if (err) {
                        console.log(`ERROR Updating new alertrule at core database: ${err.message}`);
                        callback(false);
                    } else {

                        callback(true);
                    }
                });
            } else {
                return callback(false);
            }
        })
    }

    /**
     * Checks if a rule with the given title exists in core database
     * @param {object} reqBody req.body of request containig the form fields
     * @param {*} callback exists (true, rows), or not (false, null)
     */
    doesRuleExist(reqBody, callback) {
        let title = reqBody.title;
        let query = 'SELECT * FROM alertrulesStore WHERE title = ?';
        global.coreDb.all(query, [
            title
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                if (rows.length > 0) {
                    return callback(true, rows);
                } else {
                    return callback(false, null);
                }

            }
        });
    }

    /**
     * adds groups to an alertrule in core database
     * @param {integer} alertRuleId id of alertRule the groups shall be attached to 
     * @param {array} groupsArray array holding the ids of groups to be added
     * @param {*} callback success (true/false)
     */
    addRuleGroups(alertRuleId, groupsArray, callback) {
        console.log('Referencing groups to alert rule with id: ' + alertRuleId);
        console.log('Groups Array filled: ' + Array.isArray(groupsArray));
        console.log('GroupsArray: ' + JSON.stringify(groupsArray));

        if (Array.isArray(groupsArray)) {
            // there have been rules given to store
            global.coreDb.serialize(() => {
                groupsArray.forEach((group) => {
                    var groupId = group;
                    let query = "INSERT INTO alertrulesGroupsStore (groupId, alertRuleId) VALUES (?, ?)";

                    global.coreDb.run(query, [
                        groupId,
                        alertRuleId
                    ], function (err) {
                        if (err) {
                            console.log(`ERROR Adding group to alertrule at core database: ${err.message}`);
                            return callback(false);
                        }
                    });
                })
            });
            return callback(true);
        } else {
            // not rules given --> this is not allowed, return false
            return false;
        }
    }

    /**
     * adds geargroups to an alertrule in core database
     * @param {integer} alertRuleId id of alertRule the groups shall be attached to 
     * @param {array} groupsArray array holding the ids of gear groups to be added
     * @param {*} callback success (true/false)
     */
    addRuleGearGroups(alertRuleId, groupsArray, callback) {
        console.log('Referencing gear groups to alert rule with id: ' + alertRuleId);
        console.log('Gear groups Array filled: ' + Array.isArray(groupsArray));
        console.log('gearGroupsArray: ' + JSON.stringify(groupsArray));

        if (Array.isArray(groupsArray)) {
            // there have been rules given to store
            global.coreDb.serialize(() => {
                groupsArray.forEach((group) => {
                    var groupId = group;
                    let query = "INSERT INTO alertrulesGearGroupsStore (groupId, alertRuleId) VALUES (?, ?)";

                    global.coreDb.run(query, [
                        groupId,
                        alertRuleId
                    ], function (err) {
                        if (err) {
                            console.log(`ERROR Adding geargroup to alertrule at core database: ${err.message}`);
                            return callback(false);
                        }
                    });
                })
            });
            return callback(true);
        } else {
            // no rules given --> this is not allowed, return false
            return false;
        }
    }

    /**
     * Returns all active groups to be alerted if given rule evaluates positive
     * @param {integer} alertRuleId id of alert rule
     * @param {function} callback rows or null
     */
    getRuleGroups(alertRuleId, callback) {
        let query = "SELECT * FROM alertrulesGroupsStore WHERE alertRuleId = ?";
        global.coreDb.all(query, [
            alertRuleId
        ], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all active geargroups to be alerted if given rule evaluates positive
     * @param {integer} alertRuleId id of alert rule
     * @param {function} callback rows or null
     */
    getRuleGearGroups(alertRuleId, callback) {
        let query = "SELECT * FROM alertrulesGearGroupsStore WHERE alertRuleId = ?";
        global.coreDb.all(query, [
            alertRuleId
        ], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }



    /**
     * Deletes an alertRule with its filters and groups from core database
     * @param {integer} alertRuleId id of alertRule to be deleted
     * @param {*} callback success (true/false)
     */
    removeAlertRule(alertRuleId, callback) {
        console.log('deleting rule with id: ' + alertRuleId);
        let query = "DELETE FROM alertrulesFiltersStore WHERE alertRuleId = ?";
        global.coreDb.run(query, [
            alertRuleId
        ], function (err) {
            if (err) {
                console.log(`ERROR Removing Filters for alertrule at core database: ${err.message}`);
                return callback(false);
            } else {
                let query = "DELETE FROM alertrulesGroupsStore WHERE alertRuleId = ?";
                global.coreDb.run(query, [
                    alertRuleId
                ], function (err) {
                    if (err) {
                        console.log(`ERROR Removing Groups for alertrule at core database: ${err.message}`);
                        return callback(false);
                    } else {
                        let query = "DELETE FROM alertrulesGearGroupsStore WHERE alertRuleId = ?";
                        global.coreDb.run(query, [
                            alertRuleId
                        ], function (err) {
                            if (err) {
                                console.log(`ERROR Removing gear groups for alertrule at core database: ${err.message}`);
                                return callback(false);
                            } else {
                                let query = "DELETE FROM alertrulesStore WHERE id = ?";
                                global.coreDb.run(query, [
                                    alertRuleId
                                ], function (err) {
                                    if (err) {
                                        console.log(`ERROR Removing alertrule at core database: ${err.message}`);
                                        return callback(false);
                                    } else {
                                        return callback(true);
                                    }
                                });
                            }
                        });
                    }
                });
            }
        });
    }

    /**
     * Checks which filters have been applied and prepares information for persisting filters to core database
     * @param {object} reqBody req.body, holding the form fields
     * @param {integer} alertRuleId id of rule, the filters shall be referenced to
     * @param {*} callback success (true/false)
     */
    renderRuleFilters(reqBody, alertRuleId, callback) {
        console.log('renderRuleFilters');
        var filtersToPersist = [];
        var filter = {};

        // Check for TimeRange Filter
        if ((typeof reqBody.timerange_from != 'undefined') && (typeof reqBody.timerange_till != 'undefined')) {
            // Time Range Filter active
            console.log('MODELS | ALERTRULES | FILTER | Time Range Active');
            filter = {
                filterType: 'timerange',
                filterParameters: {
                    timerange_from: reqBody.timerange_from,
                    timerange_till: reqBody.timerange_till
                },
                alertRuleId: alertRuleId
            }
            filtersToPersist.push(filter);
        }
        if (typeof reqBody.zveicodes != 'undefined') {
            // ZVEI Filter active
            console.log('MODELS | ALERTRULES | FILTER | ZVEI Active');
            var searchType_containsOne = 0;
            if (reqBody.zvei_searchType_containsOne === 'on') {
                searchType_containsOne = 1;
            }
            var searchType_containsAll = 0;
            if (reqBody.zvei_searchType_containsAll === 'on') {
                searchType_containsAll = 1;
            }

            filter = {
                filterType: 'zveis',
                filterParameters: {
                    zveicodes: reqBody.zveicodes,
                    searchType_containsOne: searchType_containsOne,
                    searchType_containsAll: searchType_containsAll
                },
                alertRuleId: alertRuleId
            }
            filtersToPersist.push(filter);
        }
        if (typeof reqBody.zveicodesdelta != 'undefined') {
            // ZVEI Filter active
            console.log('MODELS | ALERTRULES | FILTER | ZVEI DELTA Active');
            console.log(reqBody);
            var searchType_containsOne = 0;
            if (reqBody.zveidelta_searchType_containsOne === 'on') {
                searchType_containsOne = 1;
            }
            var searchType_containsAll = 0;
            if (reqBody.zveidelta_searchType_containsAll === 'on') {
                searchType_containsAll = 1;
            }

            filter = {
                filterType: 'zveisdelta',
                filterParameters: {
                    zveicodesdelta: reqBody.zveicodesdelta,
                    searchType_containsOne: searchType_containsOne,
                    searchType_containsAll: searchType_containsAll
                },
                alertRuleId: alertRuleId
            }
            filtersToPersist.push(filter);
        }
        if (typeof reqBody.keyword_searchfor != 'undefined') {
            // Keyword Filter active
            console.log('MODELS | ALERTRULES | FILTER | Keyword Active');
            // set search Type
            var searchtype_contains = 0;
            var searchtype_beginsWith = 0;
            var searchtype_fullMatch = 0;
            var keyword_searchfor = '';

            if (typeof reqBody.keyword_searchfor != 'undefined') {
                keyword_searchfor = reqBody.keyword_searchfor;
            }

            if (typeof reqBody.keyword_searchType_contains != 'undefined') {
                if (reqBody.keyword_searchType_contains === 'on') {
                    searchtype_contains = 1;
                }
            }
            if (typeof reqBody.keyword_searchType_beginsWith != 'undefined') {
                if (reqBody.keyword_searchType_beginsWith === 'on') {
                    searchtype_beginsWith = 1;
                }
            }
            if (typeof reqBody.keyword_searchType_fullMatch != 'undefined') {
                if (reqBody.keyword_searchType_fullMatch === 'on') {
                    searchtype_fullMatch = 1;
                }
            }

            // build filter
            filter = {
                filterType: 'keyword',
                filterParameters: {
                    keyword_searchfor: keyword_searchfor,
                    keyword_searchType_contains: searchtype_contains,
                    keyword_searchType_beginsWith: searchtype_beginsWith,
                    keyword_searchType_fullMatch: searchtype_fullMatch
                },
                alertRuleId: alertRuleId
            }
            filtersToPersist.push(filter);
        }
        if ((typeof reqBody.weekday_monday != 'undefined') ||
            (typeof reqBody.weekday_tuesday != 'undefined') ||
            (typeof reqBody.weekday_wednesday != 'undefined') ||
            (typeof reqBody.weekday_thursday != 'undefined') ||
            (typeof reqBody.weekday_friday != 'undefined') ||
            (typeof reqBody.weekday_saturday != 'undefined') ||
            (typeof reqBody.weekday_sunday != 'undefined')) {
            // Day of week filter active
            console.log('MODELS | ALERTRULES | FILTER | Day of Week Active');
            // set search Type
            var weekday_monday = 0;
            var weekday_tuesday = 0;
            var weekday_wednesday = 0;
            var weekday_thursday = 0;
            var weekday_friday = 0;
            var weekday_saturday = 0;
            var weekday_sunday = 0;

            if (typeof reqBody.weekday_monday != 'undefined') {
                if (reqBody.weekday_monday === 'on') {
                    weekday_monday = 1;
                }
            }
            if (typeof reqBody.weekday_tuesday != 'undefined') {
                if (reqBody.weekday_tuesday === 'on') {
                    weekday_tuesday = 1;
                }
            }
            if (typeof reqBody.weekday_wednesday != 'undefined') {
                if (reqBody.weekday_wednesday === 'on') {
                    weekday_wednesday = 1;
                }
            }
            if (typeof reqBody.weekday_thursday != 'undefined') {
                if (reqBody.weekday_thursday === 'on') {
                    weekday_thursday = 1;
                }
            }
            if (typeof reqBody.weekday_friday != 'undefined') {
                if (reqBody.weekday_friday === 'on') {
                    weekday_friday = 1;
                }
            }
            if (typeof reqBody.weekday_saturday != 'undefined') {
                if (reqBody.weekday_saturday === 'on') {
                    weekday_saturday = 1;
                }
            }
            if (typeof reqBody.weekday_sunday != 'undefined') {
                if (reqBody.weekday_sunday === 'on') {
                    weekday_sunday = 1;
                }
            }

            // build filter
            filter = {
                filterType: 'dayofweek',
                filterParameters: {
                    weekday_monday: weekday_monday,
                    weekday_tuesday: weekday_tuesday,
                    weekday_wednesday: weekday_wednesday,
                    weekday_thursday: weekday_thursday,
                    weekday_friday: weekday_friday,
                    weekday_saturday: weekday_saturday,
                    weekday_sunday: weekday_sunday
                },
                alertRuleId: alertRuleId
            }
            filtersToPersist.push(filter);
        }
        var gearActive = false;
        var gearlist = [];
        for (let i = 0; i < 1000; i++) {
            var gearid = 'gear_id_' + i;
            if (typeof reqBody[gearid] != 'undefined') {
                gearActive = true;
                gearlist.push(i);
            }
        }
        if (gearActive) {
            // Gear Filter active
            console.log('MODELS | ALERTRULES | FILTER | Gear Active');
            // build filter
            filter = {
                filterType: 'gear',
                filterParameters: {
                    gearids: gearlist
                },
                alertRuleId: alertRuleId
            }
            filtersToPersist.push(filter);
        }

        var error = false;
        global.coreDb.serialize(() => {
            let query = "DELETE FROM alertrulesFiltersStore WHERE alertRuleId = ?";
            global.coreDb.run(query, [
                alertRuleId
            ], function (err) {
                if (err) {
                    console.log(`ERROR storing alertruleFilter at core database: ${err.message}`);
                    error = true;
                }
            });
            filtersToPersist.forEach((filter) => {
                let query = "INSERT INTO alertrulesFiltersStore (filterType, filterParameters, alertRuleId) VALUES (?, ?, ?)";
                global.coreDb.run(query, [
                    filter.filterType,
                    JSON.stringify(filter.filterParameters),
                    alertRuleId
                ], function (err) {
                    if (err) {
                        console.log(`ERROR storing alertruleFilter at core database: ${err.message}`);
                        error = true;
                    }
                });
            })
        })
        return callback(error);
    }

    /**
     * Returns all filters for given alert rule id
     * @param {integer} alertRuleId id of alert rule filters shall be returned for
     * @param {*} callback rows or null
     */
    getRuleFilters(alertRuleId, callback) {
        let query = "SELECT * FROM alertrulesFiltersStore WHERE alertRuleId = ?";
        global.coreDb.all(query, [
            alertRuleId
        ], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }


    // TBD !!!!!
    renderRuleGroups(reqBody, alertRuleId, callback) {
        var groups = new Groups();
        var groupsToAdd = [];
        groups.getlistOfGroupsForUI((rowsGroups) => {

            rowsGroups.forEach((group) => {
                if (typeof reqBody['alarmableGroup[' + group.id + ']'] != 'undefined') {
                    if (reqBody['alarmableGroup[' + group.id + ']'] === 'on') {
                        groupsToAdd.push(group.id);
                    }
                }
            });

            // Now we should have an array of groups to add for alert rule
            var error = false;
            global.coreDb.serialize(() => {
                let query = "DELETE FROM alertrulesGroupsStore WHERE alertRuleId = ?";
                global.coreDb.run(query, [
                    alertRuleId
                ], function (err) {
                    if (err) {
                        console.log(`ERROR removing existing groups for alertrule at core database: ${err.message}`);
                        error = true;
                    }
                });

                groupsToAdd.forEach((group) => {
                    let query = "INSERT INTO alertrulesGroupsStore (groupId, alertRuleId) VALUES (?, ?)";
                    global.coreDb.run(query, [
                        group,
                        alertRuleId,
                    ], function (err) {
                        if (err) {
                            console.log(`ERROR storing alertruleGroup at core database: ${err.message}`);
                            error = true;
                        }
                    });
                })
            })
            if(error === false) {
                return callback(true);
            } else {
                return callback(false);
            }
            
        })
    }

    /**
     * persists gear groups for alert rule
     * @param {object} reqBody form body data
     * @param {*} alertRuleId id of alert rule the gear groups shall be added for
     * @param {*} callback success (true/false)
     */
    renderRuleGearGroups(reqBody, alertRuleId, callback) {
        var geargroups = new Geargroups();
        var groupsToAdd = [];
        
        geargroups.getListOfGroups((rowsGroups) => {
            rowsGroups.forEach((group) => {
                if (typeof reqBody['alarmableGearGroup[' + group.id + ']'] != 'undefined') {
                    if (reqBody['alarmableGearGroup[' + group.id + ']'] === 'on') {
                        groupsToAdd.push(group.id);
                    }
                }
            });

            // Now we should have an array of groups to add for alert rule
            var error = false;
            global.coreDb.serialize(() => {
                let query = "DELETE FROM alertrulesGearGroupsStore WHERE alertRuleId = ?";
                global.coreDb.run(query, [
                    alertRuleId
                ], function (err) {
                    if (err) {
                        console.log(`ERROR removing existing gear groups for alertrule at core database: ${err.message}`);
                        error = true;
                    }
                });

                groupsToAdd.forEach((group) => {
                    let query = "INSERT INTO alertrulesGearGroupsStore (groupId, alertRuleId) VALUES (?, ?)";
                    global.coreDb.run(query, [
                        group,
                        alertRuleId,
                    ], function (err) {
                        if (err) {
                            console.log(`ERROR storing alertruleGroup at core database: ${err.message}`);
                            error = true;
                        }
                    });
                })
            })
            if(error === false) {
                return callback(true);
            } else {
                return callback(false);
            }
        })
    }

    getTestOperationHTML(oId, callback) {
        var operations = new Operations();
        operations.getOperation(oId, (rowsOperation) => {
            if (rowsOperation != null) {
                if (rowsOperation.length > 0) {
                    var operation = rowsOperation[0];
                    // clear out null values
                    if (operation.alarmdate === null) { operation.alarmdate = '' };
                    if (operation.alarmtime === null) { operation.alarmtime = '' };
                    if (operation.gear === null) { operation.gear = '' };
                    if (operation.operationnumber === null) { operation.operationnumber = '' };
                    if (operation.location === null) { operation.location = '' };
                    if (operation.message === null) { operation.message = '' };
                    if (operation.community === null) { operation.community = '' };
                    if (operation.rawdata === null) { operation.rawdata = '' };
                    if (operation.object === null) { operation.object = '' };
                    if (operation.district === null) { operation.district = '' };
                    if (operation.keywordId === null) { operation.keywordId = '' };
                    if (operation.keywordCategory === null) { operation.keywordCategory = '' };
                    if (operation.keywordName === null) { operation.keywordName = '' };
                    if (operation.keywordRaw === null) { operation.keywordRaw = '' };
                    if (operation.zveis === null) { operation.zveis = '' };
                    if (operation.subject === null) { operation.subject = '' };
                    if (operation.street === null) { operation.street = '' };
                    if (operation.gkx === null) { operation.gkx = '' };
                    if (operation.gky === null) { operation.gky = '' };
                    if (operation.lat === null) { operation.lat = '' };
                    if (operation.lon === null) { operation.lon = '' };
                    if (operation.floor === null) { operation.floor = '' };
                    if (operation.ils === null) { operation.ils = '' };
                    if (operation.section === null) { operation.section = '' };

                    var retHTML = '<form class="forms-sample" method="post" enctype="multipart/form-data">' +
                        '<div class="card-body">' +
                        '   <div class="custom-control custom-switch pl-5">' +
                        '       <input type="checkbox" class="custom-control-input" id="changedOperation" name="changedOperation" onchange="enableEditMode()">' +
                        '       <label class="custom-control-label" for="changedOperation"><small>Alarmierungsdaten anpassen</small></label> ' +
                        '   </div>' +
                        '   <h5 class="card-title">Alarmierungsdaten</h5>' +
                        '   <div class="row">' +
                        '       <div class="col-md-4">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="operationnumber">Einsatznummer</label>' +
                        '               <input type="text" class="form-control" id="operationnumber" name="operationnumber" placeholder="T 5.2 200808 8058" value="' + operation.operationnumber + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '   </div>' +
                        '   <div class="row">' +
                        '       <div class="col-md-4">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="community">Gemeinde</label>' +
                        '               <input type="text" class="form-control" id="community" name="community" placeholder="Reichenschwand" value="' + operation.community + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-4">' +
                        '           <div class="form-group">' +
                        '               <label for="subject" class="required">Stichwort</label>' +
                        '               <input type="text" class="form-control" id="subject" name="subject" placeholder="THL 1" value="' + operation.subject + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-4">' +
                        '           <div class="form-group">' +
                        '               <div class="form-group">' +
                        '                   <label for="district">Ortsteil</label>' +
                        '                   <input type="text" class="form-control" id="district" name="district" placeholder="Oberndorf"  value="' + operation.district + '" disabled>' +
                        '               </div>' +
                        '           </div>' +
                        '       </div>' +
                        '   </div>' +
                        '   <div class="row">' +
                        '       <div class="col-md-6">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="gear">Einsatzmittel</label>' +
                        '               <textarea class="form-control" name="gear" id="gear" rows="5" placeholder="FF Reichenschwand" required readonly>' + operation.gear + '</textarea>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-6">' +
                        '           <div class="form-group">' +
                        '               <label for="zveis">alarmierte Schleifen</label>' +
                        '               <input type="text" class="form-control" id="zveis" name="zveis" placeholder="12345, 12345F, 34212, 23456"  value="' + operation.zveis + '" disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '   </div>' +
                        '   <div class="row">' +
                        '       <div class="col-md-6">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="gkx">GK-Koordinate X</label>' +
                        '               <input type="text" class="form-control" id="gkx" name="gkx" placeholder="4454576.17"  value="' + operation.gkx + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-6">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="gkx">GK-Koordinate Y</label>' +
                        '               <input type="text" class="form-control" id="gky" name="gky" placeholder="5485861.02"  value="' + operation.gky + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '   </div>' +
                        '   <div class="row">' +
                        '       <div class="col-md-3">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="keywordCategory">Stichwort Kategorie</label>' +
                        '               <input type="text" class="form-control" id="keywordCategory" name="keywordCategory" placeholder="VU"  value="' + operation.keywordCategory + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-3">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="keywordId">Stichwort ID</label>' +
                        '               <input type="text" class="form-control" id="keywordId" name="keywordId" placeholder="T2725"  value="' + operation.keywordId + '"  required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-3">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="keywordName">Stichwort Name</label>' +
                        '               <input type="text" class="form-control" id="keywordName" name="keywordName" placeholder="mit Motorrad"  value="' + operation.keywordName + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-3">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="keywordRaw">Stichwort komplett</label>' +
                        '               <input type="text" class="form-control" id="keywordRaw" name="keywordRaw" placeholder="#T2725#VU#mit Motorrad"  value="' + operation.keywordRaw + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '   </div>' +
                        '   <div class="row">' +
                        '       <div class="col-md-12">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="message">Mitteilung ILS</label>' +
                        '               <textarea class="form-control" name="message" id="message" rows="5" placeholder="Beispieltext der Leitstelle" required readonly>' + operation.message + '</textarea>' +
                        '           </div>' +
                        '       </div>' +
                        '   </div>' +
                        '   <div class="row">' +
                        '       <div class="col-md-3">' +
                        '           <div class="form-group">' +
                        '               <label class="required" for="location">Einsatzort</label>' +
                        '               <input type="text" class="form-control" id="location" name="location" placeholder="Reichenschwand - Reichenschwand"  value="' + operation.location + '" required disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-3">' +
                        '           <div class="form-group">' +
                        '               <label for="object">Objekt</label>' +
                        '               <input type="text" class="form-control" id="object" name="object" placeholder="DORMERO Schlosshotel"  value="' + operation.object + '" disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-3">' +
                        '           <div class="form-group">' +
                        '               <label for="section">Abschnitt</label>' +
                        '               <input type="text" class="form-control" id="section" name="section" placeholder="B14 km 22 - 23"  value="' + operation.section + '" disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-3">' +
                        '           <div class="form-group">' +
                        '               <label for="street">Straße</label>' +
                        '               <input type="text" class="form-control" id="street" name="street" placeholder="B14" value="' + operation.street + '" disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '   </div>' +
                        '   <div class="row">' +
                        '       <div class="col-md-6">' +
                        '           <div class="form-group">' +
                        '               <label for="floor">Stockwerk</label>' +
                        '               <input type="text" class="form-control" id="floor" name="floor" placeholder="1. OG" value="' + operation.floor + '" disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '       <div class="col-md-6">' +
                        '           <div class="form-group">' +
                        '               <label for="ils">alarmierende Leitstelle</label>' +
                        '               <input type="text" class="form-control" id="ils" name="ils" placeholder="ILS Nürnberg" value="' + operation.ils + '" disabled>' +
                        '           </div>' +
                        '       </div>' +
                        '   </div>' +
                        '</div>' +
                        '</form>';
                    return callback(retHTML);
                } else {
                    return callback('Einsatz konnte nicht gefunden werden');
                }
            } else {
                return callback('Einsatz konnte nicht gefunden werden');
            }
        });
    }
}

module.exports = Alertrules;