/**
 * Class handles hierachy stuff
 * @class
 * @author Stefan Bauer
 */
// require the library, main export is a function

const basedata = require('./basedata');


class BasedataHierarchy {
  
    /**
     * @constructor
     * @param {object} alogger Logger to be used by class methods (usually req.app.logger)
     */
    constructor(alogger) {
        // always initialize all instance properties
        this._logger = alogger;
    }


    getOrganizations(callback) {
        let query = `SELECT organizations.*, 
         organizationsTypes.id AS orgaTypeId,
         organizationsTypes.title AS orgaTypeTitle,
         organizationsTypes.colors AS orgaTypeColors
         FROM organizations LEFT JOIN organizationsTypes ON organizations.organizationstype = organizationsTypes.id
            `;
        global.coreDb.all(query, [
            
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    getBasedata(callback) {
      let query = `SELECT * FROM basedata`;
      global.coreDb.all(query, [
          
      ], (err, rows) => {
          if (err) {
              return null;
          } else {
              return callback(rows);
          }
      });
  }


      generateHierarchyJSON(callback) {
         this.getBasedata((basedataRows) => {
            this.getOrganizations((rows) => {
               if (rows != null) {
   
                  var result = [];
   
                  result.push(this.generateRootElement())
                  rows.forEach((entry) => {
                     result.push(this.generateOrgaElement(entry));
                     basedataRows.forEach((basedata) => {
                        if (basedata.organizationid == entry.id) {
                           result.push(this.generateBasedataElement(basedata));
                        }
                     })
                  })  
                  return callback(result);
               }
   
               return callback(null)
           })
         })
        
      }

      generateOrgaElement(entry) {
         return {
            "nodeId":entry.id,
            "parentNodeId":(entry.parent != null && entry.parent != "")  ? entry.parent : "root",
            "width":342,
            "height":146,
            "borderWidth":1,
            "borderRadius":5,
            "borderColor":{
               "red":15,
               "green":140,
               "blue":121,
               "alpha":1
            },
            "backgroundColor":{
               "red":255,
               "green":0,
               "blue":0,
               "alpha":1
            },
            "nodeImage":{
               "url": entry.image_data != null ? entry.image_data : "/assets/img/stock/userDefault.png",
               "width":100,
               "height":100,
               "centerTopDistance":0,
               "centerLeftDistance":0,
               "cornerShape":"CIRCLE",
               "shadow":true,
               "borderWidth":0,
               "borderColor":{
                  "red":19,
                  "green":123,
                  "blue":128,
                  "alpha":1
               }
            },
            "nodeIcon":{
               "icon":"https://to.ly/1yZnX",
               "size":30
            },
            "template":"<div>\n<div style=\"margin-left:70px;\nmargin-top:10px;\nfont-size:20px;\nfont-weight:bold;\n\">"+entry.name+"</div>\n<div style=\"margin-left:70px;\nmargin-top:3px;\nfont-size:16px;\n\">"+ entry.shortName+"</div>\n\n<div style=\"margin-left:70px;\nmargin-top:3px;\nfont-size:14px;\n\">"+ entry.orgaTypeTitle +"</div>\n\n                 <div style=\"margin-left:196px;\n                             margin-top:15px;\n                             font-size:13px;\n                             position:absolute;\n                             bottom:5px;\n                            \">\n                      <div>CTO office</div>\n                      <div style=\"margin-top:5px\">"+" - " +"</div>\n                 </div>\n              </div>",
            "connectorLineColor":{
               "red":220,
               "green":189,
               "blue":207,
               "alpha":1
            },
            "connectorLineWidth":5,
            "dashArray":"",
            "expanded":true,
            "directSubordinates":4,
            "totalSubordinates":1515
         }
      }

      generateBasedataElement(entry) {
         return {
            "nodeId":entry.uuid,
            "parentNodeId": entry.organizationid,
            "width":342,
            "height":146,
            "borderWidth":1,
            "borderRadius":5,
            "borderColor":{
               "red":15,
               "green":140,
               "blue":121,
               "alpha":1
            },
            "backgroundColor":{
               "red":53,
               "green":210,
               "blue":58,
               "alpha":1
            },
            "nodeImage":{
               "url": entry.image_data != null ? entry.image_data : "/assets/img/stock/userDefault.png",
               "width":100,
               "height":100,
               "centerTopDistance":0,
               "centerLeftDistance":0,
               "cornerShape":"CIRCLE",
               "shadow":true,
               "borderWidth":0,
               "borderColor":{
                  "red":19,
                  "green":123,
                  "blue":128,
                  "alpha":1
               }
            },
            "nodeIcon":{
               "icon":"https://to.ly/1yZnX",
               "size":30
            },
            "template":"<div>\n<div style=\"margin-left:70px;\nmargin-top:10px;\nfont-size:20px;\nfont-weight:bold;\n\">"+entry.lastname+ " "+ entry.firstname +"</div>\n<div style=\"margin-left:70px;\n                              margin-top:3px;\n                              font-size:16px;\n                         \">Cheaf Executive Officer  </div>\n\n                 <div style=\"margin-left:70px;\n                              margin-top:3px;\n                              font-size:14px;\n                         \">Business first</div>\n\n                 <div style=\"margin-left:196px;\n                             margin-top:15px;\n                             font-size:13px;\n                             position:absolute;\n                             bottom:5px;\n                            \">\n                      <div>CTO office</div>\n                      <div style=\"margin-top:5px\">Corporate</div>\n                 </div>\n              </div>",
            "connectorLineColor":{
               "red":220,
               "green":189,
               "blue":207,
               "alpha":1
            },
            "connectorLineWidth":5,
            "dashArray":"",
            "expanded":true,
            "directSubordinates":4,
            "totalSubordinates":1515
         }
      }

      generateRootElement(){
         return {
            "nodeId":"root",
            "parentNodeId":null,
            "width":342,
            "height":146,
            "borderWidth":1,
            "borderRadius":5,
            "borderColor":{
               "red":15,
               "green":140,
               "blue":121,
               "alpha":1
            },
            "backgroundColor":{
               "red":0,
               "green":0,
               "blue":0,
               "alpha":1
            },
            "nodeImage":{
               "url":"/assets/img/stock/userDefault.png",
               "width":100,
               "height":100,
               "centerTopDistance":0,
               "centerLeftDistance":0,
               "cornerShape":"CIRCLE",
               "shadow":true,
               "borderWidth":0,
               "borderColor":{
                  "red":19,
                  "green":123,
                  "blue":128,
                  "alpha":1
               }
            },
            "nodeIcon":{
               "icon":"https://to.ly/1yZnX",
               "size":30
            },
            "template":"<div>\n<div style=\"margin-left:70px;\nmargin-top:10px;\nfont-size:20px;\nfont-weight:bold;\n\">ALARMIATOR (SERVERNAME)</div>\n <div style=\"margin-left:70px;\n                              margin-top:3px;\n                              font-size:14px;\n                         \">Wurzel Element</div>\n\n                 <div style=\"margin-left:196px;\n                             margin-top:15px;\n                             font-size:13px;\n                             position:absolute;\n                             bottom:5px;\n                            \">\n                      <div>CTO office</div>\n                      <div style=\"margin-top:5px\">Corporate</div>\n                 </div>\n              </div>",
            "connectorLineColor":{
               "red":220,
               "green":189,
               "blue":207,
               "alpha":1
            },
            "connectorLineWidth":5,
            "dashArray":"",
            "expanded":false,
            "directSubordinates":4,
            "totalSubordinates":1515
         }
    }
}

module.exports = BasedataHierarchy;