const EventEmitter = require('events');
const XLSX = require('xlsx');

class Importer extends EventEmitter {
    
    /**
     * @constructor
     */
    constructor() {
        super();
        this.workbook = null;           // instance of workbook
    }

    /**
     * Opens an existing file from filesystem
     * @param {string} filepath path to Excel-File to be opened
     */
    openXSLX(filepath) {
        this.workbook = XLSX.readFile(filepath);
    }

    /**
     * Returns array of Sheets existing in Workbook
     */
    getSheetnames(callback) {
        var sheetNames = [];
        this.workbook.SheetNames.forEach((sheet, index) => {
            console.log('MODELS-IMPORTER | ' + index + ' | found Sheet: ' + sheet);
            var sheet = {
                name: sheet,
                index: index
            }
            sheetNames.push(sheet);
        })
        return callback(sheetNames);
    }

    /**
     * Returns content of given worksheet
     * @param {string} sheetName Name of sheet to return JSON data for
     * @param {*} callback JSON-Object containing data from worksheet
     */
    getContentOfSheetAsJSON(sheetName, callback) {
        var JSON = XLSX.utils.sheet_to_json(this.workbook.Sheets[sheetName]);
        return callback(JSON);
    };


    getFirstLineOfSheetsAsJSON(sheetName, callback) {
        var row = null;
        var JSON = XLSX.utils.sheet_to_json(this.workbook.Sheets[sheetName]);
        if (JSON != null) {
            if (JSON.length > 0) {
                row = JSON[0];
            }
        }
        return callback(row);
    }


}

module.exports = Importer;