var uuid = require('uuid'); // required for generating timebased guids
var moment = require('moment'); // require

/**
 * Core Object for operations
 * @namespace core/operation
 * @class
 */
class Operation {
    /**
     * Returns an empty operation object
     * @constructor
     */
    constructor() {

        this._createdAt = {
            value: null,
            label: 'Empfang Server'
        }
        this._alarmdate = {
            value: null,
            label: 'Datum'
        }
        this._alarmtime = {
            value: null,
            label: 'Uhrzeit'
        };
        this._alarmtimeCalc = {
            value: null,
            label: 'Alarmzeit'
        };
        this._community = {
            value: null,
            label: 'Gemeinde'
        };
        this._district = {
            value: null,
            label: 'Ortsteil'
        };
        this._gear = {
            value: null,
            label: 'Einsatzmittel'
        };
        this._gearCalc = {
            value: null,
            label: 'Liste Einsatzmittel'
        };
        this._gkx = {
            value: null,
            label: 'GK-Koordinate X'
        };
        this._gky = {
            value: null,
            label: 'GK-Koordinate Y'
        };
        this._keywordCategory = {
            value: null,
            label: 'Stichwort Kategorie'
        };
        this._keywordId = {
            value: null,
            label: 'Stichwort ID'
        };
        this._keywordName = {
            value: null,
            label: 'Stichwort Name'
        };
        this._keywordRaw = {
            value: null,
            label: 'Stichwort ABEK'
        };
        this._lat = {
            value: null,
            label: 'WGS84 Latitude'
        };
        this._location = {
            value: null,
            label: 'Ort'
        };
        this._locationCalc = {
            value: null,
            label: 'Ortsangabe'
        }
        this._lon = {
            value: null,
            label: 'WGS84 Longitude'
        };
        this._message = {
            value: null,
            label: 'Mitteilung'
        };
        this._object = {
            value: null,
            label: 'Objekt'
        };
        this._operationSourceConnector = {
            value: null,
            label: 'Quelle'
        };
        this._operationState = {
            value: null,
            label: 'Zustand'
        };
        this._operationnumber = {
            value: null,
            label: 'Einsatznummer'
        };
        this._rawdata = {
            value: null,
            label: 'Alarmierung Rohdaten'
        };
        this._section = {
            value: null,
            label: 'Abschnitt'
        };
        this._street = {
            value: null,
            label: 'Straße'
        };
        this._subject = {
            value: null,
            label: 'Stichwort'
        };
        this._zveis = {
            value: null,
            label: 'Schleifen'
        };
        this._zveisCalc = {
            value: null,
            label: 'Schleife Liste'
        };
        this._zveisDelta = {
            value: null,
            label: 'Schleifen Delta'
        },
        this._zveisDeltaCalc = {
            value: null,
            label: 'Alarmierte Schleifen'
        };
        this._gearData = {
            value: null,
            label: 'Einsatzmittel Daten'
        },
        this._gearDataCalc = {
            value: null,
            label: 'Alarmierte Einsatzmittel'
        };
        this._floor = {
            value: null,
            label: 'Etage'
        }
        this._ils = {
            value: null,
            label: 'Leitstelle'
        }
        this._uuid = {
            value: null,
            label: 'UUID'
        };
        this._isTest = {
            value: false,
            label: 'Testalarmierung'
        };

        this._alarmedGroupIds = {
            value: null,
            label: 'Alarmierte Gruppen IDs'
        };

        this.setCreatedAtTimestamp();
    }

    /**
     * Automatic timestamp of creation of this object instance
     * @type {string}
     */
    set createdAt(createdAt) {
        this._createdAt.value = createdAt;
    }
    get createdAt() {
        return this._createdAt;
    }

    /**
     * Section information (Kreuzung)
     * @type {string}
     */
    set section(section) {
        this._section.value = section;
        // Calculate locationCalc if location and object not set
        if ((this.isDefined(this._location) === false) && (this.isDefined(this._object) === false) && (this.isDefined(this._section) === true)) {
            this._locationCalc.value = section;
        }
    }
    get section() {
        return this._section;
    }

    /**
     * Date the alarm has been raised
     * @type {string}
     */
    set alarmdate(alarmdate) {
        this._alarmdate.value = alarmdate;
        if (this._alarmtime.value !== null) {
            this._alarmtimeCalc.value = this._alarmdate.value + ' ' + this._alarmtime.value;
        }
    }
    get alarmdate() {
        return this._alarmdate;
    }

    /**
     * Time the alarm has been raised
     * @type {string}
     */
    set alarmtime(alarmtime) {
        this._alarmtime.value = alarmtime;
        if (this._alarmdate.value !== null) {
            this._alarmtimeCalc.value = this._alarmdate.value + ' ' + this._alarmtime.value;
        }
    }
    get alarmtime() {
        return this._alarmtime;
    }

    /**
     * Alarm time calculated (will be calculated as soon alarmdate and alarmtime have been set)
     * @type {string}
     */
    set alarmtimeCalc(alarmtimeCalc) {
        this._alarmtimeCalc.value = alarmtimeCalc;
    }
    get alarmtimeCalc() {
        return this._alarmtimeCalc;
    }
        
    /**
     * Gear as string
     * @type {string}
     */
    set gear(gear) {
        if (gear == null) gear = ""
        this._gear.value = gear;
        var gearArray = this._gear.value.split(/\r?\n/);
        var filtered = gearArray.filter(function (el) {
            return el != null;
            });
        this._gearCalc.value = filtered;

        try {
            if (this._gearDataCalc.value == null){
                this.gearDataCalc = this.calculateZveisGearData('gear', gear);
            }
        }
        catch (exp) {
            console.log(exp);
            // Error setting gearDataCalc
        }
    }
    get gear() {
        return this._gear;
    }
      
    /**
     * List of gear from gear split by separator character "\n"
     * @type {Array}
     */
    set gearCalc(gear) {
        this._gearCalc.value = gear;
    }
    get gearCalc() {
        return this._gearCalc;
    }
      
    /**
     * Number of operation as reported by source ('Einsatznummer')
     * @type {string}
     */
    set operationnumber(operationnumber) {
        this._operationnumber.value = operationnumber;
    }
    get operationnumber() {
        return this._operationnumber;
    }
      
    /**
     * Location ('Einsatznummer')
     * @type {string}
     */
    set location(location) {
        this._location.value = location;
        if (this.isDefined(this._location) === true){
            this._locationCalc.value = location;
        }
    }
    get location() {
        return this._location;
    }

    /**
     * Message for alarm ('Bemerkung')
     * @type {string}
     */
    set message(message) {
        this._message.value = message;
    }
    get message() {
        return this._message;
    }

    /**
     * community ('Gemeinde')
     * @type {string}
     */
    set community(community) {
        this._community.value = community;
    }
    get community() {
        return this._community;
    }  

    /**
     * stores the whole alarm as a raw text field ('sds_komplett')
     * @type {string}
     */
    set rawdata(rawdata) {
        this._rawdata.value = rawdata;
    }
    get rawdata() {
        return this._rawdata;
    }      

    /**
     * Object named in alarm ('Objekt'), e.g. "Schlosshotel DORMERO Reichenschwand"
     * @type {string}
     */
    set object(object) {
        this._object.value = object;
        if (this.isDefined(this._location) === false && this.isDefined(this._object) === true) {
            this._locationCalc.value = object;
        }
    }
    get object() {
        return this._object;
    }    
        
    /**
     * district
     * @type {string}
     */
    set district(district) {
        this._district.value = district;
    }
    get district() {
        return this._district;
    }      
        
    /**
     * keywordId
     * @type {string}
     */
    set keywordId(keywordId) {
        this._keywordId.value = keywordId;
    }
    get keywordId() {
        return this._keywordId;
    }     

    /**
     * keywordCategory
     * @type {string}
     */
    set keywordCategory(keywordCategory) {
        this._keywordCategory.value = keywordCategory;
    }
    get keywordCategory() {
        return this._keywordCategory;
    }     

    /**
     * keywordRaw
     * @type {string}
     */
    set keywordRaw(keywordRaw) {
        this._keywordRaw.value = keywordRaw;
    }
    get keywordRaw() {
        return this._keywordRaw;
    }  

    /**
     * keywordName
     * @type {string}
     */
    set keywordName(keywordName) {
        this._keywordName.value = keywordName;
    }
    get keywordName() {
        return this._keywordName;
    }  
        
    /**
     * zveis listed in alarm as string. Will calculate zveiCalc after setting
     * @type {string}
     */
    set zveis(zveis) {
        if (zveis == null) zveis = ""
        this._zveis.value = zveis;
        this._zveisCalc.value = this._zveis.value.split(',').map(x => x.trim()).filter(x => x);
     
        try {
            if (this._zveisDeltaCalc.value == null){
                this.zveisDeltaCalc = this.calculateZveisGearData('zvei', zveis);
            }
        }
        catch (exp) {
            // Error setting gearDataCalc
        }
    }
    get zveis() {
        return this._zveis;
    }    
        
     /**
     * List of ZVEIs as an Array
     * @type {Array}
     */
    set zveisCalc(zveisCalc) {
        this._zveisCalc.value = zveisCalc;
    }
    get zveisCalc() {
        return this._zveisCalc;
    }     
       
    /**
     * zveis delta listed in alarm as string. Will calculate zveiDeltaCalc after setting
     * @type {string}
     */
    set zveisDelta(zveisDelta) {
        if ((typeof zveisDelta != 'undefined') && (zveisDelta != null)) {
            this._zveisDelta.value = zveisDelta;
            this._zveisDeltaCalc.value = this._zveisDelta.value.split(',').map(x => x.trim()).filter(x => x);
        }
    }
    get zveisDelta() {
        return this._zveisDelta;
    }    
        
    /**
     * List of ZVEIs delta as an Array
     * @type {Array}
     */
    set zveisDeltaCalc(zveisDeltaCalc) {
        this._zveisDeltaCalc.value = zveisDeltaCalc;
    }
    get zveisDeltaCalc() {
        return this._zveisDeltaCalc;
    }     
    
    /**
     * gear Data listed in alarm as string. Will calculate gearDataCalc after setting
     * @type {string}
     */
    set gearData(gearData) {
        if ((typeof gearData != 'undefined') && (gearData != null)) {
            this._gearData.value = gearData;
            this._gearDataCalc.value = this._gearData.value.split('\n').map(x => x.trim()).filter(x => x);
        }
    }
    get gearData() {
        return this._gearData;
    }    
        
    /**
     * List of gear data as an Array
     * @type {Array}
     */
    set gearDataCalc(gearDataCalc) {
        this._gearDataCalc.value = gearDataCalc;
    }
    get gearDataCalc() {
        return this._gearDataCalc;
    }  

     /**
     * Subject of Alarm ('Stichwort')
     * @type {string}
     */
    set subject(subject) {
        this._subject.value = subject;
    }
    get subject() {
        return this._subject;
    }   

     /**
     * street
     * @type {string}
     */
    set street(street) {
        this._street.value = street;
        if  (
            (this.isDefined(this._location) === false) &&
            (this.isDefined(this._object) === false) &&
            (this.isDefined(this._section) === false) &&
            (this.isDefined(this._street) === true)
        ) {
            this.locationCalc = street;
        }
    }
    get street() {
        return this._street;
    } 
    
     /**
     * gkx - Gauss Krueger X-Coordinate
     * @type {string}
     */
    set gkx(gkx) {
        this._gkx.value = gkx;
    }
    get gkx() {
        return this._gkx;
    } 

    /**
     * gky- Gauss Krueger Y-Coordinate
     * @type {string}
     */
    set gky(gky) {
        this._gky.value = gky;
    }
    get gky() {
        return this._gky;
    } 

     /**
     * latitude
     * @type {string}
     */
    set lat(lat) {
        this._lat.value = lat;
    }
    get lat() {
        return this._lat;
    }  
    
     /**
     * longitude
     * @type {string}
     */
    set lon(lon) {
        this._lon.value = lon;
    }
    get lon() {
        return this._lon;
    } 
    
    /**
    * Etage
    * @type {string}
    */
    set floor(floor) {
        this._floor.value = floor;
    }
    get floor() {
        return this._floor;
    }
        
     /**
     * Leitstelle
     * @type {string}
     */
    set ils(ils) {
        this._ils.value = ils;
    }
    get ils() {
        return this._ils;
    }

     /**
     * operationState (0 if new, 1 if archived)
     * @type {integer}
     */
    set operationState(operationState) {
        this._operationState.value = operationState;
    }
    get operationState() {
        return this._operationState;
    }   
        
    /**
     * namespace of source for this alarm (core or <<namespace>>)
     * @type {string}
     */
    set operationSourceConnector(operationSourceConnector) {
        this._operationSourceConnector.value = operationSourceConnector;
    }
    get operationSourceConnector() {
        return this._operationSourceConnector;
    }  
    
    /**
     * GUID of this operation object
     * @type {string}
     */
    set uuid(uuid) {
        this._uuid.value = uuid;
    }
    get uuid() {
        return this._uuid;
    }  

    /**
     * Test-property of this operation object
     * @type {boolean}
     */
    set isTest(isTest) {
        this._isTest.value = isTest;
    }
    get isTest() {
        return this._isTest;
    }  

    /**
     * alarmedGroupIds-property of this operation object
     * @type {Array}
     */
    set alarmedGroupIds(alarmedGroupIds) {
        this._alarmedGroupIds.value = alarmedGroupIds;
    }
    get alarmedGroupIds() {
        return this._alarmedGroupIds;
    }  

    /**
     * Calculated Location
     * @type {string}
     */
    set locationCalc(locationCalc) {
        this._locationCalc.value = locationCalc;
    }
    get locationCalc() {
        return this._locationCalc;
    }  

    
    /**
     * Sets a new guuid for this operation
     */
    setUUID() {
        this._uuid.value = uuid.v1();
    }

    /**
     * sets a timestamp in _createdAt.value while instanziiating this class
     */
    setCreatedAtTimestamp() {
        var timestamp = moment().format();
        this._createdAt.value = timestamp;
    }


    /**
     * Returns false if property is null or length = 0
     * @param {object} value property to be checked
     */
    isDefined(value) {
        var retVal = true;
        if (value == undefined) return false;
        if (value.value == undefined) return false;
        if (value.value === null) {
            retVal = false;
        } else {
            if (value.value.length > 0) {
                retVal = true;
            } else { 
                retVal = false;
            }
        }
        return retVal;
    }

    /**
     * Returns minified object which can be used as payload for alarming via FCM or other push services
     */
    getReduced() {
        var reduced = {};
        if (this.isDefined(this._createdAt)) {
            reduced._createdAt = this.createdAt;
            reduced._createdAt = this.createdAt;
        }
        if (this.isDefined(this._alarmdate)) {
            reduced._alarmdate = this.alarmdate;
        }
        if (this.isDefined(this._alarmtime)) {
            reduced._alarmtime = this.alarmtime;
        };
        if (this.isDefined(this._community)) {
            reduced._community = this.community;
        };
        if (this.isDefined(this._district)) {
            reduced._district = this.district;
        };
        if (this.isDefined(this._gear)) {
            reduced._gear = this.gear;
        };
        if (this.isDefined(this._keywordCategory)) {
            reduced._keywordCategory = this.keywordCategory;
        } else {
            reduced._keywordCategory = {
                value: '',
                label: 'Stichwort Kategorie'
            };
        };
        if (this.isDefined(this._keywordId)) {
            reduced._keywordId = this.keywordId;
        } else {
            reduced._keywordId =  {
                value: '',
                label: 'Stichwort ID'
            };
        };
        if (this.isDefined(this._keywordName)) {
            reduced._keywordName = this.keywordName;
        } else {
            reduced._keywordName =  {
                value: '',
                label: 'Stichwort Name'
            };
        };
        
        // LATITUDE HAS TO BE SENT ALWAY, NO CHECK NEEDED
        reduced._lat = this.lat;
        
        if (this.isDefined(this._location)) {
            reduced._location = this.location;
        };
        if (this.isDefined(this.locationCalc)) {
            reduced._locationCalc = this.locationCalc;
        }
        
        // LONGITUDE HAS TO BE SENT ALWAYS, NO CHECK NEEDED
        reduced._lon = this.lon;
        
        if (this.isDefined(this._message)) {
            reduced._message = this.message;
        };
        if (this.isDefined(this._object)) {
            reduced._object = this.object;
        };
        if (this.isDefined(this._operationnumber)) {
            reduced._operationnumber = this.operationnumber;
        };
        if (this.isDefined(this._section)) {
            reduced._section = this.section;
        };
        if (this.isDefined(this._street)) {
            reduced._street = this.street;
        };
        if (this.isDefined(this._subject)) {
            reduced._subject = this.subject;
        };
        if (this.isDefined(this._zveis)) {
            reduced._zveis = this.zveis;
        };
        /*
        if (this.isDefined(this._zveisDelta)) {
            reduced._zveisDelta = this.zveisDelta;
        };
        // ZVEIs Delta Calc
        if (this.isDefined(this._zveisDeltaCalc)) {
            reduced._zveisDeltaCalc = this.zveisDeltaCalc;
        };
        // Gear Data Calc
        if (this.isDefined(this._gearDataCalc)) {
            reduced._gearDataCalc = this.gearDataCalc;
        };
        */
        if (this.isDefined(this._uuid)) {
            reduced._uuid = this.uuid;
        };
        if (this.isDefined(this._floor)) {
            reduced._floor = this.floor;
        };
        if (this.isDefined(this._ils)) {
            reduced._ils = this.ils;
        };
        if (this.isDefined(this._isTest)) {
            reduced._isTest = this.isTest;
        };
        
        return reduced;
    }

    calculateZveisGearData(mode, allElements) {
        // mode should be "zvei" or "gear"
        let datePart = moment().format('YYYY-MM-DD');
        let datePartYesterday = moment().subtract(1, 'days').format('YYYY-MM-DD');
        let dataObj = [];

        const RANGE = 1; // Timerange +-1 Minute

        let seperator;

        switch (mode) {
            case "gear":
                seperator = /\n|;/gm;
                break;
            case "zvei":
            default:
                seperator = /\n|,|;/gm;
        }

        const timeRegex = /\d{2}:\d{2}[:\d{2}]*/m;
    
        if (allElements != null && allElements != undefined) {
            if (allElements.length > 0) {
                // split now and extract time
                // for cases where separator is ","
                var allElementsArr = allElements.split(seperator);
                if (Array.isArray(allElementsArr)) {
                    // successfully separated zveis with time into array
                    allElementsArr.forEach(elementWithTime => {
                        if (elementWithTime.match(timeRegex)) {
                            // has time
                            let elementString = elementWithTime.replace(timeRegex,'').trim();
                            let timeString = elementWithTime.match(timeRegex);
    
                            let timestamp = moment(datePart + ' ' + timeString).format('YYYY-MM-DDTHH:mm:ss');
                            let sameOrBefore = moment().add(1, 'minutes').isSameOrBefore(timestamp);
                            // console.log('PARSING: element: ' + elementString + ' | time: ' + timeString + ' | timestamp: ' + timestamp + '| same or before now: ' + sameOrBefore);
                            if (sameOrBefore) timestamp = moment(datePartYesterday + ' ' + timeString).format('YYYY-MM-DDTHH:mm:ss');
                            
                            let newElement = true;

                            if (moment(timestamp).isBetween(
                                moment().seconds(0).milliseconds(0).subtract(RANGE, 'minutes'),
                                moment().seconds(59).milliseconds(0).add(RANGE, 'minutes'),
                                undefined,
                                "[]")) {
                                newElement = true;
                            }
                            else {
                                newElement = false;
                            }

                            let deltaElement = {
                                timestamp: timestamp,
                                new: newElement
                            }
                            deltaElement[mode] = elementString;
    
                            dataObj.push(deltaElement);
    
                        }
                        else {
                            // no time
                            let deltaElement = {
                                timestamp: null,
                                new: true
                            }
                            deltaElement[mode] = elementWithTime.replace(timeRegex,'').trim();
                            dataObj.push(deltaElement);
                            // console.log('WARNING: only zvei / gear element found but no time: ' + elementWithTime);
                        }
                    });
                } else {
                    // not an array, do nothing
                    return [];
                }
            }
        }
        //console.log("Full Dataobject", dataObj);
        return dataObj;
    }

    /**
     * Fills operation object from a JSON object (e.g. transfering operation over ipc)
     * @param {object} jsonData operation json object
     */
    fillFromJson(jsonData){
        if (jsonData._alarmdate) this._alarmdate = jsonData._alarmdate;
        if (jsonData._alarmtime) this._alarmtime = jsonData._alarmtime;
        if (jsonData._alarmtimeCalc) this._alarmtimeCalc = jsonData._alarmtimeCalc;
        if (jsonData._community) this._community = jsonData._community;
        if (jsonData._district) this._district = jsonData._district;
        if (jsonData._gear) this._gear = jsonData._gear;
        if (jsonData._gearCalc) this._gearCalc = jsonData._gearCalc;
        if (jsonData._gkx) this._gkx = jsonData._gkx;
        if (jsonData._gky) this._gky = jsonData._gky;
        if (jsonData._keywordCategory) this._keywordCategory = jsonData._keywordCategory;
        if (jsonData._keywordId) this._keywordId = jsonData._keywordId;
        if (jsonData._keywordName) this._keywordName = jsonData._keywordName;
        if (jsonData._keywordRaw) this._keywordRaw = jsonData._keywordRaw;
        if (jsonData._lat) this._lat = jsonData._lat;
        if (jsonData._location) this._location = jsonData._location;
        if (jsonData._lon) this._lon = jsonData._lon;
        if (jsonData._message) this._message = jsonData._message;
        if (jsonData._object) this._object = jsonData._object;
        if (jsonData._operationSourceConnector) this._operationSourceConnector = jsonData._operationSourceConnector;
        this._operationState = 1;
        if (jsonData._operationnumber) this._operationnumber = jsonData._operationnumber;
        if (jsonData._rawdata) this._rawdata = jsonData._rawdata;
        if (jsonData._section) this._section = jsonData._section;
        if (jsonData._street) this._street = jsonData._street;
        if (jsonData._subject) this._subject = jsonData._subject;
        if (jsonData._zveis) this._zveis = jsonData._zveis;
        if (jsonData._zveisCalc) this._zveisCalc = jsonData._zveisCalc;

        // added new ZVEIs Delta
        if (jsonData._zveisDelta) this._zveisDelta = jsonData._zveisDelta;
        if (jsonData._zveisDeltaCalc) this._zveisDeltaCalc = jsonData._zveisDeltaCalc;

        // added new Gear Data
        if (jsonData._gearData) this._gearData = jsonData._gearData;
        if (jsonData._gearDataCalc) this._gearDataCalc = jsonData._gearDataCalc;

        if (jsonData._floor) this._floor = jsonData._floor;
        if (jsonData._ils) this._ils = jsonData._ils;

        if (jsonData._locationCalc) this._locationCalc = jsonData._locationCalc;

        if (jsonData._alarmedGroupIds) this._alarmedGroupIds = jsonData._alarmedGroupIds;
        if (jsonData._isTest) this._isTest = jsonData._isTest;

        if (jsonData._uuid && jsonData._uuid.value != null) {
            this._uuid = jsonData._uuid;
        }
        else {
            this.setUUID();
            console.log("NEW GUID CALCULATED FOR OPERATION -> " + this._uuid.value)
        }
    }
}

module.exports = Operation;

