
/**
 * Core Object for gear state
 * @namespace core/gearstate
 * @class
 */
class gearstate {
    /**
     * Returns an empty operation object
     * @constructor
     */
    constructor() {
        this.issi = '';                     // ISSI number of gear state references to
        this.radioState = '';               // radioState as sent by source;
        this.radioStateHumanReadable = '';  // radioState transformed to a human readable format
        this.radioStateShort = '';          // (1-9) 
        this.timestamp = '';                // Timestamp of the radio state
        this.source = '';                   // Source the state has sent in ("core", "<<pluginnamespace>>")
    }

    /**
     * ISSI Number of gear the state belongs to
     * @type {integer}
     */
    set issi(issi) {
        this._issi = issi;
    }
    get issi() {
        return this._issi;
    }

    /**
     * RadioState as sent by source
     * @type {integer}
     */
    set radioState(radioState) {
        this._radioState = radioState;
    }
    get radioState() {
        return this._radioState;
    }

    /**
     * RadioState transformed in a human readable format as sent by source
     * @type {string}
     */
    set radioStateHumanReadable(radioStateHumanReadable) {
        this._radioStateHumanReadable = radioStateHumanReadable;
    }
    get radioStateHumanReadable() {
        return this._radioStateHumanReadable;
    }

    /**
     * RadioState transformed in (1-9) as sent by source
     * @type {string}
     */
    set radioStateShort(radioStateShort) {
        this._radioStateShort = radioStateShort;
    }
    get radioStateShort() {
        return this._radioStateShort;
    }

    /**
     * Timestamp of the state
     * @type {string}
     */
    set timestamp(timestamp) {
        this._timestamp = timestamp;
    }
    get timestamp() {
        return this._timestamp;
    }

    /**
     * Source of the state (core or plugin-namespace)
     * @type {string}
     */
    set source(source) {
        this._source = source;
    }
    get source() {
        return this._source;
    }

}

module.exports = gearstate;

