const EventEmitter = require('events');
const moment = require('moment');
class FeedbackStore extends EventEmitter {

    /**
     * @constructor
     */
    constructor(alogger) {
        super();
        this.logger = alogger;
    }


    /**
     * Adds new feedback objects for basedata to core database, ususally caused by an new alarming
     * @param {integer} operationId id of operations object this feedbacks shall be added for
     * @param {array} basedataUUIDArray array of UUIDs of basedata to add, e.g. ['uuid1', 'uuid2']
     * @param {function} callback success (true/false)
     */
    generateFeedbackObjectsForOperation(operationId, basedataUUIDArray, callback) {
        if (Array.isArray(basedataUUIDArray)) {

            // render timestamp
            // Create an instance of the Date class
            var date = new Date();
            // Convert it to an ISO string
            var sqliteDate = date.toISOString();

            let query = "INSERT INTO feedbackStore " +
                "(operationId, basedataUUID, stateId, stateSet) ";
            query += 'VALUES ';

            var counter = basedataUUIDArray.length;
            basedataUUIDArray.forEach((uuid, index) => {
                query += " (" + operationId + ", '" + uuid + "', 0, '" + sqliteDate + "')";
                if ((index + 1) < counter) {
                    query += ", ";
                }
            })

            console.log(query);

            global.coreDb.run(query, [], function (err) {
                if (err) {
                    console.log(`ERROR Adding new feedbackstate objects to core database: ${err.message}`);
                    callback(false);
                } else {
                    callback(true);
                }
            });
        } else {
            // No UUIDs for basedata entires handed over
            return callback(false);
        }
    };


    /**
     * Returns basedata and set feedback state for given operation id
     * @param {integer} operationId id of operation
     * @param {*} callback rows of null
     */
    getFeedbackForOperationWithId(operationId, callback) {
        let query = "SELECT DISTINCT " +
                        "basedata.id AS basedataId, " +
                        "basedata.firstname, " +
                        "basedata.lastname, " +
                        "basedata.uuid AS basedataUUID, " +
                        "feedbackStore.stateId, " +
                        "feedbackStates.state AS feedbackState, " +
                        "feedbackStates.stateColor AS feedbackStateColor, " +
                        "feedbackStates.stateLabel AS feedbackStateLabel, " +
                        "feedbackStore.stateSet AS feedbackStateSet, " +
                        "( " +
                            "SELECT GROUP_CONCAT(qualifications.shortName) " +
                            "FROM qualifications AS qualis " +
                            "WHERE xt_basedata_qualifications.qualificationId = qualis.id " +
                        ") AS qualificationsList " +
                    "FROM basedata " +
                    "JOIN feedbackStore ON feedbackStore.basedataUUID = basedata.uuid " +
                    "JOIN feedbackStates ON feedbackStates.id = feedbackStore.stateId " +
                    "LEFT JOIN xt_basedata_qualifications ON xt_basedata_qualifications.basedataId = basedata.id " +
                    "LEFT JOIN qualifications ON qualifications.id = xt_basedata_qualifications.qualificationId " +
                    "WHERE feedbackStore.operationId = ? " +
                    "GROUP BY basedata.id " +
                    "ORDER BY basedata.lastname ASC";
        global.coreDb.all(query, [
            operationId
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns basedata and set feedback state for given operation id and organizationId
     * @param {integer} operationId id of operation
     * @param {integer} organizationId id of organization
     * @param {*} callback rows of null
     */
    getFeedbackForOperationWithIdAndOrganization(operationId,organizationId, callback) {
        let query = "SELECT DISTINCT " +
                        "basedata.id AS basedataId, " +
                        "basedata.firstname, " +
                        "basedata.lastname, " +
                        "basedata.uuid AS basedataUUID, " +
                        "feedbackStore.stateId, " +
                        "feedbackStates.state AS feedbackState, " +
                        "feedbackStates.stateColor AS feedbackStateColor, " +
                        "feedbackStates.stateLabel AS feedbackStateLabel, " +
                        "feedbackStore.stateSet AS feedbackStateSet, " +
                        "( " +
                            "SELECT GROUP_CONCAT(qualifications.shortName) " +
                            "FROM qualifications AS qualis " +
                            "WHERE xt_basedata_qualifications.qualificationId = qualis.id " +
                        ") AS qualificationsList " +
                    "FROM basedata " +
                    "JOIN feedbackStore ON feedbackStore.basedataUUID = basedata.uuid " +
                    "JOIN feedbackStates ON feedbackStates.id = feedbackStore.stateId " +
                    "LEFT JOIN xt_basedata_qualifications ON xt_basedata_qualifications.basedataId = basedata.id " +
                    "LEFT JOIN qualifications ON qualifications.id = xt_basedata_qualifications.qualificationId " +
                    "WHERE feedbackStore.operationId = ? AND basedata.organizationid = ?" +
                    "GROUP BY basedata.id " +
                    "ORDER BY basedata.lastname ASC";
        global.coreDb.all(query, [
            operationId,
            organizationId
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * returns the operationId for a given operation uuid
     * @param {string} uuid 
     * @param {function} callback rows or null
     */
    getOperationIdForUUID(uuid, callback) {
        let query = 'SELECT * FROM operationsStore WHERE uuid = ?';
        global.coreDb.all(query, [
            uuid
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * returns the operation
     * @param {string} operationId 
     * @param {function} callback rows or null
     */
    getOperationForId(id, callback) {
        let query = 'SELECT * FROM operationsStore WHERE id = ?';
        global.coreDb.all(query, [
            id
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Sets a new feedbackstate for given operationUUID and basedataUUID
     * @param {string} basedataUUID uuid of basedata
     * @param {string} operationUUID uuid of operation
     * @param {integer} state integer value from table feedbackStates
     * @param {function} callback success (true/false)
     */
    setFeedback(basedataUUID, operationUUID, state, callback) {
        this.getOperationIdForUUID(operationUUID, (rowOperation) => {
            var operationId = null;
            if (rowOperation !== null) {
                if (rowOperation.length > 0) {
                    operationId = rowOperation[0].id;
                }
            }
            // now set state
            if (operationId !== null) {
                let query = "UPDATE feedbackStore SET stateId = ?, stateSet = ? WHERE basedataUUID = ? and operationId = ?";
                global.coreDb.run(query, [
                    state,
                    moment().toISOString(),
                    basedataUUID,
                    operationId
                ], function (err) {
                    if (err) {
                        console.log(`ERROR setting new feedbackstate at core database: ${err.message}`);
                        callback(false);
                    } else {
                        callback(true);
                    }
                });
            } else {
                callback(false);
            }
            
        })
    }

    /**
     * Returns feedback for operationUUID and parentOperations basedataUUID
     * @param {integer} basedataUUID uuid of basedata
     * @param {integer} operationUUID uuid of operation
     * @param {*} callback rows of null
     */
    getParentFeedbackForOperationWithUUIDandBasedataUUID(basedataUUID, operationUUID, callback) {

        this.getOperationIdForUUID(operationUUID, (rowOperation) => {
            var operationId = null;
            var subOperationFor = null
            if (rowOperation !== null) {
                if (rowOperation.length > 0) {
                    operationId = rowOperation[0].id
                    subOperationFor = rowOperation[0].subOperationFor;
                }
            }

            if (operationId !== null && subOperationFor !== null) {
                let query = `
                SELECT DISTINCT 
                    feedbackStore.basedataUUID,

                    CASE
                        WHEN calcStateId IS NOT NULL THEN calcStateId
                        ELSE opFeedbackStore.opStateId
                    END AS stateId,

                    CASE
                        WHEN calcStateSet IS NOT NULL THEN calcStateSet
                        ELSE opFeedbackStore.opStateSet
                    END AS stateSet
                FROM feedbackStore
                LEFT JOIN operationsStore
                    ON feedbackStore.operationId = operationsStore.id
                    
                LEFT JOIN (
                    SELECT 
                        feedbackStore.stateId as calcStateId,
                        feedbackStore.stateSet as calcStateSet,
                        operationsStore.subOperationFor
                    FROM 
                        feedbackStore
                    JOIN
                        operationsStore ON operationsStore.id = feedbackStore.operationId
                    WHERE 
                        (
                            feedbackStore.operationId = ? OR 
                            (
                                operationsStore.subOperationFor = ? AND 
                                operationsStore.subOperationFor != 0
                            ) OR 
                            (
                                operationsStore.id = ? AND 
                                operationsStore.subOperationFor == 0
                            ) OR 
                            (
                                operationsStore.subOperationFor = ?
                            ) 
                        ) AND 
                        operationsStore.id <= ? AND 
                        basedataUUID = ? AND
                        stateId != 0

                    ORDER BY
                        feedbackStore.stateSet DESC
                    LIMIT 1
                ) as joinedStore
                
                LEFT JOIN
                ( 
                    SELECT 
                        feedbackStore.operationId as opId,
                        feedbackStore.stateId as opStateId,
                        feedbackStore.stateSet as opStateSet
                    FROM 
                        feedbackStore
                ) as opFeedbackStore   
                    ON opFeedbackStore.opId = feedbackStore.operationId
                    WHERE feedbackStore.operationId = ?
                        GROUP BY
                            feedbackStore.basedataUUID
                `
                global.coreDb.all(query, [
                    
                    operationId,
                    subOperationFor,
                    subOperationFor,
                    operationId,
                    operationId,
                    basedataUUID,
                    operationId
                    

                ], (err, rows) => {
                    if (err) {
                        console.log(`ERROR getting feedbackstate for a specific operation uuid: ${err.message}`);
                        callback(null);
                    } else {
                        callback(rows);
                    }
                });
            } else {
                callback(null);
            }
            
        })
    }

    /**
     * Returns feedback for operationId and parentOperations
     * @param {integer} operationId uuid of operation
     * @param {*} callback rows of null
     */
    getParentFeedbackForOperationId(operationId, callback) {

        this.getOperationForId(operationId, (rowOperation) => {
            var subOperationFor = null
            if (rowOperation !== null) {
                if (rowOperation.length > 0) {
                    subOperationFor = rowOperation[0].subOperationFor;
                }
            }

            if (operationId !== null && subOperationFor !== null) {
                let query = `
                SELECT OpFeedback.basedataUUID,
                    OpFeedback.firstname,
                    OpFeedback.lastname,
                    OpFeedback.basedataId,
                    OpFeedback.stateId,
                    OpFeedback.feedbackStateSet,
                    OpFeedback.qualificationsList,
                    feedbackStates.state AS feedbackState,
                    feedbackStates.stateColor AS feedbackStateColor,
                    feedbackStates.stateLabel AS feedbackStateLabel
                FROM (
                        SELECT DISTINCT feedbackStore.basedataUUID AS basedataUUID,
                                        basedata.firstname AS firstname,
                                        basedata.lastname AS lastname,
                                        basedata.id AS basedataId,
                                        CASE WHEN calcStateId IS NOT NULL THEN calcStateId ELSE opFeedbackStore.opStateId END AS stateId,
                                        calcStateId,
                                        opFeedbackStore.opStateId,
                                        CASE WHEN calcStateSet IS NOT NULL THEN calcStateSet ELSE opFeedbackStore.opStateSet END AS feedbackStateSet,
                                        calcStateSet,
                                        opFeedbackStore.opStateSet,
                                        feedbackStates.state AS feedbackState,
                                        feedbackStates.stateColor AS feedbackStateColor,
                                        feedbackStates.stateLabel AS feedbackStateLabel,
                                        (
                                            SELECT GROUP_CONCAT(DISTINCT qualifications.shortName) 
                                                FROM qualifications AS qualis
                                                WHERE xt_basedata_qualifications.qualificationId = qualis.id
                                        )
                                        AS qualificationsList
                            FROM feedbackStore
                                LEFT JOIN
                                operationsStore ON feedbackStore.operationId = operationsStore.id
                                LEFT JOIN
                                basedata ON feedbackStore.basedataUUID = basedata.uuid
                                LEFT JOIN
                                (
                                    SELECT feedbackStore.basedataUUID AS fbUUID,
                                            feedbackStore.stateId AS calcStateId,
                                            feedbackStore.stateSet AS calcStateSet,
                                            operationsStore.subOperationFor
                                        FROM feedbackStore
                                            JOIN
                                            operationsStore ON operationsStore.id = feedbackStore.operationId
                                    WHERE (feedbackStore.operationId = ? OR 
                                            (operationsStore.subOperationFor = ? AND 
                                            operationsStore.subOperationFor != 0) OR 
                                            (operationsStore.id = ? AND 
                                            operationsStore.subOperationFor == 0) OR 
                                            (operationsStore.subOperationFor = ?) ) AND 
                                            operationsStore.id <= ? AND 
                                            stateId != 0
                                    ORDER BY feedbackStore.stateSet DESC
                                )
                                AS joinedStore ON joinedStore.fbUUID = basedata.uuid
                                LEFT JOIN
                                feedbackStates ON feedbackStates.id = stateId
                                LEFT JOIN
                                xt_basedata_qualifications ON xt_basedata_qualifications.basedataId = basedata.id
                                LEFT JOIN
                                qualifications ON qualifications.id = xt_basedata_qualifications.qualificationId
                                LEFT JOIN
                                (
                                    SELECT feedbackStore.operationId AS opId,
                                            feedbackStore.stateId AS opStateId,
                                            feedbackStore.stateSet AS opStateSet
                                        FROM feedbackStore
                                )
                                AS opFeedbackStore ON opFeedbackStore.opId = feedbackStore.operationId
                            WHERE feedbackStore.operationId = ?
                            GROUP BY feedbackStore.basedataUUID
                            ORDER BY basedata.lastname ASC
                    )
                    AS OpFeedback
                    LEFT JOIN
                    feedbackStates ON feedbackStates.id = OpFeedback.stateId
                GROUP BY basedataUUID
                ORDER BY lastname ASC, firstname ASC;

                `
                global.coreDb.all(query, [
                    
                    operationId,
                    subOperationFor,
                    subOperationFor,
                    operationId,
                    operationId,
                    operationId
                    

                ], (err, rows) => {
                    if (err) {
                        console.log(`ERROR getting op / parent feedback at core database: ${err.message}`);
                        callback(null);
                    } else {
                        callback(rows);
                    }
                });
            } else {
                callback(null);
            }
            
        })
    }

}

module.exports = FeedbackStore;