const { closeAllOperations } = require("../routes/operations/operations");

/**
 * Class handles all methods and models regarding handling email templates
 * @class
 */
class emailtemplates {

  /**
   * @constructor
   * @param {object} alogger Logger to be used by class methods (usually req.app.logger)
   */
  constructor(alogger) {
    // always initialize all instance properties
    this._logger = alogger;
  }

  /**
   * Returns all access tokens as rows from core database
   * @param {function} callback row or null
   */
  getTemplateForAlias(alias, callback) {
    let query = "SELECT * FROM `emailTemplatesStore` WHERE alias = ?";
    console.log('MODELS-EMAILTEMPLATES | loading template for alias '  + alias);
    global.coreDb.all(query, [
      alias
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-EMAILTEMPLATES | Error loading template for alias ' + alias + ' from core database:' + err.message);
        return callback(null);
      } else {
        if (rows.length > 0) {
          return callback(rows[0]);
        } else {
          return callback(null);
        }
      }
    });
  }

  /**
   * Renders data into template and returns the rendered content
   * @param {object} data flat object containing data to be rendered into template (key, value)
   * @param {*} templateAlias alias of the template to be used
   * @param {*} callback string or null
   */
  renderDataToTemplate(data, templateAlias, callback) {
    this.getTemplateForAlias(templateAlias, (template) => {
      if (template != null) {
        for (const [key, value] of Object.entries(data)) {
          // we're now iterating through the data and fill the template
          console.log('ITERATING: {{' + key.toUpperCase()+'}}');
          const regexp = new RegExp(`{{${key.toUpperCase()}}}`, "g")
          if (template.html != null) {
            template.html = template.html.replace(regexp, value)
          }
          if (template.plain != null) {
            template.html = template.html.replace(regexp, value)
          }
        }
        return callback(template);
      } else {
        return callback(null);
      }
    })
  }


}

                  

// export the class
module.exports = emailtemplates;