const gk = require('gauss-krueger');
const moment = require('moment');

/**
 * Class offers testing of events
 * @class
 */
class EventsTest {
    /**
     * @constructor
     * @param {object} logger Logger to be used
     */
    constructor(logger) {
        this.logger = logger;
    }


    /**
     * Fires a event for testing purposes
     * @param {string} eventName name of event to be fired
     */
    testEvent(eventName, callback) {
        
    }

    /**
     * Implementation of event-specific methods
     */

    /**
     * TEST EVENT for testing an incoming alarm
     */
    event_new_alarm(reqBody, callback) {

        const Operation = require('./core/core_operation');
        let operation = new Operation();
        operation.setUUID(); // Calculate new GUID for this operation object as a unique identifier
        console.log('EVENT-TEST | Set new guid for test operation to: ' + operation.uuid.value);
        operation.operationnumber = reqBody.operationnumber;
        operation.subject = reqBody.subject;
        operation.message = reqBody.message;
        operation.object = reqBody.object;
        operation.operationSourceConnector = 'core';
        operation.operationState = 0;
        operation.section = reqBody.section;
        operation.street = reqBody.street;
        operation.community = reqBody.community;
        operation.district = reqBody.district;
        operation.gear = reqBody.gear;
        operation.keywordCategory = reqBody.keywordCategory;
        operation.keywordId = reqBody.keywordId;
        operation.keywordName = reqBody.keywordName;
        operation.keywordRaw = reqBody.keywordRaw;
        operation.location = reqBody.location;
        operation.zveis = reqBody.zveis;
        operation.alarmdate = moment().format('DD.MM.YYYY');
        operation.alarmtime = moment().format('HH:mm:ss');
        operation.gkx = reqBody.gkx;
        operation.gky = reqBody.gky;
        operation.lat = 0.0;
        operation.lon = 0.0;
        operation.floor = reqBody.floor;
        operation.ils = reqBody.ils;
        operation.isTest = true; // set this property to true as this is a testalarm
        
        if ((operation.gky.value.length > 0) && (operation.gkx.value.length > 0)) {
            // SDS delivered gauss krueger coordinates
            let wgs84 = gk.toWGS({ x: parseFloat(operation.gkx.value), y: parseFloat(operation.gky.value) });
            operation.lat = wgs84.latitude;
            operation.lon = wgs84.longitude;
        } else {
            operation.lat = 0.0;
            operation.lon = 0.0;
        }

        // Send Event to PluginManager
        global.plgManager.event_new_alarm(operation);
        callback(true);
    }

}

module.exports = EventsTest;