
const moment = require('moment');

class gearstate {
    constructor(alogger) {
        // always initialize all instance properties
        this.logger = alogger;
    }

    getListOfGearStates(callback) {
        this.logger.debug('MODELS-GEARSTATES | Reading gear states list');
        let sql = `SELECT * FROM gearStateStore ORDER BY id DESC`;

        global.coreDb.all(sql, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-GEARSTATE | Error reading list of gearstates from database: ' + err);
                return callback(null);
            }
            var convertedRows = [];
            for (let row of rows){
                // timestamp is in GMT
                row.timestamp = moment.tz(row.timestamp,"Etc/GMT").tz("Europe/Berlin").format('YYYY-MM-DD HH:mm:ss');
                convertedRows.push(row);
            }

            return callback(convertedRows);
        });
    }

    getListOfGearStatesWithAssetInfos(callback) {
        this.logger.debug('MODELS-GEARSTATES | Reading gear states list + asset infos');
        let sql = 
            `
                SELECT 
                    gearStateStore.timestamp as timestamp, 
                    gearStateStore.radioStatusShort as radioStatusShort, 
                    gearStateStore.radioStatusHumanReadable as radioStatusHumanReadable, 
                    gearStateStore.issi as issi,
                    assets.gearname as gearname, 
                    assets.title as geartitle, 
                    organizations.name as organizationsName  
                FROM 
                    gearStateStore 
                LEFT JOIN 
                    ASSETS 
                ON 
                        gearStateStore.issi = assets.issi
                LEFT JOIN 
                    organizations 
                ON 
                    assets.ownerOrgId = organizations.id
                ORDER BY 
                    gearStateStore.timestamp 
                DESC
            `;

        global.coreDb.all(sql, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-GEARSTATE | Error reading list of gearstates + asset infos from database: ' + err);
                return callback(null);
            }
            var convertedRows = [];
            for (let row of rows){
                // timestamp is in GMT
                row.timestamp = moment.tz(row.timestamp,"Etc/GMT").tz("Europe/Berlin").format('YYYY-MM-DD HH:mm:ss');
                convertedRows.push(row);
            }

            return callback(convertedRows);
        });
    }

    getListOfGearStatesWithAssetInfosFilteredBy(issi, gearId, from, till, callback) {
        var activeIssi = false;
        var activeGear = false;
        var activeFrom = false;
        var activeTill = false;
        var sqlParts = [];
        var sqlComparator = [];
        var sqlValues = [];

        if ((typeof issi != 'undefinded') & (issi != null)) {
            // issi filter is active
            activeIssi = true;
            sqlParts.push('gearStateStore.issi');
            sqlComparator.push(' = ');
            sqlValues.push(issi);    
        }
        if ((typeof gearId != 'undefinded') & (gearId != null)) {
            // issi filter is active
            activeGear = true;
            sqlParts.push('assets.id');
            sqlComparator.push(' = ');
            sqlValues.push(gearId);    
        }
        if ((typeof from != 'undefinded') & (from != null)) {
            // issi filter is active
            activeFrom = true;
            sqlParts.push('gearStateStore.timestamp');
            sqlComparator.push(' >= ');
            sqlValues.push(from);    
        }
        if ((typeof till != 'undefinded') & (till != null)) {
            // issi filter is active
            activeTill = true;
            sqlParts.push('gearStateStore.timestamp');
            sqlComparator.push(' <= ');
            sqlValues.push(till);    
        }

        var whereStr = '';
        sqlParts.forEach((sqlPart, index) => {
            console.log(index);
            if (index === 0) {
                whereStr = ' WHERE ';
            } else {
                whereStr = whereStr + ' AND ';
            }
            whereStr = whereStr + '(' + sqlPart + ' ' + sqlComparator[index] + ' ?) ';
        });


        this.logger.debug('MODELS-GEARSTATES | Reading gear states list + asset infos');
        let sql = 
            `
                SELECT 
                    gearStateStore.timestamp as timestamp, 
                    gearStateStore.radioStatusShort as radioStatusShort, 
                    gearStateStore.radioStatusHumanReadable as radioStatusHumanReadable, 
                    gearStateStore.issi as issi,
                    assets.gearname as gearname, 
                    assets.title as geartitle, 
                    organizations.name as organizationsName  
                FROM 
                    gearStateStore 
                LEFT JOIN 
                    ASSETS 
                ON 
                        gearStateStore.issi = assets.issi
                LEFT JOIN 
                    organizations 
                ON 
                    assets.ownerOrgId = organizations.id
                ` + whereStr + `
                ORDER BY 
                    gearStateStore.timestamp 
                DESC
            `;

        console.log(sqlValues);


        global.coreDb.all(sql, sqlValues, (err, rows) => {
            if (err) {
                this.logger.error('MODELS-GEARSTATE | Error reading list of gearstates + asset infos from database: ' + err);
                return callback(null);
            }
            var convertedRows = [];
            for (let row of rows){
                // timestamp is in GMT
                row.timestamp = moment.tz(row.timestamp,"Etc/GMT").tz("Europe/Berlin").format('YYYY-MM-DD HH:mm:ss');
                convertedRows.push(row);
            }

            return callback(convertedRows);
        });
    }

    getCurrentGearStatesList(callback) {
        this.logger.debug('MODELS-GEARSTATES | Reading current gear states list');
        let sql = 
            `
                SELECT 
                    assets.title as title,
                    assets.gearname as gearname,
                    gearStates.radioStatusHumanReadable as radioStatusHumanReadable,
                    gearStates.radioStatusShort as radioStatusShort,
                    gearStates.timestamp as timestamp,
                    organizations.name as organizationsLongName,
                    organizations.shortName as organizationsShortName,
                    organizations.uuid as organizationsUUID
                FROM 
                    assets 
                JOIN 
                    (
                        SELECT 
                            radioStatusHumanReadable,
                            radioStatusShort, 
                            MAX(timestamp) as timestamp, 
                            issi 
                        FROM
                            gearStateStore 
                        GROUP BY issi
                    ) gearStates on assets.issi = gearStates.issi
                JOIN 
                    organizations on assets.ownerOrgId = organizations.id
                WHERE 
                    assets.isGear = 1
                ORDER BY 
                    organizationsLongName ASC, 
                    assets.gearname ASC
            `;

        global.coreDb.all(sql, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-GEARSTATE | Error reading current list of gearstates from database: ' + err);
                return callback(null);
            }
            var convertedRows = [];
            for (let row of rows){
                // timestamp is in GMT
                row.timestamp = moment.tz(row.timestamp,"Etc/GMT").tz("Europe/Berlin").format('YYYY-MM-DD HH:mm:ss');
                convertedRows.push(row);
            }
            return callback(convertedRows);
        });
    }


    getCurrentGearStatesListForOrganization(organizationsLongName, callback) {
        let sql = 
            `
                SELECT 
                    assets.title as title,
                    assets.gearname as gearname,
                    gearStates.radioStatusHumanReadable as radioStatusHumanReadable,
                    gearStates.radioStatusShort as radioStatusShort,
                    gearStates.timestamp as timestamp,
                    organizations.name as organizationsLongName,
                    organizations.shortName as organizationsShortName,
                    organizations.uuid as organizationsUUID
                FROM 
                    assets 
                LEFT JOIN 
                    (
                        SELECT 
                            radioStatusHumanReadable,
                            radioStatusShort, 
                            MAX(timestamp) as timestamp, 
                            issi 
                        FROM
                            gearStateStore 
                        GROUP BY issi
                    ) gearStates on assets.issi = gearStates.issi
                JOIN 
                    organizations on assets.ownerOrgId = organizations.id
                WHERE 
                    assets.isGear = 1 AND organizationsLongName = ?
                ORDER BY 
                    organizationsLongName ASC, 
                    assets.gearname ASC
            `;

        global.coreDb.all(sql, [
            organizationsLongName
        ], (err, rows) => {
            if (err) {
                console.log(err);
                this.logger.error('MODELS-GEARSTATE | Error reading current list of gearstates from database: ' + err);
                return callback(null);
            }
            var convertedRows = [];
            for (let row of rows){
                // timestamp is in GMT
                row.timestamp = moment.tz(row.timestamp,"Etc/GMT").tz("Europe/Berlin").format('YYYY-MM-DD HH:mm:ss');
                convertedRows.push(row);
            }
            return callback(convertedRows);
        });
    }
}

// export the class
module.exports = gearstate;