/**
 * Class handles GIT stuff
 * @class
 * @author Stefan Bauer
 */
// require the library, main export is a function
const simpleGit = require('simple-git');

class Git {
    constructor(){
        const options = {
            baseDir: process.cwd(),
            binary: 'git',
            maxConcurrentProcesses: 6,
         };
        this.git = simpleGit(options);
    }

    getGitHeadHash(){
        return this.git.revparse("HEAD");
    }

    getGitHeadShortHash(){
        return this.git.revparse({
            '--short': null,
            'HEAD':null
        });
    }

    getGitBehindAheadMaster(){
        return this.getCurrentBranch().then((currentBranch) => {       
            return new Promise((resolve,reject) => {
                this.git.raw('rev-list', '--left-right', '--count', 'origin/master...', currentBranch).then((result) => {
                    resolve(result.trim());
                })
            })          
        })
    }

    getGitBehindMaster(){
        return this.getCurrentBranch().then((currentBranch) => {       
            return new Promise((resolve,reject) => {
                this.git.raw('rev-list', '--left-only', '--count', 'origin/master...', currentBranch).then((result) => {
                    resolve(result.trim());
                })
            })          
        })
    }

    getGitAheadMaster(){
        return this.getCurrentBranch().then((currentBranch) => {       
            return new Promise((resolve,reject) => {
                this.git.raw('rev-list', '--left-right', '--count', 'origin/master...', currentBranch).then((result) => {
                    resolve(result.trim());
                })
            })          
        })
    }

    fetch(){
        var remote = this.buildRemoteRepoLink();
        if (remote != null && remote != undefined) {
            return new Promise((resolve,reject) => {
                this.getCurrentBranch().then((currentBranch) => {  
                    console.log("Git fetching ... ");
                    this.git.fetch(remote, currentBranch).then((res) => {
                        console.log(res);
                        resolve(res);
                    });    
                
                }).catch((error) => {
                    console.error(error.message);
                })
            })
            
        }
    }

    getCurrentBranch(){
        return new Promise((resolve, reject) => {
            this.git.branch().then(( BranchSummaryResult ) => {
                resolve(BranchSummaryResult.current);
            })  
        })
    }

    pullCurrentBranch(){
        try {
            var remote = this.buildRemoteRepoLink();
            console.log("GIT Pulling ... ");
            if (remote != null && remote != undefined){                    
                return new Promise((resolve, reject) => {
                    this.getCurrentBranch().then((currentBranch) => {
                        this.fetch().then((result) => {
                            this.git.pull(remote,currentBranch /*, {'--dry-run':null}*/)
                            .then(( res ) => {
                                resolve(res);
                            })
                            .catch( (error) => {
                                console.log(error.message);
                            }) 
                        })
                    })             
                })
            }
        }
        catch (exp) {
            console.log("unable to perform pull");
            console.log(exp);
        }        
    }
    buildRemoteRepoLink(){
        var user;
        var pass;
        global.configuration.serverconfig.forEach( (entry) => {
            switch (entry.keyword){
                case "githubUsername": {
                    user = entry.value;
                    break;
                }
                case "githubPassword": {
                    pass = entry.value;
                    break;
                }
                default: {

                }
            }
        } )
        const REPO = 'github.com/BoxSo/ALARMiator-Server';
        const remote = `https://${user}:${pass}@${REPO}`;
        
        if (user != undefined && user != null && pass != undefined && pass != null) {
            console.log("GIT CREDENTIALS FOUND");
            return remote;
        }
        else {
            console.log("WARNING: NO GIT CREDENTIALS FOUND");
            return null;
        }
    }
}

module.exports = Git;