
/**
 * ALARMiator Rule Component
 * 
 * @summary: Evaluates agains Time of Day rules
 * @description:Checks if a operation alerttime is between a defined time of day
 * 
 * @author Jens Dinstühler
 * 
 */

const RuleEngineHelpers = require('../ruleEngineHelpers');

class ruleComponentDayOfWeek {

    constructor() {
        this.helper = new RuleEngineHelpers();
    }

    /**
     * sets the parameters for this rule
     * @param {object} parameters 
     * @returns {object} success (true/false)
     */
    setRuleParameters(parameters) {
        if (this.helper.isDefined(parameters)) {
            try {
                this.monday = parseInt(parameters.monday);
                this.tuesday = parseInt(parameters.tuesday);
                this.wednesday = parseInt(parameters.wednesday);
                this.thursday = parseInt(parameters.thursday);
                this.friday = parseInt(parameters.friday);
                this.saturday = parseInt(parameters.saturday);
                this.sunday = parseInt(parameters.sunday);
            } catch (err) {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Evaluates the rule against a given operation object
     * @param {operation} operation operation object
     * @returns {boolean} true / false
     */
    evaluate(operation) {
        if (typeof operation != 'undefined') {
            if (typeof operation.alarmdate != 'undefined') {
                if (typeof operation.alarmdate.value != 'undefined') {
                    // alarmdate seems to be defined
                    var alarmdate = '' + operation.alarmdate.value;
                    var alarmDateArray = alarmdate.split('.');
                    var day = null;
                    var month = null;
                    var year = null;
                    var dayOfWeek = null;
                    if (Array.isArray(alarmDateArray)) {
                        day = alarmDateArray[0];
                        month = alarmDateArray[1];
                        year = alarmDateArray[2];
                    }

                    if ((day.length > 0) && (month.length > 0) && (year > 0)) {
                        var alarmDateObj = new Date(Date.UTC(year, month-1, day, 3, 0, 0));
                        var dayOfWeek = alarmDateObj.getDay();
                        var weHaveAMatch = false;
                        if((this.monday === 1) && (dayOfWeek === 1)) { weHaveAMatch = true; }
                        if((this.tuesday === 1) && (dayOfWeek === 2)) { weHaveAMatch = true; }
                        if((this.wednesday === 1) && (dayOfWeek === 3)) { weHaveAMatch = true; }
                        if((this.thursday === 1) && (dayOfWeek === 4)) { weHaveAMatch = true; }
                        if((this.friday === 1) && (dayOfWeek === 5)) { weHaveAMatch = true; }
                        if((this.saturday === 1) && (dayOfWeek === 6)) { weHaveAMatch = true; }
                        if((this.sunday === 1) && ((dayOfWeek === 6) || (dayOfWeek === 0))) { weHaveAMatch = true; }
                        return weHaveAMatch;
                    } else { 
                        return null;
                    }
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

}

module.exports = ruleComponentDayOfWeek;