
/**
 * ALARMiator Rule Component
 * 
 * @summary: Evaluates against matches within the keyword Category listed in operation
 * @description:Checks if a operation keyword categorycontains the given keyword
 * 
 * @author Jens Dinstühler
 * 
 */

const RuleEngineHelpers = require('../ruleEngineHelpers');

class ruleComponentKeywordCategory {

    constructor() {
        this.helper = new RuleEngineHelpers();
    }

    /**
     * sets the parameters for this rule
     * @param {object} parameters 
     * @returns {object} success (true/false)
     */
    setRuleParameters(parameters) {
        if (this.helper.isDefined(parameters)) {
            this.searchFor = parameters.searchFor;
            this.searchType = parameters.searchType;    // 'full-match', 'starts-with', 'contains'
            return true;
        } else {
            return false;
        }
    }

    /**
     * Evaluates the rule against a given operation object
     * @param {operation} operation operation object
     * @returns {boolean} true / false
     */
    evaluate(operation) {
        if (typeof operation != 'undefined') {
            if (typeof operation.keywordCategory != 'undefined') {
                if (typeof operation.keywordCategory.value != 'undefined') {
                    // gearCalc seems to be defined
                    var keywordCategory = operation.keywordCategory.value;
                    var matchFound = false;

                    if (this.searchType === 'full-match') {
                        if (keywordCategory === this.searchFor) {
                            matchFound = true;
                        }
                    }

                    if (this.searchType === 'starts-with') {
                        if (keywordCategory.startsWith(this.searchFor)) {
                            matchFound = true;
                        }
                    }

                    if (this.searchType === 'contains') {
                        if (keywordCategory.includes(this.searchFor)) {
                            matchFound = true;
                        }
                    }
                    return matchFound;
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
}

module.exports = ruleComponentKeywordCategory;