
/**
 * ALARMiator Rule Component
 * 
 * @summary: Evaluates agains Time of Day rules
 * @description:Checks if a operation alerttime is between a defined time of day
 * 
 * @author Jens Dinstühler
 * 
 */

const RuleEngineHelpers = require('../ruleEngineHelpers');

class ruleComponentTimeOfDay {

    constructor() {
        this.helper = new RuleEngineHelpers();
    }

    /**
     * sets the parameters for this rule
     * @param {object} parameters 
     * @returns {object} success (true/false)
     */
    setRuleParameters(parameters) {
        if (this.helper.isDefined(parameters)) {
            this.fromHour = parameters.fromHour;
            this.fromMinute = parameters.fromMinute;
            this.tillHour = parameters.tillHour;
            this.tillMinute = parameters.tillMinute;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Evaluates the rule against a given operation object
     * @param {operation} operation operation object
     * @returns {boolean} true / false
     */
    evaluate(operation) {
        if (typeof operation != 'undefined') {
            if (typeof operation.alarmtime != 'undefined') {
                if (typeof operation.alarmtime.value != 'undefined') {
                    // alarmtime seems to be defined
                    var alarmtime = '' + operation.alarmtime.value;
                    var alarmtimeArray = alarmtime.split(':');
                    if (alarmtimeArray.length > 1) {
                        var alarmtimeHour = alarmtimeArray[0];
                        var alarmtimeMinute = alarmtimeArray[1];
                        var alarmtimeHourInt = null;
                        var alarmtimeMinuteInt = null;

                        try {
                            alarmtimeHourInt = parseInt(alarmtimeHour);
                            alarmtimeMinuteInt = parseInt(alarmtimeMinute);
                        } catch (err) {
                            console.log('Error rendering alarmtime to integer');
                            return null;
                        }

                        // Now check the range
                        var afterStart = false;
                        var beforeEnd = false;

                        if (alarmtimeHourInt == parseInt(this.fromHour)) {
                            if (alarmtimeMinuteInt >= parseInt(this.fromMinute)) {
                                // alarmtime is behind or exactly start time
                                afterStart = true;
                            } else {
                                afterStart = false;
                            }
                        }
                        if (alarmtimeHourInt > parseInt(this.fromHour)) {
                            afterStart = true;
                        }

                        if (alarmtimeHour == parseInt(this.tillHour)) {
                            if (alarmtimeMinute <= parseInt(this.tillMinute)) {
                                beforeEnd = true;
                            }
                        }

                        if (alarmtimeHour < parseInt(this.tillHour)) {
                            beforeEnd = true;
                        }

                        

                        // Return the result
                        if ((beforeEnd) && (afterStart)) {
                            return true;
                        } else {
                            return false;
                        }


                    } else {
                        return null;
                    }
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

}

module.exports = ruleComponentTimeOfDay;