
/**
 * ALARMiator Rule Component
 * 
 * @summary: Evaluates against ZVEIs listed in operation
 * @description:Checks if a operation zveilist containt the given zvei(s)
 * 
 * @author Jens Dinstühler
 * 
 */

const RuleEngineHelpers = require('../ruleEngineHelpers');

class ruleComponentZVEI {

    constructor() {
        this.helper = new RuleEngineHelpers();
    }

    /**
     * sets the parameters for this rule
     * @param {object} parameters 
     * @returns {object} success (true/false)
     */
    setRuleParameters(parameters) {
        if (this.helper.isDefined(parameters)) {
            this.zveiListRequested = parameters.zveiList;
            this.searchType = parameters.searchType;    // all or one
            return true;
        } else {
            return false;
        }
    } 


    /**
     * Evaluates the rule against a given operation object
     * @param {operation} operation operation object
     * @returns {boolean} true / false
     */
    evaluate(operation) {
        if (typeof operation != 'undefined') {
            if (typeof operation.zveisCalc != 'undefined') {
                if (typeof operation.zveisCalc.value != 'undefined') {
                    // gearCalc seems to be defined
                    var zveilist = operation.zveisCalc.value;
                    var zveiFoundOne = false;
                    var zveiFoundAll = true;

                    // Check if one of the requested zveis can be found
                    zveilist.forEach(zveiOperation => {
                        if (this.zveiListRequested.indexOf(zveiOperation.trim()) > -1) {
                            zveiFoundOne = true;
                        }
                    });
                    // now check if all requested zveis can be found
                    this.zveiListRequested.forEach((zveiRequested) => {
                        //console.log('Treffer für alle (' + zveiRequested.trim() + '): ' + zveilist.indexOf(zveiRequested.trim()));
                        if (!(zveilist.indexOf(zveiRequested.trim()) > -1)) {
                            zveiFoundAll = false;
                        } 
                    })

                    // now return result depending of requested searchType
                    if (this.searchType === 'one') {
                        return zveiFoundOne;
                    } else if (this.searchType === 'all') {
                        if (zveiFoundOne === true) {
                            return zveiFoundAll;
                        } else {
                            return false;
                        }
                        
                    }
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

}

module.exports = ruleComponentZVEI;