const EventEmitter = require('events');
const Groups = require('../models/groups');
const Operations = require('../models/operationshelper');
const Alertrules = require('./alertrules');
const Assets = require('./assets');

/**
 * Load all rule components
 */
const RuleComponentTimeOfDay = require('./ruleEngine/components/ruleComponentTimeOfDay');
const RuleComponentDayOfWeek = require('./ruleEngine/components/ruleComponentDayOfWeek');
const RuleComponentGear = require('./ruleEngine/components/ruleComponentGear');
const RuleComponentZVEI = require('./ruleEngine/components/ruleComponentZVEI');
const RuleComponentZVEIDelta = require('./ruleEngine/components/ruleComponentZVEIDelta');
const RuleComponentKeywordId = require('./ruleEngine/components/ruleComponentKeywordID');
const RuleComponentKeywordName = require('./ruleEngine/components/ruleComponentKeywordName');
const RuleComponentKeywordCategory = require('./ruleEngine/components/ruleComponentKeywordCategory');
const RuleComponentObject = require('./ruleEngine/components/ruleComponentObject');
const RuleComponentSubject = require('./ruleEngine/components/ruleComponentSubject');



class Ruleengine extends EventEmitter {

    /**
     * constructor
     */
    constructor() {
        super();
    }

    /**
     * Evaluates all existing active rules against an existing operation from core database
     * @param {integer} operationId id of operation in core database to evaluate all active rules against
     * @param {function} callback returns array of rules evaluations
     */
    evaluateOperationAgainstRules(operationId, callback) {
        var alertrules = new Alertrules();
        // load all active alert rules to be avaluated
        alertrules.getAllActiveRules((rowsActiveAlertRules) => {
            if (rowsActiveAlertRules != null) {
                // No error loading active alert rules
                if (rowsActiveAlertRules.length > 0) {
                    // There are active alert rules
                    // Now cycle through alert rules an collect alarmable members
                    var rulesCount = rowsActiveAlertRules.length;
                    var rulesEvaluationResult = [];
                    rowsActiveAlertRules.forEach((alertRule, index) => {
                        var ruleEvaluatesResult = true;
                        var ruleId = null;
                        this.evaluateExistingOperationAgainstRule(operationId, null, alertRule.id, (cycleResult) => {
                            // now check if rule evaluates positive
                            if (Array.isArray(cycleResult)) {
                                cycleResult.forEach((filterResult) => {
                                    ruleId = filterResult.alertRuleId;
                                    if (typeof filterResult.evaluates != 'undefined') {
                                        if (filterResult.evaluates != true) {
                                            ruleEvaluatesResult = false;
                                        }
                                    }
                                })
                            }
                            var ruleResult = {
                                alertRuleId: alertRule.id,
                                alertRuleTitle: alertRule.title,
                                evaluates: ruleEvaluatesResult,
                                filterResults: cycleResult
                            }
                            // console.log('MODELS | RULEENGINE | START DEBUG | evaluateOperationAgainstRules | ruleResult for index ' + index + ':');
                            // console.log(ruleResult);
                            // console.log('MODELS | RULEENGINE | END DEBUG   | evaluateOperationAgainstRules | ruleResult for index ' + index + ':');
                            
                            rulesEvaluationResult.push(ruleResult);

                            if (rulesEvaluationResult.length === rowsActiveAlertRules.length) {
                                // We've processed the last rule -> return result
                                console.log('MODELS | RULEENGINE | evaluateOperationAgainstRules | CALLBACK | ruleResult ');
                                return callback(rulesEvaluationResult);
                            }
                            /*
                            if (index === (rulesCount - 1)) {
                                // We've processed the last rule -> return result
                                return callback(rulesEvaluationResult);
                            }
                            */
                        })
                    })
                } else {
                    // no active alert rules found
                    callback(null);
                }
            } else {
                // Error loading active alert rules TBD
                callback(null);
            }
        })
    }

    /**
     * Returns all alarmable users from all positive evaluated rules
     * @param {object} evalResult result from rule evaluation
     * @param {function} callback array of alarmable users, array of all users (independant if alarmable or not)
     */
    loadAllUsersToAlarmByEvaluationResult(evalResult, callback) {
        var alarmableUsers = [];
        var allUsers = [];
        var ruleIdsPostive = [];
        var ruleIdsAll = [];
        if (Array.isArray(evalResult)) {
            var countResults = evalResult.length;
            evalResult.forEach((ruleResult, index) => {

                if (ruleResult.evaluates === true) {
                    ruleIdsPostive.push(ruleResult.alertRuleId);
                } 
                ruleIdsAll.push(ruleResult.alertRuleId);
            });
            this.getUsersToAlarmForRules(ruleIdsPostive, (resultUsersToAlarm) => {
                this.getAllUserForRules(ruleIdsAll, (resultAllUsers) => {
                    return callback(resultUsersToAlarm, resultAllUsers);
                })
            })
        } else {
            return callback(alarmableUsers, allUsers);
        }
    }


    /**
     * Evaluates one single rule against given operation
     * @param {integer} operationId id of operation to evaluate rules for
     * @param {object} operation operation object if not a operation from core database
     * @param {integer} ruleId id of rule to evaluate the operation against
     * @param {function} callback result of evaliation as array with all filters and their state
     */
    evaluateExistingOperationAgainstRule(operationId = null, operation = null, ruleId, callback) {
        console.log('MODELS | RULEENGINE | operationId: ' + operationId);
        if (operationId != null) {
            var operations = new Operations();
            operations.getOperationAsObject(operationId, (operation) => {
                // Now load the rule
                var alertrules = new Alertrules();
                alertrules.getRuleWithId(ruleId, (rowsAlertRule) => {
                    alertrules.getRuleFilters(ruleId, (rowsAlertRuleFilters) => {
                        alertrules.getRuleGroups(ruleId, (rowsAlertRuleGroups) => {
                            var assets = new Assets();
                            assets.getAllAssetsWhichCanBeAlarmed((rowsAssets) => {
                                console.log('MODELS | RULEENGINE | starting evaluating rule');
                                var alertrule = rowsAlertRule[0];
                                // Now check all filters and evaluate them
                                rowsAlertRuleFilters.forEach((filter, index) => {
                                    var params = null;
                                    try {
                                        params = JSON.parse(filter.filterParameters);
                                    } catch {
                                        params = null;
                                    }

                                    // Cycle throuh existing filters
                                    if (filter.filterType === 'timerange') {
                                        /**
                                         * TIMERANGE FILTER 
                                         */
                                        var ruleResult = null;
                                        if (params != null) {
                                            var from = params.timerange_from.split(':');
                                            if (Array.isArray(from)) {
                                                params.fromHour = from[0];
                                                params.fromMinute = from[1];
                                            } else {
                                                params.fromHour = 0;
                                                params.fromMinute = 0;
                                            }
                                            var till = params.timerange_till.split(':');
                                            if (Array.isArray(till)) {
                                                params.tillHour = till[0];
                                                params.tillMinute = till[1];
                                            } else {
                                                params.tillHour = 0;
                                                params.tillMinute = 0;
                                            }

                                            var ruleComponentTimeOfDay = new RuleComponentTimeOfDay();
                                            var ruleParametersTimeOfDay = {
                                                fromHour: params.fromHour,
                                                fromMinute: params.fromMinute,
                                                tillHour: params.tillHour,
                                                tillMinute: params.tillMinute
                                            }
                                            ruleComponentTimeOfDay.setRuleParameters(ruleParametersTimeOfDay);
                                            ruleResult = ruleComponentTimeOfDay.evaluate(operation);
                                        }
                                        rowsAlertRuleFilters[index].evaluates = ruleResult;
                                        rowsAlertRuleFilters[index].params = params;
                                        rowsAlertRuleFilters[index].probe = ruleParametersTimeOfDay;
                                        console.log('MODELS | RULEENGINE | EVALUATE | BETWEEN: ' + params.fromHour + ':' + params.fromMinute + ' -> ' + params.tillHour + ':' + params.tillMinute + ' | Time-Of-Day Result: ' + ruleResult);
                                    } else if (filter.filterType === 'dayofweek') {
                                        /**
                                         * DAY OF WEEK FILTER
                                         */
                                        var ruleResult = null;
                                        if (params != null) {
                                            var ruleComponentDayOfWeek = new RuleComponentDayOfWeek();
                                            var ruleParametersDayOfWeek = {
                                                monday: params.weekday_monday,
                                                tuesday: params.weekday_tuesday,
                                                wednesday: params.weekday_wednesday,
                                                thursday: params.weekday_thursday,
                                                friday: params.weekday_friday,
                                                saturday: params.weekday_saturday,
                                                sunday: params.weekday_sunday

                                            }
                                            ruleComponentDayOfWeek.setRuleParameters(ruleParametersDayOfWeek);
                                            ruleResult = ruleComponentDayOfWeek.evaluate(operation);
                                        }
                                        rowsAlertRuleFilters[index].evaluates = ruleResult;
                                        rowsAlertRuleFilters[index].params = params;
                                        rowsAlertRuleFilters[index].probe = ruleParametersDayOfWeek;
                                        console.log('MODELS | RULEENGINE | EVALUATE | Day-of-week Result: ' + ruleResult);
                                    } else if (filter.filterType === 'zveis') {
                                        /**
                                        * ZVEIs
                                        */
                                        console.log(params);
                                        var ruleResult = null;
                                        if (params != null) {
                                            var ruleComponentZVEI = new RuleComponentZVEI();
                                            var searchType = '';
                                            if (params.searchType_containsOne === 1) {
                                                searchType = 'one';
                                            }
                                            if (params.searchType_containsAll === 1) {
                                                searchType = 'all';
                                            }
                                            var zveiListString = '' + params.zveicodes;
                                            var zveiListArray = zveiListString.split(',').map(x => x.trim()).filter(x => x);

                                            var ruleParametersZVEIs = {
                                                zveiList: zveiListArray,
                                                searchType: searchType
                                            };
                                            // var ruleParametersZVEIs = params.zveicodes;
                                            ruleComponentZVEI.setRuleParameters(ruleParametersZVEIs);
                                            ruleResult = ruleComponentZVEI.evaluate(operation);
                                        }
                                        rowsAlertRuleFilters[index].evaluates = ruleResult;
                                        rowsAlertRuleFilters[index].params = params;
                                        rowsAlertRuleFilters[index].probe = ruleParametersZVEIs;
                                        console.log('MODELS | RULEENGINE | EVALUATE | ZVEIs: ' + ruleResult);
                                    } else if (filter.filterType === 'zveisdelta') {
                                        /**
                                        * ZVEIs Delta
                                        */
                                        console.log('-------------- ZVEIS DELTA Rule Evaluation -----------------')
                                        console.log(params);
                                        var ruleResult = null;
                                        if (params != null) {
                                            var ruleComponentZVEI = new RuleComponentZVEIDelta();
                                            var searchType = '';
                                            if (params.searchType_containsOne === 1) {
                                                searchType = 'one';
                                            }
                                            if (params.searchType_containsAll === 1) {
                                                searchType = 'all';
                                            }
                                            var zveiListString = '' + params.zveicodesdelta;
                                            var zveiListArray = zveiListString.split(',').map(x => x.trim()).filter(x => x);

                                            var ruleParametersZVEIsDelta = {
                                                zveiList: zveiListArray,
                                                searchType: searchType
                                            };
                                            // var ruleParametersZVEIs = params.zveicodes;
                                            ruleComponentZVEI.setRuleParameters(ruleParametersZVEIsDelta);
                                            ruleResult = ruleComponentZVEI.evaluate(operation);
                                        }
                                        rowsAlertRuleFilters[index].evaluates = ruleResult;
                                        rowsAlertRuleFilters[index].params = params;
                                        rowsAlertRuleFilters[index].probe = ruleParametersZVEIsDelta;
                                        console.log('MODELS | RULEENGINE | EVALUATE | ZVEIs DELTA: ' + ruleResult);
                                    } else if (filter.filterType === 'keyword') {
                                        /**
                                        * Keyword
                                        */
                                        var ruleResult = null;
                                        if (params != null) {
                                            var ruleComponentSubject = new RuleComponentSubject();
                                            var searchType = null;
                                            if (params.keyword_searchType_contains === 1) {
                                                searchType = 'contains';
                                            } else if (params.keyword_searchType_beginsWith === 1) {
                                                searchType = 'starts-with';
                                            } else if (params.keyword_searchType_contains === 1) {
                                                searchType = 'full-match';
                                            }

                                            var ruleParametersSubject = {
                                                searchFor: params.keyword_searchfor,
                                                searchType: searchType
                                            }
                                            ruleComponentSubject.setRuleParameters(ruleParametersSubject);
                                            ruleResult = ruleComponentSubject.evaluate(operation);
                                        }
                                        rowsAlertRuleFilters[index].evaluates = ruleResult;
                                        rowsAlertRuleFilters[index].params = params;
                                        rowsAlertRuleFilters[index].probe = ruleParametersSubject;
                                        console.log('MODELS | RULEENGINE | EVALUATE | Subject: ' + ruleResult);
                                    } else if (filter.filterType === 'gear') {
                                        /**
                                         * GEAR
                                         */
                                        var ruleResult = null;
                                        if (rowsAssets === null) {
                                            // no assets in Database > evaluate false
                                            rowsAlertRuleFilters[index].evaluates = false;
                                            rowsAlertRuleFilters[index].params = '';
                                        } else {
                                            var gearArray = [];
                                            // collect names of alarmable assets/gear
                                            rowsAssets.forEach((aGear) => {
                                                if (params.gearids.includes(aGear.id)) {
                                                    gearArray.push(aGear.gearname);
                                                }
                                            })

                                            if (params != null) {
                                                var ruleComponentGear = new RuleComponentGear();
                                                var ruleParametersGear = gearArray;
                                                ruleComponentGear.setRuleParameters(ruleParametersGear);
                                                ruleResult = ruleComponentGear.evaluate(operation);
                                            }

                                            rowsAlertRuleFilters[index].evaluates = ruleResult;
                                            rowsAlertRuleFilters[index].params = {
                                                gear: gearArray
                                            };
                                            rowsAlertRuleFilters[index].probe = ruleParametersGear;
                                            console.log('MODELS | RULEENGINE | EVALUATE | Gear: ' + ruleResult);
                                        }

                                    }
                                });
                                return callback(rowsAlertRuleFilters);
                            })
                        })
                    })
                });
            });
        }
    }

    /**
     * Returns List of Users to alarm for given RuleID
     * @param {integer} ruleId 
     * @param {function} callback rows or null
     */
    getUsersToAlarm(ruleId, callback) {
        console.log('getUsersToAlarm(' + ruleId + ')');
       
       let query = "SELECT DISTINCT " +
       "basedata.id AS basedataId, " +
       "basedata.firstname AS firstName, " +
       "basedata.lastname AS lastName,  " +
       "basedata.uuid AS basedataUUID,  " +
       "basedata.cellphone AS cellphone,  " +
       "basedata.email AS email, " +
       "alertdevices.state AS deviceState, " +
       "fcmtokenStore.fcmToken AS fcmtoken,  " +
       "group_concat(DISTINCT xt_basedata_alarmings.pluginNamespace) AS namespace " +
       "FROM alertrulesGroupsStore " +
       "JOIN groups ON groups.id = alertrulesGroupsStore.groupId " +
       "JOIN xt_basedata_groups ON xt_basedata_groups.groupId = alertrulesGroupsStore.groupId  " +
       "JOIN basedata ON basedata.id = xt_basedata_groups.basedataId " +
       "LEFT JOIN alertdevices ON alertdevices.basedataId = basedata.id  " +
       "LEFT JOIN fcmtokenStore ON fcmtokenStore.deviceId = alertdevices.id  " +
       "LEFT JOIN xt_basedata_alarmings ON xt_basedata_alarmings.basedataId = basedata.id " +
       "WHERE alertrulesGroupsStore.alertRuleId = ? " +
       "GROUP BY basedata.id, fcmtoken " +
       "ORDER BY basedata.id";


        global.coreDb.all(query, [
            ruleId
        ], (err, rows) => {
            if (err) {
                this._logger.error('MODELS-RULEENGINE | Error loading users to alarm from core data: ' + err.message);
                return callback(null);
            } else {
                if (rows != null) {
                    rows.forEach((row) => {
                        if (row.namespace != null) {
                            var nspaceArr = row.namespace.split(',');
                            if (Array.isArray(nspaceArr)) {
                                row.namespace = nspaceArr;
                            }
                        }
                    });
                }
                return callback(rows);
            }
        });
    }

     /*
        !!! IMPORTANT: !!! 
            Returning properties should be same as handlerAlarm.getUsersToAlarmForGroups 
        !!! IMPORTANT: !!!  
    */

    /**
     * Returns List of Users to alarm for given Array of rule ids
     * @param {array} ruleId rule Ids to return users for
     * @param {function} callback rows or null
     */
    getUsersToAlarmForRules(ruleIdArray, callback) {
        //console.log('getUsersToAlarmForRules: ' + JSON.stringify(ruleIdArray));
        if (Array.isArray(ruleIdArray) && (ruleIdArray.length > 0)) {
            let query_1 = "SELECT DISTINCT " +
                "basedata.id AS basedataId, " +
                "basedata.firstname AS firstName, " +
                "basedata.lastname AS lastName,  " +
                "basedata.uuid AS basedataUUID,  " +
                "basedata.cellphone AS cellphone,  " +
                "basedata.email AS email, " +
                "alertdevices.state AS deviceState, " +
                "fcmtokenStore.fcmToken AS fcmtoken,  " +
                "group_concat(DISTINCT xt_basedata_alarmings.pluginNamespace) AS namespace, " +
                "organizations.name AS organizationname " +
                "FROM alertrulesGroupsStore " +
                "JOIN groups ON groups.id = alertrulesGroupsStore.groupId " +
                "JOIN xt_basedata_groups ON xt_basedata_groups.groupId = alertrulesGroupsStore.groupId  " +
                "JOIN basedata ON basedata.id = xt_basedata_groups.basedataId " +
                "LEFT JOIN alertdevices ON alertdevices.basedataId = basedata.id  " +
                "LEFT JOIN fcmtokenStore ON fcmtokenStore.deviceId = alertdevices.id  " +
                "LEFT JOIN xt_basedata_alarmings ON xt_basedata_alarmings.basedataId = basedata.id "+
                "LEFT JOIN organizations ON organizations.id = basedata.organizationid ";

            let query_2 = "WHERE ";

            let query_3 = "GROUP BY basedata.id, fcmtoken " +
                "ORDER BY basedata.id";

            let query_2_addendum = '';
            if (Array.isArray(ruleIdArray)) {
                ruleIdArray.forEach((rule, index) => {
                    //console.log('LOOP ruleIdArray: ' + index);
                    query_2_addendum = query_2_addendum + " alertrulesGroupsStore.alertRuleId = " + rule + " ";
                    if (index < (ruleIdArray.length - 1)) {
                        query_2_addendum = query_2_addendum + " OR ";
                    }
                });
            }

            let query = query_1 + query_2 + query_2_addendum + query_3;

            console.log('SQL QUERY USERS TO ALARM: ' + query);
            global.coreDb.all(query, [
            ], (err, rows) => {
                if (err) {
                    console.log('MODELS-RULEENGINE | Error loading users to alarm from core data: ' + err.message);
                    return callback(null);
                } else {
                    if (rows != null) {
                        rows.forEach((row) => {
                            if (row.namespace != null) {
                                var nspaceArr = row.namespace.split(',');
                                if (Array.isArray(nspaceArr)) {
                                    row.namespace = nspaceArr;
                                }
                            }
                        });
                    }
                    return callback(rows);
                }
            });
        } else {
            return callback([]);
        }
        
    }

    /**
     * Returns all Users which might be alarmable (independent of missing device or token)
     * @param {integer} ruleId id of rule
     * @param {*} callback rows or null
     */
    getAllUserForRule(ruleId, callback) {
        let query = "SELECT " +
            "alertrulesGroupsStore.groupId AS groupId, " +
            "groups.groupname AS groupName, " +
            "basedata.firstname AS firstName, " +
            "basedata.lastname AS lastName, " +
            "basedata.uuid AS basedataUUID, " +
            "basedata.cellphone AS cellphone, " +
            "basedata.email AS email, " +
            "organizations.name AS organzationName, " +
            "organizations.shortName AS organzationShortName " +
            "FROM alertrulesGroupsStore " +
            "INNER JOIN groups ON groups.id = alertrulesGroupsStore.groupId " +
            "INNER JOIN xt_basedata_groups ON xt_basedata_groups.groupId = alertrulesGroupsStore.groupId " +
            "INNER JOIN basedata ON basedata.id = xt_basedata_groups.basedataId " +
            "INNER JOIN organizations ON organizations.id = basedata.organizationid " +
            "WHERE alertrulesGroupsStore.alertRuleId = ? " +
            "ORDER BY groupName ASC";
        global.coreDb.all(query, [
            ruleId
        ], (err, rows) => {
            if (err) {
                this._logger.error('MODELS-RULEENGINE | Error loading all users otentially to alarm from core data: ' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all Users which might be alarmable (independent of missing device or token) for all Rules handed over as an Array
     * @param {array} ruleIdsArray id of rule
     * @param {function} callback rows or null
     */
    getAllUserForRules(ruleIdsArray, callback) {
        if (Array.isArray(ruleIdsArray) && (ruleIdsArray.length > 0)) {
            let query_1 = "SELECT " +
                "alertrulesGroupsStore.groupId AS groupId, " +
                "groups.groupname AS groupName, " +
                "basedata.firstname AS firstName, " +
                "basedata.lastname AS lastName, " +
                "basedata.uuid AS basedataUUID, " +
                "basedata.cellphone AS cellphone, " +
                "basedata.email AS email, " +
                "organizations.name AS organzationName, " +
                "organizations.shortName AS organzationShortName " +
                "FROM alertrulesGroupsStore " +
                "INNER JOIN groups ON groups.id = alertrulesGroupsStore.groupId " +
                "INNER JOIN xt_basedata_groups ON xt_basedata_groups.groupId = alertrulesGroupsStore.groupId " +
                "INNER JOIN basedata ON basedata.id = xt_basedata_groups.basedataId " +
                "INNER JOIN organizations ON organizations.id = basedata.organizationid ";
            let query_2 = "WHERE ";
            var query_2_addendum = "";
            ruleIdsArray.forEach((rule, index) => {
                //console.log('INDEX: ' + index + ' LENGTH: ' + ruleIdsArray.length);
                query_2_addendum = query_2_addendum + " alertrulesGroupsStore.alertRuleId = " + rule + " ";
                if (index < (ruleIdsArray.length - 1)) {
                    query_2_addendum = query_2_addendum + ' OR ';
                }
            })
            let query_3 = "ORDER BY groupName ASC";

            let query = query_1 + query_2 + query_2_addendum + query_3;
            //console.log('QUERY: ' + query);
            global.coreDb.all(query, [
            ], (err, rows) => {
                if (err) {
                    this._logger.error('MODELS-RULEENGINE | Error loading all users otentially to alarm from core data: ' + err.message);
                    return callback(null);
                } else {
                    return callback(rows);
                }
            });
        } else {
            return callback([]);
        }
    } 

    /**
     * Returns List of Groups to alarm for given Array of rule ids
     * @param {array} ruleId rule Ids to return users for
     * @param {function} callback rows or null
     */
    getGroupsToAlarmForRules(ruleIdArray, callback) {
        if (Array.isArray(ruleIdArray) && (ruleIdArray.length > 0)) {
            let query_1 = `
                SELECT DISTINCT alertrulesGroupsStore.groupId AS groupId,
                    groups.groupname AS groupname,
                    groups.shortName AS shortName,
                    organizations.name AS organizationsName,
                    organizations.shortName AS organizationsShortName
                FROM alertrulesGroupsStore
                    JOIN
                    groups ON groups.id = alertrulesGroupsStore.groupId
                    LEFT JOIN
                    organizations ON groups.organizationid = organizations.id
            `

            let query_2 = "WHERE ";

            let query_3 = "ORDER BY organizations.name, groups.groupname";

            let query_2_addendum = '';
            if (Array.isArray(ruleIdArray)) {
                ruleIdArray.forEach((rule, index) => {
                    //console.log('LOOP ruleIdArray: ' + index);
                    query_2_addendum = query_2_addendum + " alertrulesGroupsStore.alertRuleId = " + rule + " ";
                    if (index < (ruleIdArray.length - 1)) {
                        query_2_addendum = query_2_addendum + " OR ";
                    }
                });
            }

            let query = query_1 + query_2 + query_2_addendum + query_3;

            //console.log('SQL QUERY GROUPS TO ALARM: ' + query);
            global.coreDb.all(query, [
            ], (err, rows) => {
                if (err) {
                    console.error('MODELS-RULEENGINE | Error loading all users otentially to alarm from core data: ' + err.message);
                    return callback(null);
                } else {
                    return callback(rows);
                }
            });
        } else {
            return callback([]);
        }
        
    }
}

module.exports = Ruleengine;