// 
// Parser for ILS Nürnberg
//
// Parses incoming Faxes of ILS Nürnberg and
// stores them to the operations table in alarmiator database
//
//
const Tesseract = require('tesseract.js');
const fs = require('fs');
const Path = require('path');
// required for generating timebased guids
var uuid = require('uuid');
// required for having a timestamp in front of all console output
require('console-stamp')(console, '[HH:MM:ss.l]');
// Require model for operations
const operationObj = require('./models/operation');
const operation = new operationObj();
// time / date based operations
var moment = require('moment');
// helper class for geolocation functionality
const geolocationhelper = require('../../../../models/geolocationhelper');
const geolocation = new geolocationhelper();

// Configure Service Name for log output
var svcName = 'ILSFAX-INBOUND';
function setSvcName(svcname) {
  svcName = svcname;
}

// Configure the logger to be used
var logger = null;

function setLogger(loggerInstance) {
  logger = loggerInstance;
}

// Setter for organzation ID
var orgId = 0;
function setOrgId(orgid) {
  orgId = orgid;
}

function writeOCRTextToFile(content, filename) {
  // get filename of new file for naming additional created files the same
  var extension = Path.extname(filename);
  var basename = Path.dirname(filename) + '/../txt';
  var fileName = Path.basename(filename,extension);
  var txtFileName = basename + "/" + fileName;

  let data = JSON.stringify(content, null, 4);  
  fs.writeFileSync(txtFileName + '.json', data);  
}

function writeOperationToDatabase(operation) {
  let query = "INSERT INTO `operations` " + 
                  "(acknowledged, " + 
                  "operationguid, " + 
                  "operationnumber, " + 
                  "timestampincome, " + 
                  "timestampalarm, " + 
                  "messenger, " + 
                  "comment, " + 
                  "plan, " + 
                  "priority, " + 
                  "einsatzortobject, " + 
                  "einsatzortzipcode, " + 
                  "einsatzortcity, " + 
                  "einsatzortstreet, " + 
                  "einsatzortstreetnumber, " + 
                  "einsatzortintersection, " + 
                  "einsatzortproperty, " + 
                  "einsatzortGKcoordinates, " + 
                  "einsatzortlat, " + 
                  "einsatzortlon, " + 
                  "zielortobject, " + 
                  "zielortzipcode, " + 
                  "zielortcity, " + 
                  "zielortstreet, " + 
                  "zielortstreetnumber, " + 
                  "zielortintersection, " + 
                  "zielortlat, " + 
                  "zielortlon, " + 
                  "keyword, " + 
                  "keywordmisc, " + 
                  "rawjson, " + 
                  "srcservice, " +
                  "organizationId) " + 
              " VALUES (" +
                  operation.acknowledged + ", " +
                  "'" + operation.operationguid + "', " +
                  "'" + operation.operationnumber  + "', " +
                  "'" + operation.timestampincome  + "', " +
                  "'" + operation.timestampalarm  + "', " +
                  "'" + operation.messenger  + "', " +
                  "'" + operation.comment  + "', " +
                  "'" + operation.plan  + "', " +
                  "'" + operation.priority  + "', " +
                  "'" + operation.einsatzortobject + "', " +
                  "'" + operation.einsatzortzipcode  + "', " +
                  "'" + operation.einsatzortcity  + "', " +
                  "'" + operation.einsatzortstreet  + "', " +
                  "'" + operation.einsatzortstreetnumber  + "', " +
                  "'" + operation.einsatzortintersection  + "', " +
                  "'" + operation.einsatzortproperty  + "', " +
                  "'" + operation.einsatzortGKcoordinates  + "', " +
                  "'" + operation.einsatzortlat  + "', " +
                  "'" + operation.einsatzortlon  + "', " +
                  "'" + operation.zielortobject  + "', " +
                  "'" + operation.zielortzipcode  + "', " +
                  "'" + operation.zielortcity  + "', " +
                  "'" + operation.zielortstreet  + "', " +
                  "'" + operation.zielortstreetnumber  + "', " +
                  "'" + operation.zielortintersection  + "', " +
                  "'" + operation.zielortlat  + "', " +
                  "'" + operation.zielortlon  + "', " +
                  "'" + operation.keyword  + "', " +
                  "'" + operation.keywordmisc  + "', " +
                  "'" + operation.rawjson  + "', " +
                  "'" + operation.srcservice  + "', " +
                      + orgId + 
               ");";
  
  db.query(query, (err, result) => {
      if (err) {
        logger.error(`${svcName} | Error writing operation to database`);
        logger.error(`${svcName} | SQL-Statement with Error: `, query);
        logger.error(`${svcName} | SQL-Error: `, err.sqlMessage);
      } else {
        logger.info(`${svcName} | Successfully written operation to database`);
        operation.ressources.forEach(ressource => {
          // write each single ressource in database
          // First check if we didn't catch an empty string / line
          if (ressource.trim().length > 0) {
            let query = "INSERT INTO `operations_ressources` (operationid, nameraw) VALUES (" + result.insertId + ", '" + ressource + "');"
            db.query(query, (err2, result2) => {
              if (err2) {
                logger.error(`${svcName} | Error writing ressource to database`);
                logger.error(`${svcName} | SQL-Statement with Error: `, query);
                logger.error(`${svcName} | SQL-Error: `, err2.sqlMessage);
              } else {
                logger.debug(`${svcName} | successfully written operation to database`);
                // Successfully written ressource to database
                //console.info(`${svcName} | Successfully written ressource to database: ` + ressuorce);
              }
            });
          }
        });
      }
  });
}

function getCleanFax(lines){
  logger.info(`${svcName} | Starting to clean ...`)
  // fix ocr bugs -> remove if there is a dictonary
  cleanfax=lines.replace(/=/g,':');

  cleanfax = cleanfax.replace(/[ ]*[—-]+[—-]+[ ]*/gm,"")
  cleanfax = cleanfax.replace(/EINSATZMTTTEL/g,"EINSATZMITTEL");
  return cleanfax
}

function getSections(lines){
  logger.info(`${svcName} | Try to get sections for easy setup ...`)
  return lines.match((/\n+[A-Z ]+\n/g)).map(x => x.replace(/\n/g, ''));
}

function getKeywords(lines){
  logger.info(`${svcName} | Try to get keywords for easy setup ...`)
  return lines.match(/[A-ZÄÜÖ][a-zA-Z.ßäöüÖÄÜ\-—]+:/g).map(x => x.replace(/:/g, ''))
}

function parseFaxToJSON(lines, faxtemplate) {
  logger.info(`${svcName} | Starting with Fax to JSON convertion ...`)
  var faxobject = faxtemplate;
  var parsedSecObj;
  // get Sections from Template
  var sections = Object.keys(faxtemplate)

  // iterate over all sections
  sections.forEach( section => {
    // parse Fax with section
    parsedSecObj = parse(lines, section, sections);

    // get Keywords from Template for specific section
    var allkeywords = Object.keys(faxtemplate[section])

    var keywordcounter = 0; // Counter to check whether Keywords were found or not
    // iterate over all keywords from this section
    allkeywords.forEach( keyword => {
      // parse section text with keyword
      let parsedObj = parse(parsedSecObj.parsedtext, keyword, allkeywords);

      // Check if there is a value found for this Keyword
      if (parsedObj.parsedtext !== null) {
        keywordcounter++;
        // Add parsed text to faxobject and remove starting and ending:
        // - whitespaces
        // - :
        // - /
        // - -
        // - —

        faxobject[section][keyword]=parsedObj.parsedtext.trim()
        .replace(/(^[:\/—\-]+|[:\/—\-]+$)/gm,'')
        .trim();
      }
    })
    // if no keyword is found add full section text to faxobject
    if (keywordcounter === 0 && parsedSecObj !== null && parsedSecObj.parsedtext !== null) {
      faxobject[section] = parsedSecObj.parsedtext.trim();
    }
  })
  return faxobject;
}

function JsonToOperation(operationJson, callback) {
  // generate mysql conform timestamp
  var a = moment();
  var timeStampIncome = a.format('YYYY/MM/DD HH:mm');
  logger.info(`${svcName} | Timestamp incoming: ` + timeStampIncome);

  // calculate Latitude / Longitude values out of gauss krueger coordinates
  var latitude = 0.0;
  var longitude = 0.0;
  try {
    geolocation.gkToLatLon(operationJson.EINSATZORT.Koordinate, (arrLatLon) => {
      latitude = arrLatLon.latitude;
      longitude = arrLatLon.longitude;
      logger.info(`${svcName} | calculated Lat/Lon coordinates from GK-coordinates: (` + latitude + `/` + longitude + `)`);
    });
  } catch (err) {
    logger.error(`${svcName} | Error calculating Lat/lon coordinates from GK-coordinates: ` + err);
    callback();
  }
  
  // link JSON-Values to the general operations-object here
  // if the json-member exists, the value defined will be attached
  // and written to the database into the operations-table
  
  // operation.id 
  operation.acknowledged = 0;
  operation.operationguid = uuid.v1();
  operation.operationnumber = operationJson.FAXFAXFAXFAX.Nummer;
  operation.timestampincome = timeStampIncome;
  operation.timestampalarm = timeStampIncome; // TODO - Needs to be read from fax!
  operation.messenger = operationJson.MITTEILER.Name;
  operation.comment = operationJson.BEMERKUNG;
  operation.plan = operationJson.OBJEKTINFO["Plan Nr"];
  operation.priority = operationJson.EINSATZGRUND["Prio."]; // Maybe there will be a problem cause of the dot after "Prio"
  operation.einsatzortobject = operationJson.EINSATZORT.Objekt;
  operation.einsatzortzipcode = operationJson.EINSATZORT.PLZ; // not yet available for ILS Nürnberg
  operation.einsatzortcity = operationJson.EINSATZORT.Ort;
  operation.einsatzortstreet = operationJson.EINSATZORT.Straße;
  operation.einsatzortstreetnumber = operationJson.EINSATZORT.Nr;
  operation.einsatzortintersection = operationJson.EINSATZORT.Kreuzung;
  operation.einsatzortproperty = operationJson.EINSATZORT.Objekt;
  operation.einsatzortGKcoordinates = operationJson.EINSATZORT.Koordinate;
  operation.einsatzortlat = latitude; // Needs to be calculated (GK --> LatLon)
  operation.einsatzortlon = longitude; // Needs to be calculated (GK --> LatLon)
  operation.zielortobject = operationJson.ZIELORT.Objekt;
  operation.zielortzipcode = operationJson.ZIELORT.PLZ; // Not availabnle for ILS Nürnberg
  operation.zielortcity = ""; // Not available for ILS Nürnberg
  operation.zielortstreet = operationJson.ZIELORT.Straße;
  operation.zielortstreetnumber = operationJson.ZIELORT.Nr;
  operation.zielortintersection = ""; // Not available for ILS Nürnberg
  operation.zielortlat = ""; // Not available for ILS Nürnberg
  operation.zielortlon = ""; // Not available for ILS Nürnberg
  operation.keyword = operationJson.EINSATZGRUND.Stichwort;
  operation.keywordmisc = operationJson.EINSATZGRUND.Schlagwort;
  operation.rawjson = ""; // TODO - Still Problems in converting JSON to String to be propery escaped for sql statement
  operation.srcservice = "ILSFAX-INBOUND-ilsfax";
  operation.ressources = parseOperationRessources(operationJson.EINSATZMITTEL); // Parse operation ressources to the object for later storeage to database table

  // Calling callback-function
  callback(operation);
}

function parseOperationRessources(ressourcesString) {
  var ressourcesArray = ressourcesString.split('\n');
  if (ressourcesArray.length > 1) {
    // remove first element cause it's the headline
    ressourcesArray.splice(0, 1);
    ressourcesArray.forEach(function(ressource, index) {
      // Strip unneeded Information from ressources
      var singleRessourceArray = ressource.split(':');
      if (singleRessourceArray.length > 0) {
        ressourcesArray[index] = singleRessourceArray[0];
      }
    });
  } 
  logger.info(`${svcName} | Anzahl Einsatzmittel erkannt: ` + ressourcesArray.length);
  ressourcesArray.forEach(element => {
    logger.info(`${svcName} | Einsatzmittel erkannt: ` + element);
  });
  return ressourcesArray;
}

function ocr(image){
  //console.log(`${svcName} | Transformed Image file for OCR: ` + image);
  Tesseract.recognize(image, {lang: 'deu'})
  .then(function(result){

    lines=getCleanFax(result.text);

    // ILS NÜRNBERG Template
     faxtemplate = {"FAXFAXFAXFAX":{
           "Absender":"",
           "Alarmzeit":"",
           "Fax":"",
           "Termin":"",
           "Nummer":""
        },
        "MITTEILER":{
           "Name":"",
           "Tel":""
        },
        "EINSATZORT":{
           "Straße":"",
           "Nr":"",
           "Abschnitt":"",
           "Kreuzung":"",
           "Ort":"",
           "Objekt":"",
           "Abteilung":"",
           "Zuständig":"",
           "Koordinate":""
        },
        "ZIELORT":{
           "Straße":"",
           "Nr":"",
           "Ort":"",
           "Objekt":"",
           "Station":""
        },
        "PATIENT":"",
        "EINSATZGRUND":{
           "Schlagwort":"",
           "Stichwort":"",
           "Prio.":""
        },
        "EINSATZMITTEL":"",
        "OBJEKTINFO":{
           "Plan Nr":""
        },
        "BEMERKUNG":"",
        "ENDE ALARMFAX":""
     }

    faxdata=  parseFaxToJSON(lines, faxtemplate)
    logger.info(`${svcName} | OCR successful, writing fax details to file`);

    // Write textfile in archive folder with recognized fax content

    writeOCRTextToFile(faxdata, image)
    JsonToOperation(faxdata, (operation) => {
      // Write data to operation table in databse
      writeOperationToDatabase(operation);
    });
    logger.debug(`${svcName} | Fax Processing finished.`);
  })
  .catch(err => {
    logger.error(`${svcName} | Error in OCR: ` + err);
  })
  .finally( msg => {
      // console.log( msg )
     //Tesseract.terminate();
  })
}

function parse(data, keyword, allkeywords) {
  var parsedObj = {};
  parsedObj['keyword'] = keyword;
  var keywordexists = -1;
  // Check if keyword is in data
  if (keyword !== null && data !== null) {
    keywordexists = data.indexOf(keyword);
  }

  if (keywordexists !== -1) {
    // Find next keyword
    var indexes = allkeywords.map(x => data.indexOf(x))
    // Remove not included keywords
    var filtered = indexes.filter(x => x !== -1 && x > data.indexOf(keyword));
    //console.log("Indexes: ", filtered);
    // Find next keyword
    var nextkeyword = allkeywords[indexes.indexOf(Math.min.apply(Math, filtered))]
    //console.log("next: ", nextkeyword)

    // if there is another keyword
    if (nextkeyword !== undefined) {
      parsedObj['parsedtext'] = data.substring(
        data.indexOf(keyword)+keyword.length,
        data.indexOf(nextkeyword));
      parsedObj['nextindex'] = data.indexOf(nextkeyword);
    }
    else {
      // rest of the string
      parsedObj['parsedtext'] = data.substring(
        data.indexOf(keyword)+keyword.length);
      parsedObj['nextindex'] = -1;
    }
  }
  else {
    parsedObj['parsedtext'] = null;
    parsedObj['nextindex'] = 0;
  }

  //console.log(parsedObj);
  return parsedObj;
}

module.exports = {
  setSvcName,
  setLogger,
  setOrgId,
  getCleanFax,
  getSections,
  getKeywords,
  parseFaxToJSON,
  ocr,
  parse
}

