const Auth = require('../../../../../models/auth');
const Basedata = require('../../../../../models/basedata');
const Organizations = require('../../../../../models/organizations');
const Configuration = require('../../../../../models/configuration');
const Users = require('../../../../../models/users');


module.exports = {
    login(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {
                    // Check if user needs to be enforced to change its password
                    var enforcePasswordChange = false;
                    if (row.startPassword !== null) {
                        if (row.startPassword.length > 0) {
                            // start Password set !
                            enforcePasswordChange = true;
                        }
                    }
                    var basedata = new Basedata(global.logger);
                    basedata.getBaseDataForId(row.basedataid, (basedataRow) => {
                        if (basedataRow.length > 0) {
                            var organizations = new Organizations(global.logger);
                            organizations.getOrganizationWithId(basedataRow[0].organizationid, (organizationsRow) => {
                                if (organizationsRow.length > 0) {
                                    var configuration = new Configuration(global.logger);
                                    configuration.getValueForKeyword('serveruuid', (serverUUID) => {
                                        this.logger.log('AUTH | LOGIN | User logged in successfully');
                                        response.json(
                                            {
                                                success: 1,
                                                description: 'authenticated',
                                                isSuperUser: row.superuser,
                                                serverUUID: serverUUID,
                                                organizationUUID: organizationsRow[0].uuid,
                                                organizationName: organizationsRow[0].name,
                                                organizationShortName: organizationsRow[0].shortName,
                                                basedata: basedataRow,
                                                enforcePasswordChange: enforcePasswordChange
                                            }
                                        );
                                    })
                                } else {
                                    this.logger.error('AUTH | LOGIN | Organization could not be found for user');
                                    response.json(
                                        {
                                            success: 0,
                                            description: 'organization could not be found for user. Contact the administrator.',
                                        }
                                    );
                                }
                            })
                        } else {
                            this.logger.error('AUTH | LOGIN | User not found');
                            response.json(
                                {
                                    success: 0,
                                    description: 'basedata could not be found for user. Contact the administrator.',
                                }
                            );
                        }
                    })

                } else {
                    // User invalid. Send unsuccessful response
                    this.logger.error('AUTH | LOGIN | User not found or passwords do not match');
                    return response.status(401).json(
                        {
                            success: 0,
                            description: 'Invalid Authentication Credentials'
                        }
                    );
                }
            })
        } else {
            this.logger.error('AUTH | LOGIN | Uername or password empty');
            return response.status(401).json(
                {
                    success: 0,
                    description: 'Invalid Authentication Credentials (not provided password and/or username)'
                }
            );
        }
    },

    changePassword(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        const newPasswordMd5 = request.swagger.params.newPasswordHash.value;

        if ((username.length > 0) && (passwordMd5.length > 0) && (newPasswordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, userRow) => {
                if (success) {
                    // user is authenticated
                    var users = new Users(global.logger);
                    users.setPasswordMD5HashedForUser(userRow.id, newPasswordMd5, (success) => {
                        if (success) {
                            // new password hash set in core database for user
                            return response.status(200).json(
                                {
                                    success: 1,
                                    description: 'success'
                                }
                            )
                        } else {
                            // new password could not be set in core database for user
                            return response.status(402).json(
                                {
                                    success: 0,
                                    description: 'The new password could not be set in ALARMiator Sever for user. Contact administrator.'
                                }
                            );
                        }
                    })
                } else {
                    // authentication went wrong
                    this.logger.error('AUTH | LOGIN | Uername or password empty');
                    return response.status(401).json(
                        {
                            success: 0,
                            description: 'Invalid Authentication Credentials (not provided password and/or username)'
                        }
                    );
                }
            });
        }
    },
    getMyUserGroups(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, userRow) => {
                if (success) {
                    // user is authenticated
                    if (userRow.superuser === 1) {
                        return response.json(
                            {
                                success: 1,
                                grantedNamespaces: ['superuser']
                            }
                        );
                    }
                    auth.getUsersGroupsForUserWithId(userRow.id, (accessRows) => {
                        if (accessRows != null) {
                            return response.json(
                                {
                                    success: 1,
                                    grantedNamespaces: accessRows.map(row => row.grantnamespace)
                                }
                            );
                        }
                        else {
                            return response.status(401).json(
                                {
                                    success: 0,
                                    description: 'Error fetching user groups'
                                }
                            );
                        }
                    })
                } else {
                    // authentication went wrong
                    this.logger.error('AUTH | LOGIN | Uername or password empty');
                    return response.status(401).json(
                        {
                            success: 0,
                            description: 'Invalid Authentication Credentials (not provided password and/or username)'
                        }
                    );
                }
            });
        }
    }
}