const Devices = require('../../../../../models/alertdevices');
const Basedata = require('../../../../../models/basedata');
const Auth = require('../../../../../models/auth');
const FCMToken = require('../../../../../models/fctmtokens');

module.exports = {
    getBasedataIdForUser(username, passwordMd5, callback) {
        console.log(`authenticating User (${username} | ${passwordMd5})`);
        callback(1);
    },

    register(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        
        // authenticate user
        var auth = new Auth(global.logger);
        auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
            if (success) {
                // get basedataId
                var basedataId = row.basedataId;
                // Get Device information
                const uuid = request.swagger.params.uuid.value;
                const name = request.swagger.params.name.value;
                const model = request.swagger.params.model.value;
                const platform = request.swagger.params.platform.value;
                const osVersion = request.swagger.params.osVersion.value;

                var devices = new Devices(global.logger);
                devices.doesDeviceExist(uuid, (existingDeviceRow) => {
                    global.logger.debug('API | Device existing result: ' + existingDeviceRow.length);
                    if (existingDeviceRow.length > 0) {
                        response.json(
                            {
                                success: 0,
                                description: 'Device exists in database.',
                            }
                        );
                        // there is a device with this uuid
                        // can not be created again
                    } else {
                        devices.createDevice(basedataId, null, name, model, platform, osVersion, uuid, 0, (success) => {
                            global.logger.debug('API | registered device success: ' + success);
                            if (success) {

                                var registerInfo = {
                                    basedataId: basedataId,
                                    deviceName: name,
                                    deviceModel: model,
                                    devicePlatform: platform,
                                    deviceOsVersion: osVersion,
                                    deviceUUID: uuid,
                                }

                                var sendMessage = {
                                    newDeviceRegistered: registerInfo
                                }

                                process.send(JSON.stringify(sendMessage));

                                response.json(
                                    {
                                        success: 1,
                                        description: 'device saved'
                                    }
                                );
                            } else {
                                response.json(
                                    {
                                        success: 0,
                                        description: 'device could not be stored'
                                    }
                                );
                            }
                        })
                    }
                })

            } else {
                // User not authenticated
                response.json(
                    {
                        success: 0,
                        description: 'not authenticated'
                    }
                );
            }
        })
    },

    unregister(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
    
        // authenticate user
        var auth = new Auth(global.logger);
        auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
            if (success) {
                // get basedataId
                var basedataId = row.basedataId;
                // Get Device information
                const uuid = request.swagger.params.uuid.value;

                var devices = new Devices(global.logger);
                
                devices.removeDeviceByUUID(uuid, (success) => {
                    if (success) {
                        response.json(
                            {
                                success: 1,
                                description: 'Device removed from database.',
                            }
                        );
                    } else {
                        response.json(
                            {
                                success: 0,
                                description: 'Device could not be removed from database.',
                            }
                        );
                    }
                })
            } else {
                // User not authenticated
                response.json(
                    {
                        success: 0,
                        description: 'not authenticated'
                    }
                );
            }
        })
    },

    registerfcmtoken(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        
        // authenticate user
        var auth = new Auth(global.logger);
        auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
            if (success) {
                // Get Device information
                const uuid = request.swagger.params.uuid.value;
                // get token from device
                const fcmtoken = request.swagger.params.fcmtoken.value;
                const appnamespace = request.swagger.params.appnamespace.value;

                var devices = new Devices(global.logger);
                devices.getAlertdeciceForUUID(uuid, (rowsDevice) => {
                    if (rowsDevice.length > 0) {
                        // Found a device for uuid
                        var fcmtokens = new FCMToken(global.logger);
                        fcmtokens.doesTokenExist(fcmtoken, (isExistingToken) => {
                            if (isExistingToken.length > 0) {
                                // Token exists
                                response.json(
                                    {
                                        success: 1,
                                        description: 'Token exists at database, did not create a new one.',
                                    }
                                );
                            } else {
                                // New token
                                fcmtokens.createFCMToken(rowsDevice[0].id, 1, appnamespace, 0, fcmtoken, (successToken) => {
                                    if (successToken) {
                                        response.json(
                                            {
                                                success: 1,
                                                description: 'new token stored to database',
                                            }
                                        );
                                    } else {
                                        response.json(
                                            {
                                                success: 0,
                                                description: 'token could not be stored to datbase',
                                            }
                                        );
                                    }
                                });
                            }
                        })
                    } else {
                        // Device for uuid not found
                        response.json(
                            {
                                success: 0,
                                description: 'Device with uuid does not exist in database. Please register device prior adding fcmtoken for device',
                            }
                        );
                    }
                })
            } else {
                // User not authenticated
                response.json(
                    {
                        success: 0,
                        description: 'not authenticated'
                    }
                );
            }
        })
    },

    unregisterTokensForDeviceWithUUID(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        const uuid = request.swagger.params.uuid.value;

        // authenticate user
        var auth = new Auth(global.logger);
        auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
            if (success) {
                var fcmtokens = new FCMToken(global.logger);
                fcmtokens.removeTokensForDeviceUUID(uuid, (success) => {
                    if (success) {
                        // User not authenticated
                        response.json(
                            {
                                success: 1,
                                description: 'tokens deleted'
                            }
                        );
                    } else {
                        // User not authenticated
                        response.json(
                            {
                                success: 0,
                                description: 'could not delete tokens'
                            }
                        );
                    }
                })
            } else {
                // User not authenticated
                response.json(
                    {
                        success: 0,
                        description: 'not authenticated'
                    }
                );
            }
        });
    },
    updateDevice(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
    
        // authenticate user
        var auth = new Auth(global.logger);
        auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
            if (success) {
                // get basedataId
                var basedataId = row.basedataId;
                // Get Device information
                const uuid = request.swagger.params.uuid.value;
                const appversion = request.swagger.params.appversion.value;

                var devices = new Devices(global.logger);

                if (appversion) {
                    devices.updateDeviceVersion(uuid, appversion, (success) => {
                        if (success) {
                            response.json(
                                {
                                    success: 1,
                                    description: 'Updated app version',
                                }
                            );
                        } else {
                            response.status(400).json(
                                {
                                    success: 0,
                                    description: 'Error updating app version',
                                }
                            );
                        }
                    })
                }
                else {
                    // User not authenticated
                    response.status(405).json(
                        {
                            success: 0,
                            description: 'Not available yet'
                        }
                    );
                }
                
            } else {
                // User not authenticated
                response.json(
                    {
                        success: 0,
                        description: 'not authenticated'
                    }
                );
            }
        })
    },
}