const Auth = require('../../../../../models/auth');
const Basedata = require('../../../../../models/basedata');
const Users = require('../../../../../models/users');
const Feedbackstore = require('../../../../../models/core/feedbackstore');



module.exports = {
    setFeedback(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        const operationUUID = request.swagger.params.operationUUID.value;
        const basedataUUID = request.swagger.params.basedataUUID.value;
        const feedbackState = request.swagger.params.state.value;

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {
                    // Check if user needs to be enforced to change its password
                    var feedbackstore = new Feedbackstore(global.logger);
                    feedbackstore.setFeedback(basedataUUID, operationUUID, feedbackState, (success) => {
                        if (success) {
                            var feedbackInfo = {
                                basedataUUID: basedataUUID,
                                operationUUID: operationUUID,
                                feedbackState: feedbackState
                            }
                            var sendMessage = {
                                feedbackState : feedbackInfo
                            }
                            process.send(JSON.stringify(sendMessage));
                            
                            response.json(
                                {
                                    success: 1,
                                    description: 'state set successfully.',
                                });
                        } else {
                            response.json(
                                {
                                    success: 0,
                                    description: 'state could not be set.',
                                });
                        }
                    })
                } else {
                    this.logger.error('AUTH | LOGIN | Authentication failed');
                    response.json(
                        {
                            success: 0,
                            description: 'User not found or passwords do not match.',
                        }
                    );
                }
            })
        } else {
            // User invalid. Send unsuccessful response
            this.logger.error('AUTH | LOGIN | Credentials not sent');
            return response.status(401).json(
                {
                    success: 0,
                    description: 'Invalid Authentication Credentials'
                }
            );
        }
    },

    getParentFeedbackForUUID(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        const operationUUID = request.swagger.params.operationUUID.value;
        const basedataUUID = request.swagger.params.basedataUUID.value;

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {
                    var feedbackstore = new Feedbackstore(global.logger);
                    feedbackstore.getParentFeedbackForOperationWithUUIDandBasedataUUID(basedataUUID, operationUUID, (feedback) => {
                        if (feedback != null) {
                            response.json(
                                {
                                    success: 1,
                                    feedback: feedback
                                });
                        } else {
                            response.status(404).json(
                                {
                                    success: 0,
                                    description: 'Could not get feedback for this request',
                                });
                        }
                    })
                } else {
                    response.status(400).json(
                        {
                            success: 0,
                            description: 'User not found or passwords do not match.',
                        }
                    );
                }
            })
        } else {
            // User invalid. Send unsuccessful response
            this.logger.error('AUTH | LOGIN | Credentials not sent');
            return response.status(401).json(
                {
                    success: 0,
                    description: 'Invalid Authentication Credentials'
                }
            );
        }
    },

    changePassword(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        const newPasswordMd5 = request.swagger.params.newPasswordHash.value;

        if ((username.length > 0) && (passwordMd5.length > 0) && (newPasswordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, userRow) => {
                if (success) {
                    // user is authenticated
                    var users = new Users(global.logger);
                    users.setPasswordMD5HashedForUser(userRow.id, newPasswordMd5, (success) => {
                        if (success) {
                            // new password hash set in core database for user
                            return response.status(200).json(
                                {
                                    success: 1,
                                    description: 'success'
                                }
                            )
                        } else {
                            // new password could not be set in core database for user
                            return response.status(402).json(
                                {
                                    success: 0,
                                    description: 'The new password could not be set in ALARMiator Sever for user. Contact administrator.'
                                }
                            );
                        }
                    })
                } else {
                    // authentication went wrong
                    this.logger.error('AUTH | LOGIN | Uername or password empty');
                    return response.status(401).json(
                        {
                            success: 0,
                            description: 'Invalid Authentication Credentials (not provided password and/or username)'
                        }
                    );
                }
            });
        }
    }
}