const Basedata = require('../../../../../models/basedata');
const Operations = require('../../../../../models/operationshelper');
const Auth = require('../../../../../models/auth');
const Feedbackstore = require('../../../../../models/core/feedbackstore');
const Qualifications = require('../../../../../models/qualifications');
const Feedbackstates = require('../../../../../models/feedbackstates');
const Operation = require('../../../../../models/core/core_operation');
const AccessTokens = require('../../../../../models/accesstokens');
const moment = require('moment');

const uuid = require('uuid').v1;

function transform(value) {
    var retVal = '';
    if (typeof value != 'undefined') {
        return value;
    } else {
        return '';
    }
}

module.exports = {
    getBasedataIdForUser(username, passwordMd5, callback) {
        console.log(`authenticating User (${username} | ${passwordMd5})`);
        callback(1);
    },

    // returns list of operations a user has been alarmed for und which are active
    list(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {
                    // Check if user needs to be enforced to change its password
                    var operations = new Operations(global.logger);
                    operations.getListOfActiveOperationsUserWasAlarmedFor(row.basedataUUID, (operationsList) => {
                        response.json(
                            {
                                success: 1,
                                operations: operationsList
                            }
                        )
                    })
                } else {
                    this.logger.error('AUTH | LOGIN | Authentication failed');
                    response.json(
                        {
                            success: 0,
                            description: 'User not found or passwords do not match.',
                        }
                    );
                }
            })
        } else {
            // User invalid. Send unsuccessful response
            this.logger.error('AUTH | LOGIN | Credentials not sent');
            return response.status(401).json(
                {
                    success: 0,
                    description: 'Invalid Authentication Credentials'
                }
            );
        }
    },

    getfeedback(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        const uuid = request.swagger.params.uuid.value;

        console.log('API | getFeedback');

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {
                    var organizationId = row.organizationId;

                    var qualifications = new Qualifications(global.logger);
                    var qualificationsArray = [];
                    qualifications.getQualificationsForApp(organizationId, (rowsQualifications) => {
                        if (rowsQualifications != null) {
                            if (rowsQualifications.length > 0) {
                                rowsQualifications.forEach(quali => {
                                    qualificationsArray.push(quali.shortName);
                                });
                            }
                        }
                    })

                    var feedbackstates = new Feedbackstates(global.logger);
                    feedbackstates.getAllStates((rowsFeedbackStates) => {
                        var operations = new Operations(global.logger);
                        operations.getOperationForUUID(uuid, (rowsOperation) => {
                            if (rowsOperation.length > 0) {
                                var feedbackstore = new Feedbackstore(global.logger);
                                feedbackstore.getParentFeedbackForOperationId(rowsOperation[0].id, (feedbackRows) => {
                                    /**
                                     * FOR DEBUGGING
                                    console.log('___________________FEEDBACK_____________________');
                                    console.log(feedbackRows);
                                    console.log('-------------------FEEDBACK---------------------');
                                    */


                                    /**
                                     * make sure that qualifications are distinct
                                     */
                                    if (feedbackRows != null) {
                                        if (feedbackRows.length > 0) {
                                            feedbackRows.forEach((fbRow, index) => {
                                                var qualificationsStr = fbRow.qualificationsList;
                                                if (qualificationsStr != null) {
                                                    var qualificationsArr = qualificationsStr.split(',');
                                                    if (Array.isArray(qualificationsArr)) {
                                                        var uniqueArray = qualificationsArr.filter(function (elem, pos) {
                                                            return qualificationsArr.indexOf(elem) == pos;
                                                        })
                                                        feedbackRows[index].qualificationsList = uniqueArray.join(',');
                                                    }
                                                }
                                            });
                                        }
                                    }


                                    /** NEW kind of feedback */
                                    var payloadArray = [];
                                    rowsFeedbackStates.forEach((feedbackState) => {

                                        var feedbackInState = [];
                                        feedbackRows.forEach((feedbackRow) => {
                                            if (feedbackRow.feedbackStateLabel === feedbackState.stateLabel) {
                                                feedbackInState.push(feedbackRow);
                                            }
                                        });

                                        var payloadPart = {
                                            [feedbackState.stateLabel]: feedbackInState
                                        }
                                        payloadArray.push(payloadPart);
                                    })
                                    /** END NEW Kind of feedback */


                                    if (feedbackRows !== null) {
                                        // filter qualifications to match the wallboard profile settings
                                        feedbackRows.forEach((feedback, index) => {
                                            var newQualiSting = '';
                                            var qualiSting = feedback.qualificationsList;
                                            if (qualiSting === null) {
                                                qualiSting = '';
                                            }
                                            var qualiArray = qualiSting.split(',');
                                            if (qualiArray.length > 0) {
                                                qualiArray.forEach((quali) => {
                                                    if (qualificationsArray.includes(quali)) {
                                                        newQualiSting = newQualiSting + quali + ',';
                                                    }
                                                })
                                            }
                                            if (newQualiSting.length > 0) {
                                                var checkChar = newQualiSting.slice(-1);
                                                if (checkChar = ',') {
                                                    newQualiSting = newQualiSting.slice(0, (newQualiSting.length - 1));
                                                }
                                            }
                                            feedbackRows[index].qualificationsList = newQualiSting;
                                        })
                                        // End Filter
                                        response.json(
                                            {
                                                success: 1,
                                                feedbackStates: rowsFeedbackStates,
                                                feedback: payloadArray,
                                                qualifications: qualificationsArray
                                            }
                                        )
                                    } else {
                                        response.status(204).send();
                                    }
                                })
                            } else {
                                console.log('API | getFeedback | could not find operation for uuid: ' + uuid);
                                response.status(404).send();
                            }
                        });
                    })

                } else {
                    response.status(401).send('not authenticated.');
                }
            });
        } else {
            response.status(401).send('not authenticated');
        }
    },

    archive(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;

        const operationId = request.swagger.params.operationId.value;
        const uuid = request.swagger.params.uuid.value;

        var operations = new Operations(global.logger);

        // check which identifier has been handed over for operation to be archived
        if (typeof operationId != 'undefined') {

            operations.archiveOperation(operationId, (success) => {
                if (success) {
                    response.json(
                        {
                            success: 1,
                            description: 'operation successfully archived'
                        }
                    )
                } else {
                    response.json(
                        {
                            success: 0,
                            description: 'operation could not be archived'
                        }
                    )
                }
            })
        } else if (typeof uuid != 'undefined') {
            operations.archiveOperationByUUID(uuid, (success) => {
                if (success) {
                    response.json(
                        {
                            success: 1,
                            description: 'operation successfully archived'
                        }
                    )
                } else {
                    response.json(
                        {
                            success: 0,
                            description: 'operation could not be archived'
                        }
                    )
                }
            })
        } else {
            response.json(
                {
                    success: 0,
                    description: 'operation could not be archived, no identifier handed over'
                }
            )
        }
    },

    close(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;

        const operationId = request.swagger.params.operationId.value;
        const uuid = request.swagger.params.uuid.value;

        var operations = new Operations(global.logger);

        // check which identifier has been handed over for operation to be archived
        if (typeof operationId != 'undefined') {

            operations.closeOperation(operationId, (success) => {
                if (success) {
                    response.json(
                        {
                            success: 1,
                            description: 'operation successfully closed'
                        }
                    )
                } else {
                    response.json(
                        {
                            success: 0,
                            description: 'operation could not be closed'
                        }
                    )
                }
            })
        } else if (typeof uuid != 'undefined') {
            operations.closeOperationByUUID(uuid, (success) => {
                if (success) {
                    response.json(
                        {
                            success: 1,
                            description: 'operation successfully closed'
                        }
                    )
                } else {
                    response.json(
                        {
                            success: 0,
                            description: 'operation could not be closed'
                        }
                    )
                }
            })
        } else {
            response.json(
                {
                    success: 0,
                    description: 'operation could not be archived, no identifier handed over'
                }
            )
        }
    },

    createOperation(request, response) {
        // extract API Key for alarmings
        const token = transform(request.swagger.params.token.value);
        
        // extract operation data from request
        var alarmdate = transform(request.swagger.params.alarmdate.value);
        if (alarmdate.length < 1) {
            alarmdate = moment().format('DD.MM.YYYY');
        }
        var alarmtime = transform(request.swagger.params.alarmtime.value);
        if (alarmtime.length < 1) {
            alarmtime = moment().format('HH:mm:ss');
        }
        const gear = transform(request.swagger.params.gear.value);
        var operationnumber = transform(request.swagger.params.operationnumber.value);
        // make sure we have a operationnumber set
        if ((operationnumber === null) || (operationnumber.length < 1)) {
            operationnumber = uuid();
        }
        const location = transform(request.swagger.params.location.value);
        const message = transform(request.swagger.params.message.value);
        const community = transform(request.swagger.params.community.value);
        const object = transform(request.swagger.params.object.value);
        const district = transform(request.swagger.params.district.value);
        const keywordId = transform(request.swagger.params.keywordId.value);
        const keywordCategory = transform(request.swagger.params.keywordCategory.value);
        const keywordName = transform(request.swagger.params.keywordName.value);
        const zveis = transform(request.swagger.params.zveis.value);
        const subject = transform(request.swagger.params.subject.value);
        const street = transform(request.swagger.params.street.value);
        const gkx = transform(request.swagger.params.gkx.value);
        const gky = transform(request.swagger.params.gky.value);
        const floor = transform(request.swagger.params.floor.value);
        const ils = transform(request.swagger.params.ils.value);
        const section = transform(request.swagger.params.section.value);
        var connector = transform(request.swagger.params.connector.value);
        // set connector by default if not set by caller
        if (connector.length < 1) {
            connector = 'REST API';
        }
        var istest = transform(request.swagger.params.istest.value);
        // make sure this is set by default if not set by caller
        if (istest.length < 1) {
            istest = 0;
        }
        console.log('REST-API | recieved new operation from Rest Api');

        // Check if we have a valid access token
        let accesstoken = new AccessTokens(global.logger);
        accesstoken.isTokenValidandActive(token, (valid) => {
            if (valid) {
                // Token valid and active
                var operation = new Operation();
                operation.setUUID();
                operation.alarmdate = alarmdate;
                operation.alarmtime = alarmtime;
                operation.gear = gear;
                operation.operationnumber = operationnumber;
                operation.location = location;
                operation.message = message;
                operation.community = community;
                operation.object = object;
                operation.district = district;
                operation.keywordId = keywordId;
                operation.keywordCategory = keywordCategory;
                operation.keywordName = keywordName;
                operation.zveis = zveis;
                operation.subject = subject;
                operation.street = street;
                operation.gkx = gkx;
                operation.gky = gky;
                operation.floor = floor;
                operation.ils = ils;
                operation.section = section;
                operation.operationSourceConnector = connector;
                operation.isTest = istest;

                // prepate operation to be sent to core
                var sendMessage = {
                    newAlarm : operation
                }
                process.send(JSON.stringify(sendMessage));

                // Send response to client (successful)
                return response.status(200).json(
                    {
                        success: 1,
                        description: 'success'
                    }
                );
            } else {
                // Token not found or inactive
                return response.status(200).json(
                    {
                        success: 0,
                        description: 'Invalid Authentication access token'
                    }
                );
            }
        })
    },

    // returns additional information for a specific operation
    getAdditionalOperationInfos(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        const operationUUID = request.swagger.params.uuid.value;

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {
                    // Check if user needs to be enforced to change its password
                    var operations = new Operations(global.logger);
                    operations.getAdditionalInfosForOperationUUID(operationUUID, (additionalInfos) => {
                        if (additionalInfos != null && additionalInfos.length > 0) {
                            var op = operations.createOperationFromObject(additionalInfos[0])

                            const additionalInfosFields = [
                                '_zveisDeltaCalc',
                                '_gearDataCalc'
                            ];

                            var additionalInfo = Object.keys(op)
                              .filter(key => additionalInfosFields.includes(key))
                              .reduce((obj, key) => {
                                if (op.isDefined(op[key])){
                                    obj[key] = op[key];
                                }
                                return obj;
                              }, {});


                            operations.getAlarmedGroupsForOperationUUID(operationUUID, (alarmedGroups) => {
                                if (alarmedGroups != null) {
                                    additionalInfo['_alarmedGroups'] = alarmedGroups;
                                }
                                return response.json(
                                    {
                                        success: 1,
                                        operationId: operationUUID,
                                        infos: additionalInfo
                                    }
                                )
                            })        
                        }
                        else {
                            return response.status(404).json(
                                {
                                    success: 0,
                                    description: 'UUID not found'
                                }
                            );
                        }
                    })
                    
                } else {
                    this.logger.error('AUTH | LOGIN | Authentication failed');
                    response.json(
                        {
                            success: 0,
                            description: 'User not found or passwords do not match.',
                        }
                    );
                }
            })
        } else {
            // User invalid. Send unsuccessful response
            this.logger.error('AUTH | LOGIN | Credentials not sent');
            return response.status(401).json(
                {
                    success: 0,
                    description: 'Invalid Authentication Credentials'
                }
            );
        }
    },
    // returns list of operation uuids a user has been alarmed for und which are active
    getMyActiveOperationUUIDs(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {
                    // Check if user needs to be enforced to change its password
                    var operations = new Operations(global.logger);
                    operations.getListOfActiveOperationsUserWasAlarmedFor(row.basedataUUID, (operationList) => {
                        if (operationList != null) {
                            const uuidList =  operationList
                            .reduce((array, operation) => {
                                array.push({
                                    uuid: operation.uuid
                                });
                                return array;
                            }, []);
                            return response.json(
                                {
                                    success: 1,
                                    uuids: uuidList
                                }
                            ) 
                        }
                        else {
                            response.json(
                                {
                                    success: 0,
                                    description: 'Error Occured',
                                }
                            )
                        }
                    })
                } else {
                    this.logger.error('AUTH | LOGIN | Authentication failed');
                    response.json(
                        {
                            success: 0,
                            description: 'User not found or passwords do not match.',
                        }
                    );
                }
            })
        } else {
            // User invalid. Send unsuccessful response
            this.logger.error('AUTH | LOGIN | Credentials not sent');
            return response.status(401).json(
                {
                    success: 0,
                    description: 'Invalid Authentication Credentials'
                }
            );
        }
    },
    // returns operation for uuid
    getOperationForUUID(request, response) {
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;
        const operationUUID = request.swagger.params.uuid.value;

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {
                    // Check if user needs to be enforced to change its password
                    var operations = new Operations(global.logger);
                    operations.getOperationForUUID(operationUUID, (additionalInfos) => {
                        if (additionalInfos != null && additionalInfos.length > 0) {
                            var op = operations.createOperationFromObject(additionalInfos[0])
                            return response.json(
                                {
                                    success: 1,
                                    operation: op.getReduced()
                                }
                            )
                        }
                        else {
                            return response.status(404).json(
                                {
                                    success: 0,
                                    description: 'UUID not found'
                                }
                            );
                        }
                    })
                    
                } else {
                    this.logger.error('AUTH | LOGIN | Authentication failed');
                    response.json(
                        {
                            success: 0,
                            description: 'User not found or passwords do not match.',
                        }
                    );
                }
            })
        } else {
            // User invalid. Send unsuccessful response
            this.logger.error('AUTH | LOGIN | Credentials not sent');
            return response.status(401).json(
                {
                    success: 0,
                    description: 'Invalid Authentication Credentials'
                }
            );
        }
    },
    createMinimalisticOperationFromApp(request, response) { 
        const username = request.swagger.params.username.value;
        const passwordMd5 = request.swagger.params.passwordhash.value;

        if ((username.length > 0) && (passwordMd5.length > 0)) {
            var auth = new Auth(global.logger);
            auth.authenticateUserMD5(username, passwordMd5, (success, row) => {
                if (success) {

                    auth.checkPermission(row.id, "alarmings")
                    .then(() => {
                        // Required
                        const subject = request.swagger.params.subject.value;
                        const alarmedGroupIds = request.swagger.params.alarmedGroupIds.value;

                        // Optional, if non-existent fill with basic information
                        const message = request.swagger.params.message.value ? request.swagger.params.message.value : "Händische Alarmierung von der App ausgelöst!";                    
                        const keywordName = request.swagger.params.keywordName.value ? request.swagger.params.keywordName.value : "Manueller APP Alarm";
                        const location = request.swagger.params.location.value ? request.swagger.params.location.value : "Kein Einsatzort angegeben";


                        // Fill Operation with basic information
                        const alarmdate = moment().format('DD.MM.YYYY');
                        const alarmtime = moment().format('HH:mm:ss');
                        const operationnumber = uuid();
                        const keywordId = "APP";
                        const keywordCategory = "APP-Alarm";
                        const keywordRaw = "#"+keywordId+"#"+keywordCategory+"#"+keywordName;
                        const lat = 0;
                        const lon = 0;
                        const connector = 'ALARMiator Mobile';
                        const istest = 0;
                        console.log('REST-API | received new operation for groups from ALARMiator Mobile');         

                        var operation = new Operation();
                        operation.setUUID();
                        operation.alarmdate = alarmdate;
                        operation.alarmtime = alarmtime;
                        operation.operationnumber = operationnumber;
                        if (location != null) operation.location = location;
                        operation.message = message;
                        operation.keywordId = keywordId;
                        operation.keywordCategory = keywordCategory;
                        operation.keywordName = keywordName;
                        operation.keywordRaw = keywordRaw;
                        operation.subject = subject;
                        operation.lat = lat;
                        operation.lon = lon;
                        operation.operationSourceConnector = connector;
                        operation.isTest = istest;
                        operation.alarmedGroupIds = alarmedGroupIds;

                        // prepate operation to be sent to core
                        var sendMessage = {
                            newAlarm : operation
                        }
                        process.send(JSON.stringify(sendMessage));

                        // Send response to client (successful)
                        return response.status(200).json(
                            {
                                success: 1,
                                description: 'success'
                            }
                        );
                    })
                    .catch(err => {
                        this.logger.error('OPERATIONS | sendMinimalisticAppAlarm | Missing permissions');
                        response.status(403).json(
                            {
                                success: 0,
                                description: 'Missing permissions',
                            }
                        );
                    })                    
                } else {
                    this.logger.error('AUTH | LOGIN | Authentication failed');
                    response.json(
                        {
                            success: 0,
                            description: 'User not found or passwords do not match.',
                        }
                    );
                }
            })
        } else {
            // User invalid. Send unsuccessful response
            this.logger.error('AUTH | LOGIN | Credentials not sent');
            return response.status(401).json(
                {
                    success: 0,
                    description: 'Invalid Authentication Credentials'
                }
            );
        }
    }
}