'use strict';
var Database = require('../../../models/database');

var SwaggerExpress = require('swagger-express-mw');
var app = require('express')();
module.exports = app; // for testing
var config = null;

var swaggerConfig = {
  appRoot: __dirname,  // required config
  configDir: __dirname + '/config'
};

var serviceState = 0;

/**
 * Log Function for inter process communication
 */
function LogAtMain(msg) {
  process.send('{"logmessage" : "' + msg + '"}');
}
 
/**
 * 
 * Wire messages coming from Plugin base.js
 * 
 */
process.on('message', (msg) => {
  if (typeof msg.config != 'undefined') {
      config = msg.config;
      serviceState = 1;
      LogAtMain('Recieved Configuration from CORE');
  } else if (typeof msg.start != 'undefined') {
    if (msg.start === 1) {
      startAPI();
    }
  }
});

// GENERIC Error Handling
process.on('unhandledRejection', (reason, p) => {
  console.log('PLUGIN | RESTAPI | Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
  LogAtMain('Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
});
process.on('uncaughtException', function (err) {
  console.log('PLUGIN | RESTAPI | Caught exception: ' + err);
  process.send('{"error" : "' + err.message + '"}');
});


/**
 * Start API Logger
 */

// create a rolling file logger based on date/time that fires process events
const opts = {
  logDirectory: swaggerConfig.appRoot + "/../../../logs/api", // NOTE: folder must exist and be writable...
  fileNamePattern: 'ALARMiator-API-<DATE>.log',
  dateFormat: 'YYYY.MM.DD'
};
var logmanager = require('simple-node-logger');
global.logger = logmanager.createRollingFileLogger(opts);
global.logger.setLevel('all');
global.logger.info('API | Started API');


/**
 * connect to database
 */
global.appRoot = __dirname + '/../../../';
var coreDb = new Database();
coreDb.openCoreDatabase((successDatabase) => {
  if (successDatabase) {
    global.coreDb = coreDb.db;
    LogAtMain('database started')
    global.logger.info('database started');
  } else {
    LogAtMain('Could not connect to database');
    global.logger.error('Could not connect to database');
  }
});



 /**
 * Start Swagger API Service
 */
function startAPI() {

  LogAtMain('Starting API now');
  var httpPort = 5010;
  var runOnHTTP = true;
  var httpsPort = 5443;
  var runOnHTTPS = false;

  if (typeof config != 'undefined') {
    if (typeof config.httpPort != 'undefined') {
      if (parseInt(config.httpPort) > 0) {
        httpPort = parseInt(config.httpPort);
      }
    }
    if (typeof config.httpsPort != 'undefined') {
      if (parseInt(config.httpsPort) > 0) {
        httpsPort = parseInt(config.httpsPort);
      }
    }
    if (typeof config.runOnHTTP != 'undefined') {
      if (config.runOnHTTP === 'true') {
        runOnHTTP = true;
      } else {
        runOnHTTP = false;
      }
    }
    if (typeof config.runOnHTTPS != 'undefined') {
      if (config.runOnHTTPS === 'true') {
        runOnHTTPS = true;
      } else {
        runOnHTTPS = false;
      }
    }
  }

  
  SwaggerExpress.create(swaggerConfig, function (err, swaggerExpress) {
    if (err) { throw err; }
  
    // install middleware
    swaggerExpress.register(app);
  
    var port = httpPort || 5010;
    app.listen(port);
  
    console.log('PLUGIN | RESTAPI | Rest API running on Port ' + port);
    LogAtMain('CORE | Rest API running on Port ' + port);
    // Starting HTTPS
    // check if certificate-files exist
    const fs = require('fs');
    if (
      (fs.existsSync(__dirname + '/../../../certificates/privkey.pem')) &&
      (fs.existsSync(__dirname + '/../../../certificates/cert.pem'))) {
      // Only start SSL Server if pem files are existing
  
      var sslport = httpsPort || 5443;
      var https = require('https');
      var options = {
        key: fs.readFileSync(__dirname + '/../../../certificates/privkey.pem'),
        cert: fs.readFileSync(__dirname + '/../../../certificates/cert.pem'),
        passphrase: 'ALARMiator'
      };
      https.createServer(options, app).listen(sslport, function () {
        console.log('PLUGIN | RESTAPI | SSL Server started # %s!', sslport);
        LogAtMain('SSL Server started # %s!', sslport);
      });
    } else {
      console.log('PLUGIN | RESTAPI | SSL Server NOT started cause if missing certificate files');
      LogAtMain('SSL Server NOT started cause of missing certificate files ');
    }
  
  });
}



