const moment = require('moment');
const Operation = require(__dirname + '/../../../../models/core/core_operation.js');
class ParserUtils {
    constructor() {
    
    }

    calculateZveisGearData(mode, allElements, deltaElements) {
        // mode should be "zvei" or "gear"
        let datePart = moment().format('YYYY-MM-DD');
        let datePartYesterday = moment().subtract(1, 'days').format('YYYY-MM-DD');
        let dataObj = [];
        let deltaArray = [];
    
        let seperator;

        switch (mode) {
            case "gear":
                seperator = /\n|;/gm;
                break;
            case "zvei":
            default:
                seperator = /\n|,|;/gm;
        }

        const timeRegex = /\d{2}:\d{2}[:\d{2}]*/m;
    
        if (deltaElements != null) {
            if (deltaElements.length > 0) {
                deltaElements.split(seperator).forEach((element) => {

                    if (element.match(timeRegex)) {
                        let timeString = element.match(timeRegex);
    
                        let timestamp = moment(datePart + ' ' + timeString);

                        let testTimestamp = moment(timestamp); // clone for testing;
    
                        if (testTimestamp.subtract(1, "minutes").isAfter(moment())) {
                            timestamp.subtract(1, 'days');
                        }
                        
                        timestamp = timestamp.format('YYYY-MM-DDTHH:mm:ss')
                        deltaArray.push({
                            element: element.replace(timeRegex,'').trim(),
                            timestamp: timestamp
                        });
                    }
                    else {
                        deltaArray.push({
                            element: element.replace(timeRegex,'').trim(),
                            timestamp: null
                        });
                    }
                })
            }
        }
    
        if (allElements != null) {
            if (allElements.length > 0) {
                // split now and extract time
                // for cases where separator is ","
                var allElementsArr = allElements.split(seperator);
                if (Array.isArray(allElementsArr)) {
                    // successfully separated zveis with time into array
                    allElementsArr.forEach(elementWithTime => {
                        if (elementWithTime.match(timeRegex)) {
                            // has time
                            let elementString = elementWithTime.replace(timeRegex,'').trim();
                            let timeString = elementWithTime.match(timeRegex);
    
                            let timestamp = moment(datePart + ' ' + timeString).format('YYYY-MM-DDTHH:mm:ss');
                            let sameOrBefore = moment().add(1, 'minutes').isSameOrBefore(timestamp);
                            console.log('PARSING: element: ' + elementString + ' | time: ' + timeString + ' | timestamp: ' + timestamp + '| same or before now: ' + sameOrBefore);
                            if (sameOrBefore) timestamp = moment(datePartYesterday + ' ' + timeString).format('YYYY-MM-DDTHH:mm:ss');
                            
                            let newElement = true;
    
                            if (deltaArray.length > 0) {
                                let found = deltaArray.find(x => x.element == elementString && x.timestamp == timestamp);
                                if (found != undefined && found != null) {
                                    newElement = true;
                                }
                                else {
                                    newElement = false;
                                }
                            }
                            else {
                                newElement = false;
                            }
                            
                            let deltaElement = {
                                timestamp: timestamp,
                                new: newElement
                            }
                            deltaElement[mode] = elementString;
    
                            dataObj.push(deltaElement);
    
                        }
                        else {
                            // no time
                            let deltaElement = {
                                timestamp: null,
                                new: true
                            }
                            deltaElement[mode] = elementWithTime.replace(timeRegex,'').trim();
                            dataObj.push(deltaElement);
                            console.log('WARNING: only zvei / gear element found but no time: ' + elementWithTime);
                        }
                    });
                } else {
                    // not an array, do nothing
                    return [];
                }
            }
        }
        //console.log("Full Dataobject", dataObj);
        return dataObj;
    }

    createDeltaElementsString(mode, allElements) {
        let datePart = moment().format('YYYY-MM-DD');
        var deltaArray = [];
        let seperator;

        switch (mode) {
            case "gear":
                seperator = /\n|;/gm;
                break;
            case "zvei":
            default:
                seperator = /\n|,|;/gm;
        }

        const timeRegex = /\d{2}:\d{2}[:\d{2}]*/m;

        if (allElements != null) {
            if (allElements.length > 0) {
                allElements.split(seperator).forEach((element) => {

                    if (element.match(timeRegex)) {
                        let timeString = element.match(timeRegex);
    
                        let timestamp = moment(datePart + ' ' + timeString);
    
                        if (moment(timestamp).subtract(1, "minutes").isAfter(moment())) {
                            timestamp.subtract(1, 'days');
                        }
                        if (moment(timestamp).isBetween(
                            moment().seconds(0).milliseconds(0).subtract(1, 'minutes'),
                            moment().seconds(59).milliseconds(0).add(1, 'minutes'),
                            undefined,
                            "[]"
                        )){
                            deltaArray.push(element);
                        }
                    }
                    else {
                        deltaArray.push(element);
                    }
                })
                // success case
                return deltaArray.join(';')
            }
        }
        // error case
        return null;
    }
}

module.exports = ParserUtils;