const EventEmitter = require('events');
const gk = require('gauss-krueger');
const Operation = require(__dirname + '/../../../../models/core/core_operation.js');

const ParserUtils = require('./parser-utils');

class ParserERH extends EventEmitter {

    statusMap = {
        '32770': 'Priorisierter Sprechwunsch',
        '32771': 'Einsatzbereit über Funk',
        '32772': 'Einsatzbereit auf Wache',
        '32773': 'Einsatz übernommen',
        '32774': 'Am Einsatzort eingetroffen',
        '32775': 'Sprechwunsch (einsatzbezogen)',
        '32776': 'Nicht Einsatzbereit',
        '32777': '7',
        '32778': '8',
        '32779': 'Fremdanmeldung'
    }

    statusMappingToStatusShort = {
        '32770': 0,
        '32771': 1,
        '32772': 2,
        '32773': 3,
        '32774': 4,
        '32775': 5,
        '32776': 6,
        '32777': 7,
        '32778': 8,
        '32779': 9
    }

    constructor() {
        super();

        // Parser-Log-Name
        this.parserLogName = 'PARSER-ILSERH'

        // Status Data
        this.issi = null;
        this.radioStatus = null;
        this.radioStatusHumanReadable = null;
        this.radioStatusShort = null;

        this.parserUtils = new ParserUtils();
    }

    parseRAWAlarmData(rawAlarmDataArray) {
        console.log(this.parserLogName + ' | PARSING RAW ALARM');
        console.log('----------- RAW DATA inside parser ---------- ');
        console.log(rawAlarmDataArray);
        console.log('----------- END RAW DATA inside parser ----- ');
        this.operation = new Operation();
        // ILS ERH
        // this.schlagwort_kurz = rawAlarmDataArray.schlagwort_kurz;
        //this.sds_abschnitt_kurz_tts = rawAlarmDataArray.sds_abschnitt_kurz_tts;
        // this.sds_alarmweg = rawAlarmDataArray.sds_alarmweg;
        // this.sds_datum_kurz = rawAlarmDataArray.sds_datum_kurz;
        // this.sds_einsatzdatum = rawAlarmDataArray.sds_einsatzdatum;
        // this.sds_erh_kbm = rawAlarmDataArray.sds_erh_kbm;
        // this.sds_plan = rawAlarmDataArray.sds_plan;


        this.operation.section = rawAlarmDataArray.sds_abschnitt;
        this.operation.alarmdate = rawAlarmDataArray.sds_alarmdatum;
        this.operation.alarmtime = rawAlarmDataArray.sds_alarmuhrzeit;
        this.operation.gear = rawAlarmDataArray.sds_einsatzmittel;
        this.operation.operationnumber = rawAlarmDataArray.sds_einsatznummer;
        if (rawAlarmDataArray.sds_ort != undefined) {
            this.operation.location = rawAlarmDataArray.sds_ort.trim();
        }

        this.operation.message = rawAlarmDataArray.sds_freitext;

        this.operation.rawdata = rawAlarmDataArray.sds_komplett;
        this.operation.object = rawAlarmDataArray.sds_objekt_kurz_tts;
        if (rawAlarmDataArray.sds_ortsteil != undefined) {
            this.operation.district = rawAlarmDataArray.sds_ortsteil.trim();
            this.operation.community = rawAlarmDataArray.sds_ortsteil.trim();
        }

        this.operation.keywordId = rawAlarmDataArray.schlagwort;
        this.operation.keywordCategory = rawAlarmDataArray.sds_schlagwort_tts_navi;
        this.operation.keywordRaw = rawAlarmDataArray.sds_schlagwort;
        //this.operation.keywordName = rawAlarmDataArray.schlagwort_kurz;
        this.operation.keywordName = rawAlarmDataArray.schlagwort;

        // replace double-pipe (ERH specific)
        var tmpZVEIs = '' + rawAlarmDataArray.sds_schleifen;
        tmpZVEIs = tmpZVEIs.replace(/\|/g, "");
        
        // remove times from zveis list (by extracting only 4-5 digit zveis and zveis with an "F" at the end)
        var re = /[0-9]{4,}[F]*/gm;
        var matchArray = tmpZVEIs.match(re);
        var newZveis = matchArray.join(', ');
        this.operation.zveis = newZveis;

        try {

            // parse zvei delta
            let sdsSchleifenDelta = this.parserUtils.createDeltaElementsString("zvei", rawAlarmDataArray.sds_schleifen);

            this.operation.zveisDelta = sdsSchleifenDelta;
            if (sdsSchleifenDelta != null) {
                try {
                    //this.operation.zveisDeltaCalc = this.splitZVEIsDelta(sdsSchleifenDelta);
                    this.operation.zveisDeltaCalc = this.parserUtils.calculateZveisGearData("zvei", rawAlarmDataArray.sds_schleifen, sdsSchleifenDelta)
                } catch (err) {
                    console.log('ERROR calling calculateZveisGearData: ' + err);
                }
                console.log('------------- Calculated ZVEIs Delta ----------------');
                console.log(this.operation.zveisDeltaCalc);
                console.log('.....................................................');
            }
            // end parse zvei delta
        }
        catch (exp) {

        }

        try {
            // parse gear data
            let sdsGearDelta = this.parserUtils.createDeltaElementsString("gear", rawAlarmDataArray.sds_einsatzmittel);
            this.operation.gearData = sdsGearDelta;

            if (sdsGearDelta != null) {
                try {
                    this.operation.gearDataCalc = this.parserUtils.calculateZveisGearData("gear", rawAlarmDataArray.sds_einsatzmittel, sdsGearDelta);
                    console.log(this.operation.gearDataCalc);
                } catch (err) {
                    console.log('ERROR calling calculateZveisGearData: ' + err);
                }
                console.log('------------- Calculated Gear Data ----------------');
                console.log(this.operation.gearDataCalc);
                console.log('.....................................................');
            }
            // end parse gear data
        }
        catch (exp) {

        }


        this.operation.subject = rawAlarmDataArray.sds_stichwort;

        if (rawAlarmDataArray.sds_strasse != undefined && rawAlarmDataArray.sds_hausnummernzusatz != undefined) {
            this.operation.street = (rawAlarmDataArray.sds_strasse + ' ' + rawAlarmDataArray.sds_hausnummernzusatz).trim();
        }

        this.operation.gkx = rawAlarmDataArray.sds_x;
        this.operation.gky = rawAlarmDataArray.sds_y;
        //this.operation.floor = rawAlarmDataArray.sds_etage;
        //this.operation.ils = rawAlarmDataArray.sds_ils;
        this.calculateLatLon();
        // Now caluclate the keyword parts from raw keyword
        this.calculateKeywordsFromRaw();

        console.log(this.parserLogName + ' | ----------- Operation Object inside parser prio raising event alarm_parsed ---------- ');
        console.log(this.operation);
        console.log(this.parserLogName + ' | ----------- ENDE Operation Object inside parser prior raising event alarm parsed ---- ');

        this.emit('alarm_parsed', null);
    }

    /**
     * injects a test alarm in parser to test following process
     * FOR TESTING
     */
    TestparseRAWAlarmData() {
        console.log(this.parserLogName + ' | TEST | PARSING RAW ALARM');

        this.operation = new Operation();

        this.operation.section = 'Kreuzung';
        this.operation.alarmdate = 'ALARMDATUM';
        this.operation.alarmtime = 'ALARMZEIT';
        this.operation.gear = 'EINSATZMITTEL';
        this.operation.operationnumber = 'EINSATZNUMMER';
        this.operation.location = 'LOCATION';
        this.operation.message = 'MITTEILUNG';
        this.operation.community = 'GEMEINDE';
        this.operation.rawdata = 'KOMPLETTE SDS';
        this.operation.object = 'OBJEKT';
        this.operation.district = 'ORTSTEIL';
        this.operation.keywordId = 'STICHWORT-ID';
        this.operation.keywordCategory = 'STICHWORT KATEGORIE';
        this.operation.keywordRaw = 'STICHWORT RAW';
        this.operation.keywordName = 'SCHLAGWORT NAME';
        this.operation.zveis = 'SCHLEIFEN';
        this.operation.subject = 'STICHWORT';
        this.operation.street = 'STRASSE';
        this.operation.gkx = '4447706.33';
        this.operation.gky = '5485688.77';
        this.operation.floor = '1. OG';
        this.operation.ils = 'ILS Nürnberg';

        this.calculateLatLon();
        this.emit('alarm_parsed', null);
    }

    parseRAWZVEIData(rawZVEIDataArray) {
        console.log(this.parserLogName + ' | parseRAWZVEIData: ');
        this.zveicode = rawZVEIDataArray.data.code;
        this.zveidescription = rawZVEIDataArray.data.description;
        this.emit('zveiParsed', null);
    }

    parseRAWStatusData(rawStatusDataArray) {
        console.log(this.parserLogName + ' | parseRAWStatusData: ' + rawStatusDataArray);
        this.radioStatus = rawStatusDataArray.data.radioStatus;
        this.issi = rawStatusDataArray.data.issi;
        this.translateStatusNumberToHumanReadable();
        this.emit('statusParsed', null);
    }

    translateStatusNumberToHumanReadable() {
        if (typeof this.statusMap[this.radioStatus] != 'undefined') {
            this.radioStatusHumanReadable = this.statusMap[this.radioStatus];

            if (typeof this.statusMappingToStatusShort[this.radioStatus] != 'undefined') {
                this.radioStatusShort = this.statusMappingToStatusShort[this.radioStatus];
            }

            this.emit('statusTranslate_successful', this.radioStatusHumanReadable);
        } else {
            this.emit('statusTranslate_error', null);
        }
    }

    calculateLatLon() {

        if ((this.operation.gky.value.length > 0) && (this.operation.gkx.value.length > 0)) {
            // SDS delivered gauss krueger coordinates
            let wgs84 = gk.toWGS({ x: parseFloat(this.operation.gkx.value), y: parseFloat(this.operation.gky.value) });
            this.operation.lat = wgs84.latitude;
            this.operation.lon = wgs84.longitude;
            this.emit('geocoding_successful', null);
        } else {
            // SDS did not deliver gauss krueger coordinates
            this.operation.lat = 0;
            this.operation.lon = 0;
            this.emit('geocoding_error', null);
        }
    }


    /**
     * PARSER specific methods only needed for this parser
     */


    /**
     * Calculate Keyword parts from keywordRaw of sds
     */
    calculateKeywordsFromRaw() {
        if (typeof this.operation.keywordRaw != 'undefined') {
            if (typeof this.operation.keywordRaw.value != 'undefined') {
                var keywordRaw = '' + this.operation.keywordRaw.value;
                var keywdSplitArr = keywordRaw.split('#');
                if (Array.isArray(keywdSplitArr)) {
                    // seems there is content
                    if (keywdSplitArr.length > 1) {
                        this.operation.keywordId = keywdSplitArr[1];
                    }
                    if (keywdSplitArr.length > 2) {
                        this.operation.keywordCategory = keywdSplitArr[2];
                    }
                    if (keywdSplitArr.length > 3) {
                        this.operation.keywordName = keywdSplitArr[3];
                    }
                }
            }
        }
    }
}

module.exports = ParserERH;