const EventEmitter = require('events');
const gk = require('gauss-krueger');

class ParserERH extends EventEmitter {

    statusMap = {
        '32770' : 'Priorisierter Sprechwunsch',
        '32771' : 'Einsatzbereit über Funk',
        '32772' : 'Einsatzbereit auf Wache',
        '32773' : 'Einsatz übernommen',
        '32774' : 'Am Einsatzort eingetroffen',
        '32775' : 'Sprechwunsch (einsatzbezogen)',
        '32776' : 'Nicht Einsatzbereit',
        '32777' : '7',
        '32778' : '8',
        '32779' : 'Fremdanmeldung'
    }

    constructor() {
        super();
        // Alarm Data
        this.schlagwort_kurz = '';
        this.sds_abschnitt = '';
        this.sds_abschnitt_kurz_tts = '';
        this.sds_alarmdatum= '';
        this.sds_alarmuhrzeit= '';
        this.sds_alarmweg = '';
        this.sds_datum_kurz = '';
        this.sds_einsatzdatum = '';
        this.sds_einsatzmittel = '';
        this.sds_einsatznummer = '';
        this.sds_einsatzort = '';
        this.sds_erh_kbm = '';
        this.sds_freitext = '';
        this.sds_hausnummernzusatz = '';
        this.sds_komplett = '';
        this.sds_objekt = '';
        this.sds_objekt_kurz_tts = '';
        this.sds_ort = '';
        this.sds_ortsteil = '';
        this.sds_ortsteil_kurz = '';
        this.sds_plan = '';
        this.sds_schlagwort = '';
        this.sds_schlagwort_tts_navi = '';
        this.sds_schleifen = '';
        this.sds_stichwort = '';
        this.sds_strasse = '';
        this.sds_strasse_kurz_tts = '';
        this.sds_x = '';
        this.sds_y = '';
        this.lat = null;
        this.lon = null;
        // Status Data
        this.issi = null;
        this.radioStatus = null;
        this.radioStatusHumanReadable = null;
    }

    parseRAWAlarmData(rawAlarmDataArray) {
        this.schlagwort_kurz = rawAlarmDataArray.schlagwort_kurz;
        this.sds_abschnitt = rawAlarmDataArray.sds_abschnitt;
        this.sds_abschnitt_kurz_tts = rawAlarmDataArray.sds_abschnitt_kurz_tts;
        this.sds_alarmdatum = rawAlarmDataArray.sds_alarmdatum;
        this.sds_alarmuhrzeit = rawAlarmDataArray.sds_alarmuhrzeit;
        this.sds_alarmweg = rawAlarmDataArray.sds_alarmweg;
        this.sds_datum_kurz = rawAlarmDataArray.sds_datum_kurz;
        this.sds_einsatzdatum = rawAlarmDataArray.sds_einsatzdatum;
        this.sds_einsatzmittel = rawAlarmDataArray.sds_einsatzmittel;
        this.sds_einsatznummer = rawAlarmDataArray.sds_einsatznummer;
        this.sds_einsatzort = rawAlarmDataArray.sds_einsatzort;
        this.sds_erh_kbm = rawAlarmDataArray.sds_erh_kbm;
        this.sds_freitext = rawAlarmDataArray.sds_freitext;
        this.sds_hausnummernzusatz = rawAlarmDataArray.sds_hausnummernzusatz;
        this.sds_komplett = rawAlarmDataArray.sds_komplett;
        this.sds_objekt = rawAlarmDataArray.sds_komplett;
        this.sds_objekt_kurz_tts = rawAlarmDataArray.sds_objekt_kurz_tts;
        this.sds_ort = rawAlarmDataArray.sds_ort;
        this.sds_ortsteil = rawAlarmDataArray.sds_ortsteil;
        this.sds_ortsteil_kurz = rawAlarmDataArray.sds_ortsteil_kurz;
        this.sds_plan = rawAlarmDataArray.sds_plan;
        this.sds_schlagwort = rawAlarmDataArray.sds_schlagwort;
        this.sds_schlagwort_tts_navi = rawAlarmDataArray.sds_schlagwort_tts_navi;
        this.sds_schleifen = rawAlarmDataArray.sds_schleifen;
        this.sds_stichwort = rawAlarmDataArray.sds_stichwort;
        this.sds_strasse = rawAlarmDataArray.sds_strasse;
        this.sds_strasse_kurz_tts = rawAlarmDataArray.sds_strasse_kurz_tts;
        this.sds_x = rawAlarmDataArray.sds_x;
        this.sds_y = rawAlarmDataArray.sds_y;
        this.calculateLatLon();
        this.emit('alarm_parsed', null);
    }

    parseRAWZVEIData(rawZVEIDataArray) {
        console.log('PARSER-ILSERH | parseRAWZVEIData: ');
        this.zveicode = rawZVEIDataArray.data.code;
        this.zveidescription = rawZVEIDataArray.data.description;
        this.emit('zveiParsed', null);
    }

    parseRAWStatusData(rawStatusDataArray) {
        console.log('PARSER-ILSERH | parseRAWStatusData: ' + rawStatusDataArray);
        this.radioStatus = rawStatusDataArray.data.radioStatus;
        this.issi = rawStatusDataArray.data.issi;
        this.translateStatusNumberToHumanReadable();
        this.emit('statusParsed', null);
    }

    translateStatusNumberToHumanReadable() {
        if (typeof this.statusMap[this.radioStatus] != 'undefined') {
            this.radioStatusHumanReadable = this.statusMap[this.radioStatus];
            this.emit('statusTranslate_successful', this.radioStatusHumanReadable);
        } else {
            this.emit('statusTranslate_error', null);
        }
    }

    calculateLatLon() {
        if ((this.sds_y.length > 0) && (this.sds_x.length > 0)) {
            // SDS delivered gauss krueger coordinates
            console.log('Y: ' + this.sds_y + ' X: ' + this.sds_x);
            let wgs84 = gk.toWGS({x: parseFloat(this.sds_x), y: parseFloat(this.sds_y)});
            this.lat = wgs84.latitude;
            this.lon = wgs84.longitude;
            this.emit('geocoding_successful', null);
        } else {
            // SDS did not deliver gauss krueger coordinates
            this.lat = 0;
            this.lon = 0;
            this.emit('geocoding_error', null);
        }
    }

}

module.exports = ParserERH;