const EventEmitter = require('events');

const admin = require("firebase-admin");
let serviceAccount = require("./alarmiator-oauth.json");
const FCMToken = require('../../../models/fctmtokens');
const Operation = require('../../../models/core/core_operation');
const moment = require("moment");
var uuid = require('uuid'); // required for generating timebased guids
const Assets = require('../../../models/assets');

const PayloadBuilder = require('./payload-builder');

/**
 * 
 * @class Sending new alarms to mobile apps via Firebase Cloud Messaging
 * ALARMiator Zusatzalarmierung
 * 
 * @version: 1.1.0
 * @author:  Stefan Bauer 2020
 */

class alarmiatormobile extends EventEmitter {
    /**
     *
     * @constructor
     */
    constructor() {
        super();
        global.plgManager.logger.info('-------- PLUGIN ALARMIATOR MOBILE -----------');
        //this.plgManager = global.plgManager;
        this.registerForEvents();
        this.serviceProcId = null;
        this.loadConfig((success) => {

        });    
        
        this.alarmiator = null;
        this.alarmInfo = null;
        this.fcmtokens = new FCMToken(global.plgManager.logger);
        this.assets = new Assets(global.plgManager.logger);
        this.payloadBuilder = new PayloadBuilder(global.plgManager.logger);
    }

    /**
     * Registration of Events at the plugin manager (hook in to events provided by the plugin manager)
     */
    registerForEvents() {
        global.plgManager.logger.info('PLUGIN | ALARMIATORMOBILE | Registering events')

        global.plgManager.on('event_pluginConfig_refreshed', (namespace) => {
            if (namespace === 'alarmiatormobile') {
                // plugins config has changed --> refresh
                this.loadConfig((success) => {
                    if (success) {
                        global.plgManager.logger.info('PLUGIN | ALARMIATORMOBILE | successfully reloaded plugin configuration');
                    } else {
                        global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | could not reload plugin config!');
                    }
                });
            }
        })

        global.plgManager.on('event_new_alarm', (alarmInfo) => {

            try {
                if (this.getConfigValue('alarmMethode') === "alarmAll"){
                    this.alarmInfo = alarmInfo;
                    this.setupInstance();
                    this.sendMessage(alarmInfo);
                }
            }      
            catch (error){
                console.error("PLUGIN | ALARMIATORMOBILE | Error caught while sending alarm: " + error.message);
                global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error caught while sending alarm: ' + error.message);
            }

        })

        global.plgManager.on('event_new_alarm_with_rules', (alarmInfo, ruleResult) => {

            try {
                if (this.getConfigValue('alarmMethode') === "ruleBased"){
                    this.alarmInfo = alarmInfo;
                    //this.setupInstance();
                    this.sendAlarmMessageToEveryUser(alarmInfo, ruleResult);
                }
            }      
            catch (error){
                console.error("PLUGIN | ALARMIATORMOBILE | Error caught while sending alarm: " + error.message);
                global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error caught while sending alarm: ' + error.message);
            }

        })

        global.plgManager.on('event_new_test_alarm', (baseDataId, pluginNamespace, alarmInfo) => {
            if (pluginNamespace === "alarmiatormobile"){
                global.plgManager.logger.info('PLUGIN | ALARMIATORMOBILE | EVENT event_new_test_alarm recieved -> processing now');

                try {
                    this.alarmInfo = alarmInfo;
                    this.sendTestAlarm(baseDataId, alarmInfo);
                }      
                catch (error){
                    console.error("PLUGIN | ALARMIATORMOBILE | Error caught while sending alarm: " + error.message);
                    global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error caught while sending alarm: ' + error.message);
                }
            }
        })

        global.plgManager.on('event_new_token_test_alarm', (basedataId, pluginNamespace, token) => {
            if (pluginNamespace === "alarmiatormobile"){
                global.plgManager.logger.info('PLUGIN | ALARMIATORMOBILE | EVENT event_new_token_test_alarm recieved -> processing now');

                try {
                    var tokenList = [];
                    tokenList.push(token);
                    this.sendAlarmMessageToEveryToken(this.generateTestAlarm(), tokenList);
                }      
                catch (error){
                    console.error("PLUGIN | ALARMIATORMOBILE | Error caught while sending alarm: " + error.message);
                    global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error caught while sending alarm: ' + error.message);
                }
            }
        })
    
        // !!!!!!!!! Use at own risk! You can't mute admin notifications yet + its not tested for iOS !!!!!!!!!

        global.plgManager.on('event_new_admin_notification', (notification) => {
            console.log(notification);
            try {
                console.log(notification);
                if (this.getConfigValue('alarmiatorAdminActive') == "true"){
                    this.sendAdminNotifications(notification);
                }
            }      
            catch (error){
                console.error("PLUGIN | ALARMIATORMOBILE | Error caught while sending notification: " + error.message);
                global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error caught while sending notification: ' + error.message);
            }
        })
        
        this.on('event_retry_send_fcm', (messages,fcmKeys,retryCounter) => {
            // TODO: Remove admin notification.. Test only!

            //global.plgManager.event_new_admin_notification('ALARMiator Mobile: Internal Server Error - Retry Versuch #'+retryCounter +" - für "+ messages.length + " Token.");

            const retryLimit = 5; // Maybe plugin setting?
            if (retryCounter < retryLimit){
                var timeOut = Math.pow(2,retryCounter);
                global.plgManager.logger.info('PLUGIN | ALARMIATORMOBILE | EVENT event_retry_send_fcm recieved -> wait for timeout '+timeOut +"s");
                setTimeout(() => {
                    global.plgManager.logger.info('PLUGIN | ALARMIATORMOBILE | Retry #'+ ++retryCounter + ' sending now');
                    this.sendMultipleFCMMessagesWithRetry(messages,fcmKeys, retryCounter);
                }, timeOut * 1000);
            }
            else {
                global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Retry limit reached, discarding messages');
            }
        })

        global.plgManager.on('event_server_config_changed', (emitChangedValuesArray) => {
            global.plgManager.logger.info('PLUGIN | ALARMIATORMOBILE | EVENT event_server_config_changed received -> processing now');

            try {
                emitChangedValuesArray.forEach((configValue) => {
                    if (configValue.keyword == 'externalfqdn' && configValue.changed){
                        // send configValue.newValue;  
                        if (configValue.newValue !== null && configValue.newValue !== undefined){
                            global.plgManager.loadConfigFromStore('restapi', (configRows) => {
                                if (configRows !== null) {
                                    var apiConfig;
                                    apiConfig = configRows[0];
                                    apiConfig = JSON.parse(apiConfig.configstore);

                                    var protocol = null;
                                    var port = null;

                                    apiConfig.fields.forEach((field) => {
                                        switch( field.fieldname){
                                            case "runOnHTTP" : 
                                                if (field.value == true || field.value == 'true') {
                                                    protocol = "http://";
                                                }
                                                break;
                                            case "httpPort": 
                                                port = field.value;
                                                break;
                                            default:
                                                break;
                                        }
                                    });

                                    if (port != null && protocol != null) {
                                        var newAdress = protocol + configValue.newValue + ":" + httpPort; 
                                        this.sendConfigMessages("adress_changed", newAdress)
                                    }  
                                }
                            })
                        }
                    }
                })
            }      
            catch (error){
                console.error("PLUGIN | ALARMIATORMOBILE | Error caught while sending config: " + error.message);
                global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error caught while sending config: ' + error.message);
            }
        })

        global.plgManager.on('event_new_state', (stateInfo) => {
            if (this.getConfigValue('stateActive') == "true"){
                try {
                    this.sendStateMessage(stateInfo);
                }      
                catch (error){
                    console.error("PLUGIN | ALARMIATORMOBILE | Error caught while sending state: " + error.message);
                    global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error caught while sending state: ' + error.message);
                }
            }
        });
    }


    /**
     * Plugin-specific Getters and Setters
     *
    */

    set alarmiatorActive(alarmiatorActive) {
        this._alarmiatorActive = alarmiatorActive;
    }
    get alarmiatorActive() {
        return this._alarmiatorActive;
    }

    /**
     * Plugin-specific implementation
     */

    /**
     * Creates ALARMiator mobile Instance 
     * 
     */
    setupInstance() {
        global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Setup ALARMiator Mobile Instance');

        if (admin.apps.length == 0){
            admin.initializeApp(
                {credential: admin.credential.cert(serviceAccount)}
            )
        }    

    }

    /**
     * Check plugin config, whether sending alarms to ALARMiator is active 
     * 
     * @param {operation} alarmInfo 
     */
    sendMessage(alarmInfo) {
        global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Processing event_new_alarm');

        if (this.getConfigValue('alarmiatorActive') == "true"){
            global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Sending Alarmiator messages');
            this.sendAlarmMessages(alarmInfo);
        }
    }


    /**
     * Sending ALARMiator messages to all active alarmiatormobile tokens 
     * @param {operation} alarmInfo 
     */
    sendAlarmMessages(alarmInfo){
        
        var fcmKeys = [];

        /* Get FCM token from tokenstore*/
        this.fcmtokens.getAllFCMTokensForNamespace("alarmiatormobile", (fcmrows) => {
            if (fcmrows != null){
                fcmrows.forEach((entry)=> {
                    fcmKeys.push(entry.fcmToken);
                })
            }
            var message = this.payloadBuilder.buildPayloadAlarm(alarmInfo, fcmKeys);
            this.sendFCMMessage(message, fcmKeys);
        }); 

    }

    /**
     * Sending ALARMiator config messages to all active alarmiatormobile tokens 
     * @param {string} subtype 
     * @param {string} configValue
     */
    sendConfigMessages(subtype, configValue){
        var fcmKeys = [];
        var messages = [];

        /* Get FCM token from tokenstore*/
        this.fcmtokens.getAllFCMTokensForNamespace("alarmiatormobile", (fcmrows) => {
            if (fcmrows != null){
                fcmrows.forEach((entry)=> {
                    messages.push(this.payloadBuilder.buildConfigPayload(subtype, configValue, entry.fcmToken))
                    fcmKeys.push(entry.fcmToken);
                })

                this.sendMultipleFCMMessages(messages,fcmKeys);

                var debugMessages = this.payloadBuilder.buildConfigPayload(subtype, configValue, "INSERT_TOKEN_HERE")
                // Create Payload for log
                global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------START-----------');
                global.plgManager.logger.debug(JSON.stringify(debugMessages));
                global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------END-------------');
            }

        }); 
    }


    /**
     * Sending ALARMiator messages to all users in ALARMiatorReceivers
     * @param {string} messageText 
     */
    sendAdminNotifications(messageText){
        
        var fcmKeys = [];
        var messages = [];

        var title = "Admin Benachrichtigung für " + global.serverInstanceName;

        /* Get FCM token from tokenstore*/
        this.fcmtokens.getAllFCMTokensForAccessGroup("administration", (fcmrows) => {
            if (fcmrows != null){
                fcmrows.forEach((entry)=> {
                    messages.push(this.payloadBuilder.buildPayloadForSingleMessage("admin", title , messageText, entry.fcmToken));
                    fcmKeys.push(entry.fcmToken);
                })
                this.sendMultipleFCMMessages(messages,fcmKeys);

                var debugMessages = this.payloadBuilder.buildPayloadForSingleMessage("admin", title, messageText, "INSERT_TOKEN_HERE")
                // Create Payload for log
                global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------START-----------');
                global.plgManager.logger.debug(JSON.stringify(debugMessages));
                global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------END-------------');
            }
        }); 

    }

    /**
     * Sending state messages to all users in ALARMiatorReceivers
     * @param {object} stateInfo 
     */
    sendStateMessage(stateInfo){
        var fcmKeys = [];
        var messages = [];

        this.assets.getAssetForISSI(stateInfo.issi, (assetsRows) => {
            if (assetsRows != null) {
                assetsRows.forEach((entry)=> {
                    var name;
                    
                    if (entry.gearname != null && entry.gearname != undefined) {
                        name = entry.title + "\t(" + entry.gearname + ")";
                    }
                    else {
                        name = entry.title;
                    }

                    var stateMessage= name +"\nStatus: " + stateInfo.radioStatusShort + " - " + stateInfo.radioStatusHumanReadable;
                    var title = entry.organizationsName +" - Statuswechsel";
                    /* Get FCM token from tokenstore*/
                    this.fcmtokens.getAllFCMTokensForISSI(stateInfo.issi, (fcmrows) => {
                        if (fcmrows != null){
                            fcmrows.forEach((entry)=> {
                                messages.push(this.payloadBuilder.buildPayloadForSingleMessage("gearState", title, stateMessage, entry.fcmToken))
                                fcmKeys.push(entry.fcmToken);
                            })

                            console.log(fcmKeys);
                            this.sendMultipleFCMMessages(messages,fcmKeys);

                            var debugMessages = this.payloadBuilder.buildPayloadForSingleMessage("gearState", title, stateMessage, "INSERT_TOKEN_HERE")
                            // Create Payload for log
                            global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------START-----------');
                            global.plgManager.logger.debug(JSON.stringify(debugMessages));
                            global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------END-------------');
                        }

                    }); 
                })
            }
       }); 
    }

    /**
     * Send test alarm
     * @param {number} baseDataId 
     * @param {operation} alarmInfo 
     */
    sendTestAlarm(baseDataId, alarmInfo){
        var fcmKeys = [];
        var messages = [];

        /* Get FCM token for User from tokenstore*/
        this.fcmtokens.getAllFCMTokensForBasedataAndNamespace("alarmiatormobile", baseDataId, (fcmrows) => {
            console.log(fcmrows);
            if (fcmrows != null){
                fcmrows.forEach((entry)=> {
                    fcmKeys.push(entry.fcmToken);
                    messages.push(this.payloadBuilder.buildAlarmPayloadForSingleToken(alarmInfo,entry.fcmToken, null));
                })
            }
            this.sendMultipleFCMMessages(messages,fcmKeys);
        }); 

    }

    /**
     * Sending own ALARMiator alarm message to every user
     * @param {operation} alarmInfo 
     * @param {array} ruleResult
     */
    sendAlarmMessageToEveryUser(alarmInfo, ruleResult){
        global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Processing event_new_alarm_with_rules');      
        var fcmKeys = [];
        var messages = [];

        if (ruleResult != null) {
            ruleResult.forEach((alarmedUser) => {
                if (alarmedUser.fcmtoken != undefined && alarmedUser.fcmtoken != null && alarmedUser.fcmtoken != "" && alarmedUser.deviceState == 1) {
                    if (alarmedUser.namespace != null){
                        if (alarmedUser.namespace.includes("alarmiatormobile")){
                            fcmKeys.push(alarmedUser.fcmtoken);
                            messages.push(this.payloadBuilder.buildAlarmPayloadForSingleToken(alarmInfo,alarmedUser.fcmtoken, alarmedUser.organizationname));
                        }
                    }
                }
            })
        }

        this.sendMultipleFCMMessages(messages,fcmKeys);

        var debugMessages = this.payloadBuilder.buildAlarmPayloadForSingleToken(alarmInfo, "INSERT_TOKEN_HERE")
        // Create Payload for log
        global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------START-----------');
        global.plgManager.logger.debug(JSON.stringify(debugMessages));
        global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------END-------------');
    }

    /**
     * Sending own ALARMiator alarm message to every user from tokenList
     * @param {operation} alarmInfo 
     * @param {array} fcmKeys
     */
    sendAlarmMessageToEveryToken(alarmInfo, fcmKeys){
        var messages = [];

        if (fcmKeys != null && fcmKeys.length > 0) {
            fcmKeys.forEach((token) => {
                messages.push(this.payloadBuilder.buildAlarmPayloadForSingleToken(alarmInfo,token));
            })
        }

        this.sendMultipleFCMMessages(messages,fcmKeys);

        var debugMessages = this.payloadBuilder.buildAlarmPayloadForSingleToken(alarmInfo, "INSERT_TOKEN_HERE")
        // Create Payload for log
        global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------START-----------');
        global.plgManager.logger.debug(JSON.stringify(debugMessages));
        global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | Example PAYLOAD for DEBUGGING -------END-------------');
    }

    /**
     * Sending message to tokens
     * @param {string} message 
     * @param {array} fcmKeys 
     */
    sendFCMMessage(message,fcmKeys){
        if (fcmKeys.length > 0){
            admin.messaging().sendMulticast(message)
            .then((response) => {
                // Response is a message ID string.

                const failedTokens = {};
                const successTokens = [];
                response.responses.forEach((resp, idx) => {
                    if (!resp.success) {
                        failedTokens[fcmKeys[idx]] = resp.error.message;
                    }
                    else {
                        successTokens.push(fcmKeys[idx]);
                    }
                });

                global.plgManager.logger.debug("PLUGIN | ALARMIATORMOBILE | List of successfully sent tokens: " + JSON.stringify(successTokens))
                global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | List of tokens that caused failures: ' + JSON.stringify(failedTokens));
                admin.app().delete();
            })
            .catch((error) => {
                global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error sending message: ',JSON.stringify(error));
                admin.app().delete();
        });
        }
        else {
            global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Empty FCM keys array ');
        }
    }

    /**
     * Sending multiple messages
     * @param {array} messages
     * @param {array} fcmKeys 
     */
    sendMultipleFCMMessages(messages,fcmKeys){
        if (fcmKeys.length > 0 && messages.length){
            this.sendMultipleFCMMessagesWithRetry(messages,fcmKeys,0);
        }
        else {
            global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Empty FCM keys or messages array ');
        }
    }


    sendMultipleFCMMessagesWithRetry(messages, fcmKeys, retryCounter){
        if (fcmKeys.length > 0 && messages.length){      
            var app = admin.initializeApp(
                {credential: admin.credential.cert(serviceAccount)}, uuid.v1()
                );      
            app.messaging().sendAll(messages)
            .then((response) => {
                // Response is a message ID string.

                const failedTokens = {};
                const failedTokenList = [];
                const successTokens = [];

                const retryMessages = [];
                const retryToken = [];

                response.responses.forEach((resp, idx) => {
                    if (!resp.success) {
                        failedTokens[fcmKeys[idx]] = resp.error.message;
                        failedTokenList.push(fcmKeys[idx]);
                        switch(resp.error.message) {
                            case "Internal error encountered." : {
                                retryMessages.push(messages[idx]);
                                retryToken.push(fcmKeys[idx]);
                                break;
                            }
                            default: {
                                break;
                            }
                        }
                    }
                    else {
                        successTokens.push(fcmKeys[idx]);
                    }
                });

                if (retryMessages.length > 0){
                    global.plgManager.logger.debug("PLUGIN | ALARMIATORMOBILE | Sending event_retry_send_fcm for "+ retryMessages.length +" messages");
                    this.emit("event_retry_send_fcm",retryMessages, retryToken, retryCounter);  
                }
          
                global.plgManager.event_fcm_sending_report(failedTokenList, successTokens);

                global.plgManager.logger.debug("PLUGIN | ALARMIATORMOBILE | List of successfully sent tokens: " + JSON.stringify(successTokens))
                global.plgManager.logger.debug('PLUGIN | ALARMIATORMOBILE | List of tokens that caused failures: ' + JSON.stringify(failedTokens));
                app.delete();
            })
            .catch((error) => {
                global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Error sending message: ',JSON.stringify(error));
                app.delete();
                global.plgManager.logger.debug("PLUGIN | ALARMIATORMOBILE | Sending event_retry_send_fcm for "+ messages.length +" messages");
                this.emit("event_retry_send_fcm", messages, fcmKeys, retryCounter); 
            });
        }
        else {
            global.plgManager.logger.error('PLUGIN | ALARMIATORMOBILE | Empty FCM keys or messages array ');
        }
    }

    generateTestAlarm(){
        var alarmInfo = new Operation();
        alarmInfo.setUUID();
        alarmInfo.alarmdate = moment().format("DD.MM.YYYY");
        alarmInfo.alarmtime = moment().format("HH:mm:ss");
        alarmInfo.community = 'Musterstadt';
        alarmInfo.district = 'Musterdorf';
        alarmInfo.floor = '1. OG';
        alarmInfo.gear = 'FF Musterstadt\nFL Musterstadt 00-40-01\nFL Musterstadt 00-11-01';
        alarmInfo.gkx = '5483498.63';
        alarmInfo.gky = '3461404.41';
        alarmInfo.lat = 49.5140795;
        alarmInfo.lon = 11.3713787;
        alarmInfo.ils = 'ILS Musterstadt';
        alarmInfo.keywordCategory = 'INF';
        alarmInfo.keywordId = 'T2725';
        alarmInfo.keywordName = 'Probealarm';
        alarmInfo.keywordRaw = '#T2725#INF#Probealarm'
        alarmInfo.location = 'Musterdorf';
        alarmInfo.message = 'Probealarm angefordert durch User. KEIN EINSATZ!';
        alarmInfo.object = 'Rathaus Musterstadt';
        alarmInfo.operationSourceConnector = 'alarmiator';
        alarmInfo.operationState = 1;
        alarmInfo.operationnumber = 'T 0.0 000000 00000';
        alarmInfo.rawdata = '';
        alarmInfo.section = 'Ortsgebiet';
        alarmInfo.street = 'Musterstraße 0';
        alarmInfo.subject = 'INF PROBEALARM';
        alarmInfo.zveis = '12345, 12346, 12347F';
        alarmInfo.isTest = true;
        return alarmInfo;
    }

    /**
     * Loads configuration for ALARMiator plugin
     * 
     */
    loadConfig(callback) {
        global.plgManager.loadConfigFromStore('alarmiatormobile', (configRows) => {
            if (configRows !== null) {
                this.config = configRows[0];
                this.config = JSON.parse(this.config.configstore);
                callback(true);
            }
            global.plgManager.logger.info('PLUGIN | ALARMIATORMOBILE | Configuration reloaded from core database');
        })
    }

    /**
     * returns config parameter for given fieldname
     * @param {string} fieldname 
     */
    getConfigValue(fieldname) {
        var retVal = null;
        this.config.fields.forEach((field) => {
            if (field.fieldname === fieldname) {
                retVal = field.value;
            } 
        });
        return retVal;
    }

}

module.exports = alarmiatormobile;
