const EventEmitter = require('events');
const nodemailer = require('nodemailer');
const Basedata = require('../../../models/basedata');
const Configuration = require('../../../models/configuration');

class sendemail extends EventEmitter {
    constructor() {
        super();
        global.plgManager.logger.info('-------- PLUGIN SENDEMAIL -----------');
        //this.plgManager = global.plgManager;
        this.registerForEvents();
        this.serviceProcId = null;
        this.loadConfig((success) => {
            if (success) {
                global.plgManager.logger.info('PLUGIN | SENDMAIL | successfully loaded Configuration from core database');
                this.initializePlugin((successInit) => {
                    if (successInit) {
                        global.plgManager.logger.info('PLUGIN | SENDMAIL | successfully initialized plugin. Ready to send eMails now.');
                    } else {
                        global.plgManager.logger.error('PLUGIN | SENDMAIL | something went wrong initializing plugin');
                    }
                });
            } else {
                global.plgManager.logger.error('PLUGIN | SENDMAIL | something went wrong loading configuration. Can not send eMails!!');
            }
        });
        this.transporter = null;
        this.configuation = new Configuration(global.plgManager.logger);
    }

    /**
     * Registration of Events at the plugin manager (hook in to events provided by the plugin manager)
     */
    registerForEvents() {
        global.plgManager.logger.info('PLUGIN | SENDEMAIL | Registering events')
        global.plgManager.on('event_send_email', (emailInfo) => {
            global.plgManager.logger.info('PLUGIN | SENDEMAIL | EVENT event_send_email recieved');
            this.initializePlugin((success) => {
                if (success) {
                    this.to = emailInfo.to;
                    this.subject = emailInfo.subject;
                    this.plaintext = emailInfo.plaintext;
                    this.htmltext = emailInfo.htmltext;
                    this.setupTransporter();
                    this.setupMail();
                    this.sendMail();
                }    
            });
        });

        global.plgManager.on('event_pluginConfig_refreshed', (namespace) => {
            if (namespace === 'sendemail') {
                // plugins config has changed --> refresh
                this.loadConfig((success) => {
                    if (success) {
                        global.plgManager.logger.info('PLUGIN | SENDEMAIL | successfully reloaded plugin configuration');
                    } else {
                        global.plgManager.logger.error('PLUGIN | SENDEMAIL | could not reload plugin config!');
                    }
                });
            }
        })

        global.plgManager.on('event_new_alarm', (alarmInfo) => {
            global.plgManager.logger.info('PLUGIN | SENDEMAIL | EVENT event_new_alarm received');

            try {
                if (this.getConfigValue('alarmMethode') === "alarmAll"){
                    this.initializePlugin((success) => {
                        if (success) {
                            this.to = 'jens.dinstuehler@feuerwehr-reichenschwand.de';
                            this.subject = this.subject + ' ALARM - ' + alarmInfo.subject.value;
                            this.plaintext = 'Mitteilung:\n' + alarmInfo.message.value + '\n\nOrt: ' + alarmInfo.location.value + '\n\nEinsatzmittel: ' + alarmInfo.gear.value;
                            this.htmltext = '';
                            this.setupTransporter();
                            this.setupMail();
                            this.sendMail();
                        }
                    });
                }
            }      
            catch (error){
                console.error("PLUGIN | SENDEMAIL | Error caught while sending alarm: " + error.message);
                global.plgManager.logger.error('PLUGIN | SENDEMAIL | Error caught while sending alarm: ' + error.message);
            }
        })

        global.plgManager.on('event_new_alarm_with_rules', (alarmInfo, ruleResult) => {

            try {
                if (this.getConfigValue('alarmMethode') === "ruleBased"){
                    global.plgManager.logger.debug('PLUGIN | SENDEMAIL | Processing event_new_alarm_with_rules');  
                    this.initializePlugin((success) => {
                        if (success) {

                            var receivers = [];    
                            if (ruleResult != null){
                                ruleResult.forEach((alarmedUser) => {
                                    if (alarmedUser.email != undefined && alarmedUser.email != "" && alarmedUser.email != null) {
                                        if (alarmedUser.namespace != null){
                                            if (alarmedUser.namespace.includes("sendemail")){
                                                receivers.push(alarmedUser.email);
                                            }
                                        }
                                    }
                                })
                            }
                    
                            var uniqueReceivers = receivers.filter((value, index, self) => {
                                return self.indexOf(value) === index;
                            });

                            this.to = uniqueReceivers;
                            this.subject = this.subject + ' ALARM - ' + alarmInfo.subject.value;
                            this.plaintext = 'Mitteilung:\n' + alarmInfo.message.value + '\n\nOrt: ' + alarmInfo.location.value + '\n\nEinsatzmittel: ' + alarmInfo.gear.value;
                            this.htmltext = '';
                            this.setupTransporter();
                            this.setupMail();
                            this.sendMail();
                        }
                    });
                }
            }      
            catch (error){
                console.error("PLUGIN | SENDEMAIL | Error caught while sending alarm: " + error.message);
                global.plgManager.logger.error('PLUGIN | SENDEMAIL | Error caught while sending alarm: ' + error.message);
            }

        })

        global.plgManager.on('event_new_test_alarm', (basedataId, pluginNamespace, alarmInfo) => {
            if (pluginNamespace === 'sendemail') {
                global.plgManager.logger.info('PLUGIN | SENDEMAIL | EVENT event_new_test_alarm recieved -> processing now');
                // get eMail-Adress
                var basedata = new Basedata(global.logger);
                basedata.getBaseDataForId(basedataId, (rowsBasedata) => {
                    if (rowsBasedata !== null) {
                        if (rowsBasedata.length > 0 ) {
                            var recipientaddress = rowsBasedata[0].email;
                            if (recipientaddress.length > 0 ) {
                                this.initializePlugin((success) => {
                                    if (success) {
                                        console.log('sending to: ' + recipientaddress);
                                        this.to = recipientaddress;
                                        this.subject = this.subject + ' ALARM - ' + alarmInfo.subject.value;
                                        this.plaintext = 'Mitteilung:\n' + alarmInfo.message.value + '\n\nOrt: ' + alarmInfo.location.value + '\n\nEinsatzmittel: ' + alarmInfo.gear.value;
                                        this.htmltext = '';
                                        this.setupTransporter();
                                        this.setupMail();
                                        this.sendMail();
                                    }
                                });
                            } else {
                                // Member has no email-address set
                            
                                global.plgManager.logger.info('PLUGIN | SENDEMAIL | EVENT event_new_test_alarm recieved -> Member does not have an eMail address set');
                            }
                        } else {
                            // No basedata entry found
                            global.plgManager.logger.info('PLUGIN | SENDEMAIL | EVENT event_new_test_alarm recieved -> Could not find Basedata entry for member');
                        }
                    } else {
                        // Error getting basedata from core database
                        global.plgManager.logger.info('PLUGIN | SENDEMAIL | EVENT event_new_test_alarm recieved -> Could not find Basedata entry for member');
                    }
                })
            } else {
                global.plgManager.logger.info('PLUGIN | SENDEMAIL | EVENT event_new_test_alarm recieved -> NOT FOR this plugin');
            }
        })
    }



    /**
     * Plugin-specific Getters and Setters
     * 
    */


    set from(from) {
        this._from = from;
    }
    get from() {
        return this._from;
    }

    set to(to) {
        this._to = to;
    }
    get to() {
        return this._to;
    }

    set subject(subject) {
        this._subject = subject;
    }
    get subject() {
        return this._subject;
    }

    set plaintext(plaintext) {
        this._plaintext = plaintext;
    }
    get plaintext() {
        return this._plaintext;
    }

    set htmltext(htmltext) {
        this._htmltext = htmltext;
    }
    get htmltext() {
        return this._htmltext;
    }

    set host(host) {
        this._host = host;
    }
    get host() {
        return this._host;
    }

    set port(port) {
        this._port = port;
    }
    get port() {
        return this._port;
    }

    set serverUser(serverUser) {
        this._serverUser = serverUser;
    }
    get serverUser() {
        return this._serverUser;
    }

    set serverPassword(serverPassword) {
        this._serverPassword = serverPassword;
    }
    get serverPassword() {
        return this._serverPassword;
    }

    /**
     * Plugin-specific implementation
     */
    setupTransporter() {
        global.plgManager.logger.debug('PLUGIN | SENDMAIL | Connecting to EMail Server');
        
        
        this.transporter = nodemailer.createTransport({
            host: this.host,    // dinstuehler.eu
            port: this.port,    //465
            secure: true, // upgrade later with STARTTLS
            auth: {
              user: this.serverUser, // "support@alarmiator.de",
              pass: this.serverPassword, // "ThePassword"
            },
            tls: {
                rejectUnauthorized: false
            }
        });
    }

    setupMail() {
        global.plgManager.logger.debug('PLUGIN | SENDMAIL | Composing EMail');
        // setup e-mail data
        this.mailOptions = {
            from: this.from,
            to: this.to,
            subject: this.subject,
            text: this.plaintext,
            html: this.htmltext
        };
        
    }

    sendMail() {
        global.plgManager.logger.debug('PLUGIN | SENDMAIL | Try sending EMail');
        // send mail with defined transport object
        this.transporter.sendMail(this.mailOptions, function (error, info) {
            if (error) {
                global.plgManager.logger.error('PLUGIN | SENDEMAIL | Error sending eMail: ' + error);
            }
            if (typeof info !== 'undefined') {
                global.plgManager.logger.info('PLUGIN | SENDEMAIL | EMail sent: ' + info.response);
                console.log('PLUGIN | SENDEMAIL | EMail sent: ' + info.response);
            }
        }.bind(this));
    
    }

    /**
     * Loads configuration for KatSys Service
     * TBD: Needs to be pulled from plugin config store
     */
    loadConfig(callback) {
        global.plgManager.loadConfigFromStore('sendemail', (configRows) => {
            if (configRows !== null) {
                this.config = configRows[0];
                this.config = JSON.parse(this.config.configstore);
                global.plgManager.logger.info('PLUGIN | SENDEMAIL | Configuration reloaded from core database');
                callback(true);
            } else {
                global.plgManager.logger.info('PLUGIN | SENDEMAIL | Found no configuration reloaded in core database');
                callback(false);
            }
        })
    }

    /**
     * returns config parameter for given fieldname
     * @param {string} fieldname 
     */
    getConfigValue(fieldname) {
        var retVal = null;
        this.config.fields.forEach((field) => {
            if (field.fieldname === fieldname) {
                retVal = field.value;
            } 
        });
        return retVal;
    }

    /**
     * initializes all settings. Returns false, if a needed setting is not valid or not set
     * @param {function} callback success (true/false)
     */
    initializePlugin(callback) {
        this.configuation.getValueForKeyword("emailServerdomain",  (host) => {
            if (host != null) {
                this.configuation.getValueForKeyword("emailServerSenderaddress",  (senderAdress) => {
                    if (senderAdress != null) {
                        this.configuation.getValueForKeyword("emailServerPassword",  (password) => {
                            if (password != null) {
                                this.configuation.getValueForKeyword("emailServerPort",  (port) => {
                                    if (port != null) {
                                        this.from = senderAdress;
                                        this.host = host;
                                        this.port = parseInt(port);
                                        this.serverUser = senderAdress;
                                        this.serverPassword = password;
                                        this.subject =  this.getConfigValue('mailserverTitlePrefix');
                                        return callback(true);
                                    }
                                    else {
                                        global.plgManager.logger.error('PLUGIN | SENDEMAIL | No email port found, check email config')
                                        return callback(false);
                                    }        
                                })
                            }
                            else {
                                global.plgManager.logger.error('PLUGIN | SENDEMAIL | No email password found, check email config')
                                return callback(false);
                            }        
                        })
                    }
                    else {
                        global.plgManager.logger.error('PLUGIN | SENDEMAIL | No email serveradress found, check email config')
                        return callback(false);
                    }        
                })
            } 
            else {
                global.plgManager.logger.error('PLUGIN | SENDEMAIL | No host found, check email config');
                return callback(false);
            }
        })
    }

}

module.exports = sendemail;