const EventEmitter = require('events');

/**
 * Class for KatSysProxy
 * @class plugins/outbound/katsysproxy
 * @namespace plugins/outbound/katsysproxy
 * @memberof plugins/outbound/katsysproxy
  */
class katsysproxy extends EventEmitter {

    /**
     * Constructor
     * 
     * Sets up everything, starts websocket
     */
    constructor() {
        super();
        global.plgManager.logger.info('-------- PLUGIN KatSysProxy -----------');
        this.serviceProcess = null;
        this.registerForEvents();
        this.serviceProcId = null;
        this.loadConfig((success) => {
            if (success) {
                // set plugin as configured
                this.initializePlugin((initsuccess) => {
                    global.plgManager.logger.debug('PLUGIN | KATSYSPROXY | initialized Plugin: ' + initsuccess);
                    if (initsuccess) {
                        this.isConfigured = true;
                        this.prepareService();
                        this.wireServiceCommunication();
                        this.configService();
                        this.startService();
                    }
                })
            }
        })

    }

    /**
     * Registration of Events at the plugin manager (hook in to events provided by the plugin manager)
     */
    registerForEvents() {
        global.plgManager.logger.info('PLUGIN | KATSYSPROXY | Registering events');
        /*
        global.plgManager.on('event_pluginConfig_refreshed', (namespace) => {
            if (namespace === 'wallboard') {
                // plugins config has changed --> refresh
                this.loadConfig((success) => {
                    if (success) {
                        global.plgManager.logger.info('PLUGIN | WALLBOARD | successfully reloaded plugin configuration');
                        this.restartService();
                    } else {
                        global.plgManager.logger.error('PLUGIN | WALLBOARD | could not reload plugin config!');
                    }
                });
            }
        })
        */
    }

    

    /**
     * Loads configuration for KatSysProxy Service
     * @memberof plugins/outbound/katsysproxy
     */
    loadConfig(callback) {
        global.plgManager.loadConfigFromStore('katsysproxy', (configRows) => {
            if (configRows !== null) {
                this.config = configRows[0];
                this.config = JSON.parse(this.config.configstore);
                callback(true);
            } else {
                callback(false);
            }
            global.plgManager.logger.info('PLUGIN | KATSYSPROXY | Configuration loaded from core database');
        })
    }

    /**
     * returns config parameter for given fieldname
     * @param {string} fieldname 
     * @memberof plugins/outbound/wallboard
     */
    getConfigValue(fieldname) {
        var retVal = null;
        this.config.fields.forEach((field) => {
            if (field.fieldname === fieldname) {
                retVal = field.value;
            }
        });
        return retVal;
    }

    /**
     * reconfigures plugin with new config settings
     * @param {function} callback success, true or false
     * @memberof plugins/outbound/wallboard
     */
    initializePlugin(callback) {
        
        this.katSysProxyConfig = {
            masterToken: this.getConfigValue('mastertoken'),
            subToken: this.getConfigValue('subtoken'),
            port: this.getConfigValue('port'),
            loglevel: 'debug'
        }

        if ((typeof this.katSysProxyConfig.masterToken !== 'undefined') && (typeof this.katSysProxyConfig.subToken !== 'undefined') && (typeof this.katSysProxyConfig.port !== 'undefined')) {
            global.plgManager.logger.info('PLUGIN | KATSYSPROXY | Properly inititlized settings');
            callback(true);
        } else {
            global.plgManager.logger.error('PLUGIN | KATSYSPROXY | could not initialize settings');
            callback(false);
        }
    }

    /**
     * Wires messages from service to event model of plugin
     * All Messages in service need to be wired here to be recognised in plugin
     * @memberof plugins/outbound/wallboard
     */
    wireServiceCommunication() {
        this.serviceProcess.on('message', (msg) => {
            console.log('PLUGIN | KATSYSPROXY | Message: ' + msg);
            var payload = null;
            try {
                payload = JSON.parse(msg);
            } catch (err) {
                payload = err.message;
            }
            
            /**
             * check what payload comes in
             */
            if (typeof payload.error != 'undefined') {
                // child has sent an error
                // try restarting the whole service
                // global.plgManager.event_new_admin_notification('KATSYSPROXY Service meldet einen Fehler');
                console.log(payload.error);
                //this.restartService();
            } else if (typeof payload.logmessage != 'undefined') {
                // LOGMESSAGE COMING FROM SERVICE
                console.log('PLUGIN | KATSYSPROXY | SERVICE | ' + payload.logmessage);
                global.plgManager.logger.info('PLUGIN | KATSYSPROXY | SERVICE | ' + payload.logmessage);
            } else if (typeof payload.newConnection != 'undefined') {
                // NEW CLIENT CONNECTED TO WEBSOCKET
                console.log('PLUGIN | KATSYSPROXY | SERVICE | NEW CLIENT CONNECTION');
                global.plgManager.logger.info('PLUGIN | KATSYSPROXY | SERVICE | NEW CLIENT CONNECTION');
            } else if (typeof payload.lostConnection != 'undefined') {
                // LOST A CLIENT CONNECTED TO WEBSOCKET
                console.log('PLUGIN | KATSYSPROXY | SERVICE | LOST CLIENT CONNECTION');
                global.plgManager.logger.info('PLUGIN | KATSYSPROXY | SERVICE | LOST CLIENT CONNECTION');
            } else {
                console.log('PLUGIN | KATSYSPROXY | SERVICE | UNSOLICITED MESSAGE: ' + msg);
                global.plgManager.logger.info('PLUGIN | KATSYSPROXY | SERVICE | UNSOLICITED MESSAGE: ' + msg);
            }
        });
    }

    /**
     * Registration of Events at the plugin manager (hook in to events provided by the plugin manager)
     */
    registerForEvents() {
        global.plgManager.logger.info('PLUGIN | KATSYSPROXY | Registering events')
               
        global.plgManager.on('event_katsys_incoming_data', (data) => {
            global.plgManager.logger.info('PLUGIN | KATSYSPROXY | EVENT event_katsys_incoming_data received');
            this.informatWebsocketAboutNewData(data);
        })
    }

    /**
     * Creates the service as a seperate node-process (fork)
     * sets the serviceProcId of the forked process
     * @memberof plugins/outbound/wallbord
     */
    prepareService() {
        global.plgManager.logger.info('PLUGIN | KATSYSPROXY | Starting Service');
        const { fork } = require('child_process');
        this.serviceProcess = fork(__dirname + '/service.js');
        this.serviceProcId = this.serviceProcess.pid;
        this.serviceProcess.title = 'ALARMiator Plugin - KATSYSPROXY Connector';
        global.plgManager.logger.info('PLUGIN | KATSYSPROXY | WebSocket - Service started with pid ' + this.serviceProcId);
        this.serviceProcess.on('exit', function (code) {
            this.serviceProcess = null;
            console.log("KATSYSPROXY Service EXITED " + code);
        });
    }

    /**
     * Sends the service configuration to the process
     */
    configService() {
        this.serviceProcess.send({ config: this.katSysProxyConfig });
    }

    /**
     * tells the service to connect
     */
    startService() {
        this.serviceProcess.send({ start: 1 });
    }

    /**
     * restarts the Plugin Service
     */
    restartService() {
        if (this.serviceProcess !== null) {
            //global.plgManager.logger.info('PLUGIN | KATSYSPROXY | Killing Service...');
            this.serviceProcess.kill("SIGHUP");
        }
        this.initializePlugin((initsuccess) => {
            //global.plgManager.logger.debug('PLUGIN | KATSYSPROXY | initialized Plugin: ' + initsuccess);
            if (initsuccess) {
                this.isConfigured = true;
                this.prepareService();
                this.wireServiceCommunication();
                this.configService();
                this.startService();
            }
        })
    }

    /**
     * Methods to control wallboard clients
     * 
     * Methods send event to service. WebSocket then loads corresponding data and
     * sends this to the clients.
     * 
     * Responsibility for refreshing of needed data is at the websocket implementation.
     * The following methods shall only signal the websocket to do something and no payload is transmitted
     * 
     */
    informatWebsocketAboutNewData(data) {
        this.serviceProcess.send(JSON.stringify({ event_katsys_incoming_data : data }));
    }


}

module.exports = katsysproxy;