const EventEmitter = require("events");
const nodemailer = require('nodemailer');

class maildaemon extends EventEmitter {
    constructor(logger) {
        super();
        this.logger = logger;
        this.transporter = null;
        this.mailOptions = null;
        this.queue = [];
    }

    set host(host) {
        this._host = host;
    }
    get host() {
        return this._host;
    }

    set port(port) {
        this._port = port;
    }
    get port() {
        return this._port;
    }

    set subject(subject){
        this._subject = subject;
    }
    get subject(){
        return this._subject;
    }

    set senderemail(senderemail) {
        this._senderemail = senderemail;
    }
    get senderemail() {
        return this._senderemail;
    }

    set from(from) {
        this._from = from;
    }
    get from() {
        return this._from;
    }

    set password(password) {
        this._password = password;
    }
    get password(){
        return this._password;
    }

    set htmlBody(htmlBody) {
        this._htmlBody = htmlBody;
    }
    get htmlBody() {
        return this._htmlBody;
    }

    set plainBody(plainBody) {
        this._plainBody = plainBody;
    }
    get plainBody() {
        return this._plainBody;
    }

    set recipients(recipients) {
        this._recipients = recipients;
    }
    get recipients(){
        return this._recipients;
    }

    set emailQueueId(emailQueueId) {
        this._emailQueueId = emailQueueId;
    }
    get emailQueueId() {
        return this._emailQueueId;
    }

    initialize() {
        this.setupTransporter();
    }

    loadRecipientsFromJSON(jsonRecipients) {
        let list = '';
        let jsonObj = JSON.parse(jsonRecipients);
        if (jsonObj.length > 0) {
            list = jsonObj.join(',');
        }
        this.recipients = list;
    }

    addToQueue(to, subject, html, plain) {
        let email = {
            from: this.senderemail,
            to: to, 
            subject: subject,
            html: html, 
            text: plain
        }
        this.queue.push(email);
    }

    cleanQueue() {
        this.queue = [];
    }



    /**
     * Setup the Transporter
     */
    setupTransporter() {
        this.logger.info('PLUGIN | MAILSENDER | Setting up Transporter');
        this.transporter = nodemailer.createTransport({
            host: this.host,    // dinstuehler.eu
            port: this.port,    //465
            secure: false,      // upgrade later with STARTTLS
            auth: {
                user: this.senderemail, // "support@alarmiator.de",
                pass: this.password,    // "ThePassword"
            },
            tls: {
                rejectUnauthorized: false
            }
        });
    }

    /**
     * Composes an email
     */
    composeEmail() {
        this.logger.info('PLUGIN | MAILSENDER | composing EMail');
        
        // setup e-mail data
        this.mailOptions = {
            from: this.senderemail,
            to: this.recipients,
            subject: this.subject,
            text: this.plainBody,
            html: this.htmlBody,
            queueId: this.emailQueueId
        };
    }

    /**
     * Sends email
     */
    sendEmail(callback) {
        this.logger.info('PLUGIN | MAILSENDER | Try sending EMail');
        // send mail with defined transport object
        this.transporter.sendMail(this.mailOptions, function (error, info) {
            if (error) {
                global.logger.info('PLUGIN | MAILSENDER | Error sending eMail: ' + error);
                this.cleanAfterSending();
                callback(false, error.message);
            }
            if (typeof info !== 'undefined') {
                global.logger.info('PLUGIN | MAILSENDER | EMail sent: ' + info.response);
                console.log('PLUGIN | MAILSENDER | EMail sent: ' + info.response);
                this.cleanAfterSending();
                callback(true, null);
            }
        }.bind(this));
    }
    
    /**
     * cleans all relevant properties after sending email
     */
    cleanAfterSending() {
        this.htmlBody = '';
        this.plainBody = '';
        this.recipients = null;
    }
}

module.exports = maildaemon;