const request = require('request');

class sms77ioController {

    constructor(logger) {
        this._logger = logger;
        this._apikey = ''; // default value for api key
        this.debugmode = 1; // default value for debug mode
    }

    set apikey(key) {
        this._apikey = key;
    }

    get apikey() {
        return this._apikey;
    }

    checkSMS77credits = function (callback) {
        // Helper Lösung für die Abfrage des Guthabens bei SMS77.io
        // zur Umgehung des CORS Problems in Browsern
        var url = 'https://gateway.sms77.io/api/balance?p=' + this._apikey;
        request(url, (err, res2, body) => {
            if (err) {
                this._logger.log('PLUGIN | SMS77IO | Error checking credits at SMS77io: ' + err.message);
                callback(false);
            }
            if (body === "900") {
                this._logger.log('PLUGIN | SMS77IO | Wrong API Key. Credits could not be checked.');
                callback(false);
            } else {
                this._logger.log(`PLUGIN | SMS77IO | Credits check successful. You have ${body} EUR in total left for sending SMS messages.`);
                callback(body);
            }
        });
    }

    sendSMS(text, senderNumberOrName, recievers, callback) {
        var url = 'https://gateway.sms77.io/api/sms?p=' + this._apikey;

        var recieverlist = '';
        var separator = '';
        if (Array.isArray(recievers)) {
            recievers.forEach((reciever, index) => {
                if (index > 0) {
                    separator = ',';
                }
                recieverlist +=  separator + reciever;
            })
        } else {
            recieverlist = recievers;
        }

        url += '&to=' + encodeURIComponent(recieverlist) + '&text=' + encodeURIComponent(text) + "&from=" + encodeURIComponent(senderNumberOrName);

        request(url, (err, res2, body) => {
            if (err) {
                callback(false);
                this._logger.log('PLUGIN | SMS77IO | Error sending SMS: ' + err.message);
            }
            if (body === "900") {
                this._logger.log('PLUGIN | SMS77IO | Wrong API Key. SMS has not been sent.');
                callback(false);
            } else {
                this._logger.log('PLUGIN | SMS77IO | successfully sents SMS');
                callback(true);
            }
        });

    }
}

// export the class
module.exports = sms77ioController;




