process.env.NTBA_FIX_319 = 1;

const EventEmitter = require('events');
const TelegramBot = require('node-telegram-bot-api');
const Basedata = require('../../../models/basedata');
const Assets = require('../../../models/assets');
const moment = require("moment");
const OperationsHelper = require('../../../models/operationshelper');
const Organiaztions = require('../../../models/organizations');

class telegram extends EventEmitter {
    constructor() {
        super();
        global.plgManager.logger.info('-------- PLUGIN TELEGRAM -----------');
        //this.plgManager = global.plgManager;
        this.registerForEvents();
        this.serviceProcId = null;
        this.loadConfig((success) => {

        });
        this.bot = null;
        this.basedata = new Basedata(global.plgManager.logger);
        this.assets = new Assets(global.plgManager.logger);
        this.operationsHelper = new OperationsHelper(global.plgManager.logger);
        this.organizations = new Organiaztions(global.plgManager.logger);
    }

    /**
     * Registration of Events at the plugin manager (hook in to events provided by the plugin manager)
     */
    registerForEvents() {
        global.plgManager.logger.info('PLUGIN | TELEGRAM | Registering events')

        global.plgManager.on('event_pluginConfig_refreshed', (namespace) => {
            if (namespace === 'telegram') {
                // plugins config has changed --> refresh
                this.loadConfig((success) => {
                    if (success) {
                        global.plgManager.logger.info('PLUGIN | TELEGRAM | successfully reloaded plugin configuration');
                    } else {
                        global.plgManager.logger.error('PLUGIN | TELEGRAM | could not reload plugin config!');
                    }
                });
            }
        })

        global.plgManager.on('event_new_admin_notification', function (notification) {
            if (this.getConfigValue('adminChatActive') == "true"){
                try {
                    this.sendAdminMessage(notification);
                }      
                catch (error){
                    console.error("PLUGIN | TELEGRAM | Error caught while sending admin notification: " + error.message);
                    global.plgManager.logger.error('PLUGIN | TELEGRAM | Error caught while sending admin notification: ' + error.message);
                }
            }
        }.bind(this));

        global.plgManager.on('event_new_feedback_received', (operationUUID, basedataUUID, stateId) => {
            if (this.getConfigValue('telegramFeedbackActive') == "true"){
                try {
                    this.sendFeedbackMessage(operationUUID, basedataUUID, stateId);
                }      
                catch (error){
                    console.error("PLUGIN | TELEGRAM | Error caught while sending feedback: " + error.message);
                    global.plgManager.logger.error('PLUGIN | TELEGRAM | Error caught while sending feedback: ' + error.message);
                }
            }
        });

        global.plgManager.on('event_new_alarm', function (alarmInfo) {   
            if (this.getConfigValue('telegramAlarmActive') == "true"){  
                try {
                    this.sendAlarm(alarmInfo);
                }      
                catch (error){
                    console.error("PLUGIN | TELEGRAM | Error caught while sending alarm: " + error.message);
                    global.plgManager.logger.error('PLUGIN | TELEGRAM | Error caught while sending alarm: ' + error.message);
                }
            }

            if (this.getConfigValue('telegramStaticAlarmActive') == "true"){  
                try {
                    this.sendStaticAlarmMessages(alarmInfo);
                }      
                catch (error){
                    console.error("PLUGIN | TELEGRAM | Error caught while sending alarm: " + error.message);
                    global.plgManager.logger.error('PLUGIN | TELEGRAM | Error caught while sending alarm: ' + error.message);
                }
            }
        }.bind(this));

        global.plgManager.on('event_new_state', function (stateInfo) {
            if (this.getConfigValue('telegramStateActive') == "true"){
                try {
                    this.sendState(stateInfo);
                }      
                catch (error){
                    console.error("PLUGIN | TELEGRAM | Error caught while sending state: " + error.message);
                    global.plgManager.logger.error('PLUGIN | TELEGRAM | Error caught while sending state: ' + error.message);
                }
            }
        }.bind(this));

    }


    /**
     * Plugin-specific Getters and Setters
     * 
    */
    set chatId(chatId) {
        this._chatId = chatId;
    }
    get chatId() {
        return this._chatId;
    }

    set message(message) {
        this._message = message;
    }
    get message() {
        return this._message;
    }

    /**
     * Plugin-specific implementation
     */

    /**
     * Setup TelegramBot object with API-token from config store 
     * 
     */
    setupBot() {
        global.plgManager.logger.debug('PLUGIN | TELEGRAM | Setup Telegram Bot');
        const botToken = this.getConfigValue('botToken');
        this.bot = new TelegramBot(botToken, {polling: false});
    }

    /**
     * returns user chatIds as array from configStore 
     * 
     */
    getUserArray(){
        var userArrayString = this.getConfigValue('chatIds');
        var userArray = [];

        // check if string is empty
        if (userArrayString){
            userArray = userArrayString.split(",").map(x => x.trim()).filter(x => x); // filter removes empty results
        }
        return userArray;
    }
    
    /**
     * returns admin chatIds as array from configStore 
     * 
     */
    getAdminArray(){
        var adminArrayString = this.getConfigValue('adminChatIds');
        var adminArray = [];

        // check if string is empty
        if (adminArrayString){
            adminArray = adminArrayString.split(",").map(x => x.trim()).filter(x => x); // filter removes empty results
        }
        return adminArray;
    }

    /**
     * returns static chatIds as array from configStore 
     * 
     */
    getStaticAlarmArray(){
        var staticArrayString = this.getConfigValue('chatIdsStatic');
        var staticArray = [];

        // check if string is empty
        if (staticArrayString){
            staticArray = staticArrayString.split(",").map(x => x.trim()).filter(x => x); // filter removes empty results
        }
        return staticArray;
    }

    getStateArray(){
        var stateArrayString = this.getConfigValue('stateChatIds');
        var stateArray = [];

        // check if string is empty
        if (stateArrayString){
            stateArray = stateArrayString.split(",").map(x => x.trim()).filter(x => x); // filter removes empty results
        }
        return stateArray;
    }

    /**
     * returns feedback chatIds as array from configStore 
     * 
     */
    getFeedbackArray(){
        var feedbackArrayString = this.getConfigValue('feedbackChatIds');
        var feedbackArray = [];

        // check if string is empty
        if (feedbackArrayString){
            feedbackArray = feedbackArrayString.split(",").map(x => x.trim()).filter(x => x); // filter removes empty results
        }
        return feedbackArray;
    }

     /**
     * Send a message to each chatId in chatIds array
     * @param {array} chatIds 
     * @param {string} message 
     */
    sendMessages(chatIds, message) {
        global.plgManager.logger.debug('PLUGIN | TELEGRAM | Sending '+ chatIds.length +' Telegram messages');
        chatIds.forEach(chatId => {
            try {

                this.bot.sendMessage(chatId, message, { parse_mode: 'HTML', disable_web_page_preview: true })
                .then((info) => {

                    global.plgManager.logger.info('PLUGIN | TELEGRAM | Message to ' + JSON.stringify(info.chat) +' sent successfully');
                    console.log('PLUGIN | TELEGRAM | Message to ' + JSON.stringify(info.chat) +' sent successfully');
                   },

                   (error) => {

                    global.plgManager.logger.error('PLUGIN | TELEGRAM | Error sending message: ' + error);
                    console.error(error);
                 });
            }
            catch(error) {
                console.error("PLUGIN | TELEGRAM | Error caught while sending message: " + error);
                global.plgManager.logger.error('PLUGIN | TELEGRAM | Error caught while sending message: ' + error);
            }

            
        });
    }

    /**
     * Sends message to admins
     * @param {string} notification 
     */
    sendAdminMessage(notification){
        var chatIds = this.getAdminArray();
        
        this.setupBot();
        this.sendMessages(chatIds, this.encodeStringForHTML(notification));
    }

    /**
     * Sends static alarm message 
     * 
     */
    sendStaticAlarmMessages(alarmInfo){
        var chatIds = this.getStaticAlarmArray();
        this.setupBot();
        var message;

        var zveis = alarmInfo.zveisCalc.value != null ? alarmInfo.zveisCalc.value : [];
        this.organizations.getListOfOrganizations((organizationRows) => {

            if (organizationRows != null) {
                var organizationNames = [];
                organizationRows.forEach((organization) => {
                    let orgZveisStr = organization.zveis;
                    console.log(orgZveisStr);
                    if (orgZveisStr != null) {
                        let orgZveisArr = orgZveisStr.split(',');
                        if (orgZveisArr.length > 0) {
                            //console.log('found zveis for organization');
                            orgZveisArr.forEach((zvei) => {
                                if (zveis.length > 0) {
                                    //console.log('cycle through zveis');
                                    zveis.forEach((zveiOperation) => {
                                        //console.log('cycle through operation zveis');
                                        if (zveiOperation.trim() === zvei.trim()) {
                                            // we have a match
                                            organizationNames.push(organization.name);
                                        }
                                    })
                                }
                            })
                        }
                    }
                })
                var uniqueArray = organizationNames.filter(function (elem, pos) {
                    return organizationNames.indexOf(elem) == pos;
                })

                if (uniqueArray.length > 0) {
                    message = "Einsatz für " + uniqueArray.join(", ");
                }
                else {
                    if (this.getConfigValue('staticAlarmText') == null || this.getConfigValue('staticAlarmText').trim() == "") {
                        return;
                    }
                    message = this.getConfigValue('staticAlarmText');
                }
                //console.log(message);
                this.sendMessages(chatIds, this.encodeStringForHTML(moment().format("DD.MM.YYYY, HH:mm:ss") + " - " + message.trim()));
            }
            else {
                if (this.getConfigValue('staticAlarmText') == null || this.getConfigValue('staticAlarmText').trim() == "") {
                    return;
                }
                message = this.getConfigValue('staticAlarmText'); 
                this.sendMessages(chatIds, this.encodeStringForHTML(moment().format("DD.MM.YYYY, HH:mm:ss") + " - " + message.trim()));
            }
        })
    }

    /**
     * Sends message to admins
     * @param {string} operationUUID 
     * @param {string} basedataUUID 
     * @param {integer} stateId 
     */
    sendFeedbackMessage(operationUUID, basedataUUID, stateId){
        var chatIds = this.getFeedbackArray();
        
        var stateLabel = ""
        switch (stateId) {
            case 0:
                stateLabel = "KEINE RÜCKMELDUNG";
                break;
            case 1:
                stateLabel = "KOMME";
                break;
            case 2:
                stateLabel = "KOMME NICHT";
                break;
            default:
                stateLabel = "Status nicht definiert!";
                break; 
        }
        this.setupBot();

        this.basedata.getBaseDataForUUID(basedataUUID, (basedataRows) => {
            if (basedataRows != null){
                basedataRows.forEach((entry)=> {

                    var message = "Rückmeldung \'" + stateLabel + "\' erhalten von "+ entry.lastname + " " + entry.firstname;
                    this.sendMessages(chatIds, this.encodeStringForHTML(message));

                    if (entry.organizationid != null) {
                        var orgIds = [];
                        orgIds.push(entry.organizationid);
                        this.sendMessageToOrgas(this.encodeStringForHTML(message),orgIds);
                    }
               })
           }
       }); 
    }

    /**
     * Creates message for a alarm object and sends message to users
     * @param {object} alarmInfo 
     */
    sendAlarm(alarmInfo){
        var chatIds = this.getUserArray();

        var alarmMessage= "";
        alarmMessage+="\uD83D\uDEA8 EINSATZ \uD83D\uDEA8";
        alarmMessage += "\n\ud83d\udcdf " + this.encodeStringForHTML(alarmInfo.subject.value) + " - " + this.encodeStringForHTML(alarmInfo.keywordCategory.value) + " - " + this.encodeStringForHTML(alarmInfo.keywordName.value);
        alarmMessage+="\n\n";
        alarmMessage+="## BEMERKUNG ##";
        alarmMessage += "\n\ud83d\udcc4 " + this.encodeStringForHTML(alarmInfo.message.value);
        alarmMessage+="\n\n";
        alarmMessage+="## EINSATZSTELLE ##";
        alarmMessage += "\n\uD83D\uDCCD ORT: " + this.encodeStringForHTML(alarmInfo.location.value);
        alarmMessage += "\n\uD83D\uDEE3\uFE0F STRASSE: " + this.encodeStringForHTML(alarmInfo.street.value);
        alarmMessage += "\n\uD83C\uDFE2 ETAGE: " + this.encodeStringForHTML(alarmInfo.floor.value);
        alarmMessage += "\n\uD83C\uDFE0 OBJEKT: " + this.encodeStringForHTML(alarmInfo.object.value);
        alarmMessage += "\n\uD83D\uDEA7 ABSCHNITT: " + this.encodeStringForHTML(alarmInfo.section.value);
        alarmMessage += "\n\uD83C\uDF0D MAPS: <a href='https://www.google.com/maps/search/?api=1&query=" + alarmInfo.lat.value + "," + alarmInfo.lon.value +"'>Google Maps</a> | ";
        alarmMessage += "<a href='http://openfiremap.org/?zoom=17&lat=" + alarmInfo.lat.value + "&lon=" + alarmInfo.lon.value +"'>Openfiremap</a>";
        alarmMessage+="\n\n";
        alarmMessage += "\uD83D\uDE92 EINSATZMITTEL:\n" + this.encodeStringForHTML(alarmInfo.gear.value);
        alarmMessage+="\n\n";
        alarmMessage+="## ALARMIERTE SCHLEIFEN ##";
        alarmMessage += "\n" + this.encodeStringForHTML(alarmInfo.zveis.value);
        alarmMessage+="\n\n";
        alarmMessage+="## SONSTIGES ##";
        alarmMessage += "\n\uD83D\uDCE1 LEITSTELLE: " + this.encodeStringForHTML(alarmInfo.ils.value);
        alarmMessage += "\n\u23f0 ALARMZEIT: " + this.encodeStringForHTML(alarmInfo.alarmdate.value) + " " + this.encodeStringForHTML(alarmInfo.alarmtime.value);
        alarmMessage += "\n\uD83D\uDCF3 VERSENDET: "+moment().format("DD.MM.YYYY, HH:mm:ss");
        this.setupBot();
        this.sendMessages(chatIds, alarmMessage);


        /* Orga specific */
        var zveis = alarmInfo.zveisCalc.value != null ? alarmInfo.zveisCalc.value : [];
        this.organizations.getListOfOrganizations((organizationRows) => {

            if (organizationRows != null) {
                var organizationIds = [];
                organizationRows.forEach((organization) => {
                    let orgZveisStr = organization.zveis;
                    console.log(orgZveisStr);
                    if (orgZveisStr != null) {
                        let orgZveisArr = orgZveisStr.split(',');
                        if (orgZveisArr.length > 0) {
                            //console.log('found zveis for organization');
                            orgZveisArr.forEach((zvei) => {
                                if (zveis.length > 0) {
                                    //console.log('cycle through zveis');
                                    zveis.forEach((zveiOperation) => {
                                        //console.log('cycle through operation zveis');
                                        if (zveiOperation.trim() === zvei.trim()) {
                                            // we have a match
                                            organizationIds.push(organization.id);
                                        }
                                    })
                                }
                            })
                        }
                    }
                })
                var orgIds = organizationIds.filter(function (elem, pos) {
                    return organizationIds.indexOf(elem) == pos;
                })

                /* Send alarm to each organization */
                this.sendMessageToOrgas(alarmMessage,orgIds);
            }
        })
    }

    /**
     * Creates message for a state object and sends message to users
     * @param {object} stateInfo 
     */
    sendState(stateInfo){
        var chatIds = this.getStateArray();

        this.setupBot();

        this.assets.getAssetForISSI(stateInfo.issi, (assetsRows) => {
            if (assetsRows != null) {
                if (assetsRows.length > 0){
                    assetsRows.forEach((entry)=> {
                        var name;
                        
                        if (entry.gearname != null && entry.gearname != undefined) {
                            name = entry.gearname;
                        }
                        else {
                            name = entry.title;
                        }

                        var stateMessage= name +" Status: " + stateInfo.radioStatusShort + " - " + stateInfo.radioStatusHumanReadable;
                        this.sendMessages(chatIds, this.encodeStringForHTML(stateMessage));

                        var orgIds = [];
                        orgIds.push(entry.ownerOrgId);
                        this.sendMessageToOrgas(this.encodeStringForHTML(stateMessage),orgIds);
                   })
                }
                else {
                    var stateMessage= "ISSI: "+stateInfo.issi +" Status: " + stateInfo.radioStatusShort + " - " + stateInfo.radioStatusHumanReadable;
                    this.sendMessages(chatIds, this.encodeStringForHTML(stateMessage))
                }
            }
       }); 



        //this.sendMessages(chatIds, this.encodeStringForHTML(stateMessage));
    }

    /**
     * Creates message for a state object and sends message to users
     * @param {string} message 
     */
    encodeStringForHTML(message){
        if (message != null && message != undefined){
            message = message.replace(/&/gm,"&amp;");
            message = message.replace(/</gm,"&lt;");
            message = message.replace(/>/gm,"&gt;");
            message = message.replace(/\"/gm,"&quot;");
        }
        else {
            message = "";
        }
      
        return message;
    }


    /**
     * Sends organization related Message to organizations 
     * @param {string} message
     * @param {Array} orgaIdArray 
     */
    sendMessageToOrgas(message, orgaIdArray) {

        const orgaMapString = this.getConfigValue("orgaMapping");

        if (orgaMapString == null) {
            global.plgManager.logger.error('PLUGIN | TELEGRAM | There are no organization specific chatIds, orgaMapping field not configured');
            console.log('PLUGIN | TELEGRAM | There are no organization specific chatIds, orgaMapping field not configured')
            return;
        } 

        const orgaJsonMap = JSON.parse(orgaMapString);

        if (orgaJsonMap.length == 0) {
            global.plgManager.logger.error('PLUGIN | TELEGRAM | There are no organization specific chatIds, array is empty');
            console.log('PLUGIN | TELEGRAM | There are no organization specific chatIds, array is empty')
            return;   
        }

        const chatIds = orgaJsonMap.filter(orga => orgaIdArray.includes(parseInt(orga.key)))
                                    .map(orga => orga.value);

        if (chatIds.length > 0) {
            this.sendMessages(chatIds, message);

        }
        else {
            global.plgManager.logger.debug('PLUGIN | TELEGRAM | No assigned chatids for organization found');
            console.log('PLUGIN | TELEGRAM | No assigned chatids for organization found');
        }
    }

    /**
     * Loads configuration for Telegram plugin
     * TBD: Needs to be pulled from plugin config store
     */
    loadConfig(callback) {
        global.plgManager.loadConfigFromStore('telegram', (configRows) => {
            if (configRows !== null) {
                this.config = configRows[0];
                this.config = JSON.parse(this.config.configstore);
                callback(true);
            }
            global.plgManager.logger.info('PLUGIN | TELEGRAM | Configuration reloaded from core database');
        })
    }

    /**
     * returns config parameter for given fieldname
     * @param {string} fieldname 
     */
    getConfigValue(fieldname) {
        var retVal = null;
        this.config.fields.forEach((field) => {
            if (field.fieldname === fieldname) {
                retVal = field.value;
            } 
        });
        return retVal;
    }

}

module.exports = telegram;