var cycleInterval = 15000;

// WebSocket
var ws = null;
var wsConnectionLostState = false;

var operationPanelState = false;
var statusChanged = false;

// Count of active operations reported by server
var countAciveOperations = 0;
var operationsStore = [];
var operationsStoreDetails = [];
var currentUIOperationNumber = 0;
var lastUIOperationNumber = 0;
var currentUIOperation = [];
var UICylceID = 0;
var UICycleRunning = false;

// Progressbar Variables
var progressbarStep = 0;
var progressbarCycleRunning = false;
var progressCycleID = 0;




/**
 * Function checks if two arrays contain the same values.
 *
 * @alias wallboard~arraysEqual
 */
function arraysEqual(_arr1, _arr2) {
    if (!Array.isArray(_arr1) || ! Array.isArray(_arr2) || _arr1.length !== _arr2.length)
      return false;

    var arr1 = _arr1.concat().sort();
    var arr2 = _arr2.concat().sort();

    for (var i = 0; i < arr1.length; i++) {

        if (arr1[i] !== arr2[i])
            return false;

    }
    return true;
}


/**
 * Function switches the operationPanel On.
 *
 * @alias wallboard~operationPanelOn
 */
function operationPanelOn() {
    if (statusChanged === false) {
        if (operationPanelState === false) {
            statusChanged = true;
            document.getElementById("operationOverlay").style.display = "block";
            document.getElementById("noOperationOverlay").style.display = "none";
            operationPanelState = true;
        }
    }
}


/**
 * Function switches the operationPanel Off.
 *
 * @alias wallboard~operationPanelOff
 */
function operationPanelOff() {
    var statusChanged = true;
    document.getElementById("operationOverlay").style.display = "none";
    document.getElementById("noOperationOverlay").style.display = "block";
    operationPanelState = false;
}

/**
 * Function calculates the time since timestamp, used for 
 * calculation of time since incoming alarm.
 * 
 * @returns string (00:10:14)
 *
 * @param timestamp as String
 * @alias timeSinceOperationIncoming
 */
function timeSinceOperationIncoming(timestamp) {
    var ms = moment().diff(moment(timestamp));
    var d = moment.duration(ms);
    var s = Math.floor(d.asHours()) + moment.utc(ms).format(":mm:ss");
    return s;
}


/**
 * Function writes operation information to the ui fields.
 * 
 * @returns nothing
 * @param operationdata as Array of operationdata
 * @alias wallboard~renderOperationDataToUI
 */
function renderOperationDataToUI(operationdata) {
    //console.log('Status Changed: ' + statusChanged);
    console.log('Operation: ' + operationdata.id);
    //if (statusChanged === true) {
        
        if (lastUIOperationNumber != operationdata.id) {
            // initialize Map
            initializeGoogleMap(operationdata.einsatzortlat, operationdata.einsatzortlon, operationdata.homelatitude, operationdata.homelongitude);
            clearPoints();
            setArcGisHomePoint(operationdata.homelatitude, operationdata.homelongitude);
            setArcGisOperationPoint(operationdata.einsatzortlat, operationdata.einsatzortlon);
        }

        // Render Timestamps
        operationTimestampIncome = moment(operationdata.timestampincome).format("HH:mm DD.MM.YYYY");
        timeSinceOperationAlarm = timeSinceOperationIncoming(operationdata.timestampincome);

        // Render Ressources
        if (Array.isArray(operationdata.ressources)) {
            var ressourcesHtml = '';

         
            operationdata.ressources.forEach(ressource => {
                ressourcename = ressource.ressourcenameraw;
                if (ressource.ressourcenamecustom != null) {
                    ressourcename = ressource.ressourcenamecustom;
                }
                ressourcesHtml = ressourcesHtml + '<button type="button" class="btn btn-danger btn-md mr-2"><h3>' + ressourcename + '</h3></button>'; 
            });
        }

        // set Value to fields
        document.getElementById('timestampincome').innerText = operationTimestampIncome;
        document.getElementById('keyword').innerText = operationdata.keyword;
        document.getElementById('comment').innerText = operationdata.comment;
        document.getElementById('messenger').innerText = operationdata.messenger;
        document.getElementById('address').innerText = operationdata.einsatzortstreet + ' ' + operationdata.einsatzortstreetnumber + '\n ' + operationdata.einsatzortcity;
        document.getElementById('ressources').innerHTML = ressourcesHtml;
        // persist currently displayed operation-id
        // for check in next cycle if it's the same operation or another one
        // if it's the same operation, maps don't need to be reloaded
        lastUIOperationNumber = operationdata.id;
        statusChanged = false;
    //}
}


/**
 * Function clears operation information in the ui fields.
 * 
 * @returns nothing
 * @param nothing
 * @alias wallboard~clearOperationDataToUI
 */
function clearOperationDataToUI() {
    //console.log('Status Changed: ' + statusChanged);
    console.log('clearOperationDataToUI | clearing UI');
    // set Value to fields
    document.getElementById('timestampincome').innerText = '-';
    document.getElementById('keyword').innerText = '-';
    document.getElementById('comment').innerText = '-';
    document.getElementById('messenger').innerText = '-';
    document.getElementById('address').innerText = '-';
    document.getElementById('ressources').innerHTML = '-';
    statusChanged = false;
}


/**
 * Function stops the UI reload (cycling).
 * 
 * @returns nothing
 * @param id of the setInterval Function
 * @alias wallboard~stopOperationUICycle
 */
function stopOperationUICycle() {
    console.log('Stopping UI cycling');
    // Clear all UI fields
    clearOperationDataToUI();
    // Stop UICycler
    clearInterval(UICylceID);
    UICylceID = 0;
    UICycleRunning = false;
    // Stop Progressbarcycler
    clearInterval(progressCycleID);
    progressCycleID = 0;
    progressbarCycleRunning = false;
    // Ckear Operations Store
    operationsStoreDetails = [];    
    currentUIOperationNumber = 0;
}

/**
 * Function worker for the UI Cycle
 * 
 * Function is called by the actual cycling instance
 * @alias wallboard~operationCycle
 */
function operationCycle() {
    // Code for Switching between operations
    console.log('Operation cycle');
    if(currentUIOperationNumber < 1) {
        // no operation has been displayed, grab the first one
        console.log('Start Cycle --> get first operation');
        currentUIOperation = operationsStoreDetails[0];
        currentUIOperationNumber = 1;
    } else {
        console.log('in the middle of Cycle --> get next operation');
        currentUIOperationNumber = currentUIOperationNumber + 1;
        if (operationsStoreDetails.length < currentUIOperationNumber) {
            currentUIOperationNumber = 1;
        }
        currentUIOperation = operationsStoreDetails[currentUIOperationNumber-1];
    }
    // Call UI-Refresh with selected operation
    console.log('Render Operation nr.: ' + currentUIOperationNumber + ' ' + currentUIOperation.length);
    startCylceProgressbar();
    renderOperationDataToUI(currentUIOperation);
}


/**
 * Function starts the UI reload (cycling).
 * 
 * @returns nothing
 * @param nothing
 * @alias wallboard~startOperationUICycle
 */
function startOperationUICycle() {
    console.log('startOperationUICycle');
    if (UICycleRunning === false) {
        // update UI immediately
        operationCycle();
        // start UI update periodically
        UICylceID = window.setInterval(operationCycle, cycleInterval);
        console.log('startOperationUICycle ID: ' + UICylceID);
        UICycleRunning = true;
    }
}


function startCylceProgressbar() {
    progressbarStep = 0;
    if (progressbarCycleRunning === false) {
        progressCycleID = window.setInterval(function() {
            progressbarCycleRunning = true;
            var value = (progressbarStep * 500) ;
            if (value > cycleInterval) {
                progressbarStep = 0;
                progressbarCycleRunning = false;
                clearInterval(progressCycleID);
            } else {
    
            }
            var percentage = Math.round((value / cycleInterval) * 100);
            //console.log('Progress (' + progressCycleID +  '): ' + percentage);
            document.getElementById('cycleProgress').style = 'width: ' + percentage + '%';
            progressbarStep = progressbarStep + 1;
            // Set counter showing active vs actually displayed operation
            var operationCounter = '(' + currentUIOperationNumber + ' / ' + operationsStoreDetails.length + ')';
            if (percentage < 53) {
                document.getElementById('operationCounter').classList.remove('text-white');
                document.getElementById('operationCounter').classList.add('text-dark');
            } else {
                document.getElementById('operationCounter').classList.remove('text-dark');
                document.getElementById('operationCounter').classList.add('text-white');
            }
            document.getElementById('operationCounter').innerText = operationCounter;
        }, 500);
    }
}


 /**
 * Function checks incoming operations from server and refreshes the lcoal store if changes where identified
 * 
 * @returns nothing
 * @param operationdata as Array of operationdata from websocket
 * @alias wallboard~updateLocalOperationsStore
 */
function updateLocalOperationsStore(jsonPayLoad) {
    // Build list of freshly incoming operations
    var incomingOperationsArray = [];
    if(jsonPayLoad.length > 0) {
        jsonPayLoad.forEach(operation => {
            console.log('ID: ' + operation.id);
            incomingOperationsArray.push(operation.id);
        });
    } 
    //console.log('incomingOperationsArray: ' + incomingOperationsArray.length);
    if(arraysEqual(incomingOperationsArray, operationsStore)) {
        // No new operations came in
        console.log('Message from Server recieved | no new / closed operations');
        //console.log(incomingOperationsArray);
        //console.log(operationsStore);
    } else {
        // new operation came in or one operation has been closed
        // reload operations for view in wallboard ui
        console.log('Message from Server recieved | there are new or closed operations --> reload ui');
        // set operationStore to loaded incominOperationsArray
        operationsStore = incomingOperationsArray;
        // Load recieved JSON into Store Details
        operationsStoreDetails = jsonPayLoad;
    }
    // Check if there are operations. If so, start UI Cycle
    if (operationsStoreDetails.length > 0) {
        console.log(operationsStoreDetails);
        if (UICylceID > 1) {
            // Cycler is running
        } else {
            // Cycler is not running --> start it
            startOperationUICycle();
            operationPanelOn();
        }
    } else {
        stopOperationUICycle();
        operationPanelOff();
    }
}


/**
 * Function initializes Wallboard.
 * 
 * @returns nothing
 * @param nothing
 * @alias wallboard~initializeWallboard
 */
function initializeWallboard() {
    try {
        ws = new WebSocket('ws://' + window.location.hostname + ':40510');    // event emmited when connected
        ws.onopen = function () {
            console.log('WALLBOARD websocket is connected to ALARMiator Server ...');        // sending a send event to websocket server
            ws.send('connected');
            if(wsConnectionLostState === true) {
                // reconnected WebSocket after an connection drop between server 
                // and wallboard --> reload page (to be sure page is loaded based 
                // on latest html / js files from the server
                window.location.reload(true);
            }
        }    // event emmited when receiving message 
        ws.onmessage = function (ev) {
            var jsonPayload = JSON.parse(ev.data);
            updateLocalOperationsStore(jsonPayload);
        }
        ws.onclose = function (ev) {
            console.log('Main | Verbindung zum ALARMiator Server ist unterbrochen');
            wsConnectionLostState = true;
            // Try to reconnect
            healthcheck();
        }
        ws.onerror = function (ev) {
            console.log('Main | Fehler in der Verbindung zum ALARMiator Server: ' + ev);
            wsConnectionLostState = true;
        }
    } catch (err) {
        // Error in creating WebSocket Connection

    }
}


/**
 * Function helper (kind of a sleep function)
 * 
 * @param {integer} ms Milliseconds to wait
 */
function sleep(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}

/**
 * Function checks if websocket connection is healty and trys to reestablish connection in case connection is not available aymore.
 */
function healthcheck() {
    if(!ws || ws.readyState == 3) {
        console.log('Healthcheck | Trying to reestablish connection to ALARMiator Server ...');
        sleep(500).then(() => {
            // Try to connect
            initializeWallboard();
        });
    }
}

initializeWallboard();
