var ejs  = require('ejs');
var fs   = require('fs');
var pdf = require('html-pdf');
const mysql = require('mysql');



var reportingBasePath = "../public/reports/";
var svcName = "REPORTINGENGINE";
// Read global configuration
var config = require('../config/config.js');


// Setup Logger
var log4js = require('log4js');
log4js.configure({
    appenders: { mylogger: { type:"file", filename: "../logs/reporting/reporting.log" }, console: { type:"console" } },
    categories: { default: { appenders:["mylogger", "console"], level:"ALL" } }
});
const logger = log4js.getLogger("default");

// Create connection to database
const db = mysql.createConnection (config.databaseOptions);
db.connect((err) => {
    if (err) {
        throw err;
    }
    logger.info(`${svcName} | Reporting Engine connected to database: ` + config.databaseOptions.database);
});
global.db = db;


// gather needed information
function get_hydrants_detail(hydrantId, callback) {
    var hydranthelper = require('../models/hydrants');
    const hydrants = new hydranthelper(logger);
    
    var hydrant = hydrants.getHydrantDataForId(hydrantId, (result) => {
        if (result != null) {
            var template = fs.readFileSync(reportingBasePath + 'templates/tpl_hydrant.ejs', 'utf-8');
            var html     = ejs.render ( template , result[0] );
            try {
                fs.writeFileSync(reportingBasePath + "ondemand/tpl_hydrant.html", html, 'utf8');
                callback(false);
            } catch {
                callback(true);
            }
        } else {
            
        }
    })
}  

function generatePDFDocumentForReport(htmlInputFile, pdfOutputFile, callback) {
    var html = fs.readFileSync(htmlInputFile, 'utf8');
    var options = { 
        format: 'A4',  
        border: '5mm'
    };

    pdf.create(html, options).toFile(pdfOutputFile, function(err, res) {
        if (err) return console.log(err);
        logger.info('Written Report PDF to ' + res); // { filename: '/app/businesscard.pdf' }
    });
}

logger.info(`${svcName} | start generating report`);
get_hydrants_detail(68, (err) => {
    if (err) {
        logger.error('Error generating report');
    } else {
        generatePDFDocumentForReport(reportingBasePath + "ondemand/tpl_hydrant.html", reportingBasePath + "ondemand/tpl_hydrant.pdf");
        logger.info('Report successfully generated');
    }
})


