const fs = require('fs');
const uuidv1 = require('uuid/v1');
const bcrypt = require('bcrypt');

let organizations = require('../../models/organizations');

module.exports = {
    getUsersList: (req, res) => {
        let query = "SELECT `users`.*, " +
                            "groups.groupname, " +
                            "groups.id AS groupid, " +
                            "organizations.id AS organizationid, " +
                            "organizations.name AS organizationname " +
                    "FROM `users` " +
                    "INNER JOIN `groups` ON users.group = `groups`.id " +
                    "INNER JOIN `organizations` ON `users`.`organizationid` = `organizations`.`id` " +
                    "ORDER BY `users`.id ASC";

        // execute query
        db.query(query, (err, result) => {
            if (err) {
                res.redirect('/admin/dashboard/list');
            }
            res.render('admin/users.ejs', {
                title: 'ALARMiator | Benutzer',
                navarea: 'admin',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                users: result
            });
        });
    },
    addUserPage: (req, res) => {
        let groupsQuery = "SELECT * FROM `groups`";
        let orgs = new organizations();
        orgs.getListOfOrganizations(function(resultOrgs) {
            
            db.query(groupsQuery, (err, result) => {
                if (err) {
                    return res.status(500).send(err);
                }
                res.render('admin/users_add.ejs', {
                    title: 'ALARMiator',
                    navarea: 'admin',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    organizations: resultOrgs,
                    groups: result
                });
            });
        });
    },

    addUser: (req, res) => {
       
        if (!req.files) {
            return res.status(400).send("No files were uploaded.");
        }

        let message = '';
        let firstname = req.body.firstname;
        let lastname = req.body.lastname;
        let groupid = req.body.groupid;
        let telephone = req.body.telephone;
        let username = req.body.username;
        let password = req.body.password;
        let uploadedFile = req.files.image;
        let image_name = uploadedFile.name;
        let fileExtension = uploadedFile.mimetype.split('/')[1];
        let organizationid = req.body.organizationid;
        image_name = username + '.' + fileExtension;

        let usernameQuery = "SELECT * FROM `users` WHERE username = '" + username + "'";

        db.query(usernameQuery, (err, result) => {
            if (err) {
                return res.status(500).send(err);
            }
            if (result.length > 0) {
                message = 'Username already exists';
                res.render('admin/users_add.ejs', {
                    message,
                    navarea: 'admin',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    title: 'Welcome to Socka | Add a new user'
                });
            } else {
                // check the filetype before uploading it
                if (uploadedFile.mimetype === 'image/png' || uploadedFile.mimetype === 'image/jpeg' || uploadedFile.mimetype === 'image/gif') {
                    // upload the file to the /public/assets/img directory
                    // get extension first
                    switch (uploadedFile.mimetype) {
                        case 'image/png' :
                            extension = '.png';
                            break;
                        case 'image/jpeg' :
                            extension = '.png';
                            break;
                        case 'image/gif' :
                            extension = '.gif';
                            break;
                        default:
                            extension = '.gif';
                    }
                    // Generate uuid for having more or less unique filename for image
                    uuidFilename = uuidv1() + extension;

                    uploadedFile.mv(`public/assets/img/${uuidFilename}`, (err ) => {
                        if (err) {
                            return res.status(500).send(err);
                        }
                        // Encrypt Password (Hash)
                        bcrypt.hash(password, 10, function(err, hash) {
                            if (err) { throw (err); }
                        
                            bcrypt.compare(password, hash, function(err, result) {
                                if (err) { throw (err); }
                                console.log(result);
                                // Store hash in database
                                // send the user's details to the database
                                let query = "INSERT INTO `users` (firstname, lastname, `group`, telephone, image, username, password, organizationid) VALUES ('" +
                                firstname + "', '" + lastname + "', " + groupid + ", '" + telephone + "', '" + uuidFilename + "', '" + username + "', '" + hash + "', " + organizationid + ")";
                                db.query(query, (err, result) => {
                                    if (err) {
                                        return res.status(500).send(err);
                                    }
                                    res.redirect('/admin/users/list');
                                });
                            });
                        });
                     });
                } else {
                    message = "Invalid File format. Only 'gif', 'jpeg' and 'png' images are allowed.";
                    res.render('admin/users_add.ejs', {
                        message,
                        navarea: 'admin',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        title: 'Welcome to Socka | Add a new user'
                    });
                }
            }
        });
    },
    editUserPage: (req, res) => {
        let userId = req.params.id;
        let orgs = new organizations();
        orgs.getListOfOrganizations(function(resultOrgs) {
            let query = "SELECT * FROM `users` WHERE id = '" + userId + "' ";
            
            db.query(query, (err, result) => {
                if (err) {
                    return res.status(500).send(err);
                }
                let userResult = result;
                groupsQuery = "SELECT * FROM `groups`";
                db.query(groupsQuery, (err, result) => {
                    if(err) {
                        return res.status(500).send(err);
                    }
                    res.render('admin/users_edit.ejs', {
                        title: 'Edit  user',
                        navarea: 'admin',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        user: userResult[0],
                        groups: result,
                        organizations: resultOrgs,
                        message: ''
                    });
                });
            });
        });
    },
    editUser: (req, res) => {
        let userId = req.params.id;
        let firstname = req.body.firstname;
        let lastname = req.body.lastname;
        let groupid = req.body.groupid;
        let telephone = req.body.telephone;
        let password = req.body.password;
        let organizationid = req.body.organizationid;
        let sqlPartialPassword = '';



        if (password.length > 0) {
            var hash = bcrypt.hashSync(password, 10);
            sqlPartialPassword = ", `password` = '" + hash + "' ";
        } 
        


        let query = "UPDATE `users` SET `organizationid` = " + organizationid + ", `firstname` = '" + firstname + "', `lastname` = '" + lastname + "', `group` = '" + groupid + "', `telephone` = '" + telephone + "' " + sqlPartialPassword + " WHERE `users`.`id` = '" + userId + "'";
        db.query(query, (err, result) => {
            if (err) {
                return res.status(500).send(err);
            }
            res.redirect('/admin/users/list');
        });
    },
    deleteUser: (req, res) => {
        // MARKER_IMAGE_BASE64_ISSUE (is this still in use?)
        let userId = req.params.id;
        let deleteUserQuery = 'DELETE FROM users WHERE id = "' + userId + '"';

        db.query(deleteUserQuery, (err, result) => {
            if (err) {
                return res.status(500).send(err);
            }
            res.redirect('/admin/users/list');
        });
    }
};