const Reports = require('../../models/core/core_reports');
const CoreCron = require('../../models/core/core_cron');
const { glob } = require('glob');
const moment = require("moment")

module.exports={

    /**
     * render testalarms list to manage testalarms
     * @param {*} req 
     * @param {*} res 
     */
    GetCronJobMainPage: (req,res) => {
        req.app.logger.debug('CORE | CRON | loading cronjob list');
    
        res.render('admin/cron', {
            title: 'ALARMiator | Admin | Aufgabenverwaltung',
            navarea: 'admin',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    /**
     * Page for adding a new cron job to core database
     * @param {object} req 
     * @param {object} res 
     */
    addCronJobPage: (req, res) => {
        let reports = new Reports(req.app.logger);
        reports.getListOfReports((rowsReports) => {
            res.render('admin/cron_add', {
                title: 'ALARMiator | Admin | Aufgabe hinzufügen',
                navarea: 'admin',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                reports: rowsReports
            });
        })
    },
    /**
     * Endpoint for persisting a new cron job at core database
     * @param {object} req 
     * @param {object} res 
     */
    addCronJob: (req, res) => {
        let corecron = new CoreCron(req.app.logger);
        corecron.addCronJob(req.body, (success) => {
            if (success) {
                // successfully persisted new task at core database
                // now initialise cron job and make it active
                corecron.GetCronJobFromCoreDB(req.body.name, (cronjob) => {
                    if (cronjob != null) {
                        var created = global.cron.AddCronJob(cronjob.name, cronjob.type, cronjob.cronDefinition, cronjob.data);
                        if(created){
                            res.redirect('/admin/cron/list');
                        }
                        else{
                            res.redirect(req.headers.referer);
                        }
                    } else {
                        res.redirect(req.headers.referer);
                    }
                })
               
            } else {
                res.redirect(req.headers.referer);
            }
        })
    },
    /**
     * Renders edit form for a given cron job
     * @param {object} req 
     * @param {object} res 
     */
    editCronJobPage: (req, res) => {
        let id = req.params.id;
        let reports = new Reports(req.app.logger);
        reports.getListOfReports((rowsReports) => {
            let corecron = new CoreCron(req.app.logger);
            corecron.GetCronJobWithIdFromCoreDB(id, (job) => {
                res.render('admin/cron_edit', {
                    title: 'ALARMiator | Admin | Aufgabe hinzufügen',
                    navarea: 'admin',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    reports: rowsReports,
                    job: job
                });
            })
        })
    },
    editCronJob: (req, res) => {
        console.log('------- PARAMS ----------');
        console.log(req.params);
        console.log('------- BODY ----------');
        console.log(req.body);

        let corecron = new CoreCron(req.app.logger);
        corecron.editCronJob(req.params.id, req.body, (success) => {
            if (success) {
                // successfully persisted new task at core database
                // now initialise cron job and make it active
                corecron.GetCronJobFromCoreDB(req.body.name, (cronjob) => {
                    if (cronjob != null) {
                        var deleted = global.cron.DeleteCronJob(cronjob.name);
                        if (deleted) {
                            var created = global.cron.AddCronJob(cronjob.name, cronjob.type, cronjob.cronDefinition, cronjob.data, cronjob.id);
                            if(created){
                                res.redirect('/admin/cron/list');
                            }
                            else{
                                res.redirect(req.headers.referer);
                            }
                        }
                    } else {
                        res.redirect(req.headers.referer);
                    }
                })
            } else {
                res.redirect(req.headers.referer);
            }
        })
    },
    GetCronJobMainPageData: (req, res) => {
        var jobs = global.cron.GetAllCronjobs();
        const formatDate = val => {
            if (val == null || val === "") {
                return val
            }
            return moment(val, "DD.MM.YYYY HH:mm:ss").format("YYYY-MM-DD HH:mm:ss")
        }
        res.json({ 
            data: jobs.map(cronjob => ({
                last: formatDate(cronjob.last),
                next: formatDate(cronjob.next),
                name: cronjob.name,
                type: cronjob.type,
                running: cronjob.cronjob.running,
                dbId: cronjob.dbId
            }))
        })
    },

    /**
     * set a testalarm activ od deactivate it
     * @param {*} req 
     * @param {*} res 
     */
    toggleCronState: (req, res) =>{
        if(req.params.state === '1'){
            req.app.logger.debug('CORE | CRON | activate testalarm');
            var started = global.cron.StartCronJob(req.params.name);
            if(started){
                global.cron.UpdateCronJobInCoreDB(req.params.name, 'running', true, (err) => {
                    if(err){
                        var msg = 'error at writing new value to coredatabase!\n' + err;
                        console.Log(msg);                  
                    }
                    res.redirect('/admin/cron/list');
                });
            } else {
                var msg = req.params.name + 'could not be started! Please check logs!'
                console.Log(msg);
                res.redirect('/admin/cron/list');
            }
        }
        else{
            req.app.logger.debug('CORE | CRON | deaktivate testalarm');
            var stopped = global.cron.StopCronJob(req.params.name);
            if(stopped){
                global.cron.UpdateCronJobInCoreDB(req.params.name, 'running', false, (err) => {
                    if(err){
                        var msg = 'error at writing new value to coredatabase!\n' + err;
                        console.Log(msg);                 
                    }
                    res.redirect('/admin/cron/list');
                });
            } else {
                var msg = req.params.name + 'could not be stopped! Please check logs!'
                console.Log(msg);
                res.redirect('/admin/cron/list');
            }
        }
    },

}