// Logging
var log4js = require('log4js');
log4js.configure({
    appenders: { mylogger: { type: "file", filename: "logs/server/server.log" }, console: { type: "console" } },
    categories: { default: { appenders: ["mylogger", "console"], level: "ALL" } }
});
const logger = log4js.getLogger("default");
const Plugins = require('../../models/plugins');
const EventManager = require('../../models/eventManager.js');

var plugins = new Plugins(logger);

module.exports = {
    getPluginsList: (req, res) => {
        plugins.getPluginsList((result) => {
            //console.log(result);
            res.render('admin/plugins_list.ejs', {
                title: 'ALARMiator | Plugins',
                navarea: 'admin',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                pluginsList: result,
                plgManager: global.plgManager
            });
        });
    },
    getPluginConfigForPlugin: (req, res) => {
        var pluginNameSpace = req.params.namespace;
        var pluginManifest = plugins.getPluginManifestForNameSpace(pluginNameSpace);
        plugins.getPluginConfig(pluginNameSpace, (config) => {
            var plgConfig = null;
            try {
                plgConfig = JSON.parse(config.configstore);
            } catch (err) {
                plgConfig = null;
            }
            if (plgConfig !== null) {
                // start building the config form
                var formHTML = plugins.getPluginConfigFormCode(plgConfig, config.state, pluginNameSpace);
                res.render('admin/plugins_config.ejs', {
                    title: 'ALARMiator | Plugin Einstellungen',
                    navarea: 'admin',
                    subarea: 'plugin',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    pluginManifest: pluginManifest,
                    config: plgConfig,
                    formHTML: formHTML,
                });
            }
        })
    },
    setPluginConfigForPlugin: (req, res) => {
        var namespace = req.params.namespace;
        if (namespace.length > 0) {
            plugins.setPluginConfig(namespace, req.body, req.files, (success) => {
                if(success) {
                    // Plugin Config has been changed
                    // write log and redirect
                    logger.info('Plugin config for namespace ' + namespace + ' successfully updated');
                    res.redirect('/admin/plugins/list');
                    
                } else {
                    logger.error('Plugin config for namespace ' + namespace + ' could not be updated');
                    res.redirect('/admin/plugins/list');
                }
            })

        } else {
            logger.error('No namespace given from form');
            res.redirect('/admin/plugins/list');
        }

    },
    resetManifestForNamespace: (req, res) => {
        var namespaceToRefresh = req.params.namespace;
        if (namespaceToRefresh.length > 0) {
            plugins.resetManifestForNamespace(namespaceToRefresh);
        }
        res.redirect('/admin/plugins/list');

    },
    eventslist: (req, res) => {
        var eventmanager = new EventManager();
        eventmanager.getAllEvents((resultEvents) => {
            res.render('admin/events.ejs', {
                title: 'ALARMiator | Plugin Einstellungen',
                navarea: 'admin',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                events: resultEvents
            });
        })
    },
    eventsNewAlarm: (req, res) => {
        res.render('admin/events_newalarm_config.ejs', {
            title: 'ALARMiator | Event event_new_alarm konfigurieren',
            navarea: 'admin',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session
        });
    },
    eventsNewAlarmSend: (req, res) => {
        console.log('EVENT NEW ALARM TEST');
        const EventsTest = require('../../models/events_test');
        let eventTester = new EventsTest();
        eventTester.event_new_alarm(req.body, (success) => {
            res.redirect('/admin/plugins/events');
        });
    },
    eventsGeneric: (req, res) => {
        var eventname = req.params.eventname;
        var eventmanager = new EventManager(req.app.logger);
        eventmanager.getEvent(eventname, (eventResult) => {
            res.render('admin/events_generic_config.ejs', {
                title: 'ALARMiator | Event event_generic konfigurieren',
                navarea: 'admin',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                event: eventResult[0]
            });
        })
    },
    eventsGenericSend: (req, res) => {
        var eventName = req.body.eventname;
        var payload = req.body.payload;
        console.log('ROUTES | PLUGINS | eventsGenericSend: Fire generic event ' + eventName + ' --> ' + payload);
        global.plgManager.event_generic(eventName, payload);
        res.redirect('/admin/plugins/events');
    },
    fireTestEvent: (req, res)  => {
        var eventName = req.params.eventname;
        if (eventName.length > 0) {
            // event is set
            // fire event
            const EventsTest = require('../../models/events_test');
            let eventTester = new EventsTest();
            eventTester.testEvent(eventName, (success) => {
                res.redirect('/admin/plugins/events');
            })
        } else {
            // no event name found
            // redirect to list
            res.redirect('/admin/plugins/events');
        }
    }
};