const Basedata = require('../../models/basedata');
const Groups = require('../../models/groups');
const Alarm = require('../../models/alarm');
const Alertrules = require('../../models/alertrules');
const Devices = require('../../models/alertdevices');
const Fcmtokens = require('../../models/fctmtokens');
const Assets = require('../../models/assets');
const Operation = require('../../models/core/core_operation');
const Operations = require('../../models/operationshelper');
const FeedbackStates = require('../../models/feedbackstates');
const assetshelper = require('../../models/assets');
const Wallboards = require('../../models/wallboards');
const Organizations = require('../../models/organizations');
const Qualifications = require('../../models/qualifications');
const Ruleengine = require('../../models/ruleengine');
const Geargroups = require('../../models/gearGroups');

var moment = require('moment');
const Feedbackstates = require('../../models/feedbackstates');
const Alertdevices = require('../../models/alertdevices');

module.exports = {
    getDashboard: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM-DASHBOARD | loading alarm dashboard page');
        var alertrules = new Alertrules();
        alertrules.getAllActiveRules((rowsActiveAlertrules) => {
            var alertdevices = new Alertdevices(req.app.logger);
            alertdevices.getInactiveAlertDevices((rowsInactiveAlertDevices) => {
                res.render('alarm/dashboard', {
                    title: 'ALARMiator | Alarm',
                    navarea: 'alarm',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    activeAlertRules: rowsActiveAlertrules,
                    inactiveDevices: rowsInactiveAlertDevices
                });
            })
        });
        
    },
    getMatrix: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM-MATRIX | loading matrix page');
        // read list of plugins which offer alarming by user
        var basedata = new Basedata(req.app.logger);
        var alarm = new Alarm();
        alarm.getAllAlarmings((rowsAlarmings) => {
            basedata.getListOfBasedataForTable((basedataResult) => {
                var plugins = global.plgManager.getPluginsWithAlarmingByUser();
                res.render('alarm/matrix', {
                    title: 'ALARMiator | Matrix',
                    navarea: 'alarm',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    plugins: plugins,
                    basedata: basedataResult,
                    alarmings: rowsAlarmings
                });
            })
        })
    },
    setAlarmForBasedata: (req, res) => {
        let basedataId = req.params.basedataId;
        let namespace = req.params.namespace;
        console.log('Adding alarming for ' + basedataId + ' on namespace ' + namespace);
        var alarm = new Alarm();
        alarm.setAlarmForBasedata(basedataId, namespace, (success) => {
            if (success) {
                res.send('ok');
            } else {
                res.send('nok');
            }
        })
    },
    removeAlarmForBasedata: (req, res) => {
        let basedataId = req.params.basedataId;
        let namespace = req.params.namespace;
        console.log('Removing alarming for ' + basedataId + ' on namespace ' + namespace);
        var alarm = new Alarm();
        alarm.removeAlarmingForBasedata(basedataId, namespace, (success) => {
            if (success) {
                res.send('ok');
            } else {
                res.send('nok');
            }
        })
    },
    getDevicesList: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM | loading list of devices');
        res.render('alarm/devices', {
            title: 'ALARMiator | Endgeräte',
            navarea: 'alarm',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getDevicesListData: (req, res) => {
        var devices = new Devices(req.app.logger);
        devices.getAllAlertdevices((rowsDevices) => {
            devices.getAllGearAlertdevices((rowsDevicesGear) => {
                res.json({ data: rowsDevices.map(dev => {
                    dev.displayName = `${dev.bDataFirstname} ${dev.bDataLastname}`
                    return dev
                }).concat(rowsDevicesGear.map(dev => {
                    // create the key 'bDataLastname' to allow sorting in the front-end
                    dev.displayName = dev.assetTitle
                    return dev
                }))})
            })
        })
    },
    addVirtualDevicePage: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM | loading form to add new virtual device');
        var basedata = new Basedata(req.app.logger);
        basedata.getListOfBasedata((rowsBasedata) => {
            var assets = new Assets(req.app.logger);
            assets.getAllAssetsWhichCanBeAlarmed((rowsAlarmableAssets) => {
                res.render('alarm/virtual_device_add', {
                    title: 'ALARMiator | virtuelles Device anlegen',
                    navarea: 'alarm',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    basedata: rowsBasedata,
                    assets: rowsAlarmableAssets
                });
            })
        });
    },
    addVirtualDevice: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM | persisting new virtual device');
        var name = req.body.name;
        var basedataId = req.body.basedataid;
        var assetId = req.body.assetId;

        if (typeof basedataId != null) {
            if (basedataId.length > 0) {
                assetId = null;
            } else {
                basedataId = null;
            }
        }

        if (typeof assetId != null) {
            if (assetId != null) {
                if (assetId.length > 0) {
                    basedataId = null;
                } else {
                    assetId = null;
                }
            } else {
                assetId = null;
            }
        }

        var devices = new Devices(req.app.logger);

        devices.createDevice(basedataId, assetId, name, null, null, null, null, 1, (success) => {
            if (success) {
                // Created virtual device successfully
                res.redirect('/alarm/devices/list?add=device');
            } else {
                // could not create virtual device
                res.redirect('/alarm/devices/list');
            }
        })
    },
    removeDevice: (req, res) => {
        var deviceId = req.params.id;
        var tokens = new Fcmtokens(req.app.logger);
        tokens.removeTokensForDeviceID(deviceId, (success) => {
            if (success) {
                var devices = new Devices(req.app.logger);
                devices.removeDevice(deviceId, (successRemoveDevice) => {
                    res.redirect('/alarm/devices/list?delete=device');
                })
            } else {
                res.redirect('/alarm/devices/list');
            }
        })
    },
    getTokensList: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM | loading list of tokens');
        
        res.render('alarm/tokens_list', {
            title: 'ALARMiator | FCM Tokens',
            navarea: 'alarm',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getTokensListData: (req, res) => {
        var tokens = new Fcmtokens(req.app.logger);
        tokens.getAllFCMTokensForUI((rowsTokens) => {
            res.json({ 
                data: rowsTokens.map(token => {
                    // resolve plugin name for each token
                    const pluginMatches = global.plgManager.pluginList.filter(plugin => plugin.namespace === token.pluginNamespace)
                    if (pluginMatches.length > 0) {
                        token.pluginTitle = pluginMatches[0].name
                    }
                    return token
                })
            })
        })
    },
    addTokenPage: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM | lloading form to add a new token');
        var basedata = new Basedata(req.app.logger);
        basedata.getListOfBasedata((rowsBasedata) => {
            var devices = new Devices(req.app.logger);
            devices.getAllAlertdevices((rowsDevices) => {
                devices.getAllGearAlertdevices((rowsDevicesGear) => {
                    res.render('alarm/tokens_add', {
                        title: 'ALARMiator | FCM Tokens',
                        navarea: 'alarm',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                        basedata: rowsBasedata,
                        devices: rowsDevices,
                        devicesGear: rowsDevicesGear
                    });
                })
            })
        })
    },
    addToken: (req, res) => {
        var deviceId = req.body.deviceid;
        var basedataId = req.body.basedataid;
        var fcmtoken = req.body.fcmtoken;
        var pluginnamespace = req.body.pluginnamespace;

        var fcmtokens = new Fcmtokens(req.app.logger);
        fcmtokens.createFCMToken(deviceId, 1, pluginnamespace, 0, fcmtoken, (success) => {
            res.redirect('/alarm/tokens/list?add=token');
        })
    },
    removeToken: (req, res) => {
        let tokenid = req.params.tokenid;

        console.log('Removing token with id ' + tokenid);
        var tokens = new Fcmtokens(req.app.logger);
        tokens.removeToken(tokenid, (success) => {
            if (success) {
                res.redirect('/alarm/tokens/list?delete=token');
            } else {
                res.redirect('/alarm/tokens/list');
            }
        })
    },
    activateToken: (req, res) => {
        let tokenid = req.params.tokenid;
        var tokens = new Fcmtokens(req.app.logger);
        tokens.setActive(tokenid, (success) => {
            if (success) {
                res.redirect('/alarm/tokens/list?edit=token');
            } else {
                res.redirect('/alarm/tokens/list');
            }
        })
    },
    deactivateToken: (req, res) => {
        let tokenid = req.params.tokenid;
        var tokens = new Fcmtokens(req.app.logger);
        console.log('deactivate: ' + tokenid);
        tokens.setInactive(tokenid, (success) => {
            if (success) {
                res.redirect('/alarm/tokens/list?edit=token');
            } else {
                res.redirect('/alarm/tokens/list');
            }
        })
    },
    toggleDevice: (req, res) => {
        let deviceId = req.params.deviceid;
        let state = req.params.state;

        console.log('Device: ' + deviceId + ' state: ' + state);

        var devices = new Devices(req.app.logger);
        if (state === '1') {
            console.log('activating');
            devices.setActive(deviceId, (success) => {
                if (success) {
                    res.redirect('/alarm/devices/list?edit=device');
                } else {
                    res.redirect('/alarm/devices/list');
                }
            })
        } else {
            console.log('deactivating');
            devices.setInactive(deviceId, (success) => {
                if (success) {
                    res.redirect('/alarm/devices/list?edit=device');
                } else {
                    res.redirect('/alarm/devices/list');
                }
            })
        }
    },
    getRulesList: (req, res) => {
        res.render('alarm/rules_list', {
            title: 'ALARMiator | Alarmierungsregeln',
            navarea: 'alarm',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });        
    },
    getRulesListData: (req, res) => {
        var alertrules = new Alertrules();
        alertrules.getAllRules((rows) => {
            res.json({ data: rows })
        })
    },
    addRulePage: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM | loading form to add a new alarming rule');
        var groups = new Groups(req.app.logger);
        groups.getListOfAlarmableGroups((rowsAlarmableGroups) => {
            var basedata = new Basedata(req.app.logger);
            basedata.getListOfBasedata((rowsBasedata) => {
                var assets = new Assets(req.app.logger);
                assets.getAllAssetsWhichCanBeAlarmed((rowsAssets) => {
                    var geargroups = new Geargroups(req.app.logger);
                    geargroups.getListOfAlarmableGroups((rowsAlarmableGearGroups) => {
                        res.render('alarm/rules_editor_add', {
                            title: 'ALARMiator | Alarmierungsregeln',
                            navarea: 'alarm',
                            avatarpath: req.session.avatar,
                            name: req.session.name,
                            organizationname: req.session.organization,
                            session: req.session,
                            basedata: rowsBasedata,
                            gear: rowsAssets,
                            alarmableGroups: rowsAlarmableGroups,
                            alarmableGearGroups: rowsAlarmableGearGroups
                        });
                    });
                })
            })
        })
    },
    addRule: (req, res) => {
        console.log('----------------------------');
        console.log(req.body);
        console.log('----------------------------');
        var alertrules = new Alertrules();
        var groupsArray = [];
        alertrules.addrule(req.body, (successRule, alertRuleId) => {
            if (successRule) {
                alertrules.renderRuleFilters(req.body, alertRuleId, (successFilters) => {
                    alertrules.renderRuleGroups(req.body, alertRuleId, (successGroups) => {
                        alertrules.renderRuleGearGroups(req.body, alertRuleId, (successGearGroups) => {
                            alertrules.addRuleGroups(alertRuleId, groupsArray, (successGroupsAdd) => {
                                if (successGroupsAdd) {
                                    res.redirect('/alarm/rules/list?add=rule');
                                } else {
                                    console.log('Error adding new alert rule with groups');
                                    res.redirect('/alarm/rules/list');
                                }
                            })
                        })
                    })
                })
            } else {
                res.redirect('/alarm/rules/list');
            }
        })
    },
    editRulePage: (req, res) => {
        var ruleId = req.params.id;
        req.app.logger.debug('ROUTES-ALARM | loading form to edit a new alarming rule');
        var alertrules = new Alertrules();
        alertrules.getRuleWithId(ruleId, (rowsAlertRule) => {
            alertrules.getRuleGroups(ruleId, (rowsActiveGroups) => {
                alertrules.getRuleGearGroups(ruleId, (rowsActiveGearGroups) => {
                    alertrules.getRuleFilters(ruleId, (rowsActiveFilters) => {
                        var groups = new Groups(req.app.logger);
                        groups.getListOfAlarmableGroups((rowsAlarmableGroups) => {
                            var geargroups = new Geargroups(req.app.logger);
                            geargroups.getListOfAlarmableGroups((rowsAlarmableGearGroups) => {
                                var basedata = new Basedata(req.app.logger);
                                basedata.getListOfBasedata((rowsBasedata) => {
                                    var assets = new Assets(req.app.logger);
                                    assets.getAllAssetsWhichCanBeAlarmed((rowsAssets) => {
                                        res.render('alarm/rules_editor_edit', {
                                            title: 'ALARMiator | Alarmierungsregeln',
                                            navarea: 'alarm',
                                            avatarpath: req.session.avatar,
                                            name: req.session.name,
                                            organizationname: req.session.organization,
                                            session: req.session,
                                            basedata: rowsBasedata,
                                            gear: rowsAssets,
                                            alarmableGroups: rowsAlarmableGroups,
                                            alarmableGearGroups: rowsAlarmableGearGroups,
                                            rule: rowsAlertRule[0],
                                            activeGroups: rowsActiveGroups,
                                            activeGearGroups: rowsActiveGearGroups,
                                            activeFilters: rowsActiveFilters
                                        })
                                    })
                                })
                            })
                        })
                    })
                })
            })
        })
    },
    editRule: (req, res) => {
        var alertRuleId = req.params.id;
        var alertrules = new Alertrules();
        alertrules.editrule(alertRuleId, req.body, (successRule) => {
            if (successRule) {
                alertrules.renderRuleFilters(req.body, alertRuleId, (successFilters) => {
                    alertrules.renderRuleGroups(req.body, alertRuleId, (successGroups) => {
                        if (successGroups) {
                            alertrules.renderRuleGearGroups(req.body, alertRuleId, (successGearGroups) => {
                                if (successGearGroups) {
                                    res.redirect('/alarm/rules/list?edit=rule');
                                } else {
                                    console.log('Error editing alert rule with gear groups');
                                    res.redirect('/alarm/rules/list');
                                }
                            })
                        } else {
                            console.log('Error editing alert rule with groups');
                            res.redirect('/alarm/rules/list');
                        }
                    })
                })
            } else {
                res.redirect('/alarm/rules/list');
            }
        })
    },
    removeRule: (req, res) => {
        var alertRuleId = req.params.id;
        var alertrules = new Alertrules();
        alertrules.removeAlertRule(alertRuleId, (successRemove) => {
            res.redirect('/alarm/rules/list?delete=rule');
        })
    },
    toggleRule: (req, res) => {
        let ruleId = req.params.id;
        let state = req.params.state;

        console.log('Rule: ' + ruleId + ' state: ' + state);
        var alertrules = new Alertrules();
        if (state === '1') {
            console.log('activating Rule');
            alertrules.activateRule(ruleId, (success) => {
                res.redirect('/alarm/rules/list?edit=rule');
            })
        } else {
            console.log('deactivating');
            alertrules.deactivateRule(ruleId, (success) => {
                res.redirect('/alarm/rules/list?edit=rule');
            })
        }
    },
    testModalChooser: (req, res) => {
        var ruleId = req.params.id;
        var alertrules = new Alertrules();
        alertrules.getRuleWithId(ruleId, (rowsAlertRules) => {
            var operations = new Operations(req.app.logger);
            operations.getListOfOperations((rowsOperations) => {
                res.render('alarm/rules_test_modal', {
                    title: 'ALARMiator | Auswahl Alarmierung',
                    navarea: 'alarm',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    operations: rowsOperations,
                    rule: rowsAlertRules[0]
                });
            })
        });
    },
    testAllModalChooser: (req, res) => {
        var operations = new Operations(req.app.logger);
        operations.getListOfOperations((rowsOperations) => {
            res.render('alarm/rules_testall_modal', {
                title: 'ALARMiator | Auswahl Alarmierung',
                navarea: 'alarm',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                operations: rowsOperations,
            });
        })
    },
    loadOperationDetailsForTest: (req, res)  => {
        var oId = req.params.id;
        let alertrules = new Alertrules();
        alertrules.getTestOperationHTML(oId, (HTML) => {
            res.send(HTML);
        })
    },
    testRuleWithOperation: (req, res) => {
        var operationId = req.body.operationId;
        var ruleId = req.body.ruleId;
        var changedOperation = req.body.changedOperation;

        // Check if user made changed to chosen operation
        if (typeof changedOperation != 'undefined') {
            if (changedOperation === 'on') {
                changedOperation = true;
            } else {
                changedOperation = false;
            }
        } else {
            changedOperation = false;
        }

        console.log('ROUTES-ALARM | Testing alarming rule ' + ruleId + ' with operation ' + operationId + ' | changed: ' + changedOperation);
        
        var ruleengine = new Ruleengine();
        ruleengine.evaluateExistingOperationAgainstRule(operationId, null, ruleId, (resultRule) => {
            console.log('ROUTES-ALARM | testRuleWithOperation | Result Rule-Test: ' + JSON.stringify(resultRule));
            ruleengine.getUsersToAlarm(ruleId, (rowsUsersToAlarm) => {
                ruleengine.getAllUserForRule(ruleId, (rowsUsersPotentiallyAlarmable) => {
                    res.render('alarm/rules_test_modal_result', {
                        title: 'ALARMiator | Regeltest',
                        navarea: 'alarm',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                        filters: resultRule,
                        usersToAlarm: rowsUsersToAlarm,
                        usersPotToAlarm: rowsUsersPotentiallyAlarmable
                    });
                })
            }) 
        })
    },
    testAllRulesWithOperation: (req, res) => {
        var operationId = req.body.operationId;
        console.log('Testing all alarming rules with operation ' + operationId);
        
        var ruleengine = new Ruleengine();
        ruleengine.evaluateOperationAgainstRules(operationId, (resultAllRules) => {
            ruleengine.loadAllUsersToAlarmByEvaluationResult(resultAllRules, (allUsersToAlarm, allUsers) => { 
                console.log('ROUTES-ALARM | testAllRulesWithOperation | loaded users');
                console.log('=============================================');
                console.log(allUsersToAlarm);
                console.log('=============================================');
                if (Array.isArray(resultAllRules)) {
                    res.render('alarm/rules_testall_modal_result', {
                        title: 'ALARMiator | Regeltest',
                        navarea: 'alarm',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                        rules: resultAllRules,
                        usersToAlarm: allUsersToAlarm,
                        usersPotToAlarm: allUsers
                    });
    
                } else {
                    res.redirect('/alarm/rules/list');
                }
            });
        })
    },
    sendTestAlarmForUserAndPlugin: (req, res) => {
        var basedataId = req.params.basedataId;
        var namespace = req.params.namespace;
        req.app.logger.debug('ROUTES-ALARM | sending a test alarm to user with id ' + basedataId + ' using plugin with namespace ' + namespace);
        var alarmInfo = new Operation();
        alarmInfo.setUUID();
        alarmInfo.alarmdate = moment().format('DD.MM.YYYY');
        alarmInfo.alarmtime = moment().format('HH:mm:ss');
        alarmInfo.community = 'Reichenschwand';
        alarmInfo.district = 'Oberndorf';
        alarmInfo.floor = '1. OG';
        alarmInfo.gear = 'FF Reichenschwand\nFL Reichenschwand 60-40-01\nFL Reichenschwand 60-48-01';
        alarmInfo.gkx = '4452602.63';
        alarmInfo.gky = '5487068.41';
        alarmInfo.lat = 49.5140795;
        alarmInfo.lon = 11.3713787;
        alarmInfo.ils = 'ILS Nürnberg';
        alarmInfo.keywordCategory = 'INF';
        alarmInfo.keywordId = 'T2725';
        alarmInfo.keywordName = 'Probealarm';
        alarmInfo.keywordRaw = '#T2725#INF#Probealarm'
        alarmInfo.location = 'Reichenschwand';
        alarmInfo.message = 'Probealarm ausgelöst durch Admin in ALARMiator Server. KEIN EINSATZ!';
        alarmInfo.object = 'Rathaus Reichenschwand';
        alarmInfo.operationSourceConnector = 'alarmiator';
        alarmInfo.operationState = 1;
        alarmInfo.operationnumber = 'T 0.0 000000 00000';
        alarmInfo.rawdata = '';
        alarmInfo.section = 'B14 Ortsgebiet';
        alarmInfo.street = 'Nürnberger Straße 20';
        alarmInfo.subject = 'INF PROBEALARM';
        alarmInfo.zveis = '24132, 24435, 24065';

        var alarm = new Alarm(req.app.logger);
        alarm.sendTestAlarmForBasedataAndNamespace(basedataId, namespace, alarmInfo, (success) => {
            //res.redirect('/alarm/matrix');
            res.status(200).send({ success: 1 });
        })
    },
    getFeedbackStatesList: (req, res) => {
        req.app.logger.debug('ROUTES-ALARM | loading list of feedback states');
        var feedbackstates = new Feedbackstates(req.app.logger);
        feedbackstates.getAllStates((statesRows) => {
            res.render('alarm/states_list', {
                title: 'ALARMiator | Feedback States',
                navarea: 'alarm',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                states: statesRows
            });
        });
    },
    activateFeedbackState: (req, res) => {
        let stateId = req.params.stateid;
        let feedbackstates = new FeedbackStates(req.app.logger);
        feedbackstates.setActive(stateId, (success) => {
            res.redirect('/alarm/states/list?edit=alarmstate');
        })
    },
    deactivateFeedbackState: (req, res) => {
        let stateId = req.params.stateid;
        let feedbackstates = new FeedbackStates(req.app.logger);
        feedbackstates.setInactive(stateId, (success) => {
            res.redirect('/alarm/states/list?edit=alarmstate');
        })
    },
    orderUpState: (req, res) => {
        let stateId = req.params.id;
        let feedbackstates = new FeedbackStates(req.app.logger);
        feedbackstates.orderUp(stateId, (success) => {
            res.redirect('/alarm/states/list?edit=alarmstate');
        })
    },
    orderDownState: (req, res) => {
        let stateId = req.params.id;
        let feedbackstates = new FeedbackStates(req.app.logger);
        feedbackstates.orderDown(stateId, (success) => {
            res.redirect('/alarm/states/list?edit=alarmstate');
        })
    },
    getAlarmableAssetsList: (req, res) => {
        let assets = new assetshelper(req.app.logger);
        assets.getAllAssetsWhichCanBeAlarmed((resultAssets) => {
            res.render('alarm/alarmable_assets_list.ejs', {
                title: 'ALARMiator | Alarmierbare Einsatzmittel',
                navarea: 'alarm',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                assets: resultAssets
            });
        });
    },
    setGearState: (req, res) => {
        var assetId = req.params.id;
        var state = req.params.state;
        var assets = new Assets(req.app.logger);

        statusMap = {
            '32770': 'Priorisierter Sprechwunsch',
            '32771': 'Einsatzbereit über Funk',
            '32772': 'Einsatzbereit auf Wache',
            '32773': 'Einsatz übernommen',
            '32774': 'Am Einsatzort eingetroffen',
            '32775': 'Sprechwunsch (einsatzbezogen)',
            '32776': 'Nicht Einsatzbereit',
            '32777': '7',
            '32778': '8',
            '32779': 'Fremdanmeldung'
        }
    
        statusMappingToStatusShort = {
            '32770': 0,
            '32771': 1,
            '32772': 2,
            '32773': 3,
            '32774': 4,
            '32775': 5,
            '32776': 6,
            '32777': 7,
            '32778': 8,
            '32779': 9
        }

        assets.getAssetwithId(assetId, (rowsAssets) => {
            if (rowsAssets != null) {
                if (rowsAssets[0].issi.length > 0) {
                    var stateInfo = {
                        issi: rowsAssets[0].issi,
                        radioStatus: 'manuell',
                        radioStatusHumanReadable: 'manuell',
                        radioStatusShort: state
                    };
                    global.plgManager.event_new_state(stateInfo);
                    res.redirect('/alarm/gear/list?edit=gear');
                }
            }
        })
        
    },
    getWallboardsList: (req, res) => {
        res.render('alarm/wallboards_list.ejs', {
            title: 'ALARMiator | Wallboardprofile',
            navarea: 'alarm',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getWallboardsListData: (req, res) => {
        let wallboards = new Wallboards(req.app.logger);
        wallboards.getListOfWallboards((rowsWallboards) => {
            res.json({ data: rowsWallboards })
        })
    },
    addWallboardPage: (req, res) => {
        var organizations = new Organizations(req.app.logger);
        organizations.getListOfOrganizations((rowsOrganizations) => {
            var qualifications = new Qualifications(req.app.logger);
            qualifications.getListOfQualifications((rowsQualifications) => {
                res.render('alarm/wallboard_add', {
                    title: 'ALARMiator | Alarmierungsregeln',
                    navarea: 'alarm',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    organizations: rowsOrganizations,
                    qualifications: rowsQualifications
                });
            })
        })
    },
    addWallboard: (req, res) => {
        var qualifications = new Qualifications(req.app.logger);
        var wallboards = new Wallboards(req.app.logger);
        wallboards.addWallboard(req.body, (successWallboard, wallboardId) => {
            if (successWallboard) {
                qualifications.assignQualificationsToWallboard(req.body.qualifications, wallboardId, (successQualifications) => {
                    if (successQualifications) {
                        res.redirect('/alarm/wallboards/list?add=wallboard');
                    } else {
                        res.redirect('/alarm/wallboards/list');
                    }
                })
            } else {
                res.redirect('/alarm/wallboards/list');
            }
        })
    },
    removeWallboard: (req, res) => {
        var wallboardId = req.params.id;
        var wallboards = new Wallboards(req.app.logger);
        wallboards.deleteWallboard(wallboardId, (success) => {
            res.redirect('/alarm/wallboards/list?delete=wallboard');
        })
    },
    editWallboardPage: (req, res) => {
        var wallboardId = req.params.id;
        var wallboards = new Wallboards(req.app.logger);
        global.configuration.getValueForKeyword('externalfqdn', (externalfqdn) => {
            wallboards.getWallboardForId(wallboardId, (rowsWallboard) => {
                wallboards.getQualificationsForWallboard(wallboardId, (rowsSetQualifications) => {
                    var organizations = new Organizations(req.app.logger);
                    organizations.getListOfOrganizations((rowsOrganizations) => {
                        var qualifications = new Qualifications(req.app.logger);
                        qualifications.getListOfQualifications((rowsQualifications) => {
                            res.render('alarm/wallboard_edit', {
                                title: 'ALARMiator | Alarmierungsregeln',
                                navarea: 'alarm',
                                avatarpath: req.session.avatar,
                                name: req.session.name,
                                organizationname: req.session.organization,
                                session: req.session,
                                organizations: rowsOrganizations,
                                qualifications: rowsQualifications,
                                setQualifications: rowsSetQualifications,
                                wallboard: rowsWallboard[0],
                                externalfqdn: externalfqdn
                            });
                        })
                    })
                })
            })
        });
        
    },
    editWallboard: (req, res) => {
        var wallboardId = req.params.id;
        var wallboards = new Wallboards(req.app.logger);
        wallboards.editWallboard(wallboardId, req.body, (successEdit) => {
            var qualifications = new Qualifications(req.app.logger);
            qualifications.assignQualificationsToWallboard(req.body.qualifications, wallboardId, (successQualifications) => {
                if (successQualifications) {
                    global.plgManager.event_wallboard_config_changed(wallboardId);
                    res.redirect('/alarm/wallboards/list?edit=wallboard');
                } else {
                    res.redirect('/alarm/wallboards/list');
                }
            })
        })
    },
    toggleState: (req, res) => {
        var wallboardId = req.params.id;
        var state = req.params.state;
        console.log('state: ' + state + ' wallboardid: ' + wallboardId);
        var wallboards = new Wallboards(req.app.logger);
        if (parseInt(state) === 1) {
            wallboards.setActive(wallboardId, (success) => {
                res.redirect('/alarm/wallboards/list?edit=wallboard');
            })
        } else if (parseInt(state) === 0) {
            wallboards.setInactive(wallboardId, (success) => {
                res.redirect('/alarm/wallboards/list?edit=wallboard');
            })
        } else {
            res.redirect('/alarm/wallboards/list');
        }
    },
    recalculateToken: (req, res) => {
        let wbId = req.params.id;
        let wallboard = new Wallboards(req.app.logger);
        wallboard.recalculateToken(wbId, (success) => {
            res.redirect(`${req.headers.referer}?edit=wallboard`);
        })
    },
    gearGroupsList: (req, res) => {
        res.render('alarm/gear_groups_list.ejs', {
            title: 'ALARMiator | Einsatzmittelgruppen',
            navarea: 'alarm',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    gearGroupsListData: (req, res) => {
        var geargroups = new Geargroups(req.app.logger);
        geargroups.getlistOfGroupsForUI((rowsGearGroups) => {
            res.json({ data: rowsGearGroups })
        })
    },
    getGearGroupEditForm: (req, res) => {
        var id = req.params.id;
        var organizations = new Organizations(req.app.logger);
        organizations.getListOfOrganizations((rowsOrganizations) => {
            var geargroups = new Geargroups(req.app.logger);
            geargroups.getGroupForId(id, (rowsGearGroups) => {
                geargroups.getGearForGroupId(id, (rowsGear) => {
                    res.render('alarm/gear_groups_edit.ejs', {
                        title: 'ALARMiator | Einsatzmittelgruppe bearbeiten',
                        navarea: 'alarm',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                        group: rowsGearGroups[0],
                        existingGear: rowsGear,
                        orgs: rowsOrganizations
                    });
                })
            })
        })
    },
    gearGroupEdit: (req, res) => {
        var gId = req.params.id;
        var groupName = req.body.groupname;
        var shortName = req.body.shortName;
        var gearList = req.body.gearList;
        var oId = req.body.organizationid;

        // Check if gearlist is filled
        var gearlistArray = gearList.split(',');
        var geargroups = new Geargroups(req.app.logger);
        geargroups.updateGroup(gId, groupName, oId, shortName, gearlistArray, (success) => {
            res.redirect('/alarm/gear/groups/list?edit=geargroup');
        })
    },
    gearGroupsAddPage: (req, res) => {
        var organizations = new Organizations(req.app.logger);
        organizations.getListOfOrganizations((rowsOrganizations) => {
            res.render('alarm/gear_groups_add.ejs', {
                title: 'ALARMiator | Einsatzmittelgruppe erstellen',
                navarea: 'alarm',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                orgs: rowsOrganizations
            });
        });
    },
    gearGroupsAdd: (req, res) => {
        var geargroups = new Geargroups(req.app.logger);
        var groupName = req.body.groupname;
        var shortName = req.body.shortName;
        var oId = req.body.organizationid;
        var gearList = req.body.gearList;
        // Check if gearlist is filled
        var gearlistArray = gearList.split(',');

        geargroups.addGroupforOrganization(groupName, oId, shortName, gearlistArray, (success) => {
            res.redirect('/alarm/gear/groups/list?add=geargroup');
        })
    },
    gearsGroupDelete: (req, res) => {
        var id = req.params.id;
        var geargroups = new Geargroups(req.app.logger);
        geargroups.deleteGroup(id, (success) => {
            res.redirect('/alarm/gear/groups/list?delete=geargroup');
        })
    },
    getGearForOrganization: (req, res) => {
        var orgId = req.params.id;
        var assets = new Assets(req.app.logger);
        assets.getAllAssetsWhichCanBeAlarmedForOrganization(orgId, (rowsGear) => {
            res.json(rowsGear);
        })
    },
    getGearGroupMembersAsHTML: (req, res) => {
        var gId = req.params.id;
        let geargroups = new Geargroups(req.app.logger);
        geargroups.getGearForGroupId(gId, (rowsGear) => {
            var retHTML = '';
            if (rowsGear.length > 0) {
                console.log(rowsGear)
                rowsGear.forEach(gear => {
                    retHTML += `${gear.gearName}</br>`
                });
            } else {
                retHTML = 'keine Mitglieder';
            }
            res.send(retHTML);
        })
    }
    
};



