const { glob } = require('glob');
const moment = require("moment")
const TestAlarmManager = require('../../models/testalarmmanager');
let operationsHelper = require('../../models/operationshelper');
let Geardirectory = require('../../models/geardirectory');
const Configuration = require('../../models/configuration');
const FeedbackStore = require('../../models/core/feedbackstore');

module.exports={

    /**
     * render testalarms list to manage testalarms
     * @param {*} req 
     * @param {*} res 
     */
    GetTestAlarmMainPage: (req,res) => {
        req.app.logger.debug('TESTALARM | loading testalarmmainpage');
        global.testalarmmanager.GetTestalarms((jobs) =>{
            res.render('alarm/testalarms_managment', {
                title: 'ALARMiator | Alarm | Probealarmierungen',
                navarea: 'alarm',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                jobs: jobs
            });
        });
    },

    /**
     * render the testalarms operations list which shows all fired testalarms from the operationstore in the coredb
     * @param {*} req 
     * @param {*} res 
     */
    getTestalarmsOperationsList: (req, res) => {
        res.render('alarm/testalarms_list.ejs', {
            title: 'ALARMiator | Einsätze',
            navarea: 'alarm',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        })
    },
    getTestalarmsOperationsListData: (req, res) => {
        let operations = new operationsHelper(req.app.logger);
        operations.getListOfOperations((result) => {
            res.json({ 
                data: result
                    .filter(r => r.isTest === 1)
                    .map(alarm => {
                        // map datetime to SQL format, easy to parse and sort in the frontend
                        alarm.datetime = moment(`${alarm.alarmdate} ${alarm.alarmtime}`, "DD.MM.YYYY HH:mm:ss").format("YYYY-MM-DD HH:mm:ss")
                        return alarm
                    }) 
            })
        })
    },
    deleteTestalarmOperation: (req, res) => {
        let operation = req.params.oId;
        let operations = new operationsHelper(req.app.logger);
        operations.deleteOperation(operation, (success) => {
            res.redirect('/alarm/testalarms/operations?delete=testalarm')
        })
    },
    /**
     * show details page for testalarm
     * @param {*} req 
     * @param {*} res 
     */
    showTestalarmOperationDetails: (req, res) => {
        let operation = req.params.oId;
        let opId = req.params.oId;
        let configuration = new Configuration(req.app.logger);
        let geardirectory = new Geardirectory(req.app.logger);
        let operations = new operationsHelper(req.app.logger);
        operations.getOperation(operation, (row) => {
            if (row.length > 0) {
                configuration.getValueForKeyword('ilsmessageseparator', (msgSeparator) => {
                    if (msgSeparator === null) {
                        msgSeparator = '|';
                    } else {
                        msgSeparator = '' + msgSeparator;
                        if (msgSeparator.length === 0) {
                            msgSeparator = '|';
                        }
                    }
                    var operation = row[0];
                    // render Linebreaks in ILS message
                    operation.message = operation.message.split(msgSeparator).join('<br>');
                    // calculate gear types for operation gearlistCalc
                    var newGearlistCalc = [];
                    if (typeof operation.gearListCalc !== 'undefined') {
                        if (operation.gearListCalc !== null) {
                            var gearlist = JSON.parse(operation.gearListCalc);
                            gearlist.forEach((gear, index) => {
                                newGearlistCalc.push(gear + ' <span class="text-muted">' + geardirectory.getLongNameForGearString(gear)) + '</span>';
                            });
                            operation.gearListCalc = JSON.stringify(newGearlistCalc);
                        }
                    }
                    operations.getSubOperationsForOperation(opId, (rowsSubOperations) => {
                        var feedbackstore = new FeedbackStore(req.app.logger);
                        feedbackstore.getFeedbackForOperationWithId(opId, (feedbackRows) => {
                            res.render('alarm/testalarms_operation_details.ejs', {
                                title: 'ALARMiator | Probealarm-Details',
                                navarea: 'alarm',
                                avatarpath: req.session.avatar,
                                name: req.session.name,
                                organizationname: req.session.organization,
                                session: req.session,
                                operation: operation,
                                subOperations: rowsSubOperations,
                                feedback: feedbackRows
                            });
                        })
                    })
                })
            }
        })
    },

    /**
     * render page to get data for a new testalarm
     * @param {*} req 
     * @param {*} res 
     */
    showNewTestAlarmAddPage: (req, res) => {
        req.app.logger.debug('TESTALARM | loading testalarmaddpage');
        res.render('alarm/testalarms_add', {
            title: 'ALARMiator | Alarm | Probealarmierungen',
            navarea: 'alarm',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },

    /**
     * add a new testalarm to the testalarmmanager
     * @param {*} req 
     * @param {*} res 
     */
    AddNewTestAlarmPage: (req, res) => {
        req.app.logger.debug('TESTALARM | add a testalarm with the data from testalarmaddpage');
        
        var cron_schedule = "";
        //cron_schedule += req.body.sec;
        cron_schedule += '0';
        cron_schedule += " ";
        cron_schedule += req.body.min;
        cron_schedule += " ";
        cron_schedule += req.body.hour;
        cron_schedule += " ";
        //cron_schedule += req.body.day;
        cron_schedule += '*';
        cron_schedule += " ";
        //cron_schedule += req.body.month;
        cron_schedule += '*';
        cron_schedule += " ";
        //cron_schedule += req.body.dayofweek + req.body.dayofweekinmonth;
        cron_schedule += req.body.dayofweek;
        
        //get operation
        var operation = global.testalarmmanager.GetOperationFromReqBody(req.body);

        global.testalarmmanager.AddNewTestalarm(req.body.testalarm_name, cron_schedule, operation, (err) =>{
            if(null === err){
                res.redirect('/alarm/testalarms?add=testalarm');
            }
            else {
                // implement errorhandling here!
            }
        });
    },

    /**
     * render the page for editing a existing testalarm
     * @param {} req 
     * @param {*} res 
     */
    showTestAlarmEditPage(req, res){
        req.app.logger.debug('TESTALARM | loading testalarm editpage');
        global.testalarmmanager.GetTestalarm(req.params.name, (job, err) => {
            if(err)
            {
                res.redirect('/alarm/testalarms');
            } else {
                var cronschedule = global.testalarmmanager.GetCronDataObject(job.cronjob.cronTime.source);
                res.render('alarm/testalarms_edit', {
                    title: 'ALARMiator | Alarm | Probealarmierungen',
                    navarea: 'alarm',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    job: job,
                    cronschedule: cronschedule,
                    state: req.params.state
                });
            }
        });
    },

    /**
     * process data from TestAlarmEditPage
     * @param {*} req 
     * @param {*} res 
     */
    TestAlarmEditPage(req, res){
        req.app.logger.debug('TESTALARM | edit a testalarm with the data from testalarmeditpage');
        
        var cron_schedule = "";
        //cron_schedule += req.body.sec;
        cron_schedule += '0';
        cron_schedule += " ";
        cron_schedule += req.body.min;
        cron_schedule += " ";
        cron_schedule += req.body.hour;
        cron_schedule += " ";
        //cron_schedule += req.body.day;
        cron_schedule += '*';
        cron_schedule += " ";
        //cron_schedule += req.body.month;
        cron_schedule += '*';
        cron_schedule += " ";
        //cron_schedule += req.body.dayofweek + req.body.dayofweekinmonth;
        cron_schedule += req.body.dayofweek;
        
        //get operation
        var operation = global.testalarmmanager.GetOperationFromReqBody(req.body);

        global.testalarmmanager.EditTestalarm(req.body.testalarm_name, cron_schedule, operation, req.params.state, (err) =>{
            if(null === err){
                res.redirect('/alarm/testalarms?edit=testalarm');
            }
            else {
                // implement errorhandling here!
            }
        });
    },

    /**
     * delete a testalarm with a defined name
     * @param {*} req 
     * @param {*} res 
     */
    deleteTestalarm: (req, res) => {
        req.app.logger.debug('TESTALARM | removeing testalarm');
        global.testalarmmanager.RemoveTestalarm(req.params.name, (err) =>{
            if(null !== err){
                // add errorhandling here!
                res.redirect('/alarm/testalarms');            
            }
            else{
                res.redirect('/alarm/testalarms?delete=testalarm');            
            }
        });
    },

    /**
     * set a testalarm activ od deactivate it
     * @param {*} req 
     * @param {*} res 
     */
    toggleTestalarm: (req, res) =>{
        if(req.params.state === '1'){
            req.app.logger.debug('TESTALARM | activate testalarm');
            global.testalarmmanager.StartTestalarm(req.params.name, (err) => {
                if(null === err){
                    res.redirect('/alarm/testalarms?edit=testalarm');            
                }
                else{
                    // implement errorhandling here
                    res.redirect('/alarm/testalarms');            
                }
            });
        }
        else{
            req.app.logger.debug('TESTALARM | deaktivate testalarm');
            global.testalarmmanager.StopTestalarm(req.params.name, (err) => {
                if(null === err){
                    res.redirect('/alarm/testalarms?edit=testalarm');            
                }
                else{
                    // implement errorhandling here
                    res.redirect('/alarm/testalarms');            
                }
            });
        }
    },

}