// uuid
const uuid = require('uuid').v1;

// for date handling
var moment = require('moment');
moment().format();

// helpers
let assetsgroupshelper = require('../../models/assetsgroups');
let assetshelper = require('../../models/assets');
let assetsparameterhelper = require('../../models/assetsparametershelper');
let warehousehelper = require('../../models/warehouses');
let Suppliers = require('../../models/suppliers');
let Organisations = require('../../models/organizations');
let momentHelperObj = require('../../models/momenthelper');
let momenthelper = new momentHelperObj();

/**
 * Module offers routes for assets handling
 * @module routes/assets
 */
module.exports = {
    /**
     * Route loading Assets List.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    getAssetsList: (req, res) => {
        let assets = new assetshelper(req.app.logger);
        assets.getListOfAssets((resultAssets) => {
            res.render('assets/assets.ejs', {
                title: 'ALARMiator | Geräte',
                navarea: 'assets',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                assets: resultAssets,
                groupName: 'alle'
            });
        });
    },
    /**
     * Route loading Assets List for an assets group.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    getAssetsListForGroup: (req, res) => {
        //getListOfAssetsForGroup
        let groupId = req.params.id;
        let assets = new assetshelper(req.app.logger);
        let groups = new assetsgroupshelper(req.app.logger);
        groups.getAssetGroup(groupId, (groupRows) => {
            assets.getListOfAssetsForGroup(groupId, (resultAssets) => {
                res.render('assets/assets.ejs', {
                    title: 'ALARMiator | Geräte',
                    navarea: 'assets',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    assets: resultAssets,
                    groupName: groupRows[0].name
                });
            });
        });
    },
    /**
     * Route loading Page for adding a new asset.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    addAssetsPage: (req, res) => {
        let groups = new assetsgroupshelper(req.app.logger);
        let warehouses = new warehousehelper(req.app.logger);
        let suppliers = new Suppliers();
        let organisations = new Organisations(req.app.logger);
        let aUUID = uuid();
        groups.getListOfAssetGroups((resultGroups) => {
            let assets = new assetshelper(req.app.logger);
            assets.getListOfAssetsStates((resultStates) => {
                warehouses.getListOfWarehouses((resultWarehouses) => {
                    suppliers.getSuppliers((resultSuppliers) => {
                        organisations.getListOfOrganizations((resultOrganizations) => {
                            //console.log(resultStates.length);
                            res.render('assets/assets_add.ejs', {
                                title: 'ALARMiator',
                                navarea: 'assets',
                                avatarpath: req.session.avatar,
                                name: req.session.name,
                                organizationname: req.session.organization,
                                session: req.session,
                                groups: resultGroups,
                                states: resultStates,
                                warehouses: resultWarehouses,
                                suppliers: resultSuppliers,
                                uuid: aUUID,
                                organizations: resultOrganizations,
                                message: ''
                            });
                        })
                    })
                })
            })
        });
    },
    /**
     * Route storing a new asset to core database and redirecting to asstes list afterwards.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    addAssets: (req, res) => {
        var assets = new assetshelper(req.app.logger);
        assets.addAsset(req.body, (success) => {
            if (success) {
                res.redirect('/assets/assets/list?add=asset');
            } else {
                res.redirect('/assets/assets/list');
            }
        })
    },
    /**
     * Route for edit Page of an asset.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    editAssetsPage: (req, res) => {

        let assetId = req.params.id;
        let assets = new assetshelper(req.app.logger);
        let groups = new assetsgroupshelper(req.app.logger);
        let warehouses = new warehousehelper(req.app.logger);
        let organisations = new Organisations(req.app.logger);
        let suppliers = new Suppliers();

        groups.getListOfAssetGroups((resultGroups) => {
            assets.getListOfAssetsStates((resultStates) => {
                warehouses.getListOfWarehouses((resultWarehouses) => {
                    suppliers.getSuppliers((resultSuppliers) => {
                        organisations.getListOfOrganizations((resultOrganizations) => {
                            assets.getAssetwithId(assetId, (assetRow) => {
                                if (assetRow === null) {
                                    req.app.logger.error('ROUTES-ASSETS | Error loading asset information from database for id ' + assetId + ': ' + err.message);
                                    res.redirect('/assets/assets/list');
                                } else {
                                    // Render dates to form representation
                                    if (assetRow[0]['date_bought'] !== null) {
                                        //assetRow[0]['date_bought'] = momenthelper.mysqlDateToForm(assetRow[0]['date_bought']);
                                    }
                                    if (assetRow[0]['date_manufactured'] !== null) {
                                        assetRow[0]['date_manufactured'] = momenthelper.mysqlDateToForm(assetRow[0]['date_manufactured']);
                                    }
                                    if (assetRow[0]['date_putOperational'] !== null) {
                                        assetRow[0]['date_putOperational'] = momenthelper.mysqlDateToForm(assetRow[0]['date_putOperational']);
                                    }
                                    if (assetRow[0]['date_outOfService'] !== null) {
                                        assetRow[0]['date_outOfService'] = momenthelper.mysqlDateToForm(assetRow[0]['date_outOfService']);
                                    }
                                    if (assetRow[0]['date_lastMnt'] !== null) {
                                        assetRow[0]['date_lastMnt'] = momenthelper.mysqlDateToForm(assetRow[0]['date_lastMnt']);
                                    }
                                    if (assetRow[0]['dateNextMaintenance'] !== null) {
                                        //assetRow[0]['dateNextMaintenance'] = momenthelper.mysqlDateToForm(assetRow[0]['dateNextMaintenance']);
                                    }

                                    res.render('assets/assets_edit.ejs', {
                                        title: 'ALARMiator',
                                        navarea: 'assets',
                                        avatarpath: req.session.avatar,
                                        name: req.session.name,
                                        organizationname: req.session.organization,
                                        session: req.session,
                                        groups: resultGroups,
                                        states: resultStates,
                                        warehouses: resultWarehouses,
                                        suppliers: resultSuppliers,
                                        organizations: resultOrganizations,
                                        asset: assetRow[0]
                                    });
                                }
                            })
                        })
                    })
                })
            })
        });
    },
    /**
     * Route for updating an asset in core database and redirecting to assets list afterwards.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    editAsset: (req, res) => {
        let assetId = req.body.assetId;
        let assets = new assetshelper(req.app.logger);
        assets.updateAsset(req.body, (success) => {
            if (success) {
                console.log('editAsset returned with success');
                res.redirect('/assets/assets/list?edit=asset');
            } else {
                res.redirect('/assets/assets/edit/' + assetId);
            }
        })
    },
    /**
     * Route for deleting an asset from core database.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    deleteAsset: (req, res) => {
        var assetId = req.params.id;
        let query = "DELETE FROM assets WHERE id = ?";

        let assets = new assetshelper(req.app.logger);
        assets.removeAsset(assetId, (success) => {
            if (success) {
                res.redirect('/assets/assets/list?delete=asset');
            } else {
                res.redirect('/assets/assets/list');
            }
        })
    },
    /**
     * Route loading Assets Parameters List.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
     getParametersList: (req, res) => {
        res.render('assets/parameters.ejs', {
            title: 'ALARMiator | Inventarparameter',
            navarea: 'assets',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getParametersListData: (req, res) => {
        let assets = new assetshelper(req.app.logger);
        assets.getListOfAssetParameters((resultParameters) => {
            res.setHeader('Content-Type', 'application/json');
            res.json({ data: resultParameters });
        })
    },
    /**
     * Route loading Assets Parameters Tabs List.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
     getParametersTabsList: (req, res) => {
        let assets = new assetshelper(req.app.logger);
        assets.getListOfAssetParametersTabs((resultParametersTabs) => {
            res.render('assets/parameterstabs.ejs', {
                title: 'ALARMiator | Inventarparameter',
                navarea: 'assets',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                assetsparameterstabs: resultParametersTabs
            });
        });
    },
    /**
     * returns assets parameters tabs as json for given assets group
     * @param {object} req 
     * @param {object} res 
     */
    getParametersTabsListForGroup: (req, res) => {
        let groupId = req.params.id;
        let assetsparamshelper = new assetsparameterhelper(req.app.logger);
        assetsparamshelper.getAssetsParametersTabs(groupId, (rowsTabs) => {
            res.json(rowsTabs);
        })
    },
    /**
     * Route for edit Page of an asset parameter tab.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
     editAssetsParameterTabPage: (req, res) => {
        let tabId = req.params.id;
        let groups = new assetsgroupshelper(req.app.logger);
        let paramHelper = new assetsparameterhelper(req.app.logger);

        groups.getListOfAssetGroups((resultGroups) => {
            paramHelper.getAssetsParametersTab(tabId, (rowsTab) => {
                res.render('assets/parameterstabs_edit.ejs', {
                    title: 'ALARMiator',
                    navarea: 'assets',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    assetsGroups: resultGroups,
                    tab: rowsTab[0]
                });
            })
        });
    },
    /**
     * Route for updating of an asset parameter tab.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    editAssetsParameterTab: (req, res) => {
        let paramHelper = new assetsparameterhelper(req.app.logger);
        paramHelper.editAssetsParameterTab(req.body, (success) => {
            res.redirect('/assets/parameterstabs/list?edit=parameterstab');
        })
    },
    /**
     * Route showing form for creating a new assets parameter tab to core database
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
     addAssetsParameterTabPage: (req, res) => {
        let groups = new assetsgroupshelper(req.app.logger);
        groups.getListOfAssetGroups((resultGroups) => {
            res.render('assets/parameterstabs_add.ejs', {
                title: 'ALARMiator',
                navarea: 'assets',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                assetsGroups: resultGroups
            });
        })
    },
    /**
     * Route storing a new assets parameter tab to core database and redirecting to asstes parameter tabs list afterwards.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
     addAssetsParametersTab: (req, res) => {
        let paramHelper = new assetsparameterhelper(req.app.logger);
        paramHelper.addAssetsParametersTab(req.body, (success) => {
            res.redirect('/assets/parameterstabs/list?add=parameterstab');
        })
    },
    /**
     * Route DELETING an assets parameter tab from core database and redirecting to asstes parameter tabs list afterwards.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
     removeAssetsParametersTab: (req, res) => {
        let tabId = req.params.id;
        let paramHelper = new assetsparameterhelper(req.app.logger);
        paramHelper.removeAssetsParametersTab(tabId, (success) => {
            res.redirect('/assets/parameterstabs/list?delete=parameterstab');
        })
    },
    /**
     * Route for adding Page of an asset parameter
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    addParameterPage: (req, res) => {
        let groups = new assetsgroupshelper(req.app.logger);
        let paramHelper = new assetsparameterhelper(req.app.logger);
        groups.getListOfAssetGroups((rowsGroups) => {
            res.render('assets/parameters_add.ejs', {
                title: 'ALARMiator',
                navarea: 'assets',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                assetsGroups: rowsGroups
            });
        })
    },
    /**
     * Adds a new asset parameter to core database and forwards to edit form for finalizing settings if of type select
     * @param {object} req 
     * @param {object} res 
     */
    addParameter: (req, res) => {
        let aphelper = new assetsparameterhelper(req.app.logger);
        aphelper.addAssetParameter(req.body, (success, insertId) => {
            if (success) {
                res.redirect(`/assets/parameters/edit/${insertId}?add=parameter`);
            } else {
                res.redirect('/assets/parameters/list');
            }
        })
    },
    /**
     * Route for edit Page of an asset parameter.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
     editAssetsParameterPage: (req, res) => {
        let paramId = req.params.id;
        let groups = new assetsgroupshelper(req.app.logger);
        let paramHelper = new assetsparameterhelper(req.app.logger);

        paramHelper.getAssetParameter(paramId, (rowsParameter) => {
            if ((rowsParameter != null) && (rowsParameter.length > 0)) {
                let param = rowsParameter[0];
                paramHelper.getSelectableOptionsForParameter(paramId, (rowsOptions) => {
                    paramHelper.getAssetsParametersTabs(param.assetsGroup, (tabsRows) => {
                        groups.getListOfAssetGroups((resultGroups) => {
                            res.render('assets/parameters_edit.ejs', {
                                title: 'ALARMiator',
                                navarea: 'assets',
                                avatarpath: req.session.avatar,
                                name: req.session.name,
                                organizationname: req.session.organization,
                                session: req.session,
                                parameter: param,
                                options: rowsOptions,
                                assetsGroups: resultGroups,
                                tabs: tabsRows
                            });
                        })
                    })
                })
            }
        })
    },
    /**
     * Route for updating an asset parameter.
     * @function
     * @memberof module:routes/assets
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
     editAssetsParameter: (req, res) => {
        console.log(req.body);
        let paramHelper = new assetsparameterhelper(req.app.logger);
        paramHelper.editAssetParameter(req.body, (success) => {
            res.redirect('/assets/parameters/list?edit=parameter');
        })
    },
    addParameterOption: (req, res) => {
        let pId = req.params.pId;
        let label = req.params.label;

        console.log('adding select option with label ' + label + ' for parameter with id ' + pId);

        let paramHelper = new assetsparameterhelper(req.app.logger);
        paramHelper.addSelectableOptionForParameter(pId, label, (success) => {
            
            if (success === null) {
                res.json({state:'error'});
            } else if (success === false) {
                res.json({state:'duplicate'});
            } else if (success === true) {
                res.json({state:'ok'});
            }
        })
    },
    deleteParameterOption: (req, res) => {
        let oId = req.params.id;
        let pId = req.params.pid;
        let paramHelper = new assetsparameterhelper(req.app.logger);
        paramHelper.removeSelectableParameterOption(oId, (success) => {
            res.redirect(`/assets/parameters/edit/${pId}?edit=parameter`);
        })
    }

};