const Suppliers = require('../../models/suppliers');
const Orders = require('../../models/orders');
const GoodsReceipts = require('../../models/goodsreceipts');

/**
 * Module offers routes for goods receipts handling
 * @module routes/goodsreceipts
 */
module.exports = {

    getReceiptList: (req, res) => {
        res.render('assets/receipts.ejs', {
            title: 'ALARMiator | Wareneingang',
            navarea: 'assets',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getReceiptListData: (req, res) => {
        let receipts = new GoodsReceipts();
        receipts.getReceipts((receiptsRows) => {
            res.json({ data: receiptsRows })
        });
    },
    editReceiptPage: (req, res) => {
        var oId = req.params.id;
        let orders = new Orders();
        orders.getOrder(oId, (orderRows) => {
            orders.getOrdersStates((rowsOrdersStates) => {
                orders.getOderPositionsForOrder(oId, (rowsOrderPositions) => {
                    var suppliers = new Suppliers();
                    suppliers.getSuppliers((rowsSuppliers) => {
                        res.render('assets/orders_edit.ejs', {
                            title: 'ALARMiator | Bestellung bearbeiten',
                            navarea: 'assets',
                            avatarpath: req.session.avatar,
                            name: req.session.name,
                            organizationname: req.session.organization,
                            session: req.session,
                            ordersStates: rowsOrdersStates,
                            orderPositions: rowsOrderPositions,
                            suppliers: rowsSuppliers,
                            order: orderRows[0]
                        });
                    });
                })
            });
        })
    },
    editReceipt: (req, res) => {
        var oId = req.params.id;
        let orders = new Orders();
        console.log(req.body);
        orders.editOrder(oId, req.body, (success) => {
            if (success) {
                res.redirect('/assets/orders/list?edit=receipt');
            }
        })
    },
    addReceiptPage: (req, res) => {
        var orders = new Orders();
        orders.getNotFulfilledOrders((rowsOrders) => {
            var suppliers = new Suppliers();
            suppliers.getSuppliers((rowsSuppliers) => {
                res.render('assets/receipts_add.ejs', {
                    title: 'ALARMiator | Wareneingang erfassen',
                    navarea: 'assets',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    orders: rowsOrders,
                    suppliers: rowsSuppliers
                });
            })
        });
    },
    addReceipt: (req, res) => {
        let orders = new Orders();
        orders.addOrder(req.body, (success, orderId) => {
            if (success) {
                // Order has been created, open it again in edit view to add order positions
                res.redirect(`/assets/orders/edit/${orderId}?add=receipt`);
            } else {
                res.redirect('/assets/orders/list');
            }
        })
    },
    deleteReceipt: (req, res) => {
        var oId = req.params.id;
        let orders = new Orders();
        orders.removeOrder(oId, (success) => {
            res.redirect('/assets/orders/list?delete=receipt');
        })

    }


};