let assetsgroupshelper = require('../../models/assetsgroups');
let roleshelper = require('../../models/roles');
const Organizations = require('../../models/organizations');
const assetgroups = require('../../models/assetsgroups');


/**
 * Module offers routes for assets groups handling
 * @module routes/assetsgroups
 */
module.exports = {
    /**
     * Route loading Assets Groups List.
     * @function
     * @memberof module:routes/assetsgroups
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    getAssetsGroupsList: (req, res) => {
        
        res.render('assets/groups.ejs', {
            title: 'ALARMiator | Gerätegruppen',
            navarea: 'assets',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getAssetsGroupsListData: (req, res) => {
        let assetsgroups = new assetsgroupshelper(req.app.logger);
        assetsgroups.getListOfAssetGroupsForTable((resultAssetsGroups) => {
            res.setHeader('Content-Type', 'application/json');
            res.json({ data: resultAssetsGroups });
        })
    },
    /**
     * Returns assets groups as JSON
     * @param {object} req 
     * @param {object} res 
     */
    getAssetsGroupsAsJSON: (req, res) => {
        let assetsgroups = new assetsgroupshelper(req.app.logger);
        var treeData = [];
        var flatData = [];

        const aNode = {
            text: '',
            href: '',
            tags: [],
            groupId: null,
            parentId: null,
            nodes: []
        }
        assetsgroups.getListOfAssetGroupsForTable((resultAssetsGroups) => {

                // transform rows into object-notation for tree view component
                var transformedData = [];
                resultAssetsGroups.forEach((group) => {
                    var node = {
                        text: group.name,
                        href: '/assets/assets/listForGroup/'+group.id,
                        tags: [group.assetsCount],
                        id: group.id,
                        parentGroupId: group.parentGroupId,
                        nodes: []
                    }
                    transformedData.push(node);
                })

                // now calculate tree as json
                tree = function (data, root) {
                    var r = [], o = {};
                    data.forEach(function (a) {
                        if (o[a.id] && o[a.id].nodes) {
                            a.nodes = o[a.id] && o[a.id].nodes;
                        }
                        o[a.id] = a;
                        if (a.parentGroupId === root) {
                            r.push(a);
                        } else {
                            o[a.parentGroupId] = o[a.parentGroupId] || {};
                            o[a.parentGroupId].nodes = o[a.parentGroupId].nodes || [];
                            o[a.parentGroupId].nodes.push(a);
                        }
                    });
                    return r;
                }(transformedData, null);

                const removeEmptyNodes = (obj) => {
                    Object.keys(obj).forEach(key =>
                      (key === 'nodes' && obj[key].length === 0) && delete obj[key] ||
                      (obj[key] && typeof obj[key] === 'object') && removeEmptyNodes(obj[key])
                    );
                    return obj;
                };
               
                var node = {
                    text: 'Alle',
                    href: '/assets/assets/list',
                    tags: [],
                    id: 0,
                    parentGroupId: null,
                    nodes: []
                }

                tree.unshift(node);
                
                var cleanedTree = removeEmptyNodes(tree);

                res.json(cleanedTree);
            })
    },
    /**
     * Route loading Assets Groups add page.
     * @function
     * @memberof module:routes/assetsgroups
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    addAssetsGroupPage: (req, res) => {
        let roles = new roleshelper(req.app.logger);
        let organizations = new Organizations(req.app.logger);

        organizations.getListOfOrganizations((resultOrganizations) => {
            roles.getListOfRoles((resultRoles) => {
                res.render('assets/groups_add.ejs', {
                    title: 'ALARMiator',
                    navarea: 'assets',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    roles: resultRoles,
                    organizations: resultOrganizations,
                    message: ''
                });
            });
        })
    },
    /**
     * Route loading Assets Groups add page based on existing group (add new group as copy of).
     * @function
     * @memberof module:routes/assetsgroups
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    addAssetsGroupAsCopyPage: (req, res) => {
        let copyFromGroupId = req.params.id;
        let roles = new roleshelper(req.app.logger);
        let organizations = new Organizations(req.app.logger);
        var assetsgroups = new assetsgroupshelper(req.app.logger);
        
        assetsgroups.getAssetGroup(copyFromGroupId, (rowsSourceGroup) => {
            organizations.getListOfOrganizations((resultOrganizations) => {
                roles.getListOfRoles((resultRoles) => {
                    res.render('assets/groups_add_copy.ejs', {
                        title: 'ALARMiator',
                        navarea: 'assets',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                        roles: resultRoles,
                        organizations: resultOrganizations,
                        sourceGroup: rowsSourceGroup,
                        message: ''
                    });
                });
            })
        })            
    },
    /**
     * Route adding new Assets Groups and redirecting to list aferwards.
     * @function
     * @memberof module:routes/assetsgroups
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    addAssetsGroup: (req, res) => {
        var assetgroups = new assetsgroupshelper(req.app.logger);
        assetgroups.addAssetsGroup(req.body, (success) => {
            if (success) {
                res.redirect('/assets/groups/list?add=assetgroup');
            } else {
                res.redirect('/assets/groups/list');
            }
        })
    },
    /**
     * Route loading Assets Groups edit page.
     * @function
     * @memberof module:routes/assetsgroups
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    editGroupPage: (req, res) => {
        let groupId = req.params.id;
        var assetsgroups = new assetsgroupshelper(req.app.logger);
        assetsgroups.getAssetGroup(groupId, (rows) => {
            assetsgroups.getListOfAssetGroupsForOrganization(rows[0].organizationId, (rowsGroups) => {
                let roles = new roleshelper(req.app.logger);
                roles.getListOfRoles((resultRoles) => {
                    let organizations = new Organizations(req.app.logger);
                    organizations.getListOfOrganizations((resultOrganizations) => {

                        res.render('assets/groups_edit.ejs', {
                            title: 'Edit  Group',
                            navarea: 'assets',
                            avatarpath: req.session.avatar,
                            name: req.session.name,
                            organizationname: req.session.organization,
                            session: req.session,
                            group: rows[0],
                            roles: resultRoles,
                            organizations: resultOrganizations,
                            groupsfororg: rowsGroups,
                            message: ''
                        });
                    });
                });
            });
        })
    },
    /**
     * Route updating Assets Group and redirecting to list afterwards.
     * @function
     * @memberof module:routes/assetsgroups
     * @param {object} req - Express request
     * @param {object} res - Express response.
     */
    editGroup: (req, res) => {
        let groupId = req.params.id;
        let roleId = req.body.roleid;
        let groupName = req.body.groupname;

        var assetsGroups = new assetsgroupshelper(req.app.logger);
        assetsGroups.editAssetsGroup(req.body, groupId, (success) => {
            if (success) {
                res.redirect('/assets/groups/list?edit=assetgroup');
            } else {
                res.redirect('/assets/groups/list');
            }
        })
    },
    /**
     * Route for deletion of an asset group
     * @memberof module:routes/assetsgroups
     * @param {object} req - Express request
     * @param {object} res - Express response.
     * 
     */
    deleteGroup: (req, res) => {
        let groupId = req.params.id;
        var assetsGroups = new assetsgroupshelper(req.app.logger);
        assetsGroups.deleteGroup(groupId, (success) => {
            if (success) {
                res.redirect('/assets/groups/list?delete=assetgroup');
            } else {
                res.redirect('/assets/groups/list');
            }
        })
    }
};