let hydrantshelper = require('../../models/hydrants');
let qrcodehelper = require('../../models/qrcodehelper');

                                    
const uuidv1 = require('uuid').v1;
var moment = require('moment');
moment().format();


module.exports = {
    getHydrantsList: (req, res) => {
        res.render('assets/hydrants.ejs', {
            title: 'ALARMiator | Hydranten',
            navarea: 'assets',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getHydrantsListData: (req, res) => {
        let hydrants = new hydrantshelper(req.app.logger);
        hydrants.getListOfHydrants((resultHydrants) => {
            res.json({ data: resultHydrants })
        });      
    },
    getHydrantsMap: (req, res) => {
        let latitude = req.params.lat;
        let longitude = req.params.lon;

        let hydrants = new hydrantshelper(req.app.logger);
        hydrants.getListOfHydrantsJSON((resultHydrantsJSON) => {
            res.render('assets/hydrants_map.ejs', {
                title: 'ALARMiator | Hydrantenkarte',
                navarea: 'assets',
                subarea: 'hydrants',
                avatarpath: req.session.avatar,
                name: req.session.name, 
                organizationname: req.session.organization,
                session: req.session,
                hydrants: resultHydrantsJSON,
                latitude: latitude,
                longitude: longitude
            })
        })
    },
    addHydrantsPage: (req, res) => {
        var lat = req.params.lat;
        var lon = req.params.lon;


        let hydrants = new hydrantshelper(req.app.logger);
        hydrants.getListOfHydrantTypes((resultHydrantsTypes) => {
            hydrants.getListOfHydrantStates((resultHydrantStates) => {
                res.render('assets/hydrants_add.ejs', {
                    title: 'ALARMiator | Hydranten hinzufügen',
                    navarea: 'assets',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    hydrantTypes: resultHydrantsTypes,
                    hydrantStates: resultHydrantStates,
                    latitude: lat,
                    longitude: lon
                });
            })
        })
    },
    addHydrant: (req, res) => {

        let hydrants = new hydrantshelper(req.app.logger);
        hydrants.addHydrant(req.body, req.files, (success, newID) => {
            req.app.logger.debug('ROUTES-HYDRANTS | generating QR-Code for Hydrant');
            let qrcode = new qrcodehelper(req.app.logger);
            qrcode.generateQrCodeToFileDestination('/assets/hydrants/edit/'+newID, 'public/assets/img/hydrants/qrcodes/'+newID+'.png', (err) => {
                if (err === true) {
                    // QRCode generated
                    //req.app.logger.error('ROUTES-HYDRANTS | Error generating QR-Code: ' + err.message);
                    console.log('ROUTES-HYDRANTS | Error generating QR-Code: ' + err.message);
                } else {
                    hydrants.setQrCodeForHydrantWithId(newID, (success) => {

                    });
                }
                if ((req.body.latitude.length > 0) && (req.body.longitude.length >0)) {
                    res.redirect(`/assets/hydrants/map/${req.body.latitude.length}/${req.body.longitude.length}?add=hydrant`);
                } else {
                    res.redirect('/assets/hydrants/map?add=hydrant');
                }
            })
        })
    },
    deleteHydrant: (req, res) => {
        let hydrantId = req.params.id;
        var hydrants = new hydrantshelper(req.app.logger);
        hydrants.getHydrantDataForId(hydrantId, (rowHydrant) => {
            if (rowHydrant !== null) {
                var image = rowHydrant[0].imgpath;
                if (image !== null) {
                    if (image.length > 0) {
                        console.log('hydrant has image and is set');
                        fs.unlink(`public/assets/img/${image}`, (err) => {
                            if (err) {
                                console.log('ERROR deleting image from filesystem');
                                req.app.logger.error('ROUTES-HYDRANTS | Error deleting image for hydrant: ' + err.message);
                                res.redirect('/assets/hydrants/list');
                            }
                        })
                        hydrants.deleteHydrant(hydrantId, (success) => {
                            res.redirect('/assets/hydrants/list?delete=hydrant');
                        })
                    } else {
                        hydrants.deleteHydrant(hydrantId, (success) => {
                            res.redirect('/assets/hydrants/list?delete=hydrant');
                        })
                    }
                } else {
                    hydrants.deleteHydrant(hydrantId, (success) => {
                        res.redirect('/assets/hydrants/list?delete=hydrant');
                    })    
                }
            } else {
                hydrants.deleteHydrant(hydrantId, (success) => {
                    res.redirect('/assets/hydrants/list?delete=hydrant');
                })
            }
        })
    },
    editHydrantsPage: (req, res) => {
        var id = req.params.id;
        let hydrants = new hydrantshelper(req.app.logger);
        hydrants.getHydrantDataForId(id, (rowHydrant) => {
            console.log(rowHydrant);
            hydrants.getListOfHydrantTypes((resultHydrantsTypes) => {
                hydrants.getListOfHydrantStates((resultHydrantStates) => {
            
                        res.render('assets/hydrants_edit.ejs', {
                            title: 'ALARMiator | Hydranten bearbeiten',
                            navarea: 'assets',
                            avatarpath: req.session.avatar,
                            name: req.session.name,
                            organizationname: req.session.organization,
                            session: req.session,
                            hydrantTypes: resultHydrantsTypes,
                            hydrantStates: resultHydrantStates,
                            hydrant: rowHydrant[0]
                        });
                    })
            })
        })
    },
    editHydrants: (req, res) => {
        let hydrantId = req.params.id;
        let hydrants = new hydrantshelper(req.app.logger);
        hydrants.editHydrant(hydrantId, req.body, req.files, (success) => {
            res.redirect('/assets/hydrants/list?edit=hydrant');
        })
    }
};