const md5 = require('md5');
const Authenticate = require('../models/auth');
const Users = require('../models/users');
const EmailQueue = require('../models/emailqueue');
const EmailTemplates = require('../models/emailtemplates');
const Configuration = require('../models/configuration');


module.exports = {
    getLoginPage: (req, res) => {
        // Check if the user is already logged in
        // if logged in redirect to home dashboard of user
        var auth = new Authenticate(global.app.logger);
        auth.checkForValidSession(req.session, (valid) => {
            if (valid) {
                res.redirect('/home/dashboard');
            } else {
                res.render('login.ejs', {
                    title: 'ALARMiator | Login',
                    redirect: req.query.redirect,
                });
            }
        })
    },
    authenticate: (req, res) => {
        var authenticate = new Authenticate(global.app.logger);
        // parse request body
        const username = req.body.username;
        const password = req.body.password;
        const redirect = req.body.redirect
        const continueRedirectSuffix = redirect === "" ? null : `redirect=${redirect}`
        if (username && password) {
            authenticate.authenticateUser(username, password, (success, userRow, enforcePasswordChange) => {
                if (success) {
                    // Successfully authenticated User
                    // Read user data and store to session
                    const useravatar = userRow.image_data == null ? "/assets/img/stock/userDefault.png" : userRow.image_data
          
                    req.session.loggedin = true;
                    req.session.username = username;
                    req.session.userid = userRow.id;
                    req.session.superuser = userRow.superuser;
                    req.session.basedataId = userRow.basedataid;
                    req.session.organization = userRow.organizationname;
                    req.session.organizationId = userRow.organizationid
                    req.session.avatar = useravatar;
                    req.session.theme = userRow.theme;
                    req.session.name = userRow.firstname + ' ' + userRow.lastname;
                    req.session.enforcePasswordChange = enforcePasswordChange;

                    authenticate.getAccessibleOrganizations(userRow.organizationid, (success, orgRows) => {
                        if (success) {
                            // store all the accessible org IDs in the session
                            req.session.accessibleOrganizations = orgRows.map(row => row.id)
                        }

                        // proceed with regular login process
                        if (userRow.state === 0) {
                            // User is locked! tell him and send him to a locked page
                            res.redirect('/lockedaccount');
                        }
    
                        authenticate.getUsersGroupsForUserWithId(req.session.userid, (accessRows) => {
                            req.session.accessRows = accessRows;
                            if (enforcePasswordChange === true) {
                                // User is enforced to change his password
                                res.redirect(`/changepassword${continueRedirectSuffix != null ? `?${continueRedirectSuffix}` : ""}`);
                            } else {
                                if (redirect !== "") {
                                    // redirect target is provided
                                    res.redirect(redirect)
                                } else {
                                    // redirect user to dashboard
                                    res.redirect('/home/dashboard');
                                }
                                
                            }
                        })
                    })                    
                } else {
                    // Authentication went wrong
                    res.redirect(`/?wp=1${continueRedirectSuffix != null ? `&${continueRedirectSuffix}` : ""}`);
                }
            })
        }
    },
    getChangePasswordPage: (req, res) => {
        const redirect = req.query.redirect
        res.render('login_setNewPassword.ejs', {
            title: 'ALARMiator | Passwort ändern',
            session: req.session,
            redirect,
        });
    },
    getPasswordRecoveryPage: (req, res) => {
        res.render("login_password_recovery.ejs", {
            title: "ALARMiator | Passwort vergessen",
            error: null,
            status: null,
        })
    },
    getPasswordResetPage: (req, res) => {
        // we will check the token when we actually submit the form
        res.render("login_password_reset.ejs", {
            title: "ALARMiator | Password wiederherstellen",
            token: req.query.t,
            status: null,
            error: null,
        })
    },
    sendPasswordRecovery: (req, res) => {
        const email = req.body.email
        // get users ip, we log this in case password reset emails are triggered by others
        const ip = req.headers['x-forwarded-for'] || req.connection.remoteAddress
        if (email == null) {
            // something fishy going on, just redirect to landing/login page
            res.redirect("/")
            return
        }

        var authenticate = new Authenticate(global.app.logger);
        authenticate.getUserInfoByEmail(email, (status, userInfo) => {
            if (status === false || userInfo.userId == null) {
                // failed request, user not found or some other technical error
                res.render("login_password_recovery.ejs", {
                    title: "ALARMiator | Passwort vergessen",
                    error: userInfo.error || 500,  // 500 is technical error
                    status: null,
                })
                return
            }
            
            // found the userId, generate password recovery token, store it and send email
            authenticate.setPasswordRecoveryToken(userInfo.userId, ip, (status, recoveryToken) => {
                if (status === false) {
                    // failed to store password recovery token
                    res.render("login_password_recovery.ejs", {
                        title: "ALARMiator | Passwort vergessen",
                        error: 500,  // technical error
                        status: null,
                    })
                    return
                }

                // get protocol, hostname and port
                const configuration = new Configuration()
                configuration.getWebuiBaseUrl(baseUrl => {
                    // send email to user with password recovery link
                    const data = {
                        firstName: userInfo.firstName,
                        resetToken: recoveryToken,
                        baseUrl,
                    }
                    const emailtemplates = new EmailTemplates(global.app.logger)
                    emailtemplates.renderDataToTemplate(data, "recoverPassword", (template) => {
                        if (template == null) {
                            console.log("MODEL-AUTH | PASSWORD RESET | Could not find email template for password recovery. Check your database migration")
                            res.render("login_password_recovery.ejs", {
                                title: "ALARMiator | Passwort vergessen",
                                error: 500,  // technical error
                                status: null,
                            })
                            return
                        }

                        const emailqueue = new EmailQueue(global.app.logger)
                        emailqueue.addToQueue("ALARMiator: Password Wiederherstellung", template.html, template.plain, [email], (success) => {
                            if (success) {
                                emailqueue.getEmailsToBeSent(null, (rowsEmails) => {
                                    if (rowsEmails != null) {
                                        if (rowsEmails.length > 0) {
                                            console.log('MODEL-AUTH | PASSWORD RESET | Informing about new emails by firing event_email_queue_refreshed with ' + rowsEmails.length + ' new emails');
                                            global.plgManager.event_email_queue_refreshed(rowsEmails);
                                        }
                                    }
                                })
                                res.render("login_password_recovery.ejs", {
                                    title: "ALARMiator | Passwort vergessen",
                                    error: null,
                                    status: true,  // will display success message
                                })
                                return
                            } 
                            console.log('MODEL-AUTH | PASSWORD RESET | Could not add email to email queue for password reset')
                            res.render("login_password_recovery.ejs", {
                                title: "ALARMiator | Passwort vergessen",
                                error: 500,  // technical error
                                status: null,
                            })
                            return
                        })
                    })                    
                })
            })
        })
    },
    resetPassword: (req, res) => {
        const token = req.body.token
        const password = req.body.password
        const passwordCheck = req.body.passwordCheck
        // helper function to avoid repeating the same block (error scenarios)
        const renderError = status => {
            res.render("login_password_reset.ejs", {
                title: "ALARMiator | Passwort wiederherstellen",
                error: status,
                status: null,
                token,
            })
        }
        if (password !== passwordCheck) {
            renderError(409)  // conflict
            return
        }
        if (password.length < 5) {
            renderError(406)  // not acceptable
            return
        }

        var users = new Users(global.app.logger)
        users.getUserByPasswordResetToken(token, (err, userId) => {
            if (err || userId == null) {
                renderError(404)  // token not found or technical error
                return
            }
            // set new password
            users.setPasswordHashForUser(userId, password, success => {
                if (success) {
                    res.render("login_password_reset", {
                        title: "ALARMiator | Passwort wiederherstellen",
                        error: null,
                        status: true,
                        token: null,
                    })
                    return
                }
                // setting password failed
                renderError(500)  // technical error
            })
        })

    },
    setNewPasswordFromStartPassword: (req, res) => {
        const userId = req.session.userid
        var { password, passwordCheck, startPassword, redirect } = req.body
        const errorRedirectSuffix = redirect === "" ? "" : `&redirect=${redirect}`
        
        if (password === passwordCheck) {
            if (password.length > 5) {
                var users = new Users();
                users.getUserForId(userId,(userRows) => {
                    if (userRows !== null) {
                        if (userRows.length > 0) {
                            if (userRows[0].startPassword === startPassword) {
                                // all credentials correct --> update password in Database
                                users.setPasswordHashForUser(userId, password, (success) => {
                                    if (success) {
                                        req.session.enforcePasswordChange = false;
                                        if (redirect === "") {
                                            res.redirect('/home/dashboard');
                                        } else {
                                            res.redirect(redirect)
                                        }                                        
                                    } else {
                                        res.redirect(`/changepassword/?wp=3${errorRedirectSuffix}`);        
                                    }
                                })
                            } else {
                                res.redirect(`/changepassword/?wp=4${errorRedirectSuffix}`);
                            }
                        } else {
                            res.redirect(`/changepassword/?wp=6${errorRedirectSuffix}`);
                        }
                    } else {
                        res.redirect(`/changepassword/?wp=5${errorRedirectSuffix}`);
                    }
                })
            } else {
                res.redirect(`/changepassword/?wp=2${errorRedirectSuffix}`);
            }
        } else {
            res.redirect(`/changepassword/?wp=1${errorRedirectSuffix}`);
        }

    },
    isAuthenticated(req, res, next) {
        // CHECK THE USER STORED IN SESSION FOR A CUSTOM VARIABLE
        // you can do this however you want with whatever variables you set up
        if (req.session.loggedin) {
            return next();
        }

        // IF A USER ISN'T LOGGED IN, THEN REDIRECT THEM SOMEWHERE
        res.redirect(`/?redirect=${req.originalUrl}`);
    },
    getUserGroups(req, res, next) {
        var authenticate = new Authenticate(global.app.logger);
        authenticate.getUsersGroupsForUserWithId(req.session.userid, (accessRows) => {
            req.session.accessRows = accessRows;
        })
        return next();
    },
    checkPermission(grantNamespace) {
        return (req, res, next) => {
            if (grantNamespace == null) {
                // no special permission required for this page
                next()
            } else if (req.session == null) {
                // not logged in
                res.redirect("/")
            } else {
                // helper function to render the no_access page
                const renderNoAccess = () => {

                    if (req.url.endsWith("/data")) {
                        // this is a JSON request (this is a convention !!!)
                        res.status(403).send()  // send an empty response with 403
                        return
                    }
                    
                    // regular HTML request
                    res.render('admin_no_access.ejs', {
                        title: 'ALARMiator | Kein Zugriff',
                        navarea: 'home',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                    });
                }

                if (req.session.superuser === 1) {
                    // super user has access to everything
                    next()
                    return
                }                
                if (req.session.accessRows == null) {
                    // user has no perms
                    renderNoAccess()
                    return
                }
                
                if (req.session.accessRows.map(row => row.grantnamespace).includes(grantNamespace)) {
                    // user has the required permission
                    next()
                } else {
                    // user doesn't have the required permission
                    renderNoAccess()
                }
            }
        }
    },
    endSession: (req, res) => {
        req.session.destroy();
        res.redirect('/');
    },
    changePassword: (req, res) => {
        var username = req.session.username;
        var password = req.body.password;
        var passwordNew = req.body.passwordNew;
        var passwordNewCheck = req.body.passwordNewCheck;

        if (passwordNew === passwordNewCheck) {
            var auth = new Authenticate(global.app.logger);
            auth.changePassword(username, password, passwordNew, (success) => {
                if (success) {
                    // Password successfully changed, log out user and let him login with the new password
                    req.session.destroy();
                    res.redirect('/');
                } else {
                    // Password could not be changed
                    res.redirect('/basedata/dashboard');
                }
            })
        } else {
            res.redirect('/basedata/dashboard');
        }
    },
    getLockedAccountPage: (req, res) => {
        res.render('login_lockedaccount.ejs', {
            title: 'ALARMiator | Account gesperrt',
            session: req.session
        });
    },

};