const fs = require('fs');
const path = require('path');
const uuidv1 = require('uuid').v1;
var moment = require('moment');
moment().format();

let organizations = require('../../models/organizations');
let groupsObj = require('../../models/groups');
let rolesObj = require('../../models/roles');
let ranksObj = require('../../models/ranks');
let Basedata = require('../../models/basedata');
let BasedataHierarchy = require('../../models/basedataHierachy');
let Users = require('../../models/users');
let Qualifications = require('../../models/qualifications');
let Assets = require('../../models/assets');
let Configuration = require("../../models/configuration")
let Importer = require('../../models/core/core_importer');
let Uploads = require('../../models/core/core_uploads');

// Gender Helper
let genderObj = require('../../models/gender');
let genderhelper = new genderObj();
let genderArr = genderhelper.getListOfGenders();

// DATE Handling Helper
let momentHelperObj = require('../../models/momenthelper');
let momenthelper = new momentHelperObj();
// Training Helper
let trainingsObj = require('../../models/trainings');
const qualifications = require('../../models/qualifications');
let trainingshelper = new trainingsObj();



module.exports = {
    getBasedataList: (req, res) => {
        res.render('basedata/basedata.ejs', {
            title: 'ALARMiator | Stammdaten',
            navarea: 'basedata',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        }); 
    },
    getBasedataListData: (req, res) => {
        let basedata = new Basedata(req.app.logger);
        basedata.getListOfBasedataForTable((rows) => {
            res.setHeader('Content-Type', 'application/json');
            const members = req.session.superuser === 1 ? rows : rows.filter(user => req.session.accessibleOrganizations.includes(user.organizationid))
            res.json({ data: members });
        })
    },
    getBasedataForOrganizationJSON: (req, res) => {
        let basedata = new Basedata(req.app.logger);
        basedata.getBasedataForOrganization(req.params.id, (rowsOrganizations) => {
            res.json(rowsOrganizations);
        })
    },
    addBasedataPage: (req, res) => {
        let orgs = new organizations(req.app.logger);
        const configuration = new Configuration(req.app.logger)
        configuration.getBasedataImageSizeLimits(imageLimit => {
            orgs.getListOfOrganizations(function (resultOrgs) {
                let groups = new groupsObj(req.app.logger);
                groups.getListOfGroups((groupsResult) => {
                    res.render('basedata/basedata_add.ejs', {
                        title: 'ALARMiator',
                        navarea: 'basedata',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                        organizations: resultOrgs,
                        genders: genderArr,
                        groups: groupsResult,
                        imageLimits: imageLimit,
                    });
                })
            })
        })
        
    },
    addBasedata: (req, res) => {

        let basedata = new Basedata(req.app.logger);
        basedata.addBasedata(req.body, (success) => {
            if (success) {
                console.log('Successfully added new basedata entra to core database');
                res.redirect('/basedata/basedata/list?add=basedata');
            } else {
                console.log('Could not add new user to core database');
            }
        });
    },
    editBasedataPageRankData: (req, res) => {
        let basedataId = req.params.id;
        let rankshelper = new ranksObj(req.app.logger);
        rankshelper.getListOfRanksForBaseData(basedataId, (resultRankesReferenced) => {
            res.json({ data: resultRankesReferenced })
        })
    },
    editBasedataPageTrainingData: (req, res) => {
        let basedataId = req.params.id;
        trainingshelper.getTrainingHistoryforBasedataId(basedataId, (resultTrainingsHistory) => {
            res.json({ data: resultTrainingsHistory })
        })
    },
    editBasedataPage: (req, res) => {
        let basedataId = req.params.id;
        let orgs = new organizations(req.app.logger);
        let groups = new groupsObj(req.app.logger);
        let roles = new rolesObj(req.app.logger);
        let users = new Users(req.app.logger);
        let qualifications = new Qualifications(req.app.logger);
        let assets = new Assets(req.app.logger);
        const configuration = new Configuration(req.app.logger)
        configuration.getBasedataImageSizeLimits(imageLimit => {
            assets.getAssetsForBasedata(basedataId, (rowsAssets) => {
                orgs.getListOfOrganizations(function (resultOrgs) {
                    var basedata = new Basedata(req.app.logger);
                    basedata.getBaseDataForId(basedataId, (basedataRow) => {
                        if (basedataRow !== null) {
                            // Basedata Entry loaded from database
                            groups.getListOfGroupsForOrganization(basedataRow[0]['organizationid'], (resultGroups) => {
                                groups.getListOfGroupsForBaseData(basedataId, (resultGroupsReferenced) => {
                                    roles.getListOfRoles((resultRoles) => {
                                        roles.getListOfRolesForBaseData(basedataId, (resultRolesReferenced) => {
                                            users.getUserForBasedataId(basedataId, (resultUser) => {
                                                qualifications.getListOfQualifications((resultQualifications) => {
                                                    qualifications.getQualificationsForBasedata(basedataId, (resultQualificationsReferenced) => {

                                                        // Convert date for view layer
                                                        // check for all date rows
                                                        if (!basedataRow[0]['birthday']) basedataRow[0]['birthday'] = '';
                                                        if (!basedataRow[0]['died']) basedataRow[0]['died'] = '';
                                                        if (!basedataRow[0]['date_entry']) basedataRow[0]['date_entry'] = '';
                                                        if (!basedataRow[0]['date_exit']) basedataRow[0]['date_exit'] = '';
                                                        if (!basedataRow[0]['date_active_entry']) basedataRow[0]['date_active_entry'] = '';
                                                        if (!basedataRow[0]['date_active_exit']) basedataRow[0]['date_active_exit'] = '';

                                                        // transform date in form needed in view for date chooser
                                                        if ((basedataRow[0]['birthday'].length > 0) && (basedataRow[0]['birthday'] !== 'null')) {
                                                            basedataRow[0]['birthday'] = momenthelper.mysqlDateToForm(basedataRow[0]['birthday']);
                                                        } else {
                                                            basedataRow[0]['birthday'] = '';
                                                        }
                                                        if ((basedataRow[0]['died'].length > 0) && (basedataRow[0]['died'] !== 'null')) {
                                                            basedataRow[0]['died'] = momenthelper.mysqlDateToForm(basedataRow[0]['died']);
                                                        } else {
                                                            basedataRow[0]['died'] = '';
                                                        }
                                                        if ((basedataRow[0]['date_entry'].length > 0) && (basedataRow[0]['date_entry'] !== 'null')) {
                                                            basedataRow[0]['date_entry'] = momenthelper.mysqlDateToForm(basedataRow[0]['date_entry']);
                                                        } else {
                                                            basedataRow[0]['date_entry'] = '';
                                                        }
                                                        if ((basedataRow[0]['date_exit'].length > 0) && (basedataRow[0]['date_exit'] !== 'null')) {
                                                            basedataRow[0]['date_exit'] = momenthelper.mysqlDateToForm(basedataRow[0]['date_exit']);
                                                        } else {
                                                            basedataRow[0]['date_exit'] = '';
                                                        }
                                                        if ((basedataRow[0]['date_active_entry'].length > 0) && (basedataRow[0]['date_active_entry'] !== 'null')) {
                                                            basedataRow[0]['date_active_entry'] = momenthelper.mysqlDateToForm(basedataRow[0]['date_active_entry']);
                                                        } else {
                                                            basedataRow[0]['date_active_entry'] = '';
                                                        }
                                                        if ((basedataRow[0]['date_active_exit'].length > 0) && (basedataRow[0]['date_active_exit'] !== 'null')) {
                                                            basedataRow[0]['date_active_exit'] = momenthelper.mysqlDateToForm(basedataRow[0]['date_active_exit']);
                                                        } else {
                                                            basedataRow[0]['date_active_exit'] = '';
                                                        }
                                                        // END Convert date for view layer

                                                        res.render('basedata/basedata_edit.ejs', {
                                                            title: 'Edit  basedata',
                                                            navarea: 'basedata',
                                                            avatarpath: req.session.avatar,
                                                            name: req.session.name,
                                                            organizationname: req.session.organization,
                                                            session: req.session,
                                                            basedata: basedataRow[0],
                                                            groups: resultGroups,
                                                            groupsReferenced: resultGroupsReferenced,
                                                            roles: resultRoles,
                                                            rolesReferenced: resultRolesReferenced,
                                                            organizations: resultOrgs,
                                                            audittrail: [],
                                                            user: resultUser,
                                                            qualifications: resultQualifications,
                                                            qualificationsReferenced: resultQualificationsReferenced,
                                                            assets: rowsAssets,
                                                            message: '',
                                                            imageLimits: imageLimit,
                                                        });
                                                    });
                                                })
                                            });
                                        });
                                    });
                                });
                            })
                        }
                    })
                })
            })
        })
    },
    editBasedata: (req, res) => {

        let basedataId = req.params.id;
        var basedata = new Basedata(req.app.logger);
        let groupsArray = req.body.groups;
        let qualificationsArray = req.body.qualifications;
        let rolesArray = req.body.roles;
        let groups = new groupsObj(req.app.logger);
        let roles = new rolesObj(req.app.logger);
        let qualifications = new Qualifications(req.app.logger);

        basedata.editBasedata(req.body, basedataId, (success) => {
            if (success) {
                if (!qualificationsArray) {
                    qualifications.assignQualificationsToBasedata(qualificationsArray, basedataId, (success) => {

                    })
                } else {
                    if (qualificationsArray.length > 0) {
                        // Groups have been selected for Basedata entry
                        qualifications.assignQualificationsToBasedata(qualificationsArray, basedataId, (success) => {
                            //res.redirect('/basedata/basedata/list');
                        })
                    } else {
                        //res.redirect('/basedata/basedata/list');
                        console.log('Qualifications not set or empty');
                    }
                }
                if (!groupsArray) {
                    groups.assignGroupsToUser([], basedataId, (success) => {
                        //res.redirect('/basedata/basedata/list');
                    })
                } else {
                    if (groupsArray.length > 0) {
                        // Groups have been selected for Basedata entry
                        groups.assignGroupsToUser(groupsArray, basedataId, (success) => {
                            //res.redirect('/basedata/basedata/list');
                        })
                    } else {
                        //res.redirect('/basedata/basedata/list');
                        console.log('GROUPS NOT SET or EMPTY');
                    }
                }
                if (!rolesArray) {
                    roles.assignRolesToBasedata([], basedataId, (success) => {
                        res.redirect('/basedata/basedata/list?edit=basedata');
                    })
                } else {
                    if (rolesArray.length > 0) {
                        // Groups have been selected for Basedata entry
                        roles.assignRolesToBasedata(rolesArray, basedataId, (success) => {
                            res.redirect('/basedata/basedata/list?edit=basedata');
                        })
                    } else {
                        //res.redirect('/basedata/basedata/list');
                    }
                }
            } else {
                res.redirect('/basedata/basedata/list?edit=basedata');
            }
        })
    },
    deleteBasedata: (req, res) => {
        let basedataId = req.params.id;
        let basedata = new Basedata(req.app.logger);

        // MARKER_IMAGE_BASE64_MIGRATION
        // deleting the image file on basedata delete can be removed after the migration to base64 images
        basedata.getImageForBasedata(basedataId, (imagename) => {
            if (imagename != null) {
                // Delete image
                fs.unlink(`public/assets/img/${imagename}`, (err) => {
                    if (err) {
                        req.app.logger.error('Could not delete avator for user with basedata id ' + basedataId);
                    }
                });
            };
            basedata.removeBasedata(basedataId, (success) => {
                if (success) {
                    req.app.logger.info('successfully removed basedata entry from core database with basedata id ' + basedataId);
                    res.redirect('/basedata/basedata/list?delete=basedata');
                } else {
                    req.app.logger.error('could not remove basedata entry from core database with basedataid ' + basedataId);
                    res.redirect('/basedata/basedata/list');
                }
            })
        })
    },
    setUserAvatar: (req, res) => {
        req.app.logger.info('UI | Request for updating user avatar.');
       
        let imageData = req.body.image_data;
        let basedataid = req.body.basedataid;

        // send the user's details to the database
        var basedata = new Basedata(req.app.logger);
        basedata.setAvatar(imageData, basedataid, () => {
            res.redirect(`/basedata/basedata/edit/${basedataid}?edit=useravatar`);
        })
    },
    getBaseDataExplorerPage: (req, res) => {
        res.render('basedata/basedata_basedataexplorer.ejs', {
            message: '',
            navarea: 'basedata',
            subarea: 'basedataexplorer',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
            title: 'ALARMiator | Stammdatenexplorer'
        });
    },
    getBaseDataExplorerJSON: (req, res) => {
        let basedataHierarchy = new BasedataHierarchy(req.app.logger);
        basedataHierarchy.generateHierarchyJSON((hierarchy) => {
            res.send(hierarchy);
        });
    },
    addBasedataRankPage: (req, res) => {
        let basedataid = req.params.id;
        let rankshelper = new ranksObj(req.app.logger);
        rankshelper.getListOfRanks((resultRanks) => {
            res.render('basedata/basedata_ranks_add_modal.ejs', {
                message: '',
                basedataid: basedataid,
                title: 'Welcome to Socka | Userlist',
                session: req.session,
                ranks: resultRanks
            });
        });
    },
    addBasedataRank: (req, res) => {

        let basedataId = req.params.id;
        let rankid = req.body.rank;
        let locationvocation = req.body.locationvocation;
        let datevocation = req.body.datevocation;

        // Date handling
        if (datevocation != '') {
            datevocation = momenthelper.formDateToMysql(datevocation);
        }

        var ranks = new ranksObj(req.app.logger);
        ranks.assignRankToBasedata(basedataId, rankid, datevocation, locationvocation, req.session.basedataId, (success) => {
            res.redirect(`/basedata/basedata/edit/${basedataId}?add=basedatarank`);
        })
    },
    deleteBasedataRank: (req, res) => {
        let rankIdtoDelete = req.params.id;
        let basedataId = req.params.basedataid;

        var ranks = new ranksObj(req.app.logger);
        ranks.removeRankForBasedata(rankIdtoDelete, (succes) => {
            res.redirect(`/basedata/basedata/edit/${basedataId}?delete=basedatarank`);
        })
    },
    importBasedataGetAssistant: (req, res) => {
        var step = parseInt(req.params.step);
        if (typeof step === 'undefined') {
            step = 0;
        }
        console.log('IMPORT - STEP ' + step);

        //
        // step based rendering and content handling
        //
        // STEP 1 - Offer form for uploading Excel - File
        if (step === 0) {
            console.log('Starting Import Wizard - Step 1')
            res.render('basedata/basedata_import_step1.ejs', {
                title: 'Mitglieder importieren - Schritt 1',
                navarea: 'basedata',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                message: ''
            });
        }
        // STEP 2 - check excel file and read the sheets in file
        if (step === 1) {
            if (!req.files) {
                return res.status(400).send("No files were uploaded.");
            }
            var uploads = new Uploads();

            let uploadedFile = req.files.fileupload;
            let filename = uploadedFile.name;
            let extension = path.extname(filename);

            console.log('mime-type of uploaded file: ' + uploadedFile.mimetype);
            // check the filetype before uploading it
            if (uploadedFile.mimetype === 'application/vnd.ms-excel') {
                var targetPath = 'uploads/' + uuidv1() + extension;
                uploadedFile.mv(targetPath, (err) => {
                    if (err) {
                        return res.status(500).send(err);
                    }
                    console.log('moved uploaded Excel-File to ' + targetPath);
                    // persist upload in database now, as it could be moved successfully
                    uploads.addUpload('EXCEL-Import Mitglieder', targetPath, filename, uploadedFile.mimetype, 'BASEDATA-IMPORT', null, null, 1, req.session.basedataId, '', (success, uId) => {
                        console.log('registered upload file in uploads-Table with id ' + uId);
                        // Read the sheets in uploaded Excel File
                        var importer = new Importer();
                        importer.openXSLX(targetPath);
                        importer.getSheetnames((sheetnames) => {
                            res.render('basedata/basedata_import_step2.ejs', {
                                title: 'Mitglieder importieren - Schritt 2',
                                navarea: 'basedata',
                                avatarpath: req.session.avatar,
                                name: req.session.name,
                                organizationname: req.session.organization,
                                session: req.session,
                                sheets: sheetnames,
                                uploadId: uId,
                                message: ''
                            });
                        })
                    })
                });
            } else {
                // Wrong mimetype -> redirect to step 1
                res.redirect('/basedata/basedata/import/0');
            }
        
        }
        // STEP 3 - read headers to match rows
        if (step === 2) {
            var uID = req.body.uId;
            var sheetName = req.body.sheetName;
            var uploads = new Uploads();
            uploads.getFilePathForId(uID, (filepath) => {
                var importer = new Importer();
                importer.openXSLX(filepath);
                importer.getFirstLineOfSheetsAsJSON(sheetName, (firstLineJSON) => {
                    console.log(firstLineJSON);
                    res.render('basedata/basedata_import_step3.ejs', {
                        title: 'Mitglieder importieren - Schritt 3',
                        navarea: 'basedata',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                        uploadId: uID,
                        headers: firstLineJSON,
                        sheetName: sheetName,
                        message: ''
                    });
                })

            })
        }
    }
};