const uuidv1 = require('uuid').v1;
const Organizations = require('../../models/organizations');
const Assets = require('../../models/assets');
const Configuration = require('../../models/configuration');
const OrganizationsTypes = require('../../models/organizationstypes');
const AssetsGroups = require('../../models/assetsgroups');
const AssetsWarehouses = require('../../models/warehouses');
const Basedata = require('../../models/basedata');
const GearGroups = require('../../models/gearGroups');
const Groups = require('../../models/groups');
const Locations = require('../../models/locations');
const Trainings = require('../../models/trainings');
const Wallboards = require('../../models/wallboards');
const Operations = require('../../models/operationshelper');

module.exports = {
    getOrganizationsPage: (req, res) => {
        req.app.logger.debug('ROUTE-ORGANIZATIONS | loading Organizations Page');
        let message = null;
        if (req.session.message != null) {
            if (req.session.message.length > 0) {
                message = req.session.message;
                req.session.message = null;
            }
        }
        
        res.render('basedata/organizations.ejs', {
            title: 'Organizations',
            navarea: 'basedata',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
            message: message
        });
    },
    getOrganizationsPageData: (req, res) => {
        const organizations = new Organizations();
        organizations.getListOfOrganizations(rows => {
            res.setHeader('Content-Type', 'application/json');
            const orgs = req.session.superuser === 1 ? rows : rows.filter(row => req.session.accessibleOrganizations.includes(row.id))
            res.json({ data: orgs });
        })
    },
    editOrganizationPage: (req, res) => {
        const organizations = new Organizations();
        const organizationstypes = new OrganizationsTypes(req.app.logger);
        const configuration = new Configuration(req.app.logger);
        
        let id = req.params.id;

        configuration.getOrganizationImageSizeLimits(imageSize => {
            organizations.getListOfOrganizations((rowsOrganizations) => {
                organizations.getOrganizationWithId(id, (rows) => {
                    if (rows !== null) {
                        // Get geo coordinates
                        if ((rows[0]['latitude'].length > 0) && (rows[0]['longitude'].length > 0)) {
                            coordinatesKnown = true;
                        } else {
                            coordinatesKnown = false;
                        }
                        organizationstypes.getListOfOrganizationsTypes((rowsOrganizationsTypes) => {
                            // read google maps api key from configuration
                        configuration.getValueForKeyword('googlemapskey', (key) => {
                            res.render('basedata/organizations_edit.ejs', {
                                title: 'Edit  Organization',
                                navarea: 'basedata',
                                subarea: 'map',
                                avatarpath: req.session.avatar,
                                name: req.session.name,
                                organizationname: req.session.organization,
                                session: req.session,
                                organization: rows[0],
                                googlemapskey: key,
                                coordinatesKnown: coordinatesKnown,
                                organizationsTypes: rowsOrganizationsTypes,
                                parentOrganizations: rowsOrganizations,
                                message: '',
                                imageLimits: imageSize,
                            });
                        })
                        })
                    } else {
                        // No Entry found for orgId
                        coordinatesKnown = false;
                    }
                });
            })        
        })
        
    },
    editOrganization: (req, res) => {
        let id = req.params.id;

        var organization = new Organizations(req.app.logger);
        organization.editOrganization(req.body, id, (success) => {
            res.redirect('/basedata/organizations/list?edit=organization');
        })
    },
    addOrganizationPage: (req, res) => {
        var configuration = new Configuration(req.app.logger);
        var organizations = new Organizations(req.app.logger);
        configuration.getOrganizationImageSizeLimits(imageSize => {
            organizations.getListOfOrganizations((rowsOrganizations) => {
                configuration.getValueForKeyword('googlemapskey', (googlemapskey) => {
                    var organizationstypes = new OrganizationsTypes(req.app.logger);
                    organizationstypes.getListOfOrganizationsTypes((rowsOrganizationsTypes) => {
                        res.render('basedata/organizations_add.ejs', {
                            title: 'Organizations',
                            navarea: 'basedata',
                            subarea: 'map',
                            avatarpath: req.session.avatar,
                            name: req.session.name,
                            organizationname: req.session.organization,
                            session: req.session,
                            googlemapskey: googlemapskey,
                            organizationsTypes: rowsOrganizationsTypes,
                            parentOrganizations: rowsOrganizations,
                            message: '',
                            imageLimits: imageSize,
                        });
                    })
                })
            })
        })
    },
    addOrganization: (req, res) => {
        const organizations = new Organizations();
        organizations.addOrganization(req.body, (success) => {
            if (success) {
                // successfully added organization
                res.redirect('/basedata/organizations/list?add=organization');
            } else {
                // New organization could not be added
                res.redirect('/basedata/organizations/list');
            }
        })
    },
    getOrganizationsChartPage: (req, res) => {
        res.render('basedata/organizations_chart.ejs', {
            title: 'Organizations',
            navarea: 'basedata',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
            message: ''
        });
    },
    setOrganizationAvatar: (req, res) => {
        req.app.logger.debug('ROUTE-ORGANIZATIONS | Changing Organization avatar for Organization with id ' + req.body.organizationid);
        let imageData = req.body.image_data;
        let organizationid = req.body.organizationid;
 
        // send the user's details to the database
        const organizations = new Organizations();
        organizations.setOrganizationAvatar(imageData, organizationid, (success) => {
            if (success) {
                res.redirect('/basedata/organizations/list?edit=organizationavatar');
            } else {
                req.app.logger.error('ROUTE-ORGANIZATIONS | error updating avatar-path in database for organization ' + organizationid);
                res.redirect('/basedata/organizations/list');
            }
        })
    },
    removeOrganization: (req, res) => {

        var result = {
            canBeDeleted: true,
            reasons: []
        }

        const organizations = new Organizations();
        if (typeof req.params.id !== 'undefined') {
            let assets = new Assets(req.app.logger);
            assets.getAssetsForOrganization(req.params.id, (rowsAssets) => {
                if (rowsAssets.length > 0) {
                    result.canBeDeleted = false;
                    result.reasons.push('Es referenzieren noch Inventargegenstände auf die Organisation');
                }
                let assetsgroups = new AssetsGroups(req.app.logger);
                assetsgroups.getListOfAssetGroupsForOrganization(req.params.id, (rowsAssetsGroups) => {
                    if (rowsAssetsGroups.length > 0) {
                        result.canBeDeleted = false;
                        result.reasons.push('Es referenzieren noch Inventargruppen auf die Organisation');
                    }
                    let warehouses = new AssetsWarehouses(req.app.logger);
                    warehouses.getListOfWarehousesForOrganizationId(req.params.id, (rowsWarehouses) => {
                        if (rowsWarehouses.length > 0) {
                            result.canBeDeleted = false;
                            result.reasons.push('Es referenzieren noch Lagerorte auf die Organisation');
                        }
                        let basedata = new Basedata(req.app.logger);
                        basedata.getBasedataForOrganization(req.params.id, (rowsBasedata) => {
                            if (rowsBasedata.length > 0) {
                                result.canBeDeleted = false;
                                result.reasons.push('Es referenzieren noch Mitglieder auf die Organisation');
                            }
                            let geargroups = new GearGroups(req.app.logger);
                            geargroups.getListOfGroupsForOrganization(req.params.id, (rowsGearGroups) => {
                                if (rowsGearGroups.length > 0) {
                                    result.canBeDeleted = false;
                                    result.reasons.push('Es referenzieren noch Inventargruppen auf die Organisation');
                                }
                                let groups = new Groups(req.app.logger);
                                groups.getListOfGroupsForOrganization(req.params.id, (rowsGroups) => {
                                    if (rowsGroups.length > 0) {
                                        result.canBeDeleted = false;
                                        result.reasons.push('Es referenzieren noch Gruppen auf die Organisation');
                                    }
                                    let locations = new Locations(req.app.logger);
                                    locations.getListOfLocationsForOrganizationWithId(req.params.id, (rowsLocations) => {
                                        if (rowsLocations.length > 0) {
                                            result.canBeDeleted = false;
                                            result.reasons.push('Es referenzieren noch Standorte auf die Organisation');
                                        }
                                        let trainings = new Trainings(req.app.logger);
                                        trainings.getTrainingsForOrganization(req.params.id, (rowsTrainings) => {
                                            if (rowsTrainings.length > 0) {
                                                result.canBeDeleted = false;
                                                result.reasons.push('Es referenzieren noch Übungen / Ausbildungen auf die Organisation');
                                            }
                                            let wallboards = new Wallboards(req.app.logger);
                                            wallboards.getWallboardForOrganization(req.params.id, (rowsWallboards) => {
                                                if (rowsWallboards.length > 0) {
                                                    result.canBeDeleted = false;
                                                    result.reasons.push('Es referenzieren noch Wallboards auf die Organisation');
                                                }
                                                let operations = new Operations(req.app.logger);
                                                operations.getOperationsForOrganization(req.params.id, (rowsOperations) => {
                                                    if (rowsOperations.length > 0) {
                                                        result.canBeDeleted = false;
                                                        result.reasons.push('Es referenzieren noch Einsätze auf die Organisation');
                                                    } 
                                                    // ALL CHECKS DONE, NOW check if can be deleted or not
                                                    if (result.canBeDeleted === true) {
                                                        // delete organization now
                                                        let organizations = new Organizations(req.app.logger);
                                                        organizations.removeOrganizationWithId(req.params.id, (success) => {
                                                            // redirect to organizations list
                                                            res.redirect('/basedata/organizations/list?delete=organization');
                                                        });
                                                    } else {
                                                        // calculate Error Message
                                                        let message = 'Organisation kann nicht gelöscht werden, es bestehen Abhängigkeiten.'
                                                        if (result.reasons.length > 0) {
                                                            message = message + '<div><ul>';
                                                            result.reasons.forEach((reason) => {
                                                                message = message + '<li>' + reason + '</li>';
                                                            })
                                                            message = message + '</ul></div>';
                                                        }
                                                        req.session.message = message;
                                                        // Could not be deleted
                                                        console.log('ROUTES | BASEDATA | ORGANIZATIONS | Error removing an organization from core database');
                                                        res.redirect('/basedata/organizations/list');
                                                    }
                                                })
                                            })
                                        })
                                    })
                                })
                            })
                        })
                    })
                    
                })
            })
            /*
            organizations.removeOrganizationWithId(req.params.id, (success, checkResult) => {
                if (success) {
                    console.log('ROUTES | BASEDATA | ORGANIZATIONS | successfully removed organization from core database')
                    res.redirect('/basedata/organizations/list');
                } else {
                    // calculate Error Message
                    let message = 'Organisation kann nicht gelöscht werden, es bestehen Abhängigkeiten.'
                    if (checkResult.reasons.length > 0) {
                        message = message + '<div><ul>';
                        checkResult.reasons.forEach((reason) => {
                            message = message + '<li>' + reason + '</li>';
                        })
                        message = message + '</ul></div>';
                    }
                    req.session.message = message;
                    // Could not be deleted
                    console.log('ROUTES | BASEDATA | ORGANIZATIONS | Error removing an organization from core database');
                    res.redirect('/basedata/organizations/list');
                }
            });*/
        } else {
            console.log('ROUTES | BASEDATA | ORGANIZATIONS | Error removing an organization from core database, id not set');
            res.redirect('/basedata/organizations/list');
        }
        
    }
};