let bDataObj = require('../../models/basedata');
const Users = require('../../models/users');
const Groups = require('../../models/groups');
const Usersgroups = require('../../models/usersgroups');
const Authenticate = require('../../models/auth');
const fs = require('fs');
const path = require('path');

const renderNoAccess = (req, res) => {
    // regular HTML request
    res.render('admin_no_access.ejs', {
        title: 'ALARMiator | Kein Zugriff',
        navarea: 'home',
        avatarpath: req.session.avatar,
        name: req.session.name,
        organizationname: req.session.organization,
        session: req.session,
    });
}

module.exports = {
    getUsersList: (req, res) => {
        req.app.logger.debug('ROUTES-USERS | loading users page');
        res.render('basedata/users.ejs', {
            title: 'ALARMiator | Benutzer',
            navarea: 'basedata',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getUsersListData: (req, res) => {
        req.app.logger.debug('ROUTES-USERS | loading user list');
        var users = new Users(req.app.logger);
        users.getListOfUsersForUI((rows) => {
            res.setHeader('Content-Type', 'application/json');

            // filter out users that the currently logged in user is not allowed to see
            const userList = req.session.superuser === 1 ? rows : rows.filter(
                user => req.session.accessibleOrganizations.includes(user.organizationid))
            res.json({ data: userList });
        })        
    },    
    addUserPage: (req, res) => {
        let groups = new Groups(req.app.logger);
        let bdatas = new bDataObj(req.app.logger);
        groups.getListOfGroups((rowsGroups) => {
            bdatas.getListOfBasedataForTable((resultBData) => {
                res.render('basedata/users_add.ejs', {
                    title: 'ALARMiator',
                    navarea: 'basedata',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    basedata: req.session.superuser === 1 ? resultBData : resultBData.filter(
                        bd => req.session.accessibleOrganizations.includes(bd.organizationid)),
                    groups: rowsGroups,
                });
            });
        });
    },
    addUser: (req, res) => {
        var users = new Users(req.app.logger);
        let username = req.body.username;
        let password = req.body.password;
        let basedataid = req.body.basedataid;
        let state = req.body.userActiv;

        users.addUser(username, password, basedataid, state, (success) => {
            if (success) {
                // User successfully added
                res.redirect('/basedata/users/list?add=user');
            } else {
                // User could not be added
                res.redirect('/basedata/users/list');
            }
        })
    },
    editUserPage: (req, res) => {
        let userId = req.params.id;
        let bdatas = new bDataObj(req.app.logger);
        let users = new Users(req.app.logger);
        let usersgroups = new Usersgroups(req.app.logger);
        let authenticate = new Authenticate();

        

        // get permissions of the currently logged in user
        authenticate.getUsersGroupsForUserWithId(req.session.userid, (currentUserGroups) => {
            // just remember the namespaces (identifiers)
            const grantNamespaces = currentUserGroups.map(group => group.grantnamespace)
            bdatas.getListOfBasedata(function (resultBData) {
                users.getUserForId(userId, (rowsUser) => {
                    const user = rowsUser[0]
                    usersgroups.getListOfGroups((rowsUsersGroups) => {
                        authenticate.getUsersGroupsForUserWithId(userId, (rowsSetUsersGroups) => {
                            const basedata = resultBData.filter(rbd => rbd.id === user.basedataid)
                            if (basedata.length === 0) {
                                // basedata associated with this user does not exist!
                                renderNoAccess(req, res)
                                return
                            }
                            if (req.session.superuser === 0 && !req.session.accessibleOrganizations.includes(basedata[0].organizationid)) {
                                // cannot see user account from other organizations
                                renderNoAccess(req, res)
                                return
                            }
                            res.render('basedata/users_edit.ejs', {
                                title: 'Edit  user',
                                navarea: 'basedata',
                                avatarpath: req.session.avatar,
                                name: req.session.name,
                                organizationname: req.session.organization,
                                session: req.session,
                                user,
                                // only provide user groups that the currently logged in user has access
                                usersgroups: req.session.superuser === 1 ? rowsUsersGroups : rowsUsersGroups.filter(
                                    group => grantNamespaces.includes(group.grantnamespace)),
                                setUsersGroups: rowsSetUsersGroups,
                                basedata,
                                message: ''
                            });
                        })
                    });
                });
            });
        })
            
        
    },
    editUser: (req, res) => {
        let userId = req.params.id;
        var users = new Users(userId, req.app.logger);
        let authenticate = new Authenticate();
        users.getExtendedUserInfo(userId)
            .then(userInfo => {
                if (req.session.superuser === 0 && !req.session.accessibleOrganizations.includes(userInfo.organizationid)) {
                    // this user belongs to a different org than we have access to
                    renderNoAccess(req, res)
                    return
                }

                // get permissions of the currently logged in user
                authenticate.getUsersGroupsForUserWithId(req.session.userid, (currentUserGroups) => {
                    // determine if the currently logged in user is only allowed to update certain permissions
                    const groupIdRestrictions = req.session.superuser === 1 ? null : currentUserGroups.map(grp => grp.groupId)
                    // update user and groups
                    users.updateUser(userId, req.body, groupIdRestrictions, (success) => {
                        console.log('Updated User: ' + success);
                        res.redirect('/basedata/users/list?edit=user');
                    })
                })
            })
            .catch(() => {
                // technical error
                res.redirect(`/basedata/users/edit/${userId}`)
            })
        
    },
    deleteUser: (req, res) => {
        let userId = req.params.id;
        let users = new Users(req.app.logger);
        users.removeUser(userId, (success) => {
            res.redirect('/basedata/users/list?delete=user');
        })
    },
    activateUser: (req, res) => {
        let userId = req.params.id;
        let users = new Users(req.app.logger);
        users.activateUser(userId, (success) => {
            res.redirect('/basedata/users/list?edit=user');
        })
    },
    deactivateUser: (req, res) => {
        let userId = req.params.id;
        let users = new Users(req.app.logger);
        users.deactivateUser(userId, (success) => {
            res.redirect('/basedata/users/list?edit=user');
        })
    },
    createCredentialsPdf: (req, res) => {
        let userId = req.params.id;
        let users = new Users(req.app.logger);
        users.generateCredentialsPDF(userId, (success, pdfPath) => {
            if (success) {
                var file = fs.createReadStream(pdfPath);
                var stat = fs.statSync(pdfPath);
                console.log('stat.size: ' + stat.size);
                res.setHeader('Content-Length', stat.size);
                res.setHeader('Content-Type', 'application/pdf');
                var PdfFileName = path.basename(pdfPath);
                res.setHeader('Content-Disposition', 'attachment; filename=' + PdfFileName);
                res.sendFile(pdfPath);
            } else {
                res.sendStatus(500);
            }
        })
    }
};