
const express = require("express")
const path = require("path")
const fs = require("fs")
const showdown = require("showdown")
const doc = express.Router()

// initialise converter
const converter = new showdown.Converter()
// settings for converter
converter.setFlavor("github")
converter.setOption("simplifiedAutoLink", "true")

// this is the absolute path to the documentation folder (required by sendFile)
const APP_PATH = path.dirname(__dirname)  // one level up (out of the routes folder)
const DOC_PATH = `${APP_PATH}/documentation/doc`


// main endpoint for `http://<host>:<port>/doc`
doc.get("/", (req, res) => {
    res.redirect("/doc/index")
})

// content endpoint for `http://<host>:<port>/doc/<whatever/markdown.md>` and static files
doc.get("/:path*", (req, res) => {
    // get path from request and construct requested resource's path
    const path = `${req.params.path}${req.params["0"]}`
    const filePath = `${DOC_PATH}/${path}`

    // check if the requested resource is a static resource (i.e. a file)
    if (path.includes(".") && path.startsWith("static/")) {
        // a static file is required
        if (fs.existsSync(filePath)) {
            res.sendFile(filePath)
        } else {
            // send a 404 with empty body
            res.status(404).send()
        }
        return
    }
    
    // serve markdown file
    const mdPath = filePath.endsWith(".md") ? filePath : `${filePath}.md`
    if (fs.existsSync(mdPath)) {
        // load markdown
        const mdContent = fs.readFileSync(mdPath,  { 
            encoding: "utf8", 
            flag:"r" 
        })

        // this is the plain HTML without any <html> or <body> tag
        const html = converter.makeHtml(mdContent)
        res.send(html)
    } else {
        // TODO: should send error/404 page
        res.status(404).send()
    }
})

// export router
module.exports = doc
