const moment = require("moment")
const operationsHelper = require('../../models/operationshelper');
const Geardirectory = require('../../models/geardirectory');
const Configuration = require('../../models/configuration');
const FeedbackStore = require('../../models/core/feedbackstore');

const alarmDatetimeMapper = alarm => {
    // map datetime to SQL format, easy to parse and sort in the frontend
    alarm.datetime = moment(`${alarm.alarmdate} ${alarm.alarmtime}`, "DD.MM.YYYY HH:mm:ss").format("YYYY-MM-DD HH:mm:ss")
    return alarm
}

module.exports = {
    getOperationsOverviewMap: (req, res) => {
        let operations = new operationsHelper(req.app.logger);
        operations.getListOfOperationsJSON((result) => {
            res.render('operations/operations_map_overview', {
                title: 'ALARMiator | Karte Einsätze',
                navarea: 'operations',
                subarea: 'operations',
                avatarpath: req.session.avatar,
                name: req.session.name,
                organizationname: req.session.organization,
                session: req.session,
                operations: result
            });
        })
    },
    getOperationsList: (req, res) => {
        res.render('operations/operations', {
            title: 'ALARMiator | Einsätze',
            navarea: 'operations',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        })
    },
    getOperationsListData: (req, res) => {
        let operations = new operationsHelper(req.app.logger);
        operations.getListOfOperations((result) => {
            res.json({
                data: result.map(alarmDatetimeMapper),
            })
        })
    },
    getActiveOperationsList: (req, res) => {        
        res.render('operations/operations_active', {
            title: 'ALARMiator | Einsätze',
            navarea: 'operations',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getActiveOperationsListData: (req, res) => {
        let operations = new operationsHelper(req.app.logger);
        operations.getListOfActiveOperations((result) => {
            res.json({
                data: result.map(alarmDatetimeMapper),
            })
        })
    },
    archiveOperation: (req, res) => {
        let operation = req.params.oId;
        let operations = new operationsHelper(req.app.logger);
        operations.archiveOperation(operation, (success) => {
            res.redirect('/operations/active');
        })
    },
    closeOperation: (req, res) => {
        let operation = req.params.oId;
        let operations = new operationsHelper(req.app.logger);
        operations.closeOperation(operation, (success) => {
            res.redirect('/operations/active');
        })
    },
    closeAllOperations: (req, res) => {
        let operations = new operationsHelper(req.app.logger);
        operations.closeAllOperations((success) => {
            res.redirect('/operations/active');
        })
    },
    activateOperation: (req, res) => {
        let operation = req.params.id;
        let operations = new operationsHelper(req.app.logger);
        operations.activateOperation(operation, (success) => {
            if (success) {
                global.plgManager.event_new_alarm_persisted(operation);
            }
            res.redirect('/operations/active');
        })
    },
    deleteOperation: (req, res) => {
        let operation = req.params.oId;
        let operations = new operationsHelper(req.app.logger);
        operations.deleteOperation(operation, (success) => {
            res.redirect('/operations/list')
        })
    },
    showOperationDetails: (req, res) => {
        let operation = req.params.oId;
        let opId = req.params.oId;
        let configuration = new Configuration(req.app.logger);
        let geardirectory = new Geardirectory(req.app.logger);
        let operations = new operationsHelper(req.app.logger);
        operations.getOperation(operation, (row) => {
            if (row.length > 0) {
                configuration.getValueForKeyword('ilsmessageseparator', (msgSeparator) => {
                    if (msgSeparator === null) {
                        msgSeparator = '|';
                    } else {
                        msgSeparator = '' + msgSeparator;
                        if (msgSeparator.length === 0) {
                            msgSeparator = '|';
                        }
                    }
                    var operation = row[0];
                    // render Linebreaks in ILS message
                    operation.message = operation.message.split(msgSeparator).join('<br>');
                    // calculate gear types for operation gearlistCalc
                    var newGearlistCalc = [];
                    if (typeof operation.gearListCalc !== 'undefined') {
                        if (operation.gearListCalc !== null && operation.gearListCalc.length > 0) {
                            var gearlist = JSON.parse(operation.gearListCalc);
                            gearlist.forEach((gear, index) => {
                                newGearlistCalc.push(gear + ' <span class="text-muted">' + geardirectory.getLongNameForGearString(gear)) + '</span>';
                            });
                            operation.gearListCalc = JSON.stringify(newGearlistCalc);
                        }
                    }
                    operations.getSubOperationsForOperation(opId, (rowsSubOperations) => {
                        var feedbackstore = new FeedbackStore(req.app.logger);
                        feedbackstore.getFeedbackForOperationWithId(opId, (feedbackRows) => {
                            res.render('operations/operations_details', {
                                title: 'ALARMiator | Einsatzdetails',
                                navarea: 'operations',
                                avatarpath: req.session.avatar,
                                name: req.session.name,
                                organizationname: req.session.organization,
                                session: req.session,
                                operation: operation,
                                subOperations: rowsSubOperations,
                                feedback: feedbackRows
                            });
                        })
                    })
                })
            }
        })
    }
};



