let hydrantshelper = require('../../models/hydrants');
let reportshelper = require('../../models/reports');

module.exports = {
    getReportHydrantDetails: (req, res) => {
        let hydrantId = req.params.id;
        const hydrants = new hydrantshelper(req.app.logger);
        hydrants.getHydrantDataForId(hydrantId, (resultHydrant) => {
            if (resultHydrant !== null) {
                // Found hydrant data in database
                // procceed with generating of pdf document
                let reports = new reportshelper(req.app.logger);
                reports.setTemplate('tpl_hydrant.ejs');
                reports.setData(resultHydrant[0]);
                reports.setOutPutFile('hydrant_details.pdf');
                reports.generateHTMLDocumentForReport(req, res, (resultHTML) => {
                    if (resultHTML) {
                        // error during creation of HTML Document
                        req.app.logger.error('ROUTES-REPORTS | Error while generating html report for hydrant details with id ' + hydrantId + ' with Error: ' + resultHTML.message);
                    } else {
                        reports.generatePDFDocumentForReport(req, res, (resultPDF) => {
                            if (resultPDF) {
                                // error during pdf generation
                                req.app.logger.error('ROUTES-REPORTS | Error while generating pdf report for hydrant details with id ' + hydrantId);
                            } else {
                                // successfully generated report
                                req.app.logger.info('ROUTES-REPORTS | successfully generated pdf report for hydrant details with id ' + hydrantId);
                                res.redirect('/reports/ondemand/hydrant_details.pdf');
                            }
                        });
                    }
                })
            }
        });
    },

    getReportHydrantsList: (req, res) => {
        
        const hydrants = new hydrantshelper(req.app.logger);
        hydrants.getListOfHydrants((resultHydrants) => {
            if (resultHydrants !== null) {
                // Found hydrant data in database
                // procceed with generating of pdf document
                let reports = new reportshelper(req.app.logger);
                reports.setTemplate('tpl_hydrant_list.ejs');
                reports.setData(resultHydrants);
                
                reports.setOutPutFile('hydrant_list.pdf');
                reports.generateHTMLDocumentForReport(req, res, (resultHTML) => {
                    if (resultHTML) {
                        // error during creation of HTML Document
                        req.app.logger.error('ROUTES-REPORTS | Error while generating html report for hydrants list with Error: ' + resultHTML.message);
                    } else {
                        reports.generatePDFDocumentForReport(req, res, (resultPDF) => {
                            if (resultPDF) {
                                // error during pdf generation
                                req.app.logger.error('ROUTES-REPORTS | Error while generating pdf report for hydrants list');
                            } else {
                                // successfully generated report
                                req.app.logger.info('ROUTES-REPORTS | successfully generated pdf report for hydrants list');
                                res.redirect('/reports/ondemand/hydrant_list.pdf');
                            }
                        });
                    }
                })
            }
        });
    }
};