const Respiratory = require('../../models/respiratory');
const Basedata = require('../../models/basedata');
const Momenthelper = require('../../models/momenthelper');
const Uploads = require('../../models/core/core_uploads');
const path = require('path');
const uuidv1 = require('uuid').v1;
const { readSync } = require('fs');


module.exports = {
    getDashboard: (req, res) => {
        var respiratory = new Respiratory(req.app.logger);
        respiratory.getValidMembers((rowsValid) => {
            respiratory.getExpiringMembers(3, (rowsExpiring) => {
                res.render('respiratory/dashboard', {
                    title: 'ALARMiator | Atemschutz',
                    navarea: 'respiratory',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    validMembers: rowsValid,
                    expiringMembers: rowsExpiring
                });
            })
        })
    },
    getRespiratoryList: (req, res) => {
        res.render('respiratory/respiratory_list', {
            title: 'ALARMiator | Einsätze',
            navarea: 'respiratory',
            avatarpath: req.session.avatar,
            name: req.session.name,
            organizationname: req.session.organization,
            session: req.session,
        });
    },
    getRespiratoryListData: (req, res) => {
        let respiratory = new Respiratory(req.app.logger);
        respiratory.loadList((rows) => {
            res.json({ data: rows })
        })
    },
    addRespiratoryPage: (req, res) => {
        let basedata = new Basedata(req.app.logger);
        basedata.getListOfBasedata((basedataRows) => {
            var respiratory = new Respiratory(req.app.logger);
            respiratory.getBasedataForWhichRespiratoryEntryExists((rowsBasedataIdsExisting) => {
                res.render('respiratory/respiratory_add', {
                    title: 'ALARMiator | Einsätze',
                    navarea: 'respiratory',
                    avatarpath: req.session.avatar,
                    name: req.session.name,
                    organizationname: req.session.organization,
                    session: req.session,
                    basedata: basedataRows,
                    basedataExisting: rowsBasedataIdsExisting
                });
            })
        })
    },
    addRespiratory: (req, res) => {
        var respiratory = new Respiratory(req.app.logger);
        respiratory.addRespiratory(req.body, (success) => {
            if (success) {
                res.redirect('/respiratory/respiratory/list?add=respiratory');
            } else {
                res.redirect('/respiratory/respiratory/add');
            }
        })
    },
    addRespiratoryAttachment: (req, res) => {
        var attachmentTitle = req.body.attachmentTitle;
        var objectId = req.body.objectId;
    
        if (!req.files) {
            return res.status(400).send("No files were uploaded.");
        }
        console.log(req.files);

        var uploads = new Uploads();

        let uploadedFile = req.files.attachment;
        let filename = uploadedFile.name;
        let extension = path.extname(filename);


        if (attachmentTitle.length > 0) {
        } else {
            attachmentTitle = filename;
        }


        console.log('ROUTES-RESPIRATORY | mime-type of uploaded file: ' + uploadedFile.mimetype);
    
        var targetPath = 'uploads/' + uuidv1() + extension;
        uploadedFile.mv(targetPath, (err) => {
            if (err) {
                return res.status(500).send(err);
            }
            console.log('ROUTES-RESPIRATORY | moved uploaded Excel-File to ' + targetPath);
            // persist upload in database now, as it could be moved successfully
            uploads.addUpload(attachmentTitle, targetPath, filename, uploadedFile.mimetype, 'RESPIRATORY', objectId, 'respiratoryStore', 1, req.session.basedataId, '', (success, uId) => {
                console.log('ROUTES-RESPIRATORY | registered upload file in uploads-Table with id ' + uId);
                // Read the sheets in uploaded Excel File
                uploads.getUploadsForObjectInTable(objectId, 'respiratoryStore', (rowsAttachments) => {
                    res.json(rowsAttachments);
                })
            })
        })
    },
    editRespiratoryPage: (req, res) => {
        var respId = req.params.id;
        var respiratory = new Respiratory(req.app.logger);
        respiratory.getEntryForId(respId, (rowMember) => {
            if (rowMember !== null) {
                // Check for uploads
                var uploads = new Uploads();
                uploads.getUploadsForObjectInTable(respId, 'respiratoryStore', (rowsUploads) => {
                    // Render page
                    res.render('respiratory/respiratory_edit', {
                        title: 'ALARMiator | Atemschutz',
                        navarea: 'respiratory',
                        avatarpath: req.session.avatar,
                        name: req.session.name,
                        organizationname: req.session.organization,
                        session: req.session,
                        member: rowMember[0],
                        uploads: rowsUploads
                    });
                })
            }
        })
    },
    editRespiratory: (req, res) => {
        var respId = req.params.id;
        var respiratory = new Respiratory(req.app.logger);
        respiratory.editRespiratory(respId, req.body, (success) => {
            res.redirect('/respiratory/respiratory/list?edit=respiratory');
        })
    }
};



