const EventEmitter = require('events');
const Excel = require('exceljs');
const fs = require('fs');

class ExcelCore extends EventEmitter {

    /**
     * Class offers printing related functionality to ALARMiator core
     * @constructor
     */
    constructor(logger) {
        super();
        this.logger = logger;
        this.workbook = null;   // holds an instance of a workbook
        this.worsheets = [];    // holds all worksheets
        this.selectedWorksheet = 1;
    }

    set xlsxFileInput(xlsxFileInput) {
        this._xlsxFileInput = xlsxFileInput;
    }
    get xlsxFileInput() {
        return this._xlsxFileInput;
    }

    set selectedWorksheet(selectedWorksheet) {
        this._selectedWorksheet = selectedWorksheet;
    }
    get selectedWorksheet() {
        return this._selectedWorksheet;
    }

    async openExcelFile(filePath, callback) {
        this.workbook = new Excel.Workbook();
        await this.workbook.xlsx.readFile(filePath);
        return callback(true);
    }

    
    getWorksheets(callback) {
        var that = this;
        this.worksheets = [];
        this.workbook.eachSheet(function(worksheet, sheetId) {
            var sheet = {
                name: worksheet.name,
                sheetId: sheetId
            }
            that.worksheets.push(sheet);
        });
        return callback(this.worksheets);
    }

    getFirstRowForWorksheet(sheetId, callback) {
        const worksheet = this.workbook.getWorksheet(sheetId);
        var row = worksheet.getRow(1);
        var cellHeaders = [];
        for (var i=0; i<worksheet.columnCount; i++) {
            var cell = {
                id: i+1,
                name: row.getCell(i+1).value
            }
            cellHeaders.push(cell);
        }
        return callback(cellHeaders);
    }

    getAllRowsForWorksheet(skipFirst, sheetId, callback) {
        const worksheet = this.workbook.getWorksheet(sheetId);
        var startLine = 1;
        if (skipFirst) {
            startLine = 2;
        }
        var rows = [];
        for (var i=startLine; i < worksheet.rowCount; i++) {
            var row = worksheet.getRow(i);
            var cellContent = [];
            for (var i2=0; i2<worksheet.columnCount; i2++) {
                var cell = {
                    id: i2+1,
                    name: row.getCell(i2+1).value
                }
                cellContent.push(cell);
            }
            rows.push(cellContent);
        }
        return callback(rows);
    }

    getColumnsForWorksheet(sheetId, callback) {
        const worksheet = this.workbook.getWorksheet(sheetId);
        var row = worksheet.getRow(5);
        return callback(row);
    }




    // Public Methos
    /**
     * Returns array of worksheets (name and sheet id)
     * @param {function} callback Array of worksheets found in defined xlsx file
     */
    getListOfWorksheets(callback) {
        this.openExcelFile(this._xlsxFileInput, (success) => {
            if (success) {
                this.getWorksheets((worksheets) => {
                    return callback(worksheets);
                })
            } else {
                return callback(null);
            }
        })
    }
    
    
    /**
     * Returns an array with the cells of first line in worksheet
     * @param {function} callback 
     */
    getFirstLine(callback) {
        this.openExcelFile(this._xlsxFileInput, (success) => {
            if (success) {
                this.getWorksheets((worksheets) => {
                    console.log('selected Worksheet: ' + this._selectedWorksheet);
                    this.getFirstRowForWorksheet(this._selectedWorksheet, (row) => {
                        return callback(row);
                    })
                })
            } else {
                return callback(null);
            }
        })
    }

    /**
     * Returns the content of the excel - worksheet, optionally skipping first line
     * @param {bool} skipFirst skip first line or not
     * @param {array} callback returns all lines as array
     */
    getAllLines(skipFirst = true, callback) {
        this.openExcelFile(this._xlsxFileInput, (success) => {
            if (success) {
                this.getWorksheets((worksheets) => {
                    this.getAllRowsForWorksheet(skipFirst, this._selectedWorksheet, (rows) => {
                        return callback(rows);
                    })
                })
            } else {
                return callback(null);
            }
        })
    }
}

module.exports = ExcelCore;
