const gk = require('gauss-krueger');
const fs = require('fs');
const Operation = require('../../../models/core/core_operation');

const testRawData = fs.readFileSync('./katsys_alarmierung_erh.json');


let rawKatSysObject = JSON.parse(testRawData);
let rawAlarmDataArray = rawKatSysObject.data.textElements;




function calculateLatLon() {

    if ((operation.gky.value.length > 0) && (operation.gkx.value.length > 0)) {
        // SDS delivered gauss krueger coordinates
        let wgs84 = gk.toWGS({ x: parseFloat(operation.gkx.value), y: parseFloat(operation.gky.value) });
        operation.lat = wgs84.latitude;
        operation.lon = wgs84.longitude;
    } else {
        // SDS did not deliver gauss krueger coordinates
        operation.lat = 0;
        operation.lon = 0;
    }
}

function calculateKeywordsFromRaw() {
    if (typeof operation.keywordRaw != 'undefined') {
        if (typeof operation.keywordRaw.value != 'undefined') {
            var keywordRaw = '' + operation.keywordRaw.value;
            var keywdSplitArr = keywordRaw.split('#');
            if (Array.isArray(keywdSplitArr)) {
                // seems there is content
                if (keywdSplitArr.length > 1) {
                    operation.keywordId = keywdSplitArr[1];
                }
                if (keywdSplitArr.length > 2) {
                    operation.keywordCategory = keywdSplitArr[2];
                }
                if (keywdSplitArr.length > 3) {
                    operation.keywordName = keywdSplitArr[3];
                }
            }
        }
    }
}

let operation = new Operation();
operation.section = '';
operation.alarmdate = rawAlarmDataArray.sds_alarmdatum;
operation.alarmtime = rawAlarmDataArray.sds_alarmuhrzeit;
operation.gear = rawAlarmDataArray.sds_einsatzmittel;
operation.operationnumber = rawAlarmDataArray.sds_einsatznummer;
operation.location = rawAlarmDataArray.sds_ort;
operation.message = rawAlarmDataArray.sds_freitext;
operation.community = rawAlarmDataArray.sds_ortsteil;
operation.rawdata = rawAlarmDataArray.sds_komplett;
operation.object = rawAlarmDataArray.sds_objekt_kurz_tts;
operation.district = rawAlarmDataArray.sds_ortsteil;
operation.keywordId = rawAlarmDataArray.schlagwort;
operation.keywordCategory = rawAlarmDataArray.sds_schlagwort_tts_navi;
operation.keywordRaw = rawAlarmDataArray.sds_schlagwort;
//operation.keywordName = rawAlarmDataArray.schlagwort_kurz;
operation.keywordName = rawAlarmDataArray.schlagwort;

// replace double-pipe (ERH specific)
var tmpZVEIs = '' + rawAlarmDataArray.sds_schleifen;
tmpZVEIs.replace('|', '');
operation.zveis = tmpZVEIs;

operation.subject = rawAlarmDataArray.sds_stichwort;
operation.street = rawAlarmDataArray.sds_strasse + ' ' + rawAlarmDataArray.sds_hausnummernzusatz;
operation.gkx = rawAlarmDataArray.sds_x;
operation.gky = rawAlarmDataArray.sds_y;
//operation.floor = rawAlarmDataArray.sds_etage;
//operation.ils = rawAlarmDataArray.sds_ils;
calculateLatLon();
calculateKeywordsFromRaw();

console.log(this.parserLogName + ' | ----------- Operation Object inside parser prio raising event alarm_parsed ---------- ');
console.log(operation);
console.log(this.parserLogName + ' | ----------- ENDE Operation Object inside parser prior raising event alarm parsed ---- ');

