const Operation = require('../../models/core/core_operation');

const RuleComponentTimeOfDay = require('../../models/ruleEngine/components/ruleComponentTimeOfDay');
const RuleComponentDayOfWeek = require('../../models/ruleEngine/components/ruleComponentDayOfWeek');
const RuleComponentGear = require('../../models/ruleEngine/components/ruleComponentGear');
const RuleComponentZVEI = require('../../models/ruleEngine/components/ruleComponentZVEI');
const RuleComponentKeywordId = require('../../models/ruleEngine/components/ruleComponentKeywordID');
const RuleComponentKeywordName = require('../../models/ruleEngine/components/ruleComponentKeywordName');
const RuleComponentKeywordCategory = require('../../models/ruleEngine/components/ruleComponentKeywordCategory');
const RuleComponentObject = require('../../models/ruleEngine/components/ruleComponentObject');
const RuleComponentSubject = require('../../models/ruleEngine/components/ruleComponentSubject');


var operation = new Operation();
operation.alarmtime = '16:02:00';
operation.alarmdate = '31.10.2020';
operation.gear = 'KBI Info\nFL Schwaig FwA-VSA-096\nFL Schwaig 96-55-01\nFF Schwaig\nFL Schwaig 96-40-01\nFL Schwaig 96-61-01';
operation.keywordRaw = '#T2726#VU#mit PKW';
operation.keywordId = 'T2726';
operation.keywordCategory = 'VU';
operation.keywordName = 'mit PKW';
operation.subject = 'THL 1';
operation.object = 'DORMERO Schlosshotel';
operation.zveis = '12345, 34567, 23456, 23456F';





var ruleParametersTimeOfDay = {
    fromHour : 7,
    fromMinute: 00,
    tillHour: 16,
    tillMinute: 59
}

var ruleParametersDayOfWeek = {
    monday: 1,
    tuesday: 1,
    wednesday: 1,
    thursday: 1,
    friday: 1,
    saturday: 0,
    sunday: 0
}

var ruleParametersGear = ['KBI Info', 'FF Reichenschwand'];

var ruleParametersZVEIs = {
    zveiList: ['12345', '34567', '23456F', '23456'],
    searchType : 'all'
};

var ruleParametersKeywordId = {
    searchFor : 'T2726',
    searchType : 'full-match'
}

var ruleParametersKeywordName = {
    searchFor : 'PKW',
    searchType : 'contains'
}

var ruleParametersKeywordCategory = {
    searchFor : 'V',
    searchType : 'starts-with'
}

var ruleParametersObject = {
    searchFor : 'DORMERO',
    searchType : 'starts-with'
}

var ruleParametersSubject = {
    searchFor : 'THL',
    searchType : 'starts-with'
}




// Rule Component Time of Day
var ruleComponentTimeOfDay = new RuleComponentTimeOfDay();
ruleComponentTimeOfDay.setRuleParameters(ruleParametersTimeOfDay);
var ruleResult = ruleComponentTimeOfDay.evaluate(operation);
console.log('Rule-Time-Of-Day Result: ' + ruleResult);

// Rule Component Day of Week
var ruleComponentDayOfWeek = new RuleComponentDayOfWeek();
ruleComponentDayOfWeek.setRuleParameters(ruleParametersDayOfWeek);
var ruleResultDayOfWeek = ruleComponentDayOfWeek.evaluate(operation);
console.log('Rule-Day-of-week Result: ' + ruleResultDayOfWeek);

// Rule Component Gear
var ruleComponentGear = new RuleComponentGear();
ruleComponentGear.setRuleParameters(ruleParametersGear);
var ruleResultGear = ruleComponentGear.evaluate(operation);
console.log('Rule-Gear Result: ' + ruleResultGear);

// Rule Component ZVEIs
var ruleComponentZVEIs = new RuleComponentZVEI();
ruleComponentZVEIs.setRuleParameters(ruleParametersZVEIs);
var ruleResultZVEIs = ruleComponentZVEIs.evaluate(operation);
console.log('Rule-ZVEIs Result: ' + ruleResultZVEIs);

// Rule Component KeywordId
var ruleComponentKeywordId = new RuleComponentKeywordId();
ruleComponentKeywordId.setRuleParameters(ruleParametersKeywordId);
var ruleResultKeywordId = ruleComponentKeywordId.evaluate(operation);
console.log('Rule-KeywordId Result: ' + ruleResultKeywordId);

// Rule Component KeywordName
var ruleComponentKeywordName = new RuleComponentKeywordName();
ruleComponentKeywordName.setRuleParameters(ruleParametersKeywordName);
var ruleResultKeywordName = ruleComponentKeywordName.evaluate(operation);
console.log('Rule-KeywordName Result: ' + ruleResultKeywordName);

// Rule Component KeywordCategoy
var ruleComponentKeywordCategory = new RuleComponentKeywordCategory();
ruleComponentKeywordCategory.setRuleParameters(ruleParametersKeywordCategory);
var ruleResultKeywordCategory = ruleComponentKeywordCategory.evaluate(operation);
console.log('Rule-KeywordCategory Result: ' + ruleResultKeywordCategory);

// Rule Component Object
var ruleComponentObject = new RuleComponentObject();
ruleComponentObject.setRuleParameters(ruleParametersObject);
var ruleResultObject = ruleComponentObject.evaluate(operation);
console.log('Rule-Object Result: ' + ruleResultObject);

// Rule Component Object
var ruleComponentSubject = new RuleComponentSubject();
ruleComponentSubject.setRuleParameters(ruleParametersSubject);
var ruleResultSubject = ruleComponentSubject.evaluate(operation);
console.log('Rule-Subject Result: ' + ruleResultSubject);