const moment = require('moment');
const Operation = require('../../../models/core/core_operation');
var uuid = require('uuid'); // required for generating timebased guids

class PayloadBuilder {


    constructor(logger) {
        this.logger = logger;
    }

    /**
     * Building alarm payload
     * @param {operation} alarmInfo 
     * @param {array} fcmKeys 
     */
    buildPayloadAlarm(alarmInfo, fcmKeys){

        var sound = "default";

        var messageType = "alarm";

        var organization = global.serverInstanceName;

        /* Customize notification here for Android + iOS */
        var notification_title = "EINSATZ " + organization;
        var notification_subtitle = alarmInfo.subject.value +" - "+ alarmInfo.keywordName.value;
        var notification_body = "Einsatzort: " + alarmInfo.locationCalc.value;
        /*************************************************/

        var payload = {  
            /* NEW */
            operation: alarmInfo.getReduced(),
            /* Android Notification */
            notification_payload: {
                title: notification_subtitle,   // switched with subtitle for android!!
                subtitle : notification_title,  // switched with title for android!!
                body: notification_body,
            }
          };
          
          // REFERENCE: https://firebase.google.com/docs/reference/fcm/rest/v1/projects.messages#top_of_page
          var message = {
            data: {
              payload: JSON.stringify(payload),
              messageType: messageType,
              feedback: "true",
              serverUUID: global.serverUUID,
              content_available: "1"
            },
            fcmOptions: {
                analyticsLabel: messageType
            },
            android: {
                priority: "high",
            },
            apns: {
              payload: {
                aps: {
                  alert : {
                    title : notification_title,
                    subtitle : notification_subtitle,
                    body: notification_body,
                  },
                  sound: {
                    critical: true,
                    name: sound,
                    volume: 1.0
                  },
                  category: messageType,
                  "mutable-content": 1,
                  "content-available": 1
                }
              }
            },
            tokens: fcmKeys
          };

        this.logger.debug('PLUGIN | ALARMIATORMOBILE | PAYLOAD for DEBUGGING -------START-----------');
        this.logger.debug(JSON.stringify(message));
        this.logger.debug('PLUGIN | ALARMIATORMOBILE | PAYLOAD for DEBUGGING -------END-------------');
        
        return message;
    }

    /**
     * Building alarm payload for single token
     * @param {operation} alarmInfo 
     * @param {array} fcmToken
     */
    buildAlarmPayloadForSingleToken(alarmInfo, fcmToken, orgName){

        var sound = "default";

        var messageType = "alarm";

        var organization

        if (orgName != undefined && orgName != null) {
            organization = orgName;
        }
        else {
            organization = global.serverInstanceName;
        }

        /* Customize notification here for Android + iOS */
        var notification_title = "EINSATZ " + organization;
        var notification_subtitle = alarmInfo.subject.value +" - "+ alarmInfo.keywordName.value;
        var notification_body = "Einsatzort: " + alarmInfo.locationCalc.value;
        /*************************************************/

        var payload = {  
            /* NEW */
            operation: alarmInfo.getReduced(),
            /* Android Notification */
            notification_payload: {
                title: notification_subtitle,   // switched with subtitle for android!!
                subtitle : notification_title,  // switched with title for android!!
                body: notification_body,
            }
          };
          
          // REFERENCE: https://firebase.google.com/docs/reference/fcm/rest/v1/projects.messages#top_of_page
          var message = {
            data: {
              payload: JSON.stringify(payload),
              messageType: messageType,
              feedback: "true",
              serverUUID: global.serverUUID,
              content_available: "1"
            },
            fcmOptions: {
                analyticsLabel: messageType
            },
            android: {
                priority: "high",
            },
            apns: {
              payload: {
                aps: {
                  alert : {
                    title : notification_title,
                    subtitle : notification_subtitle,
                    body: notification_body,
                  },
                  sound: {
                    critical: true,
                    name: sound,
                    volume: 1.0
                  },
                  category: messageType,
                  "mutable-content": 1,
                  "content-available": 1
                }
              }
            },
            token: fcmToken
          };
        
        return message;
    }

    /**
     * Building message payload
     * @param {string} message 
     * @param {string} subtype
     * @param {array} fcmKeys 
     */
    buildPayloadForNotificationMessage(message, subtype, fcmKeys){

        // Refactor to single token!
        var sound = "default";

        var messageType = "message";

        var organization = global.serverInstanceName;;

        /* Customize notification here for Android + iOS */
        var notification_title = "Nachricht für " + organization;
        var notification_subtitle = "Neue "+ subtype + "-Nachricht";
        var notification_body = message;
        /*************************************************/

        var payload = {  
            message: message,
            timeStamp: moment().format("DD.MM.YYYY, HH:mm:ss"),
            messageUUID: uuid.v1(),
            subtype: subtype,
            title: subtype + "-" + notification_title,
            notification_payload: {
                title: notification_title,
                subtitle : notification_subtitle,
                body: notification_body,
            }
          };
          
          // REFERENCE: https://firebase.google.com/docs/reference/fcm/rest/v1/projects.messages#top_of_page
          var fcmMessage = {
            data: {
              payload: JSON.stringify(payload),
              messageType: messageType,
              serverUUID: global.serverUUID,
              content_available: "1"
            },
            fcmOptions: {
                analyticsLabel: messageType
            },
            android: {
                priority: "high",
            },
            apns: {
              payload: {
                aps: {
                    alert : {
                        title : notification_title,
                        subtitle : notification_subtitle,
                        body: notification_body,
                    },
                    sound: {
                        critical: false,
                        name: sound,
                        volume: 1.0
                    },
                    category: messageType,
                    "mutable-content": 1,
                    "content-available": 1
                }
              }
            },
            tokens: fcmKeys
          };

        this.logger.debug('PLUGIN | ALARMIATORMOBILE | PAYLOAD for DEBUGGING -------START-----------');
        this.logger.debug(JSON.stringify(fcmMessage));
        this.logger.debug('PLUGIN | ALARMIATORMOBILE | PAYLOAD for DEBUGGING -------END-------------');
        
        return fcmMessage;
    }

    /**
     * Building payload for config message
     * @param {string} subtype 
     * @param {string} configValue
     * @param {string} fcmToken
     */
    buildConfigPayload(subtype, configValue, fcmToken){

        var sound = "default";

        var messageType = "config";

        var organization = global.serverInstanceName;;

        /* Customize notification here for Android + iOS */
        var notification_title = organization;
        var notification_subtitle = "Konfiguration wurde geändert";
        var notification_body = "Klicke Bitte auf diese Benachrichtigung um die Änderungen anzuwenden.";
        /*************************************************/

        var payload = {  
            subtype: subtype,
            configValue: configValue,
            notification_payload: {
                title: notification_subtitle,   // switched with subtitle for android!!
                subtitle : notification_title,  // switched with title for android!!
                body: notification_body,
            }
          };
          
          // REFERENCE: https://firebase.google.com/docs/reference/fcm/rest/v1/projects.messages#top_of_page
          var message = {
            data: {
              payload: JSON.stringify(payload),
              messageType: messageType,
              serverUUID: global.serverUUID,
              content_available: "1"
            },
            fcmOptions: {
                analyticsLabel: messageType
            },
            android: {
                priority: "high",
            },
            apns: {
              payload: {
                aps: {
                  alert : {
                    title : notification_title,
                    subtitle : notification_subtitle,
                    body: notification_body,
                  },
                  sound: {
                    critical: false,
                    name: sound,
                    volume: 1.0
                  },
                  category: messageType,
                  "mutable-content": 1,
                  "content-available": 1
                }
              }
            },
            token: fcmToken
          };
        
        return message;
    }

    /**
     * Building payload for single token message
     * @param {string} subtype 
     * @param {string} title
     * @param {string} message
     * @param {string} fcmToken
     */
    buildPayloadForSingleMessage(subtype, title, message, fcmToken){

        // Refactor to single token!
        var sound = "default";

        var messageType = "message";

        var organization = global.serverInstanceName;


        /* Customize notification here for Android + iOS */
        var notification_title = "Nachricht für " + organization;
        var notification_subtitle = "Neue "+ subtype + "-Nachricht";
        var notification_body = message;
        /*************************************************/
        
        switch (subtype) {
            case "gearState":
                notification_title = "Statuswechsel " + organization;
                notification_subtitle = "Statuswechsel";
                notification_body = message; 
                break;
            default: 
                notification_title = "Nachricht für " + organization;
                notification_subtitle = "Neue "+ subtype + "-Nachricht";
                notification_body = message;
                break;
        }

        

        var payload = {  
            message: message,
            timeStamp: moment().format("DD.MM.YYYY, HH:mm:ss"),
            messageUUID: uuid.v1(),
            subtype: subtype,
            title: title,
            notification_payload: {
                title: notification_subtitle,
                subtitle : notification_title,
                body: notification_body,
            }
        };
          
          // REFERENCE: https://firebase.google.com/docs/reference/fcm/rest/v1/projects.messages#top_of_page
          var message = {
            data: {
              payload: JSON.stringify(payload),
              messageType: messageType,
              serverUUID: global.serverUUID,
              content_available: "1"
            },
            fcmOptions: {
                analyticsLabel: messageType
            },
            android: {
                priority: "high",
            },
            apns: {
              payload: {
                aps: {
                  alert : {
                    title : notification_title,
                    subtitle : notification_subtitle,
                    body: notification_body,
                  },
                  sound: {
                    critical: false,
                    name: sound,
                    volume: 1.0
                  },
                  category: messageType,
                  "mutable-content": 1,
                  "content-available": 1
                }
              }
            },
            token: fcmToken
          };
        
        return message;
    }
}

module.exports = PayloadBuilder;